-- _file_.e

include get.e
include machine.e
include _common_.e
include _types_.e
include _search_.e
include _debug_.e
include _conv_.e
include _sequence_.e
include _dll_.e
include _external_.e
include _w32routines_.e

atom mem4, mem8
mem4 = allocate(4)
mem8 = allocate(8)

global constant
  FILETYPE_UNDEFINED = -1,
  FILETYPE_NOT_FOUND = 0,
  FILETYPE_FILE = 1,
  FILETYPE_DIRECTORY = 2

------------------------------------------------------------------------------

global function remove_eol(sequence s)
--<function>
--<name>remove_eol</name>
--<digest>remove end of line characters from given sequence</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
-- puts(1, remove_eol("First Line\nSecond Line\r\nThird Line\rFourth Line") & "\n")
-- First Line Second Line Third Line Fourth Line
--</example>
--<see_also>
--</see_also>
--</function>
  s = replace_all(s, "\r\n", "")
  s = replace_all(s, "\r", "")
  s = replace_all(s, "\n", "")
  return s
end function

------------------------------------------------------------------------------

global function split_filename(sequence s)
--<function>
--<name>split_filename</name>
--<digest>splits a complete filename in drive, path, name and extension</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
-- s = split_filename("C:\path\filename.txt")
-- s is:
--.  [1] "C"
--.  [2] "\path"
--.  [3] "filename"
--.  [4] "txt"
--</example>
--<see_also>
--</see_also>
--</function>
  sequence drive, path, name, ext
  integer d, p, e     -- separators for drive, path and extension
  integer l

  l = length(s)
  drive = ""
  path = ""
  name = ""
  ext = ""
  if l = 0 then return {drive,path,name,ext} end if

  if equal(s, ".") then
    return {drive,".",name,ext}
  elsif equal(s, "..") then
    return {drive, "..",name,ext}
  end if
  d = find(':', s)
  if d > 1 then
    drive = s[1..d-1]
    s = remove(s, 1, d)
  end if
  p = rfind(SLASH, s, {})
  e = rfind('.', s, {})
  if (e > 1) and (e > p) then
    ext = s[e+1..$]
    s = remove(s, e, length(s))
  end if
  p = rfind(SLASH, s, {})
  if p > 0 then
    if p = 1 then
      path = {SLASH}
    else
      path = s[1..p-1]
    end if
    s = remove(s, 1, p)
  end if
  name = s
  return {drive,path,name,ext}
end function

------------------------------------------------------------------------------

global function file_drive(sequence filename)
--<function>
--<name>file_path</name>
--<digest>returns file drive</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
-- s = file_drive("C:\path\filename.txt") --> "C"
--</example>
--<see_also>file_path(), file_base(), file_ext(), file_name()</see_also>
--</function>
  sequence s

  s = split_filename(filename)
  return s[1]
end function

------------------------------------------------------------------------------

global function file_path(sequence filename)
--<function>
--<name>file_path</name>
--<digest>returns file path</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
-- s = file_path("C:\path\filename.txt") --> "\path"
--</example>
--<see_also>file_drive(), file_base(), file_ext(), file_name()</see_also>
--</function>
  sequence s

  s = split_filename(filename)
  return s[2]
end function

------------------------------------------------------------------------------

global function file_base(sequence filename)
--<function>
--<name>file_base</name>
--<digest>returns filename without extension or path</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
-- s = file_base("C:\path\filename.txt") --> "filename"
--</example>
--<see_also>file_drive(), file_path(), file_ext(), file_name()</see_also>
--</function>
  sequence s

  s = split_filename(filename)
  return s[3]
end function

------------------------------------------------------------------------------

global function file_ext(sequence filename)
--<function>
--<name>file_ext</name>
--<digest>returns file extension</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
-- s = file_ext("C:\path\filename.txt") --> "txt"
--</example>
--<see_also>file_drive(), file_path(), file_base(), file_name()</see_also>
--</function>
  sequence s

  s = split_filename(filename)
  return s[4]
end function

------------------------------------------------------------------------------

global function file_name(sequence filename)
--<function>
--<name>file_name</name>
--<digest>returns filename with extension, without path</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
-- s = file_name("C:\path\filename.txt") --> "filename.txt"
--</example>
--<see_also>file_drive(), file_path(), file_base(), file_ext()</see_also>
--</function>
  sequence s

  s = split_filename(filename)
  if length(s[4]) then
    return s[3] & "." & s[4]
  else
    return s[3]
  end if
end function

------------------------------------------------------------------------------

global function file_type(sequence filename)
--<function>
--<name>file_type</name>
--<digest>Get the type of a file</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc>name of the file to query. It must not have wildcards.</desc>
--</param>
--<return>
-- integer:
--     * -1 if file could be multiply defined
--     *  0 if filename does not exist
--     *  1 if filename is a file
--     *  2 if filename is a directory
--</return>
--<example>
--</example>
--<see_also>dir()</see_also>
--</function>
  object dirfil

  if find('*', filename) or find('?', filename) then
    return FILETYPE_UNDEFINED
  end if

  if platform() = WIN32 then
    if length(filename) = 2 and filename[2] = ':' then
      filename &= "\\"
    end if
  end if

  dirfil = dir(filename)
  if sequence(dirfil) then
    if length( dirfil ) > 1 or find('d', dirfil[1][2]) or (length(filename)=3 and filename[2]=':') then
      return FILETYPE_DIRECTORY
    else
      return FILETYPE_FILE
    end if
  else
    return FILETYPE_NOT_FOUND
  end if
end function

------------------------------------------------------------------------------

global function file_exists(sequence filename)
--<function>
--<name>file_exists</name>
--<digest>check if a file exists</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc>file to check for</desc>
--</param>
--<return>
--</return>
--<example>
-- ? file_exists("test_file_.bak")
-- 1
-- ? file_exists("test_file_.old")
-- 0
--</example>
--<see_also>
--</see_also>
--</function>
  if atom(dir(filename)) then
    return 0
  else
    return 1
  end if
end function

------------------------------------------------------------------------------

global function file_length(sequence filename)
--<function>
--<name>file_length</name>
--<digest>Returns the size of a file</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc>name of the file to search size of</desc>
--</param>
--<return>
-- atom: size of file if existing or -1 else
--</return>
--<example>
-- a = file_length("C:\\path\\filename")
--</example>
--<see_also>
--</see_also>
--</function>
  object   x

  x = dir(filename)        -- Get DIR information of file
  if atom(x) then return -1 end if
  if length(x) != 1 then return -1 end if
  return x[1][D_SIZE]
end function

------------------------------------------------------------------------------

integer xGetFileVersionInfoSizeA
xGetFileVersionInfoSizeA = 0

function getVerInfoSize(atom lpstrFilename)
  atom lpdwHandle -- Dummy handle parameter: DWORD
  integer res
  
  if platform() = WIN32 then
    lpdwHandle = allocate(4)
    res = exec_w32func(xGetFileVersionInfoSizeA, {lpstrFilename, lpdwHandle}, "GetFileVersionInfoSizeA")
    free(lpdwHandle)
    return res
  else
    return 0
  end if
end function

------------------------------------------------------------------------------

integer xGetFileVersionInfoA
xGetFileVersionInfoA = 0

function getVerInfo(atom lpstrFilename, integer dwLen, atom lpData)
  if platform() = WIN32 then
    if not exec_w32func(xGetFileVersionInfoA, {lpstrFilename, NULL, dwLen, lpData}, "GetFileVersionInfoA") then
      error_message("Can't load version information", 0)
    end if
    return lpData
  else
    return 0
  end if
end function

--------------------------------------------------------------------------------

-- from win32lib tk_maths.e
function lo_word( atom pData)
  return and_bits(pData, #FFFF)
end function

--------------------------------------------------------------------------------

-- from win32lib tk_maths.e
function hi_word( atom pData)
  return and_bits(and_bits(pData, #FFFF0000) / #10000, #FFFF)
end function

------------------------------------------------------------------------------

integer xVerQueryValueA
xVerQueryValueA = 0

function getVerInfoStr(atom lpData, sequence Path)
  atom pData     -- pointer to info data
  atom lDataLen  -- length of info pointed at by pData
  atom index
  sequence s

  if platform() = WIN32 then
    pData = allocate(4)
    lDataLen = allocate(4)
    s = {}
    -- Read the string: return "" if string doesn't exist
    if exec_w32func(xVerQueryValueA, {lpData, allocate_string(Path), pData, lDataLen}, "VerQueryValueA") then
      index = peek4u(pData)
      s = append(s, hi_word(peek4u(index+8)))
      s = append(s, lo_word(peek4u(index+8)))
      s = append(s, hi_word(peek4u(index+12)))
      s = append(s, lo_word(peek4u(index+12)))
      return sprintf("%d.%d.%d.%d", s)
    end if
    free(pData)
    free(lDataLen)
    return ""
  else
    return ""
  end if
end function

------------------------------------------------------------------------------

global function file_version(sequence FileName)
--<function>
--<name>file_version</name>
--<digest>returns file version</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>FileName</name>
--<desc>name of the bound executable program</desc>
--</param>
--<return>string: file version</return>
--<example>
-- cmd = command_line()
-- s = get_file_version(cmd[1])
--</example>
--<see_also></see_also>
--</function>
  integer dwLen     -- size of ver info buffer
  sequence res
  atom fVerInfo, lpstrFilename

  if platform() = WIN32 then
    lpstrFilename = allocate_string(FileName)
    res = {}
    -- Get size of buffer: no ver info if size = 0
    dwLen = getVerInfoSize(lpstrFilename)
    if dwLen > 0 then
      -- Read ver info into buffer
      fVerInfo = allocate(dwLen)
      fVerInfo = getVerInfo(lpstrFilename, dwLen, fVerInfo)
      res = getVerInfoStr(fVerInfo, "\\")
      -- Free ver info buffer
      free(fVerInfo)
      free(lpstrFilename)
    else
      return "none"
    end if
    return res
  else
    return ""
  end if
end function

------------------------------------------------------------------------------

global function is_absolute_path(sequence filename)
--<function>
--<name>is_absolute_path</name>
--<digest>Determine if the supplied string is an absolute path or a relative path</digest>
--<desc>
-- A <i>relative</i> path is one which is relative to the current directory and
-- an <i>absolute</i> path is one that doesn't need to know the current directory
-- to find the file.
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc>name of the file path</desc>
--</param>
--<return>
-- integer: 0 if <b>filename</b> is a relative path or 1 otherwise
--</return>
--<example>
-- ? is_absolute_path("") -- returns 0
-- ? is_absolute_path("/usr/bin/abc") -- returns 1
-- ? is_absolute_path("\\temp\\somefile.doc") -- returns 1
-- ? is_absolute_path("../abc") -- returns 0
-- ? is_absolute_path("local/abc.txt") -- returns 0
-- ? is_absolute_path("abc.txt") -- returns 0
-- ? is_absolute_path("c:..\\abc") -- returns 0
-- ? is_absolute_path("c:\\windows\\system32\\abc") -- returns 1
-- ? is_absolute_path("c:/windows/system32/abc") -- returns 1
--</example>
--<see_also>
--</see_also>
--</function>
  if length(filename) = 0 then
    return 0
  end if

  if find(filename[1], "\\/:") then
    return 1
  end if

    if length(filename) = 1 then
      return 0
    end if

    if filename[2] != ':' then
      return 0
    end if

    if length(filename) < 3 then
      return 0
    end if

    if find(filename[3], "\\/:") then
      return 1
    end if
  return 0
end function

------------------------------------------------------------------------------

global function dir_size(sequence rootdir)
--<function>
--<name>dir_size</name>
--<digest>Returns sum of all file sizes from rootdir and all subdirs</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>rootdir</name>
--<desc>directory to search size of</desc>
--</param>
--<return>
-- atom
--</return>
--<example>
-- a = dir_size("C:\\path\\subpath")
--</example>
--<see_also>
--</see_also>
--</function>
  object   x
  sequence s, name
  atom     bytes

  s = split_filename(rootdir)
  name = rootdir[length(s[1])+2..$]
  if platform() = WIN32 then
    x = dir(rootdir & SLASH & "*.*")        -- Get DIR information of all files
  else
    x = dir(rootdir & SLASH & "*")        -- Get DIR information of all files
  end if
  bytes = 0
  if sequence(x) then             -- Exclude forbidden DIRs
    for i = 1 to length(x) do   -- Loop through DIR content
      s = x[i]
      if find('d',s[D_ATTRIBUTES]) then  -- Check for SubDIRs
        if not (equal(s[D_NAME],".") or equal(s[D_NAME],"..")) then
          bytes += dir_size(rootdir & "\\" & s[D_NAME])
        end if
      else
        bytes += s[D_SIZE]       -- Sum Bytes!
      end if
    end for
  end if
  return bytes
end function

------------------------------------------------------------------------------

global function parent_dir(sequence path)
--<function>
--<name>parent_dir</name>
--<digest>returns parent dir of given path</digest>
--<desc>
-- remove last sub directory from path if there is one and return it.
-- if root return empty sequence
-- path has to point to a folder and not to a file
--</desc>
--<param>
--<type>sequence</type>
--<name>path</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
-- parent_dir(".") --> ".."
-- parent_dir("C:\path\subpath") --> "\path"
--</example>
--<see_also>
--</see_also>
--</function>
  if ends({SLASH}, path) then
    if length(path) = 1 then
      return 0
    else
      path = path[1..$-1]
    end if
  end if
  if equal(path, ".") then
    return ".."
  elsif equal(path, "..") then
    return ".." & SLASH & ".."
  elsif ends("..", path) then
    return path & SLASH & ".."
  elsif ends(".", path) then
    return file_path(path) & SLASH & ".."
  end if
  if equal(path, ".") then
    return ".."
  elsif equal(path, "..") then
    return ".." & SLASH & ".."
  else
    return file_path(path)
  end if
end function -- parent_dir()

------------------------------------------------------------------------------

global procedure replace_text(sequence old_file, sequence new_file, sequence old, sequence new)
--<procedure>
--<name>replace_text</name>
--<digest>replaces all occurences of a sub-string by another in a file</digest>
--<desc>
-- result is written in a new file
-- old file is not affected
--</desc>
--<param>
--<type>sequence</type>
--<name>old_file</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>new_file</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>old</name>
--<desc>text to be replaced</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>new</name>
--<desc>new text</desc>
--</param>
--<example>
-- replace_text("test_file_.bak", "new_file_.bak", "this", "that")
--</example>
--<see_also>
--</see_also>
--</procedure>
  sequence s
  integer f_in, f_tmp
  object line

  s = {}
  f_in = open(old_file, "r")
  f_tmp = open("temp.$$$", "w")
  if (f_in != -1) and (f_tmp != -1) then
    while 1 do
      line = gets(f_in)
      if atom(line) then
        exit
      end if
      puts(f_tmp, replace_all(line, old, new))
    end while
    close(f_in)
    close(f_tmp)
  end if
  system("copy temp.$$$ " & new_file, 2)
  system("del temp.$$$", 2)
end procedure

------------------------------------------------------------------------------

global function read_lines(sequence filename)
--<function>
--<name>read_lines</name>
--<digest>reads content of a text file as a sequence of strings</digest>
--<desc>
-- one line of text per string
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
-- s = read_lines("new_file_.bak")
-- s may be (example):
-- .  [1] "-- _file_.e"
-- .  [2] ""
-- .  [3] "include lib/_file_.e"
-- .  [4] "include lib/_debug_.e"
-- .  [5] "include get.e"
-- ...
--</example>
--<see_also>write_lines(), read_file()</see_also>
--</function>
  sequence lines
  integer f_in, n
  object line, s

  lines = {}
  f_in = open(filename, "r")
  if f_in != -1 then
    while 1 do
      line = gets(f_in)
      if atom(line) then exit end if
      if begins({#EF,#BB,#BF}, line) then  -- UTF-8 BOM
        line = remove(line, 1, 3)
      end if
      s = match_any({"\r\n", "\r", "\n"}, line, {})
      if sequence(s) then
        n = s[1]
        if n then line = line[1..n-1] end if
      end if
      lines = append(lines, line)
    end while
    close(f_in)
  end if
  return lines
end function

------------------------------------------------------------------------------

global procedure write_lines(sequence filename, sequence lines)
--<procedure>
--<name>write_lines</name>
--<digest>writes a sequence of strings in a text file</digest>
--<desc>
-- one line of text per string
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>lines</name>
--<desc></desc>
--</param>
--<example>
-- write_lines("new_file.txt", {"first line","second line"})
-- puts(1, read_file("new_file.txt") & "\n")
-- first line
-- second line
--</example>
--<see_also>read_lines(), append_lines(), write_file()</see_also>
--</procedure>
  integer f_out

  f_out = open(filename, "w")
  if f_out != -1 then
    for i = 1 to length(lines) do
      puts(f_out, lines[i]&"\n")
    end for
    close(f_out)
  end if
end procedure

------------------------------------------------------------------------------

global procedure append_lines(sequence filename, sequence lines)
--<procedure>
--<name>append_lines</name>
--<digest>appends a sequence of strings to a text file</digest>
--<desc>
-- one line of text per string
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>lines</name>
--<desc></desc>
--</param>
--<example>
-- write_lines("file.txt", {"first line","second line"})
-- append_lines("file.txt", {"next line","second next line"})
-- puts(1, read_file("file.txt") & "\n")
-- first line
-- second line
-- next line
-- second next line
--</example>
--<see_also>read_lines(), write_lines(), write_file()</see_also>
--</procedure>
  integer f_out

  f_out = open(filename, "a")
  if f_out != -1 then
    for i = 1 to length(lines) do
      puts(f_out, lines[i]&"\n")
    end for
    close(f_out)
  end if
end procedure

------------------------------------------------------------------------------

global function read_file(sequence filename)
--<function>
--<name>read_file</name>
--<digest>reads content of a text file as a sequence of bytes</digest>
--<desc>file may be text or binary</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
-- write_file("new_file.txt", {1,2,3})
-- ? read_file("new_file.txt")
-- {1,2,3}
--</example>
--<see_also>write_file(), read_lines()</see_also>
--</function>
  integer fn
  sequence whole_file, chunk

  whole_file = {}
  fn = open(filename, "rb")  -- an existing file
  while 1 do
    chunk = get_bytes (fn, 100) -- read 100 bytes at a time
    whole_file &= chunk        -- chunk might be empty, that's ok
    if length(chunk) < 100 then exit end if
  end while
  close(fn)
  return whole_file
end function

------------------------------------------------------------------------------

global procedure write_file(sequence filename, sequence content)
--<procedure>
--<name>write_file</name>
--<digest>writes a sequence of byte in a file</digest>
--<desc>file may be text or binary</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>content</name>
--<desc></desc>
--</param>
--<example>
-- write_file("new_file.txt", {1,2,3})
-- ? read_file("new_file.txt")
-- {1,2,3}
--</example>
--<see_also>read_file(), read_lines()</see_also>
--</procedure>
  integer f_out

  f_out = open(filename, "wb")
  if f_out != -1 then
    puts(f_out, content)
    close(f_out)
  end if
end procedure

------------------------------------------------------------------------------

global function read_csv(sequence filename, integer skip_headers)
--<function>
--<name>read_csv</name>
--<digest>reads a CSV file in a sequence</digest>
--<desc>
-- CSV separator is a semicolon
-- CSV strings may be surrounded by quotes
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc>CSV file to read</desc>
--</param>
--<param>
--<type>integer</type>
--<name>skip_headers</name>
--<desc>if set skip first line in CSV file</desc>
--</param>
--<return>
--</return>
--<example>
-- analyze_object(read_csv("test.csv", 0), "read_csv", {})
-- read_csv =
-- .  [1]
-- .  .  [1] "Name"
-- .  .  [2] "Bytes"
-- .  .  [3] "Size"
-- .  .  [4] "#Files"
-- .  .  [5] "Most recent"
-- .  [2]
-- .  .  [1] "Z:\"
-- .  .  [2] 56144659864
-- .  .  [3] "52,29 GB"
-- .  .  [4] 23855
-- .  .  [5] 2016
-- .  [3]
-- .  .  [1] "Z:" & SLASH & "BlackBox"
-- .  .  [2] 81699302
-- .  .  [3] "77,91 MB"
-- .  .  [4] 4
-- .  .  [5] 2012
-- .  [4]
-- .  .  [1] "Z:" & SLASH & "Dying_Gasp"
-- .  .  [2] 7757074
-- .  .  [3] "7,40 MB"
-- .  .  [4] 66
-- .  .  [5] 2015
-- .  [5]
-- .  .  [1] "Z:" & SLASH & "Temp"
-- .  .  [2] 812629032
-- .  .  [3] "774,98 MB"
-- .  .  [4] 641
-- .  .  [5] 2016
-- analyze_object(read_csv("test.csv", 1), "read_csv", {})
-- read_csv =
-- .  [1]
-- .  .  [1] "Z:\"
-- .  .  [2] 56144659864
-- .  .  [3] "52,29 GB"
-- .  .  [4] 23855
-- .  .  [5] 2016
-- .  [2]
-- .  .  [1] "Z:" & SLASH & "BlackBox"
-- .  .  [2] 81699302
-- .  .  [3] "77,91 MB"
-- .  .  [4] 4
-- .  .  [5] 2012
-- .  [3]
-- .  .  [1] "Z:" & SLASH & "Dying_Gasp"
-- .  .  [2] 7757074
-- .  .  [3] "7,40 MB"
-- .  .  [4] 66
-- .  .  [5] 2015
-- .  [4]
-- .  .  [1] "Z:" & SLASH & "Temp"
-- .  .  [2] 812629032
-- .  .  [3] "774,98 MB"
-- .  .  [4] 641
-- .  .  [5] 2016
--</example>
--<see_also>
--</see_also>
--</function>
  sequence section
  integer f_in
  object line

  section = {}
  f_in = open(filename, "r")
  if f_in != -1 then
    if skip_headers then
      line = gets(f_in)
      if atom(line) then
        close(f_in)
        return section
      end if
    end if

    while 1 do
      line = gets(f_in)
      if atom(line) then
        close(f_in)
        return section
      end if
--      line = trim(line[1..length(line)-1])
      line = trim(line)
      if length(line)>0 then
        section = append(section, split_n_convert(line, ";") )
      end if
    end while
    close(f_in)
  else
    error_message("Cannot open " & filename & "!", 1)
  end if
  return section
end function

------------------------------------------------------------------------------

global function sequence_to_csv(sequence section)
--<function>
--<name>sequence_to_csv</name>
--<digest>converts a sequence to a CSV line</digest>
--<desc>
-- CSV separator is a semicolon
-- CSV strings are surrounded by quotes
--</desc>
--<param>
--<type>sequence</type>
--<name>section</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
-- puts(1, sequence_to_csv({1,"Two",3,"Four"}) & "\n")
-- 1;"Two";3;"Four"
--</example>
--<see_also>
--</see_also>
--</function>
  integer l
  sequence line

  if atom(section) then
    line = to_string(section)
  else
    l = length(section)
    line = {}
    if l > 0 then
      for j = 1 to l-1 do
        if atom(section[j]) or is_date(section[j]) then
          line &= to_string(section[j])&";"
        else
          line &= "\""&to_string(section[j])&"\";"
        end if
      end for
      if atom(section[l]) or is_date(section[l]) then
        line &= to_string(section[l])
      else
        line &= "\""&to_string(section[l])&"\""
      end if
    end if
  end if
  return line
end function

------------------------------------------------------------------------------

global procedure write_csv(sequence filename, sequence section)
--<procedure>
--<name>write_csv</name>
--<digest>writes a sequence of lines in a CSV file</digest>
--<desc>
-- CSV separator is a semicolon
-- CSV strings are surrounded by quotes
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>section</name>
--<desc></desc>
--</param>
--<example>
-- write_csv("test2.csv", {{1,"Two",3,"Four"},{1,"Two",3,"Four"}})
-- analyze_object(read_csv("test2.csv", 1), "read_csv", {})
-- read_csv =
-- .  [1]
-- .  .  [1] 1
-- .  .  [2] "Two"
-- .  .  [3] 3
-- .  .  [4] "Four"
--</example>
--<see_also>
--</see_also>
--</procedure>
  integer f_out

  f_out = open(filename, "w")
  if f_out != -1 then
    for i = 1 to length(section) do
      puts(f_out, sequence_to_csv(section[i])&"\n")
    end for
    close(f_out)
  end if
end procedure

------------------------------------------------------------------------------

global procedure append_csv(sequence filename, sequence section)
--<procedure>
--<name>append_csv</name>
--<digest>appends a sequence of lines to a CSV file</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>section</name>
--<desc></desc>
--</param>
--<example>
-- append_csv("test2.csv", {{1,"Two",3,"Four"},{1,"Two",3,"Four"}})
--</example>
--<see_also>
--</see_also>
--</procedure>
  integer f_out

  f_out = open(filename, "a")
  if f_out != -1 then
    for i = 1 to length(section) do
      puts(f_out, sequence_to_csv(section[i])&"\n")
    end for
    close(f_out)
  end if
end procedure

-----------------------------------------------------------------------------

global function find_param_name(sequence list, sequence param)
--<function>
--<name>find_param_name</name>
--<digest>find the index of a parameter in a list of strings "parameter=value"</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>list</name>
--<desc>list of strings to search in</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>param</name>
--<desc>parameter name to search for</desc>
--</param>
--<return>
-- integer: requested index
--</return>
--<example>
-- sequence params
-- params = {"first=1","second=2","third=three"}
-- ? find_param_name(params, "third")
-- 3
--</example>
--<see_also>set_param_value(), get_param_value()</see_also>
--</function>
  for i = 1 to length(list) do
    if match(param & "=", list[i]) then  -- param matches
      return i
    end if
  end for
  return 0
end function

-----------------------------------------------------------------------------

global function get_param_value(sequence list, sequence param, sequence default)
--<function>
--<name>get_param_value</name>
--<digest>find the value of a parameter in a list of strings "parameter=value"</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>list</name>
--<desc>list of strings to search in</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>param</name>
--<desc>parameter name to search for</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>default</name>
--<desc>default value if not found. defaults to ""</desc>
--</param>
--<return>
-- sequence: requested value if found, default value if not
--</return>
--<example>
-- puts(1,get_param_value(params, "second") & "\n")
-- 2
--</example>
--<see_also>find_param_name(), set_param_value()</see_also>
--</function>
  integer i, n

  i = find_param_name(list, param)
  if i then  -- param matches
    n = find('=', list[i])
    if n then
      return dequote(trim(list[i][n+1..$]), {})
    else
      return ""
    end if
  end if
  return default
end function

-----------------------------------------------------------------------------

global function set_param_value(sequence list, sequence param, sequence value, sequence optional)
--<function>
--<name>set_param_value</name>
--<digest>sets the value of a parameter in a list of strings "parameter=value"</digest>
--<desc>
-- if sequence optional is empty, a parameter value can only be modified
-- if optional "add" is set, a non-existing parameter will be added
--</desc>
--<param>
--<type>sequence</type>
--<name>list</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>param</name>
--<desc>parameter name</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>value</name>
--<desc>parameter value</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>optional</name>
--<desc>
-- optional "add" indicates to add value if not already existing else nothing is added
--</desc>
--</param>
--<return>
-- sequence: modified list
--</return>
--<example>
-- sequence params
-- params = {"first=1","second=2","third=three"}
-- analyze_object(set_param_value(params, "third", "3", {}), "set_param_value", {})
-- set_param_value =
-- .  [1] "first=1"
-- .  [2] "second=2"
-- .  [3] "third=3"
-- analyze_object(set_param_value(params, "fourth", "4", {}), "set_param_value", {})
-- set_param_value =
-- .  [1] "first=1"
-- .  [2] "second=2"
-- .  [3] "third=three"
--    ("add" is not set so parameter "fourth" is not added)
-- analyze_object(set_param_value(params, "fourth", "4", {{"add", 1}}), "set_param_value", {})
-- set_param_value =
-- .  [1] "first=1"
-- .  [2] "second=2"
-- .  [3] "third=three"
-- .  [4] "fourth=4"
--</example>
--<see_also>find_param_name(), get_param_value()</see_also>
--</function>
  integer i, eq, add

  add = get_option("add", optional, 0)
  i = find_param_name(list, param)
  if i then  -- param matches
    eq = find('=', list[i])
    list[i] = list[i][1..eq] & value
  else
    if add then
      list = append(list, param & "=" & value)
    end if
  end if
  return list
end function

------------------------------------------------------------------------------

global function read_ini_file(sequence filename)
--<function>
--<name>read_ini_file</name>
--<digest>reads a file structured as an INI file</digest>
--<desc>
-- result is a sequence { {section name, section content}, ... }
-- each section content is a list of parameters and values { "parameter=value", ... }
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
-- file test.ini contains following lines:
-- [Telnet]
-- host=127.0.0.1
-- port=23
--
-- [SSH]
-- host=127.0.0.1
-- port=22
--
-- [FTP]
-- host=127.0.0.1
-- port=21
--
-- analyze_object(read_ini_file("test.ini"), "read_ini", {})
-- read_ini =
-- .  [1]
-- .  .  [1] "Telnet"
-- .  .  [2]
-- .  .  .  [1] "host=127.0.0.1"
-- .  .  .  [2] "port=23"
-- .  [2]
-- .  .  [1] "SSH"
-- .  .  [2]
-- .  .  .  [1] "host=127.0.0.1"
-- .  .  .  [2] "port=22"
-- .  [3]
-- .  .  [1] "FTP"
-- .  .  [2]
-- .  .  .  [1] "host=127.0.0.1"
-- .  .  .  [2] "port=21"
--</example>
--<see_also>write_ini_file()</see_also>
--</function>
  integer p
  sequence content, section, params, lines

  content = {}
  section = ""
  params = {}
  if not file_exists(filename) then return 0 end if
  lines = read_lines(filename)
  if begins({#EF,#BB,#BF}, lines[1]) then  -- UTF-8 BOM
    lines[1] = remove(lines[1], 1, 3)
  end if
  for i = 1 to length( lines ) do
    if length(lines[i]) then
      -- check section
      if lines[i][1] = '[' then
        p = find(']', lines[i])
        if p = 0 then
          log_puts("Missing closing bracket: exiting")
          return 0
        end if
        if length(section) then
          content = append(content, {section, params})
        end if
        section = lines[i][2..p-1]
        params = {}
      else
        if length(section)=0 then
          log_puts("Missing section: exiting")
          return 0
        end if
        -- read params
        lines[i] = trim_head(lines[i])
        if (length(lines[i]) > 0) and not begins(";", lines[i]) then
          params = append(params, lines[i])
        end if
      end if
    end if
  end for
  if length(section) then
    content = append(content, {section, params})
  end if
  return content
end function

------------------------------------------------------------------------------

global procedure write_ini_file(sequence filename, sequence content)
--<procedure>
--<name>write_ini_file</name>
--<digest>writes a sequence of lines in an INI file</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>content</name>
--<desc>
-- { {section name, section content}, ... }
-- each section content is a list of parameters and values { "parameter=value", ... }
--</desc>
--</param>
--<example>
-- write_ini_file("test2.ini", {{"Section",{"name=value"}}})
-- [Section]
-- name=value
--</example>
--<see_also>read_ini_file()</see_also>
--</procedure>
  integer f_out

  f_out = open(filename, "w")
  for i = 1 to length(content) do
    puts(f_out, "["&content[i][1]&"]\n")
    for j = 1 to length(content[i][2]) do
      if length(content[i][2]) > 0 then
        puts(f_out, content[i][2][j]&"\n")
      end if
    end for
    puts(f_out, "\n")
  end for
  close(f_out)
end procedure

------------------------------------------------------------------------------

global function get_ini_section(sequence ini, sequence section)
--<function>
--<name>get_ini_section</name>
--<digest>extracts a section from an INI file content</digest>
--<desc>
-- file content has to be read previously with read_ini_file()
--</desc>
--<param>
--<type>sequence</type>
--<name>ini</name>
--<desc>INI file content</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>section</name>
--<desc>section name</desc>
--</param>
--<return>
-- sequence: selected section content
--</return>
--<example>
-- ini = read_ini_file("test.ini")
-- s = get_ini_section(ini, "SSH")
-- s may be (example):
-- .  [1] "host=127.0.0.1"
-- .  [2] "port=22"
--</example>search_ini(), read_ini()<see_also>
--</see_also>
--</function>
  object x

  x = find_in_array(section, ini, {{"target_field", -1}})
  if atom(x) then return 0 end if
  if length(x)<2 then return 0 end if
  return x[2]
end function

------------------------------------------------------------------------------

global function search_ini(sequence ini, sequence section, sequence key,
                           sequence default)
--<function>
--<name>search_ini</name>
--<digest>reads an element of an INI file</digest>
--<desc>
-- file content has to be read previously with read_ini_file()
--</desc>
--<param>
--<type>sequence</type>
--<name>ini</name>
--<desc>INI file content</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>section</name>
--<desc>section of ini file to search in</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>key</name>
--<desc>key to search for</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>default</name>
--<desc>default value returned if search fails</desc>
--</param>
--<return>
-- sequence: searched parameter value or default value if not found
--</return>
--<example>
-- puts(1, search_ini(ini, "FTP", "port") & "\n")
-- 21
--</example>
--<see_also>read_ini(), write_ini()</see_also>
--</function>
  object words

  words = get_ini_section(ini, section)
  -- analyze_object(words, "words", {{"output",f_debug}})
  if sequence(words) and length(words) then
    return get_param_value(words, key, default)
  end if
  return default
end function

------------------------------------------------------------------------------

global function read_ini(sequence file, sequence section, sequence name, object default)
--<function>
--<name>read_ini</name>
--<digest>reads an element of an INI file</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>file</name>
--<desc>INI file name</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>section</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>name</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>default</name>
--<desc>default value to return if read failed</desc>
--</param>
--<return>
--</return>
--<example>
-- puts(1, read_ini("test.ini", "Telnet", "port", -1) & "\n")
-- 23
--</example>
--<see_also>search_ini(), write_ini()</see_also>
--</function>
  sequence secParams
  object ini, x

  ini = read_ini_file(file)
  if atom(ini) then return default end if
  x = get_ini_section(ini, section)
  if atom(x) then return default else secParams = x end if
  x = get_param_value(secParams, name, "")
  if atom(x) then return default else return x end if
end function

------------------------------------------------------------------------------

global procedure write_ini(sequence file, sequence section, sequence name, object value)
--<procedure>
--<name>write_ini</name>
--<digest>writes an element of an INI file</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>file</name>
--<desc>INI file name</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>section</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>name</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>value</name>
--<desc></desc>
--</param>
--<example>
-- write_ini("test.ini", "Telnet", "port", "223")
-- [Telnet]
-- host=127.0.0.1
-- port=223
--
-- write_ini("test.ini", "HTTP", "host", "127.0.0.1")
-- [HTTP]
-- host=127.0.0.1
--
-- write_ini("test.ini", "HTTP", "port", "80")
-- [HTTP]
-- host=127.0.0.1
-- port=80
--</example>
--<see_also>search_ini(), read_ini()</see_also>
--</procedure>
  sequence s
  object ini
  integer sec

  ini = read_ini_file(file)
  if atom(ini) then return end if
  if atom(value) then value = to_string(value) end if
  sec = find_in_array(section, ini, {})   --find section
  if sec=0 then    -- new section
    ini = append(ini, {section, {name & "=" & value}})
  else   -- existing section
    s = ini[sec]
    if length(s)<2 then  -- empty section
      s = append( s, {name & "=" & value} )
    else
      s[2] = set_param_value(s[2], name, value, {{"add", 1}})
    end if
    ini[sec] = s
  end if
  write_ini_file(file, ini)
end procedure


------------------------------------------------------------------------------

global function get_uint8(integer fn)
--<function>
--<name>get_uint8</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- read a single unsigned byte at current position in file
  integer i

  i = getc(fn)
  if i = -1 then   -- end of file
    return {GET_EOF, 0}
  else
    return {GET_SUCCESS, i}
  end if
end function

------------------------------------------------------------------------------

global function get_int8(integer fn)
--<function>
--<name>get_int8</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- read a single signed byte at current position in file
  integer i

  i = getc(fn)
  if i = -1 then   -- end of file
    return {GET_EOF, 0}
  else
    if i >= #80 then
      return {GET_SUCCESS, i-#100}
    else
      return {GET_SUCCESS, i}
    end if
  end if
end function

------------------------------------------------------------------------------

global function get_uint16(integer fn)
--<function>
--<name>get_uint16</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- read a 2 byte unsigned word at current position in file
  sequence s

  s = get_bytes(fn, 2)
  if length(s) < 2 then   -- end of file
    return {GET_EOF, 0}
  else
    return {GET_SUCCESS, s[1] + s[2]*#100}
  end if
end function

------------------------------------------------------------------------------

global function get_int16(integer fn)
--<function>
--<name>get_int16</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- read a 2 byte signed word at current position in file
  integer i
  sequence s

  s = get_bytes(fn, 2)
  if length(s) < 2 then   -- end of file
    return {GET_EOF, 0}
  else
    i = s[1] + s[2]*#100
    if i >= #8000 then
      return {GET_SUCCESS, i - #10000}
    else
      return {GET_SUCCESS, i}
    end if
  end if
end function

------------------------------------------------------------------------------

global function get_uint32(integer fn)
--<function>
--<name>get_uint32</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- read a 4 byte unsigned integer at current position in file
  sequence s

  s = get_bytes(fn, 4)
  if length(s) < 4 then   -- end of file
    return {GET_EOF, 0}
  else
    return {GET_SUCCESS, bytes_to_int(s)}
  end if
end function

------------------------------------------------------------------------------

global function get_int32(integer fn)
--<function>
--<name>get_int32</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- read a 4 byte signed integer at current position in file
  atom a
  sequence s

  s = get_bytes(fn, 4)
  if length(s) < 4 then   -- end of file
    return {GET_EOF, 0}
  else
    a = bytes_to_int(s)
    if a >= #80000000 then
      return {GET_SUCCESS, a - #100000000}
    else
      return {GET_SUCCESS, a}
    end if
  end if
end function

------------------------------------------------------------------------------

global function get_uint64(integer fn)
--<function>
--<name>get_uint64</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- read a 8 byte long integer at current position in file
  sequence s

  s = get_bytes(fn, 8)
  if length(s) < 8 then   -- end of file
    return {GET_EOF, 0.0}
  else
    return {GET_SUCCESS, bytes_to_int(s[1..4]) + bytes_to_int(s[5..8]*#10000)}
  end if
end function

------------------------------------------------------------------------------

global function get_float32(integer fn)
--<function>
--<name>get_float32</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- read a 4 byte float at current position in file
  sequence s

  s = get_bytes(fn, 4)
  if length(s) < 4 then   -- end of file
    return {GET_EOF, 0.0}
  else
    return {GET_SUCCESS, float32_to_atom(s)}
  end if
end function

------------------------------------------------------------------------------

global function get_float64(integer fn)
--<function>
--<name>get_float64</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- read a 8 byte float at current position in file
  sequence s

  s = get_bytes(fn, 8)
  if length(s) < 8 then   -- end of file
    return {GET_EOF, 0.0}
  else
    return {GET_SUCCESS, float64_to_atom(s)}
  end if
end function

------------------------------------------------------------------------------

global function get_string(integer fn)
--<function>
--<name>get_string</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- read a variable size 0-terminated string at current position in file
  sequence s
  integer n

  s = {}
  while 1 do
    n = getc(fn)
    if n = -1 then return {GET_EOF, ""} elsif n = 0 then exit end if
    s = append(s, n)
  end while
  return {GET_SUCCESS, s}
end function

------------------------------------------------------------------------------

global procedure put_int8(integer fn, integer i)
--<procedure>
--<name>put_int8</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>i</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
-- ok = put a single signed/unsigned byte at current position in file
  if i > #100 then return end if
  puts(fn, i)
end procedure

------------------------------------------------------------------------------

global procedure put_int16(integer fn, integer i)
--<procedure>
--<name>put_int16</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>i</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
-- ok = put a 2 byte signed/unsigned word at current position in file
  integer l, h

  if i > #10000 then return end if
  l = remainder(i, #100)
  h = floor(i / #100)
  puts(fn, {l, h})
end procedure

------------------------------------------------------------------------------

global procedure put_int32(integer fn, atom a)
--<procedure>
--<name>put_int32</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>a</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
-- ok = put a 4 byte signed/unsigned integer at current position in file
  sequence s

  if a > #100000000 then return end if
  poke4(mem4, a)
  s = peek({mem4, 4})
  puts(fn, s)
end procedure

------------------------------------------------------------------------------

global procedure put_int64(integer fn, atom a)
--<procedure>
--<name>put_int64</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>a</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
-- ok = put a 8 byte signed/unsigned long integer at current position in file
  sequence s

  if a > #10000000000000000 then return end if
  poke4(mem8, remainder(a, #100000000))
  poke4(mem8+4, floor(a / #100000000))
  s = peek({mem8, 8})
  puts(fn, s)
end procedure

------------------------------------------------------------------------------

global procedure put_float32(integer fn, atom a)
--<procedure>
--<name>put_float32</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>a</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
-- ok = put a 4 byte float at current position in file
  sequence s

  s = atom_to_float32(a)
  puts(fn, s)
end procedure

------------------------------------------------------------------------------

global procedure put_float64(integer fn, atom a)
--<procedure>
--<name>put_float64</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>a</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
-- ok = put a 8 byte float at current position in file
  sequence s

  s = atom_to_float64(a)
  puts(fn, s)
end procedure

------------------------------------------------------------------------------

global procedure put_string(integer fn, sequence s)
--<procedure>
--<name>put_string</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>fn</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc></desc>
--</param>
--<see_also>
--</see_also>
--</procedure>
-- put a variable size 0-terminated string at current position in file
  sequence buffer

  buffer = s&0
  puts(fn, buffer)
end procedure

------------------------------------------------------------------------------

integer xDeleteFile
xDeleteFile = 0

global function delete_file(sequence name)
--<function>
--<name>delete_file</name>
--<digest>Delete a file.</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>name</name>
--<desc>name of the file to delete.</desc>
--</param>
--<return>
-- integer: 0 on failure, 1 on success.
--</return>
--<example>
--</example>
--<see_also></see_also>
--</function>
  integer ret
  atom pfilename

  pfilename = allocate_string(name)
  if platform() = LINUX then
    ret = not exec_func(xDeleteFile, {pfilename}, "unlink")
  elsif platform() = WIN32 then
    ret = exec_w32func(xDeleteFile, {pfilename}, "DeleteFileA")
  end if
  free(pfilename)
  return ret
end function

------------------------------------------------------------------------------

integer xCopyFile
xCopyFile = 0

global function copy_file(sequence src, sequence dest, sequence optional)
--<function>
--<name>copy_file</name>
--<digest>Copy a file.</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>src</name>
--<desc>name of the file or directory to copy</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>dest</name>
--<desc>new name or location of the file</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>optional</name>
--<desc>
-- optional "overwrite" set to 0 (the default) will prevent an existing
--   destination file from being overwritten.
--   Non-zero will overwrite the destination file.
--</desc>
--</param>
--<return>
-- integer: 0 on failure, 1 on success.
--</return>
--<example>
-- copy_file("test_file_.exw", "test_file_.bak", {})
--         1 file(s) copied.
--</example>
--<see_also>move_file(), rename_file()</see_also>
--</function>
  integer s, d, success, overwrite
  sequence chunk
  atom psrc, pdest

  overwrite = get_option("overwrite", optional, 0)
  if length(dest) then
    if file_type( dest ) = FILETYPE_DIRECTORY then
      if dest[$] != SLASH then
        dest &= SLASH
      end if
      dest &= file_name(src)
    end if
  end if

  if platform() = WIN32 then
    psrc = allocate_string(src)
    pdest = allocate_string(dest)
    success = exec_w32func(xCopyFile, {psrc, pdest, not overwrite}, "CopyFileA")
    free(pdest)
    free(psrc)
  else
    if file_exists(src) then
      if not file_exists(dest) or overwrite then
        s = open(src, "rb")
        if s = -1 then return 0 end if
        d = open(dest, "wb")
        if d = -1 then close(s) return 0 end if
        while 1 do
          chunk = get_bytes(s, 4096) -- read 4096 bytes at a time
          puts(d, chunk)
          if length(chunk) < 4096 then exit end if
        end while
        close(s)
        close(d)
      end if
    end if
    success = 1
  end if

  return success

end function

------------------------------------------------------------------------------

integer xMoveFile
xMoveFile = 0

global function move_file(sequence src, sequence dest, sequence optional)
--<function>
--<name>move_file</name>
--<digest>Move a file to another location</digest>
--<desc>
-- If <b>overwrite</b> was requested but the move fails, any existing
-- destination file is preserved.
--</desc>
--<param>
--<type>sequence</type>
--<name>src</name>
--<desc>name of the file or directory to move</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>dest</name>
--<desc>new location for the file</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>optional</name>
--<desc>
-- optional "overwrite" set to 0 (default) prevents overwriting an existing
--   destination file,
--   If set to 1, existing destination file is overwritten
--</desc>
--</param>
--<return>
-- integer: 0 on failure, 1 on success.
--</return>
--<example>
--</example>
--<see_also>rename_file(), copy_file()</see_also>
--</function>
  atom psrc, pdest, ret
  sequence tempfile
  integer overwrite

  overwrite = get_option("overwrite", optional, 0)
  psrc = 0
  pdest = 0
  tempfile = ""

  if not file_exists(src) then return 0 end if

  if not overwrite then
    if file_exists( dest ) then return 0 end if
  end if

  psrc  = allocate_string(src)
  pdest = allocate_string(dest)

  if overwrite then
    -- return value is ignored, we don't care if it existed or not
    tempfile = "temp.$$$"
    void = move_file(dest, tempfile, {})
  end if

  if platform() = LINUX then
    ret = not exec_func(xMoveFile, {psrc, pdest}, "rename")
  elsif platform() = WIN32 then
    ret = exec_w32func(xMoveFile, {psrc, pdest}, "MoveFileA")
  end if

  free(pdest)
  free(psrc)

  if overwrite then
    if not ret then
      -- move was unsuccessful so restore tempfile
      void = move_file(tempfile, dest, {})
    end if
    void = delete_file(tempfile)
  end if
  return ret
end function

------------------------------------------------------------------------------

global function rename_file(sequence old_name, sequence new_name, sequence optional)
--<function>
--<name>rename_file</name>
--<digest>Rename a file.</digest>
--<desc>
-- If <b>new_name</b> contains a path specification, this is equivalent to
-- moving the file, as well as possibly changing its name. However, the path
-- must be on the same drive for this to work.
--
-- If <b>overwrite</b> was requested but the rename fails, any existing
-- destination file is preserved.
--</desc>
--<param>
--<type>sequence</type>
--<name>old_name</name>
--<desc>name of the file or directory to rename</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>new_name</name>
--<desc>new name for the renamed file</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>optional</name>
--<desc>
-- optional "overwrite" set to 0 (default) prevents renaming if destination file
--   already exists,
--   If set to 1, existing destination file is deleted first
--</desc>
--</param>
--<return>
-- integer: 0 on failure, 1 on success.
--</return>
--<example>
--</example>
--<see_also>move_file(), copy_file()</see_also>
--</function>
  atom psrc, pdest, ret
  sequence tempfile
  integer overwrite

  overwrite = get_option("overwrite", optional, 0)
  if not overwrite then
    if file_exists(new_name) then
      return 0
    end if
  else
    if file_exists(new_name) then
      tempfile = "temp.$$$"
      ret = move_file(new_name, tempfile, {})
    end if
  end if

  tempfile = ""
  psrc = allocate_string(old_name)
  pdest = allocate_string(new_name)
  if platform() = LINUX then
    ret = not exec_func(xMoveFile, {psrc, pdest}, "rename")
  elsif platform() = WIN32 then
    ret = exec_w32func(xMoveFile, {psrc, pdest}, "MoveFileA")
  end if

  free(pdest)
  free(psrc)

  if overwrite then
    if not ret then
      if length(tempfile) > 0 then
        -- rename was unsuccessful so restore from tempfile
        ret = move_file(tempfile, new_name, {})
      end if
    end if
    void = delete_file(tempfile)
  end if

  return ret
end function

------------------------------------------------------------------------------

integer xCreateDirectory
xCreateDirectory   = 0

global function create_directory(sequence name, sequence optional)
--<function>
--<name>create_directory</name>
--<digest>Create a new directory.</digest>
--<desc>This is OpenEuphoria 4.1 create_directory()</desc>
--<param>
--<type>sequence</type>
--<name>name</name>
--<desc>name of the new directory to create</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>optional</name>
--<desc>
-- * optional "mode" : on <i>Unix</i> systems, permissions for the new directory.
--     Defaults to 448 (all rights for owner, none for others).
--     mode is ignored on non-Unix platforms.
-- * optional "mkparent" : If true (default) the parent directories are also
--     created if needed.
--</desc>
--</param>
--<return>
--  integer: 0 on failure, 1 on success.
--</return>
--<example>
-- <code>
-- if not create_directory("the_new_folder", {}) then
--   error_message("Filesystem problem - could not create the new folder", 1)
-- end if
--
-- -- This example will also create "myapp/" and "myapp/interface/"
-- -- if they don't exist.
-- if not create_directory("myapp/interface/letters", {}) then
--   error_message("Filesystem problem - could not create the new folder", 1)
-- end if
--
-- -- This example will NOT create "myapp/" and "myapp/interface/"
-- -- if they don't exist.
-- if not create_directory("myapp/interface/letters", {{"mkparent",0}}) then
--  error_message("Filesystem problem - could not create the new folder", 1)
-- end if
-- </code>
--</example>
--<see_also>remove_directory(), chdir()</see_also>
--</function>
  atom pname, ret
  integer pos, mode, mkparent

  if length(name) = 0 then
    return 0 -- failed
  end if

  mode = get_option("mode", optional, 448)
  mkparent = get_option("mkparent", optional, 1)

  -- Remove any trailing slash.
  if name[$] = SLASH then
    name = name[1 .. $-1]
  end if

  if mkparent != 0 then
    pos = rfind(SLASH, name, {})
    if pos != 0 then
      ret = create_directory(name[1.. pos-1], {
              {"mode", mode},{"mkparent", mkparent}
            })
    end if
  end if

  pname = allocate_string(name)

  if platform() = LINUX then
    ret = not exec_func(xCreateDirectory, {pname, mode}, "mkdir")
  elsif platform() = WIN32 then
    ret = exec_w32func(xCreateDirectory, {pname, 0}, "CreateDirectoryA")
    mode = mode -- get rid of not used warning
  end if

  return ret
end function

------------------------------------------------------------------------------

function delete_it(sequence path_name, sequence entry)
  void = delete_file(path_name & SLASH & entry[D_NAME])
  return 0 -- keep going
end function

global procedure clear_directory(sequence path)
--<procedure>
--<name>clear_directory</name>
--<digest>Clear (delete) a directory of all files, but retaining sub-directories.</digest>
--<desc>
-- This never removes a directory. It only ever removes files. It is used to
-- clear a directory structure of all existing files, leaving the structure
-- intact.
--</desc>
--<param>
--<type>sequence</type>
--<name>path</name>
--<desc>name of the directory you want to remove files from</desc>
--</param>
--<example>
-- <code>
-- clear_directory("the_old_folder")
-- </code>
--</example>
--<see_also>remove_directory(), delete_file()</see_also>
--</procedure>
  void = walk_dir(path, routine_id("delete_it"), TRUE)
end procedure

------------------------------------------------------------------------------

integer xRemoveDirectory
xRemoveDirectory   = 0

function rmdir(sequence dirname)
  atom pname, ret

  pname = allocate_string(dirname)
  if platform() = LINUX then
    ret = not exec_func(xRemoveDirectory, {pname}, "rmdir")
  elsif platform() = WIN32 then
    ret = exec_w32func(xRemoveDirectory, {pname}, "RemoveDirectoryA")
  end if
  free(pname)
  return ret
end function

------------------------------------------------------------------------------

global function remove_directory(sequence dirname, sequence optional)
--<function>
--<name>remove_directory</name>
--<digest>Remove a directory.</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>dirname</name>
--<desc>name of the directory to remove</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>optional</name>
--<desc>
-- optional "force" set to 1 will also remove files and sub-directories in the
--   directory. The default is 0, which means that it will only remove the
--   directory if it is already empty.
--</desc>
--</param>
--<return>
-- integer: on failure, 1 on success.
--</return>
--<example>
-- <code>
-- if not remove_directory("the_old_folder", {}) then
--   error_message("Filesystem problem - could not remove the old folder", 1)
-- end if
-- </code>
--</example>
--<see_also>create_directory(), chdir(), clear_directory()</see_also>
--</function>
  object info
  integer force

  force = get_option("force", optional, 0)
  if ends(SLASH, dirname[length(dirname)]) then
    dirname = dirname[1..length(dirname)-1]
  end if
  info = dir(dirname)
  if atom(info) then return 0 end if
  if length(info) < 2 then return rmdir(dirname) end if
  info = info[3..length(info)]
  for loop = 1 to length(info) do
    if not match(info[loop][D_ATTRIBUTES],"d") then
      if not force then
        return 0 -- Directory is not already emptied.
      end if
      if not delete_file(dirname & "\\" & info[loop][D_NAME]) then return 0 end if
    else
      if not remove_directory(dirname & "\\" & info[loop][D_NAME], optional) then return 0 end if
      void = rmdir(dirname & "\\" & info[loop][D_NAME])
    end if
  end for
  return rmdir(dirname)
end function
