-- md5.e - The MD5 Message-Digest Algorithm - version 1.11
-- Copyright (C) 2000  Davi Tassinari de Figueiredo
--
-- If you wish to contact me, send an e-mail to davitf@usa.net .
--
-- You can get the latest version of this program from my Euphoria page:
-- http://davitf.n3.net/
--
--
-- License terms and disclaimer:
--
-- Permission is granted to anyone to use this software for any purpose,
-- including commercial applications, and to alter it and redistribute it
-- freely, subject to the following restrictions:
--
-- 1. The origin of this software must not be misrepresented; you must not
--    claim that you wrote the original software or remove the original
--    authors' names.
-- 2. Altered source versions must be plainly marked as such, and must not
--    be misrepresented as being the original software.
-- 3. All source distributions, with or without modifications, must be
--    distributed under this license. If this software's source code is
--    distributed as part of a larger product, this item does not apply to
--    the rest of the product.
-- 4. If you use this software in a product, an acknowledgment in the
--    product documentation is required. If the source code for the product
--    is not freely distributed, you must include information on how to
--    freely obtain the original software's source code.
--
-- This software is provided 'as-is', without any express or implied
-- warranty.  In no event will the authors be held liable for any damages
-- arising from the use of this software.
--
-- If you want to distribute this software in a way not allowed by this
-- license, or distribute the source under different license terms, contact
-- the authors for permission.


include machine.e
include _file_.e

-- 'General-purpose' routines

constant a_address = allocate(16), b_address = a_address + 4,
	 c_address = b_address + 4, d_address = c_address + 4,
	 data_address = allocate(64)

constant md5asm = allocate (1679)

poke (md5asm, and_bits(#FF,
".1/0435iiiޜggյVH߶gg"+34&
"ѱ0[0ۜccѱ([J۫ccѱ"+38&
"<_̬ߩggյ$Zӟ߶ggյ4_"+34&
"e&`aa(td(vv~ `G``p(lt(nvf b4`bj("+97&
"؄@؎׼O7 xw@|ؔ@~؆\r8yfBxyøy@|@ؖ"+73&
"!aQaaq)mu)owg!c^w)ack)}m)wwХ±0"+96&
"$#jd;(@*Z:2l`dMf%`$%od%8(:Z:Bl`dLf$\t$$td40"-99&
"KEmM=swayԲ9~*79wASKMmMEsw^yz{؛l76{wN;CEmMUsw"-118&
"!bS`abb)u})wg!aaaq)me)gww!cJgVa"+96&
"em+w+yyq趧xcbz+go+qyÁ#dcdd+w+"+94&
"wg!a!Axaaq)me)gww!chsack)}u)wwo"+96&
"vji2nv2xʈ*kljkk2~ʆ2p*j0~jjz"+87&
")me)gww!cack)}u)wwoԥIa`x)em)ow"+96&
"4_ցڟߠggյ,_ME߯ggյ<"+34&
"&$&wd.@08dKfwh]b$#gd;(@0dMf%Y$%nd%8(@dLf"-99&
"!!!pa1-5%aKc#lE]!#wa+=-5aHcdeJ"-96&
"]C`_w(d|l aHn}`aa(td| `Z``p(ltd b"+97&
"!mportȲ|:~↲w/Lrq:v~2sRrss:v⎲2r6"+79&
"taaq)mue!c(ack)}muĥptya`x)e}m!b9{"+96&
"rss:v⎲2r-Srr:~v"+79&
"~9\":79wAKmKSCw^y|{j76|wNCmK;Sw`y̒8\ru78w8SmK"-118&
"ue!aG4Kaaq)eumu!c@3ack)uu}mХc"+96&
"$#id;0Z8l(@dMfo%/o$%md%@Z8l8(dLf$WRb$$rd4(Z8l08dNf"-99&
"f%3#%wc-7Y7k?/cJeg\n#\"hc:/Y7k\'?cLe$BH`#$lc$?Y7"-98&
"td `B``p(dtlt b@`bj(tt|l寤!"+97&
"&ih1u}mم)jݚeijj1}}m)icziiy1m}u}"+88&
"8[k`śۥ9784352"+38))

poke4(md5asm + 8, a_address)
poke4(md5asm + 14, b_address)
poke4(md5asm + 20, c_address)
poke4(md5asm + 26, d_address)
poke4(md5asm + 31, data_address)
poke4(md5asm + 1649, a_address)
poke4(md5asm + 1655, b_address)
poke4(md5asm + 1661, c_address)
poke4(md5asm + 1667, d_address)

procedure init_md5()
    poke4(a_address, #67452301)
    poke4(b_address, #EFCDAB89)
    poke4(c_address, #98BADCFE)
    poke4(d_address, #10325476)
end procedure

function pad_message(sequence message)
    -- Add bytes to the end of the message so it can be divided
    -- in an exact number of 64-byte blocks.

    atom bytes_to_add
    bytes_to_add=64-remainder(length(message)+9,64)
    if bytes_to_add=64 then bytes_to_add=0 end if

    message=message&128&repeat(0,bytes_to_add)&
      int_to_bytes(length(message)*8)&{0,0,0,0}

    return message
end function


global function md5(sequence message)
--<function>
--<name>md5</name>
--<digest>Given a string, returns a 16-byte hash of it.</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>message</name>
--<desc>string to compute</desc>
--</param>
--<return>
-- sequence of 16-byte hash
--</return>
--<example>
-- whole_file = read_file(filename)
-- return hex_string(md5(whole_file))
--</example>
--<see_also>md5sum()</see_also>
--</function>

    init_md5()

    message=pad_message(message)    -- Add bytes to the message

    -- Process each 64-byte block
    for pos_in_message=1 to length(message) by 64 do

	-- Write data into memory
	poke(data_address, message[pos_in_message..pos_in_message+63])
	-- Call calculation routine
	call(md5asm)

    end for

    -- Get hash from memory
    return peek( {a_address, 16} )    -- Return the hash

end function

------------------------------------------------------------------------------
-- added function

global function md5sum(sequence filename)
--<function>
--<name>md5sum</name>
--<digest>returns MD5 checksum of a file</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc>the filename to compute</desc>
--</param>
--<return>
-- sequence: an hexadecimal MD5 hash string of the file content
--</return>
--<example>
-- s = md5sum(sequence filename)
--</example>
--<see_also>md5()</see_also>
--</function>
  sequence whole_file

  whole_file = read_file(filename)
  return hex_string(md5(whole_file))
end function
