------------------------------------------------------------
-- Windows part extracted from Pete Stoner's code

-- wrapper for CreateToolhelpSnapshot. 
-- Will list running processes along with any associated 
-- files/dlls
--
-- Pete Stoner 16/06/2006
------------------------------------------------------------

-- Linux part is of my own
-- Jean-Marc Duro 25/08/2017

------------------------------------------------------------

--include dll.e
--include machine.e
include file.e
include _types_.e
include lib/_w32routines_.e
without warning

without trace

-- Extract of Pete Stoner's code

-- flags/constantss
constant 
 TH32CS_SNAPHEAPLIST = #1,
 TH32CS_SNAPPROCESS  = #2,
 TH32CS_SNAPTHREAD   = #4,
 TH32CS_SNAPMODULE   = #8,
 TH32CS_INHERIT      = #80000000,
 TH32CS_SNAPALL      = or_all({TH32CS_SNAPHEAPLIST, TH32CS_SNAPPROCESS, TH32CS_SNAPTHREAD, TH32CS_SNAPMODULE}),
 INVALID_HANDLE_VALUE = -1,--#0FFFFFFFF, 
 MAX_PATH            = 260,
 ERROR_NO_MORE_FILES = 18
 
-- typedef struct tagPROCESSENTRY32 
constant 
 PE32_th32ProcessID     = 8,   -- DWORD  Process identifier.
 PE32_szExeFile        = 36,   -- MAX_PATH, Pntr to a null-termed string for the executable file for the process.
 SIZEOF_PE32STRUCT      = 40 + MAX_PATH
-- end typedef struct _tagPROCESSENTRY32  

--------------------------------------------------------------------------------

integer dllSnapshot dllSnapshot = 0
integer dllCloseHandle dllCloseHandle = 0
integer dllProcess32First dllProcess32First = 0
integer dllProcess32Next dllProcess32Next = 0
function get_win_process_id(sequence processname)
  atom snapshotHandle
  object void, ret
  atom PROCESSENTRY32
  sequence processes

  -- Set the size of the structure before using it.
  PROCESSENTRY32 = allocate(SIZEOF_PE32STRUCT)
  poke4(PROCESSENTRY32, SIZEOF_PE32STRUCT)

  processes = {}

  -- Take a snapshot of all processes in the system.
  snapshotHandle = exec_w32func(dllSnapshot, {TH32CS_SNAPALL, 0})--PROCESS, 0}, "CreateToolhelp32Snapshot")
  if snapshotHandle = INVALID_HANDLE_VALUE then
    free(PROCESSENTRY32)
    return 0
  end if
  
  -- Retrieve information about the first process 
  ret = exec_w32func(dllProcess32First, {snapshotHandle, PROCESSENTRY32}, "Process32First")  
  if ret != 0 then
    -- Now walk the snapshot of processes, and
    -- get information about each process in turn
    while 1 do
      sequence name = peek_string(PROCESSENTRY32+PE32_szExeFile)
      integer pid = peek4u(PROCESSENTRY32+ PE32_th32ProcessID)
      if length(name) then
        if match(processname, name) then
          processes = append(processes, pid)
        end if
      end if
     
      if not exec_w32func(dllProcess32Next, {snapshotHandle, PROCESSENTRY32}, "Process32Next") then
        exit
      end if
     
    end while
  end if
   
  free(PROCESSENTRY32)
  void = exec_w32func(dllCloseHandle, {snapshotHandle}, "CloseHandle")
  return processes
 
end function   

function get_lnx_process_id(sequence processname)
  sequence processes
  object x
  integer ownPID

  processes = {}
  x = dir("/proc/*")
  ownPID = get_pid()
  if sequence(x) then           -- Exclude forbidden DIRs
    for i = 1 to length(x) do   -- Loop through DIR content
      sequence s = x[i]
      if find('d',s[D_ATTRIBUTES]) and
         not (equal(s[D_NAME],".") or equal(s[D_NAME],"..")) then  -- Check for SubDIRs
        if is_integer(s[D_NAME]) and (ownPID != to_number(s[D_NAME])) then
          integer fn = open("/proc/" & s[D_NAME] & "/cmdline", "rb")
          sequence name = ""
          integer c = getc(fn)
          while (c != EOF) do
            if c then name &= c end if  -- remove NULL
            c = getc(fn)
          end while
          close(fn)
          if sequence(name) and length(name) then
            if match(processname, name) then
              processes = append(processes, to_number(s[D_NAME]))
            end if
          end if
        end if
      end if
    end for
  end if
  return processes
end function

------------------------------------------------------------------------------

global function get_process_id(sequence processname)
--<function>
--<name>get_process_id</name>
--<digest>returns a list of Process IDs (PID) which command line matches given name</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>processname</name>
--<desc>name of the process to check</desc>
--</param>
--<return>
-- sequence: list of pids of processname instances
--</return>
--<example>
-- s = get_process_id("/usr/bin/nmbd")
--</example>
--<see_also>is_running()</see_also>
--</function>
  if platform() = WIN32 then
    return get_win_process_id(processname)
  elsif platform() = LINUX then
    return get_lnx_process_id(processname)
  end if
end function

------------------------------------------------------------------------------

global function is_running(sequence processname)
--<function>
--<name>is_running</name>
--<digest>Checks if a process of given name is running</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>processname</name>
--<desc>name of the process to check</desc>
--</param>
--<return>
-- integer: true if at least one instance of processname is running else false
--</return>
--<example>
-- if is_running("/usr/sbin/nmbd") then
--   -- do some processing
-- end if
--</example>
--<see_also>get_process_id()</see_also>
--</function>
  sequence processes
  
  processes = get_process_id(processname)
  return length(processes) > 0
end function
