-- _sequence_.e

include _common_.e
include _types_.e
include _conv_.e

------------------------------------------------------------------------------

global function trim_head(sequence s)
--<function>
--<name>trim_head</name>
--<digest>remove space chars from the beginning of a string</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc>string to modify</desc>
--</param>
--<return>
-- modified string
--</return>
--<example>
--</example>
--<see_also>trim_tail(), trim(), trim_spaces()</see_also>
--</function>
  integer l

  l = length(s)
  if l=0 then return "" end if
  for i=1 to l do
    if not is_space(s[i]) then
      return s[i..l]
    end if
  end for
  return ""
end function

------------------------------------------------------------------------------

global function trim_tail(sequence s)
--<function>
--<name>trim_tail</name>
--<digest>remove space chars from the end of a string</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc>string to modify</desc>
--</param>
--<return>
-- modified string
--</return>
--<example>
--</example>
--<see_also>trim_head(), trim(), trim_spaces()</see_also>
--</function>
  integer l

  l = length(s)
  if l=0 then return "" end if
  for i=l to 1 by -1 do
    if not is_space(s[i]) then
      return s[1..i]
    end if
  end for
  return ""
end function

------------------------------------------------------------------------------

global function trim(sequence s)
--<function>
--<name>trim</name>
--<digest>remove space chars from both ends of a string</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc>string to modify</desc>
--</param>
--<return>
-- modified string
--</return>
--<example>
--</example>
--<see_also>trim_head(), trim_tail(), trim_spaces()</see_also>
--</function>
  return trim_head(trim_tail(s))
end function

-------------------------------------------------------------------------------

global function trim_spaces(sequence s)
--<function>
--<name>trim_spaces</name>
--<digest></digest>
--<desc>
-- remove space chars from both ends of a string
-- and removes exceeding space chars between words
-- tab spaces are converted to normal spaces before removal
--</desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc>string to modify</desc>
--</param>
--<return>
-- modified string
--</return>
--<example>
--</example>
--<see_also>trim_head(), trim_tail(), trim()</see_also>
--</function>
  sequence result
  integer lg

  result = ""
  s = trim(s)
  lg = length(s)
  if lg = 0 then return "" end if
  if s[1] = '\t' then s[1] = " " end if
  if s[1] != ' ' then result &= {s[1]} end if
  if lg > 1 then
    for i = 2 to lg do
      if s[i] = '\t' then s[i] = ' ' end if
      if s[i] != ' ' then
        result &= {s[i]}
      else
        if s[i-1] != ' ' then result &= {s[i]} end if
      end if
    end for
  end if
  return result
end function

------------------------------------------------------------------------------

global function pad_head(sequence s, atom len)
--<function>
--<name>pad_head</name>
--<digest>Return a string left-padded to length len</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc>string to modify</desc>
--</param>
--<param>
--<type>atom</type>
--<name>len</name>
---<desc>length of string to reach</desc>
--</param>
--<return>
-- modified string
--</return>
--<example>
--</example>
--<see_also>pad_tail()</see_also>
--</function>

  if length(s) >= len then
    return s
  else
    return repeat(' ', len-length(s)) & s
  end if
end function

------------------------------------------------------------------------------

global function pad_tail(sequence s, atom len)
--<function>
--<name>pad_tail</name>
--<digest>Return a string right-padded to length len</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc>string to modify</desc>
--</param>
--<param>
--<type>atom</type>
--<name>len</name>
--<desc>length of string to reach</desc>
--</param>
--<return>
-- modified string
--</return>
--<example>
--</example>
--<see_also>pad_head()</see_also>
--</function>

  if length(s) >= len then
    return s
  else
    return s & repeat(' ', len-length(s))
  end if
end function

------------------------------------------------------------------------------

-- returns the n characters at the right of string s
global function left(sequence s, integer n)
--<function>
--<name>left</name>
--<digest>returns the given number of characters at the left of a string</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc>string to look at</desc>
--</param>
--<param>
--<type>integer</type>
--<name>n</name>
--<desc></desc>
--</param>
--<return>
-- sequence: extracted sub-string
--</return>
--<example>
--</example>
--<see_also>mid(), right()</see_also>
--</function>
  integer l

  l = length(s)
  if (l=0) or (n=0) then return "" end if
  if n >= l then return s end if
  return s[1..n]
end function

------------------------------------------------------------------------------

global function mid(sequence s, atom start, atom len)
--<function>
--<name>mid</name>
--<digest>returns the sub-string at given position and given length of a string</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc>string to look at</desc>
--</param>
--<param>
--<type>atom</type>
--<name>start</name>
--<desc>start index</desc>
--</param>
--<param>
--<type>atom</type>
--<name>len</name>
--<desc>sub-string length</desc>
--</param>
--<return>
-- sequence: extracted sub-string
--</return>
--<example>
--</example>
--<see_also>left(), right()</see_also>
--</function>
  integer lg

  lg = length(s)
  if start > lg then
    return {}
  elsif len > lg then
    return s[start..lg]
  else
    return s[start..start+len-1]
  end if
end function

------------------------------------------------------------------------------

--
global function right(sequence s, integer n)
--<function>
--<name>right</name>
--<digest>returns the given number of characters at the right of a string</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc>string to look at</desc>
--</param>
--<param>
--<type>integer</type>
--<name>n</name>
--<desc></desc>
--</param>
--<return>
-- sequence: extracted sub-string
--</return>
--<example>
--</example>
--<see_also>left(), mid()</see_also>
--</function>
  integer l

  l = length(s)
  if (l=0) or (n=0) then return "" end if
  if n >= l then return s end if
  return s[l-n+1..l]
end function

------------------------------------------------------------------------------

global function quote(sequence str, sequence optional)
--<function>
--<name>quote</name>
--<digest>insert string between quotes</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>str</name>
--<desc>string to modify</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>optional</name>
--<desc>
-- optional "delims" specifies the quotes to remove: defaults to '"'
-- delims can be an integer when starting and ending quotes are equal (most cases)
-- delims is a sequence with starting and ending quotes if these are different
--</desc>
--</param>
--<return>
-- modified string
--</return>
--<example>
--</example>
--<see_also>dequote()</see_also>
--</function>
  object delims

  delims = get_option("delims", optional, '"')
  if atom(delims) then
    return delims & str & delims
  elsif (length(delims) = 2) then
    return delims[1] & str & delims[2]
  end if
end function

------------------------------------------------------------------------------

global function dequote(sequence str, sequence optional)
--<function>
--<name>dequote</name>
--<digest>remove specified quotes if existing</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>str</name>
--<desc>string to modify</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>optional</name>
--<desc>
-- optional "delims" specifies the quotes to remove: defaults to '"'
-- delims can be an integer when starting and ending quotes are equal (most cases)
-- delims is a sequence with starting and ending quotes if these are different
--</desc>
--</param>
--<return>
-- modified string
--</return>
--<example>
--</example>
--<see_also> quote()</see_also>
--</function>
  object delims

  delims = get_option("delims", optional, '"')
  if (length(str) > 1) then
    if atom(delims) then
      if (str[1]=delims) and (str[$]=delims) then
        return str[2..$-1]
      end if
    else
      if (length(delims) = 2) and (str[1]=delims[1]) and (str[$]=delims[2]) then
        return str[2..$-1]
      end if
    end if
  end if
  return str
end function

------------------------------------------------------------------------------

global function begins(object x, sequence str)
--<function>
--<name>begins</name>
--<digest>checks whether a string begins with given sub-string</digest>
--<desc>
--</desc>
--<param>
--<type>object</type>
--<name>x</name>
--<desc>object to search for</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>str</name>
--<desc>string to search in</desc>
--</param>
--<return>
-- 1 if string begins with given object, 0 if not
--</return>
--<example>
--</example>
--<see_also>ends()</see_also>
--</function>
  if length(str) = 0 then
    return 0
  end if
  
  if atom(x) then
    if equal(x, str[1]) then
      return 1
    else
      return 0
    end if
  end if
  
  if length(x) > length(str) then
    return 0
  end if
  
  if equal(x, str[1.. length(x)]) then
    return 1
  else
    return 0
  end if
end function

------------------------------------------------------------------------------

global function ends(object x, sequence str)
--<function>
--<name>ends</name>
--<digest>checks whether a string ends with given sub-string</digest>
--<desc>
--</desc>
--<param>
--<type>object</type>
--<name>x</name>
--<desc>object to search for</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>str</name>
--<desc>string to search in</desc>
--</param>
--<return>
-- 1 if string ends with given object, 0 if not
--</return>
--<example>
--</example>
--<see_also>begins()</see_also>
--</function>
  if length(str) = 0 then
    return 0
  end if
  
  if atom(x) then
    if equal(x, str[$]) then
      return 1
    else
      return 0
    end if
  end if
  
  if length(x) > length(str) then
    return 0
  end if
  
  if equal(x, str[$ - length(x) + 1 .. $]) then
    return 1
  else
    return 0
  end if
end function

------------------------------------------------------------------------------

global function split(sequence str, object sep, sequence optional)
--<function>
--<name>split</name>
--<digest>splits a string according to a separator</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>str</name>
--<desc>string to split</desc>
--</param>
--<param>
--<type>object</type>
--<name>sep</name>
--<desc>
-- separator
-- either an object or a sequence
-- if separator is a sequence then:
-- * if as_is is set then the whole string is the separator
-- * is as_is is not set then each character of the string is considered
--</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>optional</name>
--<desc>
-- optional "keep"  indicates that separator is kept in result. Defaults to 0 (no).
-- optional "as_is" indicates that separator is the wole string instead of each character separately.
--</desc>
--</param>
--<return>
-- splitted sequence
--</return>
--<example>
-- analyze_object(split("my name is Bond", ' ', {}), "split", {})
-- split =
-- .  [1] "my"
-- .  [2] "name"
-- .  [3] "is"
-- .  [4] "Bond"
-- analyze_object(split("my name is Bond", ' ', {{"keep",1}}), "split", {})
-- split =
-- .  [1] "my"
-- .  [2] " "
-- .  [3] "name"
-- .  [4] " "
-- .  [5] "is"
-- .  [6] " "
-- .  [7] "Bond"
-- analyze_object(split("my name is: Bond", ": ", {}), "split", {})
-- split =
-- .  [1] "my"
-- .  [2] "name"
-- .  [3] "is"
-- .  [4] ""
-- .  [5] "Bond"
-- analyze_object(split("my name is: Bond", ": ", {{"as_is",1}}), "split", {})
-- split =
-- .  [1] "my name is"
-- .  [2] "Bond"
--</example>
--<see_also>
--</see_also>
--</function>
  integer i, slen, lg, keep, as_is
  sequence result, s, t
  integer tmpStr

  result = {}
  slen = length(str)
  if slen = 0 then return result end if
  keep = get_option("keep", optional, 0)
  as_is = get_option("as_is", optional, 0)
  if atom(sep) then sep = {sep} end if

  s = {}
  tmpStr = 0      -- if 1, index is within double quotes
  lg = length(sep)
  i = 1
  while i <= slen do
    if (str[i] = '"') and not find('"', sep) then
      s = s & str[i]
      if (i = 1) or (str[i-1] != '\\') then
        if tmpStr = 0 then
          tmpStr = 1
        else
          tmpStr = 0
        end if
      end if
    elsif as_is = 0 then
      if tmpStr = 0 and find(str[i], sep) then
        t = trim(s)
        -- if length(t) then result = append(result, t) end if
        result = append(result, t)
        if keep then result = append(result, str[i]) end if
        s = {}
      else
        s = s & str[i]
      end if
    elsif as_is = 1 then
      if (tmpStr = 0) and (i < slen-lg) and (match(sep, str[i..$])=1) then
        t = trim(s)
        -- if length(t) then result = append(result, t) end if
        result = append(result, t)
        i += lg-1
        if keep then result = append(result, sep) end if
        s = {}
      else
        s = s & str[i]
      end if
    end if
    i += 1
  end while
  t = trim(s)
  if length(t) then result = append(result, t) end if
  return result
end function

------------------------------------------------------------------------------

global function escape(sequence str)
--<function>
--<name>escape</name>
--<digest>escape chars from a string</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>str</name>
--<desc>string to convert chars from</desc>
--</param>
--<return>
-- escaped sequence
--</return>
--<example>
-- escape("\"string1\string2\"") => \"string1\\string2\"
-- escape("'char'") => \'char\'
-- escape(col1\tcol2) => col1\\tcol2
-- escape(line1\rline2) => line1\\rline2
-- escape(line1\nline2) => line1\\nline2
-- escape(line\n) => line\\n
--</example>
--<see_also>unescape()</see_also>
--</function>
  sequence result
  integer i

  result = ""
  if length(str) = 0 then
    return result
  end if
  i = 1
  while i < length(str) do
    if str[i] = '\n' then
      result &= "\\n"
    elsif str[i] = '\r' then
      result &= "\\r"
    elsif str[i] = '\t' then
      result &= "\\t"
    elsif str[i] = '"' then
      result &= "\\\""
    elsif str[i] = '\\' then
      result &= "\\\\"
    elsif str[i] = #27 then
      result &= "\'"
    else
      result &= str[i]
    end if
    i += 1
  end while
  if i <= length(str) then result &= str[i] end if
  return result
end function

------------------------------------------------------------------------------

global function unescape(sequence str)
--<function>
--<name>unescape</name>
--<digest>unescape chars from a string</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>str</name>
--<desc>string to convert escaped chars from</desc>
--</param>
--<return>
-- unescaped sequence
--</return>
--<example>
-- unescape(\"string1\\string2\") => "string1\string2"
-- unescape(\'char\') => 'char'
-- unescape(col1\tcol2) => col1[tab]col2
-- unescape(line1\rline2) => line1[cr]line2
-- unescape(line1\nline2) => line1[lf]line2
-- unescape(line\n) => line[lf]
--</example>
--<see_also>escape()</see_also>
--</function>
  sequence result
  integer i

  result = ""
  if length(str) = 0 then
    return result
  end if
  i = 1
  while i < length(str) do
    if equal(str[i..i+1], "\\n") then
      result &= "\n"
      i += 1
    elsif equal(str[i..i+1], "\\r") then
      result &= "\r"
      i += 1
    elsif equal(str[i..i+1], "\\t") then
      result &= "\t"
      i += 1
    elsif not find(str[i..i+1], {"\\\"", "\\'", "\\\\"}) then
      result &= str[i]
    end if
    i += 1
  end while
  if i <= length(str) then result &= str[i] end if
  return result
end function

------------------------------------------------------------------------------

global function split_n_convert(sequence str, object sep)
--<function>
--<name>split_n_convert</name>
--<digest>splits a string in objects according to given separator</digest>
--<desc>
-- splitted elements are converted to atoms when possible
--</desc>
--<param>
--<type>sequence</type>
--<name>str</name>
--<desc>string to split</desc>
--</param>
--<param>
--<type>object</type>
--<name>sep</name>
--<desc>separator</desc>
--</param>
--<return>
-- splitted sequence
--</return>
--<example>
-- analyze_object(split_n_convert("my,code,is,7", ","), "split_n_convert", {})
-- split_n_convert =
-- .  [1] "my"
-- .  [2] "code"
-- .  [3] "is"
-- .  [4] 7
--</example>
--<see_also>convert_n_join()</see_also>
--</function>
  sequence s

  s = split(str, sep, {})
  for i = 1 to length(s) do
    if is_number(s[i]) or is_hex(s[i]) then
      s[i] = to_number(s[i])
    elsif is_string(s[i]) then
      s[i] = unescape(dequote(s[i], {}))
    end if
  end for
  return s
end function

------------------------------------------------------------------------------

global function join(sequence str, sequence sep)
--<function>
--<name>join</name>
--<digest>merges sub-strings with given separator into a global string</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>str</name>
--<desc>list of sub-strings</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>sep</name>
--<desc>separator</desc>
--</param>
--<return>
-- merged string
--</return>
--<example>
-- puts(1, join({"my","code","is","007"}, " ") & "\n")
-- my code is 007
--</example>
--<see_also>split()</see_also>
--</function>
  integer slen
  sequence result

  slen = length(str)
  if slen = 0 then return "" end if

  result = str[1]
  for i = 2 to slen do
    result &= sep & str[i]
  end for
  return result
end function

------------------------------------------------------------------------------

global function convert_n_join(sequence s, sequence sep)
--<function>
--<name>convert_n_join</name>
--<digest>merge converted objects in a string with given separator</digest>
--<desc>
-- objects are converted to strings before being joined
--</desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc>sequence of objects</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>sep</name>
--<desc>separator</desc>
--</param>
--<return>
-- merged string
--</return>
--<example>
-- puts(1, convert_n_join({"my","code","is", 7}, ",") & "\n")
-- "my","code","is",7
--</example>
--<see_also>split_n_merge()</see_also>
--</function>
  for i = 1 to length(s) do
    if atom(s[i]) then
      s[i] = to_string(s[i])
    elsif is_string(s[i]) then
      s[i] = quote(s[i], {})
    end if
  end for
  return join(s, sep)
end function

------------------------------------------------------------------------------

global function insert(sequence target, object what, integer index)
--<function>
--<name>insert</name>
--<digest>insert an object at given index in a sequence</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>target</name>
--<desc>sequence to modify</desc>
--</param>
--<param>
--<type>object</type>
--<name>what</name>
--<desc>object to insert</desc>
--</param>
--<param>
--<type>integer</type>
--<name>index</name>
--<desc>position to insert object</desc>
--</param>
--<return>
-- modified sequence
--</return>
--<example>
-- puts(1, insert("George Bush", "W. ", 8) & "\n")
-- George W. Bush
--</example>
--<see_also>
--</see_also>
--</function>
  integer lg

  lg = length(target)
  if index <=1 then
    return what & target
  elsif index > lg then
    return target & what
  else
    return target[1..index-1] & what & target[index..$]
  end if
end function

------------------------------------------------------------------------------

global function remove(sequence target, atom start, atom stop)
--<function>
--<name>remove</name>
--<digest>remove from start index to stop index in given sequence</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>target</name>
--<desc>sequence to modify</desc>
--</param>
--<param>
--<type>atom</type>
--<name>start</name>
--<desc>start index</desc>
--</param>
--<param>
--<type>atom</type>
--<name>stop</name>
--<desc>stop index</desc>
--</param>
--<return>
-- modified sequence
--</return>
--<example>
-- puts(1, remove("George W. Bush", 8, 10) & "\n")
-- George Bush
--</example>
--<see_also>
--</see_also>
--</function>
  integer lg

  lg = length(target)
  if (start<1) or (start>lg) then return target end if
  if start = 1 then
    return target[stop+1..lg]
  elsif start = lg then
    return target[1..lg-1]
  else
    return target[1..start-1] & target[stop+1..lg]
  end if
end function

------------------------------------------------------------------------------

global function replace(sequence s, sequence old, sequence new)
--<function>
--<name>replace</name>
--<digest>replaces first occurence of sub-sequence old by new in sequence target</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>target</name>
--<desc>sequence to replace in</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>old</name>
--<desc>sub-sequence to be replaced</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>new</name>
--<desc>new sub-sequence</desc>
--</param>
--<return>
-- modified sequence
--</return>
--<example>
-- puts(1, replace("George W. Bush", "W. ", "") & "\n")
-- George Bush
--</example>
--<see_also>
--</see_also>
--</function>
  integer index

  index = match(old,s)
  if index > 0 then
    s = remove(s,index,index+length(old)-1)
    if length(new) then
      s = insert(s,new,index)
    end if
  end if
  return s
end function

------------------------------------------------------------------------------

global function replace_all(sequence s, sequence old, sequence new)
--<function>
--<name>replace_all</name>
--<digest>replaces all occurences of sub-sequence old by new in sequence target</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>target</name>
--<desc>sequence to replace in</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>old</name>
--<desc>sub-sequence to be replaced</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>new</name>
--<desc>new sub-sequence</desc>
--</param>
--<return>
-- modified sequence
--</return>
--<example>
-- puts(1, replace_all("atoms, integers and objects", "s", "") & "\n")
-- atom, integer and object
--</example>
--<see_also>
--</see_also>
--</function>
  integer index, lg
  sequence res

  lg = length(old)
  res = ""
  index = match(old, s)
  while index != 0 do
    if index > 1 then
      res &= s[1..index-1]&new
    elsif index = 1 then
      res &= new
    end if
    if (index+lg) > length(s) then
      s = ""
      exit
    end if
    s = s[index+lg..$]
    index = match(old, s)
  end while
  if length(s) then res &= s end if
  return res
end function

------------------------------------------------------------------------------

global function append_if_new(sequence s, object o)
--<function>
--<name>append_if_new</name>
--<digest>append an object to a sequence only if not already present</digest>
--<desc></desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc>sequence to append to</desc>
--</param>
--<param>
--<type>object</type>
--<name>o</name>
--<desc>object to append</desc>
--</param>
--<return>
-- modified sequence
--</return>
--<example>
-- puts(1, append_if_new("abcdefghij", 'a') & "\n")
-- abcdefghij
-- puts(1, append_if_new("abcdefghij", 'k') & "\n")
-- abcdefghijk
--</example>
--<see_also>append()</see_also>
--</function>
  if find(o, s) then
    return s
  else
    return append(s, o)
  end if
end function

------------------------------------------------------------------------------

global function store(sequence structure, sequence path, object x)
--<function>
--<name>store</name>
--<digest>store an object in a sequence at given nested position</digest>
--<desc>
-- this is store function from OpenEuphoria 4
--</desc>
--<param>
--<type>sequence</type>
--<name>structure</name>
--<desc>sequence to store in</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>path</name>
--<desc>nested position to store at</desc>
--</param>
--<param>
--<type>object</type>
--<name>x</name>
--<desc>object to store</desc>
--</param>
--<return>
-- modified sequence
--</return>
--<example>
-- analyze_object(store({1,{2,{3}}}, {2, 2, 1}, 4), "store 2.2.1", {})
-- store 2.2.1 =
-- .  [1] 1
-- .  [2]
-- .  .  [1] 2
-- .  .  [2] {4}
--</example>
--<see_also>fetch()</see_also>
--</function>
  sequence partials, result, branch
  object last_idx

  partials = repeat(structure,length(path)-1)
  branch = structure
  for i=1 to length(path)-1 do
    branch=branch[path[i]]
    partials[i]=branch
  end for

  last_idx = path[$]
  if atom(last_idx) then
    branch[last_idx]=x
  else
    branch[last_idx[1]..last_idx[2]]=x
  end if

  partials = prepend(partials,0) -- avoids computing temp=i+1 a few times

  for i=length(path)-1 to 2 by -1 do
    result = partials[i]
    result[path[i]] = branch
    branch = result
  end for
  structure[path[1]] = branch
  return structure
end function

------------------------------------------------------------------------------

global function fetch(sequence structure, sequence path)
--<function>
--<name>fetch</name>
--<digest>get an object in a sequence from its nested position</digest>
--<desc>
-- this is fetch function from OpenEuphoria 4
--</desc>
--<param>
--<type>sequence</type>
--<name>structure</name>
--<desc>sequence to fetch from</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>path</name>
--<desc>nested position to get at</desc>
--</param>
--<return>
-- object searched for
--</return>
--<example>
-- ? fetch({1,{2,{3,4}}}, {2, 2, 1})
-- 3
--</example>
--<see_also>store()</see_also>
--</function>
  for i=1 to length(path)-1 do
    structure = structure[path[i]]
  end for
  return structure[path[$]]
end function

------------------------------------------------------------------------------

global function extract_column(sequence array, integer column)
--<function>
--<name>extract_column</name>
--<digest>extract a column of an array</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>array</name>
--<desc>array to extract column from</desc>
--</param>
--<param>
--<type>integer</type>
--<name>column</name>
--<desc>column to be extracted from array</desc>
--</param>
--<return>
-- sequence: selected column of array
--</return>
--<example>
-- puts(1, extract_column({"Start","Tank","Own","Patent"}, 1) & "\n")
-- STOP
--</example>
--<see_also>store()</see_also>
--</function>
  sequence result

  result = {}
  if length(array) = 0 then return {} end if
  for i = 1 to length(array) do
    if column <= length(array[i]) then
      result = append(result, array[i][column])
    else
      result = append(result, {})
    end if
  end for
  return result
end function

------------------------------------------------------------------------------

function split_template(sequence s)
  sequence result, variable, fixed
  integer inside_variable
  
  result = {}
  inside_variable = 0
  fixed = ""
  variable = ""
  for i = 1 to length(s) do
    if s[i] = '<' then
      inside_variable = 1
      variable = "<"
      if length(fixed) then result = append(result, fixed) end if
    elsif s[i] = '>' then
      inside_variable = 0
      result = append(result, variable & ">")
      fixed = ""
    else
      if inside_variable then
        variable &= s[i]
      else
        fixed &= s[i]
      end if
    end if
  end for
  if length(fixed) then result = append(result, fixed) end if
  return result
end function

------------------------------------------------------------------------------

global function parse_sequence(sequence s, sequence template)
--<function>
--<name>parse_sequence</name>
--<digest>extract variables from a string according to a template</digest>
--<desc>
-- Variables are encoded as XML tags in the template
-- Example: "http://<url>:<port>/search?q=<query>"
--</desc>
--<param>
--<type>sequence</type>
--<name>s</name>
--<desc>sequence to be parsed</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>template</name>
--<desc>reference template</desc>
--</param>
--<return>
-- sequence: list of pairs {name, value} of all encountered variables in string
--</return>
--<example>
-- parse_sequence("http://www.google.com:80/search?q=euphoria",
--                "http://<url>:<port>/search?q=<query>")
-- will return
-- {{"<url>", "www.google.com"}, {"<port>", "80"}, {"<query>", "euphoria"}}
--</example>
--<see_also></see_also>
--</function>
  integer start, n
  sequence result, tmpl
  
  tmpl = split_template(template)
  result = {}
  start = 1
  for i = 1 to length(tmpl) do
    if begins("<", tmpl[i]) then
      if i < length(tmpl) then
        n = match_from(tmpl[i+1], s, start)
        if n then
          result = append(result, {tmpl[i], s[start..n-1]})
          start = n
        else
          return 0
        end if
      else
        result = append(result, {tmpl[i], s[start..$]})
      end if
    else
      n = match_from(tmpl[i], s, start)
      if n then
        start = n + length(tmpl[i])
      else
        return 0
      end if
    end if
  end for
  return result
end function

------------------------------------------------------------------------------

function index_and_clean_lines(sequence lines)
  sequence result, s, t
  integer lg

  result = {}
  for i = 1 to length(lines) do
    s = trim(lines[i])
    lg = length(s)
    t = ""
    if lg then
      t &= s[1]
      for j = 2 to length(s) do
        if (s[j] != ' ') then
          t = append(t, s[j])
        else
          if (s[j-1] != ' ') then
            t = append(t, s[j])
          end if
        end if
      end for
    end if
    result = append(result, {i, t})
  end for
  return result
end function

-----------------------------------------------------------------------------

global function compare_lines(sequence lines1, sequence lines2)
--<function>
--<name>compare_lines</name>
--<digest>compares lines2 to lines1 and returns differences</digest>
--<desc>
-- Comparison does not take spaces into account but tabs are sigificant.
--</desc>
--<param>
--<type>sequence</type>
--<name>lines1</name>
--<desc>reference sequence</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>lines2</name>
--<desc>sequence to be compared to first one</desc>
--</param>
--<return>
-- sequence: list of differences as sequences in following format:
-- {comparison, line1, sequence1, line2, sequence 2}
-- where
-- * comparison is either:
--   * '-' if reference line was removed in line2
--   * '+' if compared line was added to line1
--   * '#' if reference line differs from compared line
-- * line1 is number of reference line in line1
-- * sequence1 is reference line in line1
-- * line2 is number of compared line in line2
-- * sequence2 is compared line in line1
--</return>
--<example>
-- constant SEQ1 = {
--   "First",
--   "Second",
--   "Third",
--   "Fourth",
--   "Fifth"
-- }
--
-- constant SEQ2 = {
--   "First",
--   "Third",
--   "Not Fourth",
--   "Fifth",
--   "Sixth"
-- }

-- compare_lines(SEQ1, SEQ2)
-- will return
-- {
--   { '-', 2, "Second", 0, "" },
--   { '#', 4, "Fourth", 3, "Not Fourth" },
--   { '+', 0, "", 5, "Sixth" }
-- }
--</example>
--<see_also></see_also>
--</function>
  sequence result
  integer l2in1, l1in2
  sequence s1, s2

  result = {}

  lines1 = index_and_clean_lines(lines1)
  lines2 = index_and_clean_lines(lines2)
  while length(lines1) and length(lines2) do
    l2in1 = find_in_array(lines2[1][2], lines1, {{"search_field", 2}})
    l1in2 = find_in_array(lines1[1][2], lines2, {{"search_field", 2}})

    if l2in1 then
      s1 = lines1[l2in1]
    else
      s1 = ""
    end if
    if l1in2 then
      s2 = lines2[l1in2]
    else
      s2 = ""
    end if

    if equal(lines1[1], lines2[1]) then
      -- equality: nothing to do
      lines1 = remove(lines1, 1, 1)
      lines2 = remove(lines2, 1, 1)
    elsif (l1in2 * l2in1) > 0 then
      lines2 = remove(lines2, l1in2, l1in2)
      lines1 = remove(lines1, l2in1, l2in1)
    elsif l1in2 > 0 then
      for i = 1 to l1in2-1 do
        result = append(result, {'+', 0, "", lines2[i][1], lines2[i][2]})
      end for
      lines2 = remove(lines2, 1, l1in2-1)
    elsif l2in1 > 0 then
      for i = 1 to l2in1-1 do
        result = append(result, {'-', lines1[i][1], lines1[i][2], 0, ""})
      end for
      lines1 = remove(lines1, 1, l2in1-1)
    else
      result = append(result, {'#', lines1[1][1], lines1[1][2],
                               lines2[1][1], lines2[1][2]})
      lines1 = remove(lines1, 1, 1)
      lines2 = remove(lines2, 1, 1)
    end if
  end while
  for i = 1 to length(lines1) do
    result = append(result, {'-', lines1[i][1], lines1[i][2], 0, ""})
  end for
  for i = 1 to length(lines2) do
    result = append(result, {'+', 0, "", lines2[i][1], lines2[i][2]})
  end for
  return result
end function

