-------------------------------------------------------------------------------
-- Sockets.ew
--
-- Author:  Brett A. Pantalone
--          pantalone2001@yahoo.com
--
-- Description:
--     Euphoria wrappers for Winsock API function calls.
--
-- License: Copyright (C) 2005 Brett A. Pantalone
--
--     This program is free software; you can redistribute it and/or modify
--     it under the terms of the GNU General Public License version 2 or
--     later as published by the Free Software Foundation.
--
--     This program is distributed in the hope that it will be useful,
--     but WITHOUT ANY WARRANTY; without even the implied warranty of
--     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--     GNU General Public License for more details.
--
-------------------------------------------------------------------------------

include dll.e
include msgbox.e
include _w32routines_.e
include _w32errors_.e

-- Socket types and address families
global constant 
  SOCKET_ERROR          = -1,
  SOCK_STREAM           = 1,
  SOCK_DGRAM            = 2,
  SOCK_RAW              = 3,
  AF_UNSPEC             = 0,
  AF_UNIX               = 1,
  AF_INET               = 2,
  PF_UNSPEC             = AF_UNSPEC,
  PF_UNIX               = AF_UNIX,
  PF_INET               = AF_INET,
  INADDR_ANY            = 0,
  INADDR_LOOPBACK       = #7F000001,
  INADDR_BROADCAST      = #FFFFFFFF,
  INADDR_NONE           = #FFFFFFFF,
  INVALID_SOCKET        = #FFFFFFFF
--<constant>
--<name>SOCKET_ERROR</name>
--<value>-1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SOCK_STREAM</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SOCK_DGRAM</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SOCK_RAW</name>
--<value>3</value>
--<desc></desc>
--</constant>
--<constant>
--<name>AF_UNSPEC</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>AF_UNIX</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>AF_INET</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>PF_UNSPEC</name>
--<value>AF_UNSPEC</value>
--<desc></desc>
--</constant>
--<constant>
--<name>PF_UNIX</name>
--<value>AF_UNIX</value>
--<desc></desc>
--</constant>
--<constant>
--<name>PF_INET</name>
--<value>AF_INET</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INADDR_ANY</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INADDR_LOOPBACK</name>
--<value>#7F000001</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INADDR_BROADCAST</name>
--<value>#FFFFFFFF</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INADDR_NONE</name>
--<value>#FFFFFFFF</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INVALID_SOCKET</name>
--<value>#FFFFFFFF</value>
--<desc></desc>
--</constant>

-- Top-level socket options
global constant 
  SOL_SOCKET            = #FFFF ,
  SO_BROADCAST          = #20,
  SO_CONDITIONAL_ACCEPT = #3002,
  SO_DEBUG              = #1,
  SO_DONTROUTE          = #10,
  SO_KEEPALIVE          = #8,
  SO_LINGER             = #80,
  SO_OOBINLINE          = #100,
  SO_RCVBUF             = #1002,
  SO_RCVTIMEO           = #1006,
  SO_REUSEADDR          = #4,
  SO_SNDBUF             = #1001,
  SO_SNDTIMEO           = #1005,
  SO_USELOOPBACK        = #40
--<constant>
--<name>SOL_SOCKET</name>
--<value>#FFFF</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SO_BROADCAST</name>
--<value>#20</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SO_CONDITIONAL_ACCEPT</name>
--<value>#3002</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SO_DEBUG</name>
--<value>#1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SO_DONTROUTE</name>
--<value>#10</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SO_KEEPALIVE</name>
--<value>#8</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SO_LINGER</name>
--<value>#80</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SO_OOBINLINE</name>
--<value>#100</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SO_RCVBUF</name>
--<value>#1002</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SO_RCVTIMEO</name>
--<value>#1006</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SO_REUSEADDR</name>
--<value>#4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SO_SNDBUF</name>
--<value>#1001</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SO_SNDTIMEO</name>
--<value>#1005</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SO_USELOOPBACK</name>
--<value>#40</value>
--<desc></desc>
--</constant>

global constant
  WSA_IO_PENDING        = ERROR_IO_PENDING,
  WSA_IO_INCOMPLETE     = ERROR_IO_INCOMPLETE,
  WSA_INVALID_HANDLE    = ERROR_INVALID_HANDLE,
  WSA_INVALID_PARAMETER = ERROR_INVALID_PARAMETER,
  WSA_NOT_ENOUGH_MEMORY = ERROR_NOT_ENOUGH_MEMORY,
  WSA_OPERATION_ABORTED = ERROR_OPERATION_ABORTED
--<constant>
--<name>WSA_IO_PENDING</name>
--<value>ERROR_IO_PENDING</value>
--<desc></desc>
--</constant>
--<constant>
--<name>WSA_IO_INCOMPLETE</name>
--<value>ERROR_IO_INCOMPLETE</value>
--<desc></desc>
--</constant>
--<constant>
--<name>WSA_INVALID_HANDLE</name>
--<value>ERROR_INVALID_HANDLE</value>
--<desc></desc>
--</constant>
--<constant>
--<name>WSA_INVALID_PARAMETER</name>
--<value>ERROR_INVALID_PARAMETER</value>
--<desc></desc>
--</constant>
--<constant>
--<name>WSA_NOT_ENOUGH_MEMORY</name>
--<value>ERROR_NOT_ENOUGH_MEMORY</value>
--<desc></desc>
--</constant>
--<constant>
--<name>WSA_OPERATION_ABORTED</name>
--<value>ERROR_OPERATION_ABORTED</value>
--<desc></desc>
--</constant>

-------------------------------------------------------------------------------
-- Sizes and offsets of some critical data structures used by the Winsock API
-------------------------------------------------------------------------------
constant SIZEOF_IN_ADDR             -- struct in_addr {
    = 4                             --   union {
                                    --     struct {u_char s_b1, s_b2, s_b3, s_b4;} S_un_b;
                                    --     struct {u_short s_w1, s_w2;} S_un_w;
                                    --     u_long S_addr;
                                    --   } S_un;
                                    -- };

constant SIZEOF_SOCKADDR_IN         -- struct sockaddr_in {
    = 2                             --   short   sin_family;
    + 2                             --   u_short sin_port;
    + SIZEOF_IN_ADDR                --   struct  in_addr sin_addr;
    + 8                             --   char    sin_zero[8];
                                    -- };

constant SIZEOF_WSADATA             -- typedef struct WSAData {
    = 2                             --   WORD wVersion;
    + 2                             --   WORD wHighVersion;
    + 257                           --   char szDescription[WSADESCRIPTION_LEN+1];
    + 129                           --   char szSystemStatus[WSASYS_STATUS_LEN+1];
    + 2                             --   unsigned short iMaxSockets;
    + 2                             --   unsigned short iMaxUdpDg;
    + 4                             --   char * lpVendorInfo;
                                    -- } WSADATA;

constant OFFSET_HOST_ADDR_LIST      -- typedef struct hostent {
    = 4                             --   char FAR* h_name;
    + 4                             --   char FAR FAR** h_aliases;
    + 2                             --   short h_addrtype;
    + 2                             --   short h_length;
                                    --   char FAR FAR** h_addr_list;
                                    -- } hostent;

constant SIZEOF_FD_SET              -- typedef struct fd_set {
    = 4                             --   u_int fd_count;
    + 4 * 64                        --   SOCKET fd_array[FD_SETSIZE];
                                    -- } fd_set;

constant SIZEOF_TIMEVAL             -- struct timeval {
    = 4                             --   long tv_sec;
    + 4                             --   long tv_usec;
                                    -- };

-------------------------------------------------------------------------------
-- Define Winsock API calls and other variables with file scope.
-------------------------------------------------------------------------------
object void
atom WSAData WSAData = 0

-------------------------------------------------------------------------------

integer xStartup
xStartup = 0

global function WSAStartup()
--<function>
--<name>WSAStartup</name>
--<digest>Load the WS2_32.DLL winsock library version 2.2.</digest>
--<desc>
-- Must be called successfully before using any Winsock functions.
--</desc>
--<return>
-- Returns zero if successful, otherwise returns a non-zero error code.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    atom result
    if WSAData = 0 then
        WSAData = allocate(SIZEOF_WSADATA)
        result = exec_w32func( xStartup, {#0202, WSAData}, "WSAStartup" )
        if result != 0 then free(WSAData) end if
        return result
    end if
    return 0
end function

-------------------------------------------------------------------------------

integer xCleanup
xCleanup = 0

global function WSACleanup()
--<function>
--<name>WSACleanup</name>
--<digest>Free socket resources and unload the Winsock library.</digest>
--<desc>
--</desc>
--<return>
-- Returns zero if successful, otherwise returns SOCKET_ERROR.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    atom result
    if WSAData then
        result = exec_w32func( xCleanup, {}, "WSACleanup" )
        free(WSAData)
        WSAData = 0
        return result
    end if
    return 0
end function

-------------------------------------------------------------------------------

integer xLastErr
xLastErr = 0

global function WSAGetLastError()
--<function>
--<name>WSAGetLastError</name>
--<digest>Returns the error status for the last operation that failed.</digest>
--<desc>
--</desc>
--<return>
-- integer: error code
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    return exec_w32func( xLastErr, {}, "WSAGetLastError" )
end function

-------------------------------------------------------------------------------
-- htonl()

integer xHtonl
xHtonl = 0

global function htonl(atom a)
--<function>
--<name>htonl</name>
--<digest>Perform host to network type conversion on a long (4-byte) integer.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>a</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    return exec_w32func(xHtonl, {a}, "htonl")
end function

-------------------------------------------------------------------------------

integer xHtons
xHtons = 0

global function htons(atom a)
--<function>
--<name>htons</name>
--<digest>Perform host to network type conversion on a short (2-byte) integer.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>a</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    return exec_w32func(xHtons, {a}, "htons")
end function

-------------------------------------------------------------------------------

integer xSocket
xSocket = 0

global function socket(integer af, integer socktype, integer pf)
--<function>
--<name>socket</name>
--<digest>Creates a new socket of the specified family and type.</digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>af</name>
--<desc>Address family, typically AF_INET for internet addresses.</desc>
--</param>
--<param>
--<type>integer</type>
--<name>socktype</name>
--<desc>Socket type, use SOCK_DGRAM for UDP or SOCK_STREAM for TCP.</desc>
--</param>
--<param>
--<type>integer</type>
--<name>pf</name>
--<desc>
-- Protocol family, set to zero and let Windows determine it
-- based on the address family.
--</desc>
--</param>
--<return>
-- If successful, returns a pointer referencing the new socket.
-- Otherwise, INVALID_SOCKET is returned.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    return exec_w32func( xSocket, {af, socktype, pf}, "socket" )
end function

-------------------------------------------------------------------------------

integer xSetOpt
xSetOpt = 0

global function setsockopt(integer Socket, integer Option, object OptVal)
--<function>
--<name>setsockopt</name>
--<digest>Sets the current value for a socket option.</digest>
--<desc>
-- Sets the current value for a socket option associated with a socket of any 
-- type, in any state. Only top-level socket options (SOL_SOCKET) are supported
-- here, to make things simple. 
--</desc>
--<param>
--<type>integer</type>
--<name>Socket</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>Option</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>OptVal</name>
--<desc></desc>
--</param>
--<return>Returns zero or SOCKET_ERROR on failure.</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    atom objLen, pValue, result
    if atom(OptVal) then -- option is type integer or boolean
        objLen = 4
        pValue = allocate(objLen)
        poke4(pValue,OptVal)
    else -- option is a structure (option SO_LINGER only)
        objLen = length(OptVal) 
        pValue = allocate(objLen)
        poke(pValue,OptVal)
    end if
    result = exec_w32func( xSetOpt, {Socket, SOL_SOCKET, Option, pValue, objLen}, "setsockopt" )
    free(pValue)
    return result
end function

-------------------------------------------------------------------------------

integer xGetHost
xGetHost = 0

global function gethostbyname(sequence Host)
--<function>
--<name>gethostbyname</name>
--<digest>Retrieves host information corresponding to a host name from a host database.</digest>
--<desc>
-- This function returns only the address portion of Winsock hostent structure.
--</desc>
--<param>
--<type>sequence</type>
--<name>Host</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    atom pString, pHostent, ppInAddr
    pString = allocate_string(Host)
    pHostent = exec_w32func( xGetHost, {pString}, "gethostbyname" )
    free(pString)
    if pHostent != 0 then
        ppInAddr = peek4u(pHostent + OFFSET_HOST_ADDR_LIST)
        return peek4u(peek4u(ppInAddr))
    end if
    return SOCKET_ERROR
end function

-------------------------------------------------------------------------------

global function sockaddr_in(integer Family, sequence Host, integer Port)
--<function>
--<name>sockaddr_in</name>
--<digest>Populate a sockaddr_in structure with an internet address and port number.</digest>
--<desc>
-- This function calls gethostbyname() to convert a domain name or
-- dotted decimal string into a machine readable address.
-- It also takes care of all host-to-network type conversions.
-- 
-- To fill the structure with the IP address of the local host,
-- pass an empty sequence as the host parameter. To have the system
-- automatically select an unused port on the local host, pass 0 
-- as the port parameter.
--</desc>
--<param>
--<type>integer</type>
--<name>Family</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>Host</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>Port</name>
--<desc></desc>
--</param>
--<return>
-- A sequence representing the socket address (struct sockaddr_in)
-- or SOCKET_ERROR on failure.
--</return>
--<example>
-- sockAddrIn("www.rapideuphoria.com", 80)
-- sockAddrIn("127.0.0.1", 8080) -- Port 8080 on the localhost
-- sockAddrIn("", 0)             -- Any port on the localhost
--</example>
--<see_also>
--</see_also>
--</function>
    atom InAddr
    sequence SockAddrIn

    if equal(Host,{}) then
        InAddr = INADDR_ANY
    else
        InAddr = gethostbyname(Host)
        if InAddr = 0 then 
            return SOCKET_ERROR
        end if
    end if

    Port = htons(Port)
    SockAddrIn = int_to_bytes(Family)
    SockAddrIn = SockAddrIn[1..2] & int_to_bytes(Port)
    SockAddrIn = SockAddrIn[1..4] & int_to_bytes(InAddr)
    SockAddrIn = SockAddrIn & repeat(0,8)
    return SockAddrIn

end function

-------------------------------------------------------------------------------

integer xBind
xBind = 0

global function bind(integer sockfd, sequence sockaddr)
--<function>
--<name>bind</name>
--<digest>Associates a local address with a socket.</digest>
--<desc>
-- Must be used on an unconnected socket before a subsequent call to the 
-- listen() function.
--</desc>
--<param>
--<type>integer</type>
--<name>sockfd</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>sockaddr</name>
--<desc></desc>
--</param>
--<return>
-- Returns zero or SOCKET_ERROR on failure.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    atom cptr, retval
    cptr = allocate_string(sockaddr)
    retval = exec_w32func( xBind, {sockfd, cptr, length(sockaddr)}, "bind" )
    free(cptr)
    return retval
end function

-------------------------------------------------------------------------------

integer xConnect
xConnect = 0

global function connect(integer sockdesc, sequence sockaddr)
--<function>
--<name>connect</name>
--<digest>Establishes a connection to a specified socket.</digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>sockdesc</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>sockaddr</name>
--<desc></desc>
--</param>
--<return>
-- Returns zero if successful, otherwise returns SOCKET_ERROR.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    atom cptr, sizeof, retval
    sizeof = length(sockaddr)
    cptr = allocate(sizeof)
    poke(cptr, sockaddr)
    retval = exec_w32func( xConnect, {sockdesc, cptr, sizeof}, "connect" )
    free(cptr)
    return retval
end function

-------------------------------------------------------------------------------

integer xListen
xListen = 0

global function listen(integer sockdesc, integer backlog)
--<function>
--<name>listen</name>
--<digest>Places a bound socket in a state in which it is listening for an incoming connection.</digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>sockdesc</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>backlog</name>
--<desc>Maximum number of pending connections.</desc>
--</param>
--<return>
-- Returns zero if successful, otherwise returns SOCKET_ERROR.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    return exec_w32func( xListen, {sockdesc, backlog}, "listen" )
end function

-------------------------------------------------------------------------------

integer xAccept
xAccept = 0

global function accept(integer sockdesc)
--<function>
--<name>accept</name>
--<digest>Permits an incoming connection attempt on a socket.</digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>sockdesc</name>
--<desc></desc>
--</param>
--<return>
-- Returns a value that is a descriptor for a new socket. The returned value 
-- is a handle for the socket on which the actual connection is made. In case
-- of error, this function returns a value of INVALID_SOCKET.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    return exec_w32func( xAccept, {sockdesc, NULL, NULL}, "accept" )
end function

-------------------------------------------------------------------------------

integer xSelect
xSelect = 0

global function select(sequence ReadSet, sequence WriteSet,
                        sequence ErrorSet, sequence Timeout)
--<function>
--<name>select</name>
--<digest>Determine the status of one or more sockets.</digest>
--<desc>
-- For each socket, the caller can request information on read, write, or 
-- error status. The set of sockets for which a given status is requested is 
-- indicated by a sequence of socket handles.
--</desc>
--<param>
--<type>sequence</type>
--<name>ReadSet</name>
--<desc>Sockets to check for read status</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>WriteSet</name>
--<desc>Sockets to check for write status</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>ErrorSet</name>
--<desc>Sockets to check for error status</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>Timeout</name>
--<desc>
-- Timeout is a sequence of the form {seconds, microsecs} that specifies the 
-- maximum time select() should wait before returning. If the timeout value is 
-- {0,0} then select() will check the status of sockets and return immediately. 
-- If Timeout is an empty sequence, select() will block indefinitely until at 
-- least one of the sockets meets the specified criteria.
--</desc>
--</param>
--<return>
-- {integer result, sequence readSet, sequence writeSet, sequence errorSet}
-- Status = SOCKET_ERROR : An error occurred.
-- Status = 0 : The time limit expired.
-- Status > 0 : The total number of sockets that meet the criteria;
--              sequences contain handles of the matching sockets.
--</return>
--<example>
-- Wait for 10 seconds or until Sock1 or Sock2 have data to read:
--     select({Sock1,Sock2}, {}, {}, {10,0})
--
-- Check for errors on Sock1 and return immediately:
--     select({}, {}, {Sock1}, {0,0})
--
-- Wait indefinitely for incoming Sock1 data, or Sock2 is ready to send:
--     select({Sock1},{Sock2},{},{})
--</example>
--<see_also>
--</see_also>
--</function>
    atom ReadPtr, WritePtr, ErrorPtr, TimeoutPtr
    integer retval

    if not equal(ReadSet,{}) then
        ReadPtr = allocate(SIZEOF_FD_SET)
        poke4(ReadPtr, length(ReadSet) & ReadSet)
    else ReadPtr = NULL end if

    if not equal(WriteSet,{}) then
        WritePtr = allocate(SIZEOF_FD_SET)
        poke4(WritePtr, length(WriteSet) & WriteSet)
    else WritePtr = NULL end if

    if not equal(ErrorSet,{}) then
        ErrorPtr = allocate(SIZEOF_FD_SET)
        poke4(ErrorPtr, length(ErrorSet) & ErrorSet)
    else ErrorPtr = NULL end if

    if not equal(Timeout,{}) then
        TimeoutPtr = allocate(SIZEOF_TIMEVAL)
        poke4(TimeoutPtr, Timeout)
    else TimeoutPtr = NULL end if

    retval = exec_w32func (xSelect,{0, ReadPtr, WritePtr, ErrorPtr, TimeoutPtr}, "select")

    if ReadPtr != NULL then 
        ReadSet = peek4u({ReadPtr+4, peek4u(ReadPtr)})
        free(ReadPtr)
    end if

    if WritePtr != NULL then 
        WriteSet = peek4u({WritePtr+4, peek4u(WritePtr)})
        free(WritePtr)
    end if

    if ErrorPtr != NULL then 
        ErrorSet = peek4u({ErrorPtr+4, peek4u(ErrorPtr)})
        free(ErrorPtr)
    end if

    free(TimeoutPtr)
    return {retval, ReadSet, WriteSet, ErrorSet}
end function

-------------------------------------------------------------------------------

integer xGetName
xGetName = 0

global function GetSockName(integer hSocket)
--<function>
--<name>GetSockName</name>
--<digest>Retrieves the local name (family, address, and port) for a socket.</digest>
--<desc>
-- This is the only way to determine the local association of socket/name
-- when a connect() call has been made without doing a bind() first.
--</desc>
--<param>
--<type>integer</type>
--<name>hSocket</name>
--<desc></desc>
--</param>
--<return>
-- A sequence representing the socket address (struct sockaddr_in)
-- or SOCKET_ERROR on failure.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  object retval
  atom name, namelen

  name = allocate(SIZEOF_SOCKADDR_IN)
  namelen = allocate(4)
  poke4(namelen, SIZEOF_SOCKADDR_IN)
  retval = exec_w32func (xGetName, {hSocket, name, namelen}, "getsockname")
  if retval != SOCKET_ERROR then
    retval = peek({name, peek4u(namelen)}) 
  end if
  free(name)
  free(namelen)
  return retval
end function

-------------------------------------------------------------------------------

integer xGetPeer
xGetPeer = 0

global function GetPeerName(integer hSocket)
--<function>
--<name>GetPeerName</name>
--<digest>Retrieves the name of the peer to which a socket is connected.</digest>
--<desc>
-- This does not work for unconnected datagram sockets.
--</desc>
--<param>
--<type>integer</type>
--<name>hSocket</name>
--<desc></desc>
--</param>
--<return>
-- A sequence representing the peer address (struct sockaddr_in)
-- or SOCKET_ERROR on failure.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  object retval
  atom name, namelen

  name = allocate(SIZEOF_SOCKADDR_IN)
  namelen = allocate(4)
  poke4(namelen, SIZEOF_SOCKADDR_IN)
  retval = exec_w32func (xGetPeer, {hSocket, name, namelen}, "getpeername")
  if retval != SOCKET_ERROR then
    retval = peek({name, peek4u(namelen)}) 
  end if
  free(name)
  free(namelen)
  return retval
end function

------------------------------------------------------------------------------

global function GetPort(atom hsock)
--<function>
--<name>GetPort</name>
--<digest>get the port that is bound to the socket</digest>
--<desc>
-- low-level command
--</desc>
--<param>
--<type>atom</type>
--<name>hsock</name>
--<desc>TCP socket</desc>
--</param>
--<return>
-- integer: port number
--</return>
--<example>
-- port_used = GetPort(tcpSocket)
-- port_used = GetPort(activeConnections[cnx][TCP_SOCKET])
--</example>
--<see_also>
--</see_also>
--</function>
  sequence Address

  Address = GetSockName(hsock)
  return Address[4] + Address[3] * 256
end function

-------------------------------------------------------------------------------

integer xSend
xSend = 0

global function send(integer socket, sequence message)
--<function>
--<name>send</name>
--<digest>Sends data on a connected socket.</digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>socket</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>message</name>
--<desc></desc>
--</param>
--<return>
-- Returns the actual number of bytes transmitted, which may be less than the
-- total message length. Returns SOCKET_ERROR on failure.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    atom cptr
    integer result
    cptr = allocate_string(message)
    result = exec_w32func (xSend, {socket, cptr, length(message), 0}, "send")
    free(cptr)
    return result
end function

-------------------------------------------------------------------------------

integer xSendTo
xSendTo = 0

global function sendto(integer Socket, sequence Message, integer Flags,
                       sequence InetSockAddr)
--<function>
--<name>sendto</name>
--<digest>Sends data on an unconnected socket to a specific destination.</digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>Socket</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>Message</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>Flags</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>InetSockAddr</name>
--<desc></desc>
--</param>
--<return>
-- Returns the actual number of bytes transmitted, or SOCKET_ERROR on failure.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    atom Buffer, AddrPtr
    integer result
    Buffer = allocate_string(Message)
    AddrPtr = allocate(SIZEOF_SOCKADDR_IN)
    poke(AddrPtr, InetSockAddr)
    result = exec_w32func( xSendTo, {Socket, Buffer, length(Message), Flags, AddrPtr, SIZEOF_SOCKADDR_IN}, "sendto" )
    free(Buffer)
    free(AddrPtr)
    return result
end function

-------------------------------------------------------------------------------

integer xRecv
xRecv = 0

global function recv(integer socket, integer bytes)
--<function>
--<name>recv</name>
--<digest>Receives data from a connected or bound socket.</digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>socket</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>bytes</name>
--<desc></desc>
--</param>
--<return>
-- Returns a sequence of the form {result, buffer} where
--     result = the actual number of bytes received or SOCKET_ERROR, and
--     buffer = a sequence containing the received characters.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    atom cptr
    sequence buffer
    integer result
    cptr = allocate(bytes)
    result = exec_w32func (xRecv, {socket, cptr, bytes, 0}, "recv")
    if result = SOCKET_ERROR then buffer = {}
    else buffer = peek({cptr, result}) end if
    free(cptr)
    return {result, buffer}
end function

-------------------------------------------------------------------------------

integer xClose
xClose = 0

global function closesocket(integer socket)
--<function>
--<name>closesocket</name>
--<digest>Closes an existing socket.</digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>socket</name>
--<desc></desc>
--</param>
--<return>
-- Returns zero if successful, otherwise returns SOCKET_ERROR.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
    return exec_w32func (xClose, {socket}, "closesocket")
end function

------------------------------------------------------------------------------

global procedure handle_error()
--<procedure>
--<name>handle_error</name>
--<digest>Displays details on last error</digest>
--<desc>
--</desc>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  integer err
  sequence msg, title

  err   = WSAGetLastError()
  msg   = FormatSystemError(err)
  title = sprintf("Winsock Error %d", err)
  void  = message_box(msg, title, MB_OK)
end procedure
