--
-- euSQLite 0.3 - A SQLite wrapper
--
-- (c) 2002 Ray Smith
-- smithr@ix.net.au
--
-- converted SQLite 2.8.15 21/12/2004 C M Burch
-- converted SQLite 3.0.8 21/12/2004 C M Burch
--
-- Version 0.3 by Tone Skoda on 24th August 2005:
-- - additional wrappers were added
-- - fatal errors are automatically reported
-- - renamed my_sqlite3_* to xsqlite3_* and made them global
-- - renamed myLib to sqlite3_dll and made it global
-- - and other small additions to code

-- Name :       EuSQLite3 - an SQLite wrapper
-- Version :    0.3
-- Author :     Ray Smith
-- License :    None
-- Updated :    Chris Burch
-- Updated :    August 2005 Tone Skoda

------------------------------------------------------------------------------

-- Euphoria object compression and decompression
--
-- Copied from database.e and modified so that it works with RAM and not with file.
-- Tone Skoda, 26. August 2005

-- global constant DEBUG = 0, EXTRA_DEBUG = 0, APP_NAME = ""

------------------------------------------------------------------------------

include dll.e
include machine.e
include get.e
include misc.e
include _dll_.e
include _external_.e
with warning

sequence Bytes -- read from here instead from file
integer Byte_pos -- instead of file position

-- Compressed format of Euphoria objects on disk
--
-- First byte:
--          0..248    -- immediate small integer, -9 to 239
          -- since small negative integers -9..-1 might be common
constant
  I2B = 249,   -- 2-byte signed integer follows
  I3B = 250,   -- 3-byte signed integer follows
  I4B = 251,   -- 4-byte signed integer follows
  F4B = 252,   -- 4-byte f.p. number follows
  F8B = 253,   -- 8-byte f.p. number follows
  S1B = 254,   -- sequence, 1-byte length follows, then elements
  S4B = 255    -- sequence, 4-byte length follows, then elements

constant
  MIN1B = -9,
  MAX1B = 239,
  MIN2B = -power(2, 15),
  MAX2B =  power(2, 15)-1,
  MIN3B = -power(2, 23),
  MAX3B =  power(2, 23)-1,
  MIN4B = -power(2, 31)

-- emulation of getc(); this one reads from sequence and not from file.
-- this helps for easier translation of decompress_() to read from sequence and not from file.
-- i didn't need to modify decompress_() except change getc() to getc_() and
-- get4() to get4_() 
function getc_ ()
  integer c

  c = Bytes [Byte_pos]
  Byte_pos += 1
  return c
end function

atom mem0, mem1, mem2, mem3
mem0 = allocate(4)
mem1 = mem0 + 1
mem2 = mem0 + 2
mem3 = mem0 + 3

-- emulation of get4()
function get4_()
-- read 4-byte value at current position in sequence
  poke(mem0, getc_())
  poke(mem1, getc_())
  poke(mem2, getc_())
  poke(mem3, getc_())
  return peek4u(mem0)
end function

function decompress_(integer c)
  sequence s
  integer len
  
  if c = 0 then
    c = getc_()
    if c < I2B then
      return c + MIN1B
    end if
  end if
    
  if c = I2B then
    return getc_() + #100 * getc_() + MIN2B
  elsif c = I3B then
    return getc_() + #100 * getc_() + #10000 * getc_() + MIN3B
  elsif c = I4B then
    return get4_() + MIN4B
  elsif c = F4B then
    return float32_to_atom({getc_(), getc_(), getc_(), getc_()})
  elsif c = F8B then
    return float64_to_atom({getc_(), getc_(), getc_(), getc_(), getc_(), getc_(),  getc_(), getc_()})
  else
    -- sequence
    if c = S1B then
      len = getc_()
    else
      len = get4_()
    end if
    s = repeat(0, len)
    for i = 1 to len do
      -- in-line small integer for greater speed on strings
      c = getc_()
      if c < I2B then
        s[i] = c + MIN1B
      else
        s[i] = decompress_(c)
      end if
    end for
    return s
  end if
end function

global function decompress (sequence bytes)
--<function>
--<name>decompress</name>
--<digest>
-- decompress a sequence of bytes which should represent
-- a Euphoria object, and return it
--</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>bytes</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
   Bytes = bytes
   Byte_pos = 1
   return decompress_ (0)
end function

global function compress(object x)
--<function>
--<name>compress</name>
--<digest>
-- return the compressed representation of a Euphoria object 
-- as a sequence of bytes
--</digest>
--<desc>
--</desc>
--<param>
--<type>object</type>
--<name>x</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  sequence x4, s
    
  if integer(x) then
    if x >= MIN1B and x <= MAX1B then
      return {x - MIN1B}
    elsif x >= MIN2B and x <= MAX2B then
      x -= MIN2B
      return {I2B, and_bits(x, #FF), floor(x / #100)}
    elsif x >= MIN3B and x <= MAX3B then
      x -= MIN3B
      return {I3B, and_bits(x, #FF), and_bits(floor(x / #100), #FF), floor(x / #10000)}
    else
      return I4B & int_to_bytes(x-MIN4B)    
    end if
    
  elsif atom(x) then
    -- floating point
    x4 = atom_to_float32(x)
    if x = float32_to_atom(x4) then
      -- can represent as 4-byte float
      return F4B & x4
    else
      return F8B & atom_to_float64(x)
    end if

  else
    -- sequence
    if length(x) <= 255 then
      s = {S1B, length(x)}
    else
      s = S4B & int_to_bytes(length(x))
    end if  
    for i = 1 to length(x) do
      s &= compress(x[i])
    end for
    return s
  end if
end function

------------------------------------------------------------------------------

global constant 
  SQLITE_OK           = 0,
--<constant>
--<name>SQLITE_OK</name>
--<value>0</value>
--<desc>Successful result</desc>
--</constant>
  SQLITE_ERROR        = 1,
--<constant>
--<name>SQLITE_ERROR</name>
--<value>1</value>
--<desc>SQL error or missing database</desc>
--</constant>
  SQLITE_INTERNAL     = 2,
--<constant>
--<name>SQLITE_INTERNAL</name>
--<value>2</value>
--<desc>An internal logic error in SQLite</desc>
--</constant>
  SQLITE_PERM         = 3,
--<constant>
--<name>SQLITE_PERM</name>
--<value>3</value>
--<desc>Access permission denied</desc>
--</constant>
  SQLITE_ABORT        = 4,
--<constant>
--<name>SQLITE_ABORT</name>
--<value>4</value>
--<desc>Callback routine requested an abort</desc>
--</constant>
  SQLITE_BUSY         = 5,
--<constant>
--<name>SQLITE_BUSY</name>
--<value>5</value>
--<desc>The database file is locked</desc>
--</constant>
  SQLITE_LOCKED       = 6,
--<constant>
--<name>SQLITE_LOCKED</name>
--<value>6</value>
--<desc>A table in the database is locked</desc>
--</constant>
  SQLITE_NOMEM        = 7,
--<constant>
--<name>SQLITE_NOMEM</name>
--<value>7</value>
--<desc>A malloc() failed</desc>
--</constant>
  SQLITE_READONLY     = 8,
--<constant>
--<name>SQLITE_READONLY</name>
--<value>8</value>
--<desc>Attempt to write a readonly database</desc>
--</constant>
  SQLITE_INTERRUPT    = 9,
--<constant>
--<name>SQLITE_INTERRUPT</name>
--<value>9</value>
--<desc>Operation terminated by sqlite_interrupt()</desc>
--</constant>
  SQLITE_IOERR        = 10,
--<constant>
--<name>SQLITE_IOERR</name>
--<value>10</value>
--<desc>Some kind of disk I/O error occurred</desc>
--</constant>
  SQLITE_CORRUPT      = 11,
--<constant>
--<name>SQLITE_CORRUPT</name>
--<value>11</value>
--<desc>The database disk image is malformed</desc>
--</constant>
  SQLITE_NOTFOUND     = 12,
--<constant>
--<name>SQLITE_NOTFOUND</name>
--<value>12</value>
--<desc>(Internal Only) Table or record not found</desc>
--</constant>
  SQLITE_FULL         = 13,
--<constant>
--<name>SQLITE_FULL</name>
--<value>13</value>
--<desc>Insertion failed because database is full</desc>
--</constant>
  SQLITE_CANTOPEN     = 14,
--<constant>
--<name>SQLITE_CANTOPEN</name>
--<value>14</value>
--<desc>Unable to open the database file</desc>
--</constant>
  SQLITE_PROTOCOL     = 15,
--<constant>
--<name>SQLITE_PROTOCOL</name>
--<value>15</value>
--<desc>Database lock protocol error</desc>
--</constant>
  SQLITE_EMPTY        = 16,
--<constant>
--<name>SQLITE_EMPTY</name>
--<value>16</value>
--<desc>(Internal Only) Database table is empty</desc>
--</constant>
  SQLITE_SCHEMA       = 17,
--<constant>
--<name>SQLITE_SCHEMA</name>
--<value>17</value>
--<desc>The database schema changed</desc>
--</constant>
  SQLITE_TOOBIG       = 18,
--<constant>
--<name>SQLITE_TOOBIG</name>
--<value>18</value>
--<desc>Too much data for one row of a table</desc>
--</constant>
  SQLITE_CONSTRAINT   = 19,
--<constant>
--<name>SQLITE_CONSTRAINT</name>
--<value>19</value>
--<desc>Abort due to constraint violation</desc>
--</constant>
  SQLITE_MISMATCH     = 20,
--<constant>
--<name>SQLITE_MISMATCH</name>
--<value>20</value>
--<desc>Data type mismatch</desc>
--</constant>
  SQLITE_MISUSE       = 21,
--<constant>
--<name>SQLITE_MISUSE</name>
--<value>21</value>
--<desc>Library used incorrectly</desc>
--</constant>
  SQLITE_NOLFS        = 22,
--<constant>
--<name>SQLITE_NOLFS</name>
--<value>22</value>
--<desc>Uses OS features not supported on host</desc>
--</constant>
  SQLITE_AUTH         = 23,
--<constant>
--<name>SQLITE_AUTH</name>
--<value>23</value>
--<desc>Authorization denied</desc>
--</constant>
  SQLITE_ROW          = 100,
--<constant>
--<name>SQLITE_ROW</name>
--<value>100</value>
--<desc>sqlite_step() has another row ready</desc>
--</constant>
  SQLITE_DONE         = 101,
--<constant>
--<name>SQLITE_DONE</name>
--<value>101</value>
--<desc>sqlite_step() has finished executing</desc>
--</constant>
  SQLITE_STATIC       = 0,
--<constant>
--<name>SQLITE_STATIC</name>
--<value>0</value>
--<desc></desc>
--</constant>
  SQLITE_TRANSIENT    = -1
--<constant>
--<name>SQLITE_TRANSIENT</name>
--<value>-1</value>
--<desc></desc>
--</constant>

--*************************************************
-- Globals
--*************************************************

global integer sqlite_last_err_no
--<variable>
--<type>integer</type>
--<name>sqlite_last_err_no</name>
--<desc></desc>
--</variable>
sqlite_last_err_no = SQLITE_OK
global sequence sqlite_last_err_desc
--<variable>
--<type>sequence</type>
--<name>sqlite_last_err_desc</name>
--<desc></desc>
--</variable>
sqlite_last_err_desc = ""

-- the maximum length of an individual column returned from exec and get_table
global integer SQLITE_MAX_FIELD_LENGTH
--<variable>
--<type>integer</type>
--<name>SQLITE_MAX_FIELD_LENGTH</name>
--<desc></desc>
--</variable>
SQLITE_MAX_FIELD_LENGTH = 32768

-- the maximum length of error messages 
global integer SQLITE_MAX_ERR_LENGTH
--<variable>
--<type>integer</type>
--<name>SQLITE_MAX_ERR_LENGTH</name>
--<desc></desc>
--</variable>
SQLITE_MAX_ERR_LENGTH = 128

-- the maximum length of the version string
global integer SQLITE_MAX_VERSION_LENGTH
--<variable>
--<type>integer</type>
--<name>SQLITE_MAX_VERSION_LENGTH</name>
--<desc></desc>
--</variable>
SQLITE_MAX_VERSION_LENGTH = 64

-------------------------------------------------------------------------------------

integer xsqlite3_errmsg
xsqlite3_errmsg = 0

global function sqlite_errmsg(atom db)
--<function>
--<name>sqlite_errmsg</name>
--<digest>sqlite3 error messages</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  sequence message
  atom message_addr

  message_addr = exec_func(xsqlite3_errmsg, {db}, "+sqlite3_errmsg")
  message = peek_string(message_addr)
  return message
end function

-------------------------------------------------------------------------------------

procedure default_fatal(atom db, sequence on_command)
-- default fatal error handler - you can override this
   printf (1, "Fatal SQLite Error: %s\nWhen Executing: %s\nError Number: %d\n\n",
      {sqlite_last_err_desc, on_command, sqlite_last_err_no})
    ?1/0 -- to see call stack
end procedure

-- exception handler
global integer sqlite_fatal_id -- TODO: document this
--<variable>
--<type>integer</type>
--<name>sqlite_fatal_id</name>
--<desc></desc>
--</variable>
sqlite_fatal_id = routine_id("default_fatal") -- you can set it to your own handler

-------------------------------------------------------------------------------------

integer xsqlite3_close
xsqlite3_close = 0

global procedure sqlite_close(atom db)
--<procedure>
--<name>sqlite_close</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  exec_proc(xsqlite3_close, {db}, "+sqlite3_close")
end procedure

-------------------------------------------------------------------------------------

procedure fatal(atom db, sequence on_command)
  if db > 0 then sqlite_close(db) end if
  call_proc(sqlite_fatal_id, {db, on_command})
end procedure

-------------------------------------------------------------------------------------

integer xsqlite3_free
xsqlite3_free = 0

global procedure sqlite_free (atom addr)
--<procedure>
--<name>sqlite_free</name>
--<digest>Frees memory allocated from mprintf() or vmprintf().</digest>
--<desc>
-- Used internally by sqlite
--</desc>
--<param>
--<type>atom</type>
--<name>addr</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  exec_proc(xsqlite3_free, {addr}, "+sqlite3_free")
end procedure

-------------------------------------------------------------------------------------

integer xsqlite3_open
xsqlite3_open = 0

global function sqlite_open(sequence filename, integer mode)
--<function>
--<name>sqlite_open</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>filename</name>
--<desc>filename of the database</desc>
--</param>
--<param>
--<type>integer</type>
--<name>mode</name>
--<desc>mode of the database access</desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom filename_addr, err_no, db, db_addr

  filename_addr = allocate_string(filename)   --put the filename into memory
  db_addr = allocate(4)                       --allocate memory for the db handle
        
  err_no = exec_func(xsqlite3_open, {filename_addr, db_addr}, "+sqlite3_open")
  db = peek4u(db_addr)
  sqlite_last_err_no = err_no
  if err_no != SQLITE_OK then
    sqlite_last_err_desc = sqlite_errmsg(db)
    fatal(db, "sqlite_open()")
  else 
    sqlite_last_err_desc = ""
  end if
  return db
end function

-------------------------------------------------------------------------------------

integer xsqlite3_exec
xsqlite3_exec = 0

-- might need this one day
-----------------------------
-- sqlite_exec_callback
-----------------------------
--procedure sqlite_exec_callback(atom null_addr, integer cols, atom data_ptr, 
--                               atom col_name_ptr)
--
--
--   return 0
--end function

global function sqlite_exec(atom db, sequence cmd)
--<function>
--<name>sqlite_exec</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>cmd</name>
--<desc></desc>
--</param>
--<return>
-- returns SQLITE_OK, SQLITE_ABORT or SQLITE_BUSY
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>

  integer ret
  atom cmd_addr, err_ptr_addr, err_addr

  err_ptr_addr = allocate(4)
  poke4(err_ptr_addr, 0)
  cmd_addr = allocate_string(cmd)
   ret = exec_func(xsqlite3_exec,{db, cmd_addr, NULL, NULL, err_ptr_addr},
                   "+sqlite3_exec")

-- if using a callback TODO
-- ret = exec_func(xsqlite3_exec,{db, cmd_addr, 
--                 call_back(routine_id(sqlite_exec_callback)), NULL, err_ptr_addr},
--                 "+sqlite3_exec")

  free(cmd_addr)

  sqlite_last_err_no = SQLITE_OK
  sqlite_last_err_desc = ""
  if ret != SQLITE_OK then
    sqlite_last_err_no = ret  
    err_addr = peek4u(err_ptr_addr)
    if err_addr > 0 then
      sqlite_last_err_desc = peek_string(err_addr)  --, SQLITE_MAX_ERR_LENGTH)
      sqlite_free(err_addr)
    end if
    if not find (ret, {SQLITE_ABORT, SQLITE_BUSY}) then
      fatal (db, "sqlite_exec(\"" & cmd & "\"")
    end if
  end if
  return ret
end function

-------------------------------------------------------------------------------------

integer xsqlite3_free_table
xsqlite3_free_table = 0

global procedure sqlite_free_table(atom data_addr)
--<procedure>
--<name>sqlite_free_table</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>data_addr</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  exec_proc(xsqlite3_free_table, {data_addr}, "+sqlite3_free_table")
end procedure

-------------------------------------------------------------------------------------

integer xsqlite3_get_table
xsqlite3_get_table = 0

global function sqlite_get_table(atom db, sequence cmd)
--<function>
--<name>sqlite_get_table</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>cmd</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer ret, col, row 
  atom cmd_addr, col_addr, row_addr, data_ptr_addr, data_addr
  sequence data, tmp_row, tmp_field
  atom tmp_ptr_addr, err_ptr_addr, err_addr, field_addr

  cmd_addr = allocate_string(cmd)
  col_addr = allocate(4)
  row_addr = allocate(4)
  err_ptr_addr = allocate(4)
  poke4(err_ptr_addr, 0)
  data_ptr_addr = allocate(4)

  ret = exec_func(xsqlite3_get_table, {db, cmd_addr, data_ptr_addr, row_addr,
                  col_addr, err_ptr_addr}, "+sqlite3_get_table")   
  sqlite_last_err_no = SQLITE_OK
  sqlite_last_err_desc = ""
  if ret != SQLITE_OK then
    sqlite_last_err_no = ret  
    err_addr = peek4u(err_ptr_addr)
    if err_addr > 0 then
      sqlite_last_err_desc = peek_string(err_addr)  --, SQLITE_MAX_ERR_LENGTH)
      sqlite_free(err_addr)
    end if
    if not find (ret, {SQLITE_ABORT, SQLITE_BUSY}) then
      fatal (db, "sqlite_get_table(\"" & cmd & "\"")
    end if
  end if

  col = peek4u(col_addr)
  row = peek4u(row_addr)
  data_addr = peek4u(data_ptr_addr)

  free(cmd_addr)
  free(col_addr)
  free(row_addr)
  free(err_ptr_addr)
  free(data_ptr_addr)

  data = {}
  if row > 0 then
    tmp_ptr_addr = data_addr
    for r = 0 to row do
      tmp_row = {}
      for c = 1 to col do
        field_addr = peek4u(tmp_ptr_addr)
        if field_addr != 0 then
          tmp_field = peek_string(field_addr)  --, SQLITE_MAX_FIELD_LENGTH)
        else
          tmp_field = {}
        end if
        -- tmp_field = peek_string(peek4u(tmp_ptr_addr))  --, SQLITE_MAX_FIELD_LENGTH)
        tmp_ptr_addr += 4
        tmp_row = append(tmp_row, tmp_field)
      end for
      data = append(data, tmp_row)
    end for
  end if
  sqlite_free_table(data_addr) 
  return data
end function

-------------------------------------------------------------------------------------

integer xsqlite3_libversion
xsqlite3_libversion = 0

global function sqlite_libversion()
--<function>
--<name>sqlite_libversion</name>
--<digest></digest>
--<desc>
--</desc>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom addr

  addr = exec_func(xsqlite3_libversion, {}, "+sqlite3_libversion")
  return peek_string(addr)  -- , SQLITE_MAX_VERSION_LENGTH)
end function

-------------------------------------------------------------------------------------

integer xsqlite3_prepare
xsqlite3_prepare = 0

global function sqlite_prepare(atom db, sequence cmd)
--<function>
--<name>sqlite_prepare</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>cmd</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer ret
  atom cmd_addr, stmt_ptr_addr, stmt_addr

  cmd_addr = allocate_string(cmd)
  stmt_ptr_addr = allocate (4)
  ret = exec_func(xsqlite3_prepare,{db, cmd_addr, length (cmd), stmt_ptr_addr, 0},
                  "+sqlite3_prepare")
  sqlite_last_err_no = ret
  sqlite_last_err_desc = ""
  if ret != SQLITE_OK then  
    sqlite_last_err_no = ret      
    sqlite_last_err_desc = sqlite_errmsg (db)
    fatal (db, "sqlite_prepare(\"" & cmd & "\")")   
  end if
  stmt_addr = peek4u (stmt_ptr_addr)
  free(cmd_addr)
  free (stmt_ptr_addr)
  return stmt_addr
end function

-------------------------------------------------------------------------------------

integer xsqlite3_step
xsqlite3_step = 0

global function  sqlite_step(atom db, atom stmt)
--<function>
--<name>sqlite_step</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>stmt</name>
--<desc></desc>
--</param>
--<return>
-- Returns SQLITE_DONE, SQLITE_ROW, SQLITE_BUSY,
-- SQLITE_ERROR or SQLITE_MISUSE, see http://sqlite.org/capi3ref.html#sqlite3_step
-- SQLITE_ROW is returned if there were any row(s) found
-- and are ready to be read with sqlite_column_* functions.
-- SQLITE_DONE is returned if there was no row found.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer ret
   
  ret = exec_func(xsqlite3_step, {stmt}, "+sqlite3_step")
  sqlite_last_err_no = ret
  sqlite_last_err_desc = ""
  if ret = SQLITE_DONE
    or ret = SQLITE_ROW 
    or ret = SQLITE_BUSY then
      -- do nothing
  elsif ret = SQLITE_ERROR
    or ret = SQLITE_MISUSE then
    sqlite_last_err_desc = sqlite_errmsg (db)
    fatal (db, "sqlite_step()")
  end if
  return ret
end function

-------------------------------------------------------------------------------------

integer xsqlite3_reset
xsqlite3_reset = 0

global procedure sqlite_reset(atom db, atom stmt)
--<procedure>
--<name>sqlite_reset</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>stmt</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  integer ret
   
  ret = exec_func(xsqlite3_reset, {stmt}, "+sqlite3_reset")
  sqlite_last_err_no = ret
  sqlite_last_err_desc = ""
  if ret != SQLITE_OK then
    sqlite_last_err_desc = sqlite_errmsg (db)
    fatal (db, "sqlite_reset()")
  end if
end procedure

-------------------------------------------------------------------------------------

integer xsqlite3_finalize
xsqlite3_finalize = 0

global function sqlite_finalize(atom db, atom stmt)
--<function>
--<name>sqlite_finalize</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>stmt</name>
--<desc></desc>
--</param>
--<return>
-- can return SQLITE_ABORT, see http://sqlite.org/capi3ref.html#sqlite3_finalize
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer ret
   
  ret = exec_func(xsqlite3_finalize, {stmt}, "+sqlite3_finalize")
  sqlite_last_err_no = ret
  sqlite_last_err_desc = ""
  if ret != SQLITE_OK then
    sqlite_last_err_desc = sqlite_errmsg (db)
    if ret != SQLITE_ABORT then
      fatal (db, "sqlite_finalize()")
    end if
  end if
  return ret
end function

-------------------------------------------------------------------------------------

integer xsqlite3_bind_int
xsqlite3_bind_int = 0

global procedure sqlite_bind_int(atom db, atom stmt, integer param_index, integer val)
--<procedure>
--<name>sqlite_bind_int</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>stmt</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>param_index</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>val</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  integer ret
   
  ret = exec_func(xsqlite3_bind_int,{stmt, param_index, val}, "+sqlite3_bind_int")   
  sqlite_last_err_no = ret
  sqlite_last_err_desc = ""
  if ret != SQLITE_OK then
    sqlite_last_err_desc = sqlite_errmsg (db)
    fatal (db, "sqlite_bind_int()")
  end if
end procedure

-------------------------------------------------------------------------------------

integer xsqlite3_bind_double
xsqlite3_bind_double = 0

global procedure sqlite_bind_double(atom db, atom stmt, integer param_index, atom val)
--<procedure>
--<name>sqlite_bind_double</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>stmt</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>param_index</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>val</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  integer ret
   
  ret = exec_func(xsqlite3_bind_double,{stmt, param_index, val},
                  "+sqlite3_bind_double")
  sqlite_last_err_no = ret
  sqlite_last_err_desc = ""
  if ret != SQLITE_OK then
    sqlite_last_err_desc = sqlite_errmsg (db)
    fatal (db, "sqlite_bind_double()")
  end if
end procedure

-------------------------------------------------------------------------------------

integer xsqlite3_bind_text
xsqlite3_bind_text = 0

global procedure sqlite_bind_text(atom db, atom stmt, integer param_index, sequence val)
--<procedure>
--<name>sqlite_bind_text</name>
--<digest></digest>
--<desc>
-- You don't have to put text into ' quotes.
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>stmt</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>param_index</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>val</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  integer ret
  atom val_addr
    
  val_addr = allocate_string (val)
  ret = exec_func(xsqlite3_bind_text,{stmt, param_index, val_addr,
                  length (val), SQLITE_TRANSIENT}, "+sqlite3_bind_text")
  -- SQLITE_STATIC})            
  -- call_back (routine_id ("my_free"))}) -- this is very slow, i don't know why
  -- free (val_addr)
  sqlite_last_err_no = ret
  sqlite_last_err_desc = ""
  if ret != SQLITE_OK then
    sqlite_last_err_desc = sqlite_errmsg (db)
    fatal (db, "sqlite_bind_text()")
  end if
end procedure

-------------------------------------------------------------------------------------

integer xsqlite3_bind_blob
xsqlite3_bind_blob = 0

global procedure sqlite_bind_blob(atom db, atom stmt, integer param_index, object val)
--<procedure>
--<name>sqlite_bind_blob</name>
--<digest>used to store any Euphoria object into database</digest>
--<desc>
-- it compresses Euphoria object on the same way as EDS
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>stmt</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>param_index</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>val</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  integer ret
  sequence val_string
  atom val_addr   
    
  val_string = compress(val)
  val_addr = allocate (length (val_string))
  poke (val_addr, val_string)
      
  ret = exec_func(xsqlite3_bind_blob,{stmt, param_index, val_addr,
                  length (val_string), SQLITE_TRANSIENT}, "+sqlite3_bind_blob")
  -- SQLITE_STATIC})      
  -- call_back (routine_id ("my_free"))}) -- this is very slow, i don't know why
  -- free (val_addr)
   
  sqlite_last_err_no = ret
  sqlite_last_err_desc = ""
  if ret != SQLITE_OK then
    sqlite_last_err_desc = sqlite_errmsg (db)
    fatal (db, "sqlite_bind_blob()")
  end if   
end procedure

-------------------------------------------------------------------------------------

integer xsqlite3_column_int
xsqlite3_column_int = 0

global function sqlite_column_int(atom db, atom stmt, integer column_num)
--<function>
--<name>sqlite_column_int</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>stmt</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>column_num</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- column_num is 1-based
  return exec_func(xsqlite3_column_int,{stmt, column_num - 1},
                    "+sqlite3_column_int")
end function

-------------------------------------------------------------------------------------

integer xsqlite3_column_bytes
xsqlite3_column_bytes = 0

global function sqlite_column_bytes(atom db, atom stmt, integer column_num)
--<function>
--<name>sqlite_column_bytes</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>stmt</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>column_num</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- column_num is 1-based
  return exec_func(xsqlite3_column_bytes, {stmt, column_num - 1},
                    "+sqlite3_column_bytes")
end function

-------------------------------------------------------------------------------------

integer xsqlite3_column_text
xsqlite3_column_text = 0

global function sqlite_column_text(atom db, atom stmt, integer column_num)
--<function>
--<name>sqlite_column_text</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>stmt</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>column_num</name>
--<desc>column_num is 1-based</desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom addr, nBytes
      
  addr = exec_func(xsqlite3_column_text, {stmt, column_num - 1},
                   "+sqlite3_column_text")
  if addr then
    nBytes = exec_func(xsqlite3_column_bytes, {stmt, column_num - 1},
                       "+sqlite3_column_bytes")
    return peek ({addr, nBytes})
  else
    fatal(db, "sqlite_column_text(): can't get text")
    return ""
  end if
end function

-------------------------------------------------------------------------------------

integer xsqlite3_column_blob
xsqlite3_column_blob = 0

global function sqlite_column_blob(atom db, atom stmt, integer column_num)
--<function>
--<name>sqlite_column_blob</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>stmt</name>
--<desc></desc>
--</param>
--<param>
--<type>integer</type>
--<name>column_num</name>
--<desc>column_num is 1-based</desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom addr, nBytes
  sequence val_string
      
  addr = exec_func(xsqlite3_column_blob, {stmt, column_num - 1},
                   "+sqlite3_column_blob")
  if addr then
    nBytes = exec_func(xsqlite3_column_bytes, {stmt, column_num - 1},
                       "+sqlite3_column_bytes")
    val_string = peek ({addr, nBytes})
    return decompress (val_string)
  else
    fatal(db, "sqlite_column_blob(): can't get data")
    return -1
  end if
end function

-------------------------------------------------------------------------------------

integer xsqlite3_last_insert_rowid
xsqlite3_last_insert_rowid = 0

global function sqlite_last_insert_rowid(atom db)
--<function>
--<name>sqlite_last_insert_rowid</name>
--<digest>gets the last inserted row_id from open database db</digest>
--<desc>
--remember, to get the last inserted row, need to  work on an open database,
-- where the data has just been inserted
--</desc>
--<param>
--<type>atom</type>
--<name>db</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom row_id

  row_id = exec_func(xsqlite3_last_insert_rowid, {db}, "+sqlite3_last_insert_rowid")
  return row_id
end function


-------------------------------------------------------------------------------------
global function sqlite_query_database(sequence database, sequence cmd)
--<function>
--<name>sqlite_query_database</name>
--<digest>a universal get info from a database function</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>database</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>cmd</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  sequence data
  atom sql_db
  integer wk_void

  sql_db = sqlite_open(database, 0)
  if sql_db <= 0 then
    puts(1,"Unable to open database " & database)
  end if

  data = sqlite_get_table(sql_db, cmd)
  if sqlite_last_err_no != SQLITE_OK then
  --      errors here originally handled by my own handlers, replaced here by puts
    puts(1,"Error in " & database & " : " & sqlite_last_err_desc)
    wk_void = wait_key()
    if wk_void = '?' then
      position(1,1)
      puts(1, cmd)
      wk_void = wait_key()
    end if
  end if

  sqlite_close(sql_db)

  --remember 1st row of data is the headers
  --will pass it back, but in most cases just discard it

  return data
end function

--------------------------------------------------------------------------------

global function exec_sql_command(atom sql_db, sequence cmd)
--<function>
--<name>exec_sql_command</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>sql_db</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>cmd</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  sequence data, msg

  data = sqlite_get_table(sql_db, cmd)
  if sqlite_last_err_no != SQLITE_OK then
    msg = sprintf("Command=%s\n", {cmd}) &
          sprintf("Error %d : %s\n", {sqlite_last_err_no, sqlite_last_err_desc})
    log_puts(msg & "\n")
  end if
  return data
end function
