include dll.e
include machine.e
include _w32constants_.e
include _w32routines_.e
include _w32errors_.e

error_code = -1

-- =============================================================================
--                        General Win32 Internet Functions
-- =============================================================================
--
-- The general Microsoft(r) Win32(r) Internet functions described in this section perform basic Internet
-- file manipulations.

--------------------------------------------------------------------------------

integer xInternetAttemptConnect
xInternetAttemptConnect = 0

global function InternetAttemptConnect()
--<function>
--<name>InternetAttemptConnect</name>
--<digest>Attempts to make a connection to the Internet.</digest>
--<desc>
--   This function allows an application to first attempt to connect before issuing any requests. A client
--   program can use this to evoke the dial-up dialog box. If the attempt fails, the application should
--   enter offline mode.
--</desc>
--<return>
-- Returns ERROR_SUCCESS if successful, or a Win32(r) error value otherwise.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer result

  result = exec_w32func(xInternetAttemptConnect, {0}, "InternetAttemptConnect")
  if result != ERROR_SUCCESS then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xInternetCheckConnection
xInternetCheckConnection = 0

global function InternetCheckConnection(object Url, integer dwFlags)
--<function>
--<name>InternetCheckConnection</name>
--<digest>Allows an application to check if a connection to the Internet can be established.</digest>
--<desc>
--</desc>
--<param>
--<type>object</type>
--<name>Url</name>
--<desc>URL to use to check the connection. This value can be set to NULL.</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwFlags</name>
--<desc>
-- FLAG_ICC_FORCE_CONNECTION is the only flag that is currently available.
-- If this flag is set, it forces a connection.
--</desc>
--</param>
--<return>
--   Returns TRUE if a connection is made successfully, or FALSE otherwise. Use GetLastError to
--   retrieve the error code. ERROR_NOT_CONNECTED is returned by GetLastError if a connection
--   cannot be made or if the sockets database is unconditionally offline.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszUrl
  integer result
  
  if sequence(Url) then
    lpszUrl = allocate_string(Url)
  elsif Url = NULL then
    lpszUrl = NULL
  else
    error_msg = "InternetCheckConnection: Url should be a sequence or NULL"
    return FALSE
  end if
  
  result = exec_w32func(xInternetCheckConnection, {lpszUrl, dwFlags, 0},
                       "InternetCheckConnectionA")

  if lpszUrl != NULL then free(lpszUrl) end if
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xInternetCloseHandle
xInternetCloseHandle = 0

global function InternetCloseHandle(atom hInternet)
--<function>
--<name>InternetCloseHandle</name>
--<digest>Closes a single Internet handle.</digest>
--<desc>
-- Remarks
-- The function terminates any pending operations on the handle and discards any outstanding data. If
-- a thread is blocking a call to Wininet.dll, another thread in the application can call
-- InternetCloseHandle on the Internet handle being used by the first thread to cancel the operation
-- and unblock the first thread.
--
-- If there is a status callback registered for the handle being closed and the handle was created with a
-- non-NULL context value, an INTERNET_STATUS_HANDLE_CLOSING callback will be made.
-- This indication will be the last callback made from a handle and indicates that the handle is being
-- destroyed.
--
-- If asynchronous requests are pending for the handle or any of its child handles, the handle cannot be
-- closed immediately, but it will be invalidated. Any new requests attempted using the handle will
-- return with an ERROR_INVALID_HANDLE notification. The asynchronous requests will
-- complete with INTERNET_STATUS_REQUEST_COMPLETE. Applications must be prepared to
-- receive any INTERNET_STATUS_REQUEST_COMPLETE indications on the handle before the
-- final INTERNET_STATUS_HANDLE_CLOSING indication is made, which indicates that the
-- handle is completely closed.
--
-- An application can call GetLastError to determine if requests are pending. If GetLastError
-- returns ERROR_IO_PENDING, there were outstanding requests when the handle was closed.
--</desc>
--<param>
--<type>atom</type>
--<name>hInternet</name>
--<desc>Valid HINTERNET handle to be closed.</desc>
--</param>
--<return>
--   Returns TRUE if the handle is successfully closed, or FALSE otherwise. To get extended error
--   information, call GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer result

  result = exec_w32func(xInternetCloseHandle, {hInternet}, "InternetCloseHandle")
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xInternetConfirmZoneCrossing
xInternetConfirmZoneCrossing = 0

global function InternetConfirmZoneCrossing(atom hWnd, sequence UrlPrev,
                                              sequence UrlNew, integer bPost)
--<function>
--<name>InternetConfirmZoneCrossing</name>
--<digest>Checks for changes between secure and nonsecure URLs.</digest>
--<desc>
-- When a change occurs in security between two URLs, an application should allow
-- the user to acknowledge this change, typically by displaying a dialog box.
--</desc>
--<param>
--<type>atom</type>
--<name>hWnd</name>
--<desc>Handle to the parent window for any needed dialog box.</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>UrlPrev</name>
--<desc>
-- Address of a string variable containing the URL that was viewed before the
-- current request was made.</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>UrlNew</name>
--<desc>
-- Address of a string variable containing the new URL that the user has requested
-- to view.</desc>
--</param>
--<param>
--<type>integer</type>
--<name>bPost</name>
--<desc>
-- Boolean value that determines if a post is being made by this request. If bPost is set
-- to TRUE, a post is being made in this request. This flag is ignored in this release.
--</desc>
--</param>
--<return>
--   Returns one of the following values:
--   ERROR_SUCCESS             The user confirmed that it was okay to continue, or there was no user
--                             input needed.
--   ERROR_CANCELLED           The user canceled the request.
--   ERROR_NOT_ENOUGH_MEMORY   There is not enough memory to carry out the request.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom szUrlPrev, szUrlNew
  integer result

  if sequence(UrlPrev) then
    szUrlPrev = allocate_string(UrlPrev)
  else
    error_msg = "InternetConfirmZoneCrossing: UrlPrev should be a sequence"
    return ERROR_CANCELLED
  end if
  if sequence(UrlNew) then
    szUrlNew = allocate_string(UrlNew)
  else
    error_msg = "InternetConfirmZoneCrossing: UrlNew should be a sequence"
    return ERROR_CANCELLED
  end if
  result = exec_w32func(xInternetConfirmZoneCrossing, {hWnd, szUrlPrev,
                     szUrlNew, bPost}, "InternetConfirmZoneCrossing")
  free(szUrlPrev)
  free(szUrlNew)
  if result != ERROR_SUCCESS then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xInternetConnect
xInternetConnect = 0

global function InternetConnect(atom hInternet, sequence ServerName,
                  integer nServerPort, object UserName,
                  object Password, integer dwService,
                  integer dwFlags, atom dwContext)
--<function>
--<name>InternetConnect</name>
--<digest>Opens an FTP, Gopher, or HTTP session for a given site.</digest>
--<desc>
--   For FTP sites, InternetConnect actually establishes a connection with the server; for others, such
--   as Gopher, the actual connection is not established until the application requests a specific
--   transaction.
--
--   For maximum efficiency, applications using the Gopher and HTTP protocols should try to
--   minimize calls to InternetConnect and avoid calling this function for every transaction requested
--   by the user. One way to accomplish this is to keep a small cache of handles returned from
--   InternetConnect; when the user makes a request to a previously accessed server, that session
--   handle is still available.
--</desc>
--<param>
--<type>atom</type>
--<name>hInternet</name>
--<desc>Valid HINTERNET handle returned by a previous call to InternetOpen.</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>ServerName</name>
--<desc>
--     Address of a null-terminated string that contains the host name of an Internet server. Alternately,
--     the string can contain the IP number of the site, in ASCII dotted-decimal format (for example,
--     11.0.1.45).
--</desc>
--</param>
--<param>
--<type>integer</type>
--<name>nServerPort</name>
--<desc>
--     Number of the TCP/IP port on the server to connect to. These flags set only the port that will be
--     used. The service is set by the value of dwService. This can be any one of the following values:
--     INTERNET_DEFAULT_FTP_PORT      Uses the default port for FTP servers (port 21).
--     INTERNET_DEFAULT_GOPHER_PORT   Uses the default port for Gopher servers (port 70).
--     INTERNET_DEFAULT_HTTP_PORT     Uses the default port for HTTP servers (port 80).
--     INTERNET_DEFAULT_HTTPS_PORT    Uses the default port for HTTPS servers (port 443).
--     INTERNET_DEFAULT_SOCKS_PORT    Uses the default port for SOCKS firewall servers (port 1080).
--     INTERNET_INVALID_PORT_NUMBER   Uses the default port for the service specified by dwService.
--</desc>
--</param>
--<param>
--<type>object</type>
--<name>UserName</name>
--<desc>
--     Address of a null-terminated string that contains the name of the user to log on. If this parameter
--     is NULL, the function uses an appropriate default, except for HTTP; a NULL parameter in
--     HTTP causes the server to return an error. For the FTP protocol, the default is "anonymous".
--</desc>
--</param>
--<param>
--<type>object</type>
--<name>Password</name>
--<desc>
--     Address of a null-terminated string that contains the password to use to log on. If both
--     lpszPassword and lpszUsername are NULL, the function uses the default "anonymous"
--     password. In the case of FTP, the default password is the user's e-mail name. If lpszPassword is
--     NULL, but lpszUsername is not NULL, the function uses a blank password.
--</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwService</name>
--<desc>
--     Unsigned long integer value that contains the type of service to access. This can be one of the
--     following values:
--     INTERNET_SERVICE_FTP      FTP service.
--     INTERNET_SERVICE_GOPHER   Gopher service.
--     INTERNET_SERVICE_HTTP     HTTP service.
--</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwFlags</name>
--<desc>
--     Unsigned long integer value that contains the flags specific to the service used. When the value
--     of dwService is INTERNET_SERVICE_FTP, INTERNET_FLAG_PASSIVE causes the
--     application to use passive FTP semantics.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwContext</name>
--<desc>
--     Address of an unsigned long integer value that contains an application-defined value that is used
--     to identify the application context for the returned handle in callbacks.
--</desc>
--</param>
--<return>
--   Returns a valid handle to the FTP, Gopher, or HTTP session if the connection is successful, or
--   NULL otherwise. To get extended error information, call GetLastError. An application can also
--   use InternetGetLastResponseInfo to determine why access to the service was denied.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszServerName, lpszUserName, lpszPassword
  atom result

  if sequence(ServerName) then
    lpszServerName = allocate_string(ServerName)
  else
    error_msg = "InternetConnect: ServerName should be a sequence"
    return NULL
  end if
  if sequence(UserName) then
    lpszUserName = allocate_string(UserName)
  elsif UserName = NULL then
    lpszUserName = NULL
  else
    error_msg = "InternetConnect: UserName should be a sequence or NULL"
    return NULL
  end if
  if sequence(Password) then
    lpszPassword = allocate_string(Password)
  elsif Password = NULL then
    lpszPassword = NULL
  else
    error_msg = "InternetConnect: Password should be a sequence or NULL"
    return NULL
  end if
  result = exec_w32func(xInternetConnect, {hInternet, lpszServerName, nServerPort,
                                        lpszUserName, lpszPassword, dwService,
                                        dwFlags, dwContext}, "InternetConnectA")
  free(lpszServerName)
  if lpszUserName != NULL then free(lpszUserName) end if
  if lpszPassword != NULL then free(lpszPassword) end if
  if result = NULL then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xInternetErrorDlg
xInternetErrorDlg = 0

global function InternetErrorDlg(atom hWnd, atom hRequest, integer dwError,
                  integer dwFlags, atom lppvData)
--<function>
--<name>InternetErrorDlg</name>
--<digest>Displays a dialog box for the error that is passed to InternetErrorDlg,
-- if an appropriate dialog box exists.</digest>
--<desc>
-- If the FLAGS_ERROR_UI_FILTER_FOR_ERRORS flag is used, the function also checks the
-- headers for any hidden errors and displays a dialog box if needed.
--
-- Remarks
--   Authentication errors are hidden, because the call to HttpSendRequest will complete
--   successfully. However, the status code would indicate that the proxy or server requires
--   authentication. The FLAGS_ERROR_UI_FILTER_FOR_ERRORS flag causes the function to
--   search the headers for status codes that indicate user input is needed.
--</desc>
--<param>
--<type>atom</type>
--<name>hWnd</name>
--<desc>Handle to the parent window for any needed dialog box. This parameter can be NULL if no
--   dialog box is needed.</desc>
--</param>
--<param>
--<type>atom</type>
--<name>hRequest</name>
--<desc>HINTERNET handle to the Internet connection used in the call to HttpSendRequest.</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwError</name>
--<desc>
--   Error value for which to display a dialog box. This can be one of the following values:
--   ERROR_INTERNET_HTTP_TO_HTTPS_ON_REDIR  Notifies the user of the zone crossing to and from a secure site.
--   ERROR_INTERNET_INCORRECT_PASSWORD      Displays a dialog box for obtaining the user's name and password.
                                       --   (On Microsoft(r) Windows(r) 95, the function first attempts to
                                       --   use any cached authentication information for the server being
                                       --   accessed, before displaying a dialog box.)
--   ERROR_INTERNET_INVALID_CA              Notifies the user that the Win32(r) Internet function does not
                                       --   recognize the certificate authority that generated the certificate
                                       --   for this Secure Sockets Layer (SSL) site.
--   ERROR_INTERNET_POST_IS_NON_SECURE      Displays a warning about posting data to the server through a
                                       --   nonsecure connection.
--   ERROR_INTERNET_SEC_CERT_CN_INVALID     Indicates that the SSL certificate Common Name (hostname field)
                                       --   is incorrect. Displays an Invalid SSL Common Name dialog box,
                                       --   and lets the user view the incorrect certificate. Also allows the
                                       --   user to select a certificate in response to a server request.
--   ERROR_INTERNET_SEC_CERT_DATE_INVALID   Tells the user that the SSL certificate has expired.
--</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwFlags</name>
--<desc>
--   Unsigned long integer value that contains the action flags. This can be a combination of these
--   values:
--   FLAGS_ERROR_UI_FILTER_FOR_ERRORS      cans the returned headers for errors. Call after using
                                       --  HttpSendRequest. This option detects any hidden errors, such as an
                                       --  authentication error.
--   FLAGS_ERROR_UI_FLAGS_CHANGE_OPTIONS   If the function succeeds, stores the results of the dialog box in
                                       --  the Internet handle.
--   FLAGS_ERROR_UI_FLAGS_GENERATE_DATA    Queries the Internet handle for needed information. The function
                                       --  constructs the appropriate data structure for the error. (For
                                       --  example, for Cert CN failures, the function grabs the certificate.)
--   FLAGS_ERROR_UI_SERIALIZE_DIALOGS      Serializes authentication dialog boxes for concurrent requests on a
                                       --  password cache entry. The lppvData parameter should contain the
                                       --  address of a pointer to an INTERNET_AUTH_NOTIFY_DATA structure, and
                                       --  the client should implement a thread-safe, nonblocking callback
                                       --  function.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lppvData</name>
--<desc>
--   Address of pointer to a data structure. The structure can be different for each error that needs to
--   be handled.
--</desc>
--</param>
--<return>
--   Returns one of the following values, or an error code otherwise.
--   ERROR_SUCCESS                 The function completed successfully.
--   ERROR_CANCELLED               The function was canceled by the user.
--   ERROR_INTERNET_FORCE_RETRY    The Win32 function needs to redo its request.
--   ERROR_INVALID_HANDLE          The handle to the parent window is invalid.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer result

  result = exec_w32func(xInternetErrorDlg, {hWnd, hRequest, dwError, dwFlags, lppvData},
                     "InternetErrorDlg")
  if result != ERROR_SUCCESS then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xInternetFindNextFile
xInternetFindNextFile = 0

global function InternetFindNextFile(atom hFind, atom lpvFindData)
--<function>
--<name>InternetFindNextFile</name>
--<digest>
-- Continues a file search started as a result of a previous call to
-- FtpFindFirstFile or GopherFindFirstFile.
--</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>hFind</name>
--<desc>
--     Valid HINTERNET handle returned from either FtpFindFirstFile or
--     GopherFindFirstFile, or from InternetOpenUrl (directories only).
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lpvFindData</name>
--<desc>
--     Address of the buffer that receives information about the found file or
--     directory. The format of the information placed in the buffer depends on
--     the protocol in use. The FTP protocol returns a WIN32_FIND_DATA
--     structure, and the Gopher protocol returns a GOPHER_FIND_DATA structure.
--</desc>
--</param>
--<return>
--     Returns TRUE if the function succeeds, or FALSE otherwise. To get
--     extended error information, call GetLastError. If the function finds no
--     matching files, GetLastError returns ERROR_NO_MORE_FILES.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer result
  
  result = exec_w32func(xInternetFindNextFile, {hFind, lpvFindData}, "InternetFindNextFileA")
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xInternetGetLastResponseInfo
xInternetGetLastResponseInfo = 0

global function InternetGetLastResponseInfo()
--<function>
--<name>InternetGetLastResponseInfo</name>
--<digest>
-- Retrieves the last Win32(r) Internet function error description or server
-- response on the thread calling this function.
--</digest>
--<desc>
--
-- Remarks
--   The FTP and Gopher protocols can return additional text information along
--   with most errors. This extended error information can be retrieved by
--   using the InternetGetLastResponseInfo function whenever GetLastError
--   returns ERROR_INTERNET_EXTENDED_ERROR (occurring after an unsuccessful
--   function call).
--
--   The buffer pointed to by lpszBuffer must be large enough to hold both the
--   error string and a zero terminator at the end of the string. However, note
--   that the value returned in lpdwBufferLength does not include the
--   terminating zero.
--
--   InternetGetLastResponseInfo can be called multiple times until another
--   Win32 Internet function is called on this thread. When another function
--   is called, the internal buffer that is storing the last response
--   information is cleared.
--</desc>
--<return>
--   Returns TRUE if error text was successfully written to the buffer, or FALSE
--   otherwise. To get extended error information, call GetLastError.
--   If the buffer is too small to hold all the error text, GetLastError returns
--   ERROR_INSUFFICIENT_BUFFER, and the lpdwBufferLength parameter contains the
--   minimum buffer size required to return all the error text.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpdwError, lpszBuffer, lpdwBufferLength
  integer result

  error_code = -1
  error_msg  = ""
  lpdwError = allocate(4)
  lpszBuffer = allocate(128 + 1)  -- 128 Chars initially
  lpdwBufferLength = allocate(4)
  poke4(lpdwBufferLength, 128)    -- set size of message

  result = exec_w32func(xInternetGetLastResponseInfo, {lpdwError, lpszBuffer,
                     lpdwBufferLength}, "InternetGetLastResponseInfoA")
  if result = FALSE then
    if GetLastError() = ERROR_INSUFFICIENT_BUFFER  then
     -- the buffer was smaller than needed
      free(lpszBuffer)
      lpszBuffer = allocate(peek4u(lpdwBufferLength))

      result = exec_w32func(xInternetGetLastResponseInfo, {lpdwError, lpszBuffer,
                         lpdwBufferLength}, "InternetGetLastResponseInfoA")
    end if
  end if
  if result = TRUE then
    error_code = peek4u(lpdwError)
    error_msg  = peek({lpszBuffer, peek4u(lpdwBufferLength)})
  else
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if

  free(lpdwError)
  free(lpszBuffer)
  free(lpdwBufferLength)
  return result
end function

--------------------------------------------------------------------------------

-- InternetInitializeAutoProxyDll
--
-- Not currently supported.

--------------------------------------------------------------------------------

integer xInternetLockRequestFile
xInternetLockRequestFile = 0

global function InternetLockRequestFile(atom hInternet, atom lphLockReqHandle)
--<function>
--<name>InternetLockRequestFile</name>
--<digest>Allows the user to place a lock on the file that is being used.</digest>
--<desc>
-- Remarks
--   If the HINTERNET handle passed to hInternet was created using
--   INTERNET_FLAG_NO_CACHE_WRITE or INTERNET_FLAG_DONT_CACHE, the function
--   creates a temporary file with the extension .tmp, unless it is an HTTPS
--   resource. If the handle was created using INTERNET_FLAG_NO_CACHE_WRITE or
--   INTERNET_FLAG_DONT_CACHE and it is accessing an HTTPS resource,
--   InternetLockRequestFile fails.
--</desc>
--<param>
--<type>atom</type>
--<name>hInternet</name>
--<desc>
--     HINTERNET handle returned by FtpOpenFile, GopherOpenFile,
--     HttpOpenRequest, or InternetOpenUrl.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lphLockReqHandle</name>
--<desc>
--     Address of a handle to store the lock request handle.
--</desc>
--</param>
--<return>
--   Returns TRUE if successful, or FALSE otherwise. To get a specific error
--   message, call GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom result

  result =  exec_w32func(xInternetLockRequestFile, {hInternet, lphLockReqHandle}, "InternetLockRequestFile")
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
    return 0
  else
    return lphLockReqHandle
  end if
end function

--------------------------------------------------------------------------------

integer xInternetOpen
xInternetOpen = 0

global function InternetOpen(sequence Agent, integer dwAccessType,
                  object ProxyName, object ProxyBypass, integer dwFlags)
--<function>
--<name>InternetOpen</name>
--<digest>
-- Initializes an application's use of the Win32(r) Internet functions.
--</digest>
--<desc>
-- Remarks
--   InternetOpen is the first Win32 Internet function called by an application.
--   It tells the Internet DLL to initialize internal data structures and
--   prepare for future calls from the application. When the application
--   finishes using the Internet functions, it should call InternetCloseHandle
--   to free the handle and any associated resources.
--
--   The application can make any number of calls to InternetOpen, although a
--   single call is normally sufficient. The application might need to have
--   separate behaviors defined for each InternetOpen instance, such as
--   different proxy servers configured for each.
--</desc>
--<param>
--<type>sequence</type>
--<name>Agent</name>
--<desc>
--     String variable that contains the name of the application
--     or entity calling the Internet functions (for example, Microsoft(r)
--     Internet Explorer). This name is used as the user agent in the HTTP
--     protocol.
--</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwAccessType</name>
--<desc>
--     Type of access required. This can be one of the following values:
--     INTERNET_OPEN_TYPE_DIRECT      Resolves all host names locally.
--     INTERNET_OPEN_TYPE_PRECONFIG   Retrieves the proxy or direct
--                                    configurationfrom the registry.
--     INTERNET_OPEN_TYPE_PRECONFIG_WITH_NO_AUTOPROXY   Retrieves the proxy or
--                                    direct configurationfrom the registry and
--                                    prevents the use of a startup JScript(r)
--                                    (compatible with ECMA 262 language
--                                    specification) or Internet Setup (INS)
--                                    file.
--     INTERNET_OPEN_TYPE_PROXY       Passes requests to the proxy unless a
--                                    proxybypass list is supplied and the name
--                                    to be resolved bypasses the proxy. In this
--                                    case, the function uses
--                                    INTERNET_OPEN_TYPE_DIRECT.
--</desc>
--</param>
--<param>
--<type>object</type>
--<name>ProxyName</name>
--<desc>
--     String variable that contains the name of the proxy
--     server(s) to use when proxy access is specified by setting dwAccessType
--     to INTERNET_OPEN_TYPE_PROXY. Do not use an empty string, because
--     InternetOpen will use it as the proxy name. The Win32 Internet functions
--     recognize only CERN type proxies (HTTP only) and the TIS FTP gateway
--     (FTP only). If Internet Explorer is installed, the Win32 Internet
--     functions also support SOCKS proxies. FTP and Gopher requests can be
--     made through a CERN type proxy either by changing them to an HTTP request
--     or by using InternetOpenUrl. If dwAccessType is not set to
--     INTERNET_OPEN_TYPE_PROXY, this parameter is ignored and should be set to
--     NULL. For more information about listing proxy servers, see Listing Proxy
--     Servers section of the Enabling Internet Functionality article.
--</desc>
--</param>
--<param>
--<type>object</type>
--<name>ProxyBypass</name>
--<desc>
--     String variable that contains an optional list of host names
--     or IP addresses, or both, that should not be routed through the proxy
--     when dwAccessType is set to INTERNET_OPEN_TYPE_PROXY. The list can
--     contain wildcards. Do not use an empty string, because InternetOpen will
--     use it as the proxy bypass list. If this parameter specifies the
--     "<local>" macro as the only entry, the function bypasses any host name
--     that does not contain a period. If dwAccessType is not set to
--     INTERNET_OPEN_TYPE_PROXY, this parameter is ignored and should be set
--     to NULL.
--</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwFlags</name>
--<desc>
--     Unsigned long integer value that contains the flags that indicate various
--     options affecting the behavior of the function. This can be a combination
--     of these values:
--       INTERNET_FLAG_ASYNC
--       INTERNET_FLAG_FROM_CACHE
--       INTERNET_FLAG_OFFLINE
--</desc>
--</param>
--<return>
--   Returns a valid handle that the application passes on to subsequent Win32
--   Internet functions. If InternetOpen fails, it returns NULL. To get a
--   specific error message, call GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszAgent, lpszProxyName, lpszProxyBypass
  atom result

  if sequence(Agent) then
    lpszAgent = allocate_string(Agent)
  else
    error_msg = "InternetOpen: Agent should be a sequence"
    return NULL
  end if
  if sequence(ProxyName) then
    lpszProxyName = allocate_string(ProxyName)
  elsif ProxyName = NULL then
    lpszProxyName = NULL
  else
    error_msg = "InternetOpen: ProxyName should be a sequence or NULL"
    return NULL
  end if
  if sequence(ProxyBypass) then
    lpszProxyBypass = allocate_string(ProxyBypass)
  elsif ProxyBypass = NULL then
    lpszProxyBypass = NULL
  else
    error_msg = "InternetOpen: ProxyBypass should be a sequence or NULL"
    return NULL
  end if
  result =  exec_w32func(xInternetOpen, {lpszAgent, dwAccessType, lpszProxyName,
                                   lpszProxyBypass, dwFlags}, "InternetOpenA")
  free(lpszAgent)
  if lpszProxyName != NULL then free(lpszProxyName) end if
  if lpszProxyBypass != NULL then free(lpszProxyBypass) end if
  if result = NULL then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xInternetQueryDataAvailable
xInternetQueryDataAvailable = 0

global function InternetQueryDataAvailable(atom hFile)
--<function>
--<name>InternetQueryDataAvailable</name>
--<digest>
-- This function returns the number of bytes of data that are available to be
-- read immediately by a subsequent call to InternetReadFile.
--</digest>
--<desc>
-- Remarks
-- If there is   currently no data available and the end of the file has not been
-- reached, the request waits until data becomes available. The amount of data
-- remaining will not be recalculated until all available data indicated by
-- the call to InternetQueryDataAvailable is read.
--
-- For HINTERNET handles created by HttpOpenRequest and sent by
-- HttpSendRequestEx, a call to HttpEndRequest must be made on the handle
-- before InternetQueryDataAvailable can be used.
--</desc>
--<param>
--<type>atom</type>
--<name>hFile</name>
--<desc>
--     Valid HINTERNET handle, as returned by InternetOpenUrl, FtpOpenFile,
--     GopherOpenFile, or HttpOpenRequest.
--</desc>
--</param>
--<return>
--   Returns TRUE if the function succeeds, or FALSE otherwise. To get extended
--   error information, call GetLastError. If the function finds no matching
--   files, GetLastError returns ERROR_NO_MORE_FILES.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpdwNumberOfBytesAvailable
  integer result, BytesAvailable

  lpdwNumberOfBytesAvailable = allocate(4)
  result =  exec_w32func(xInternetQueryDataAvailable, {hFile,
                     lpdwNumberOfBytesAvailable, 0, 0}, "InternetQueryDataAvailable")
  if result = TRUE then
    BytesAvailable = peek4u(lpdwNumberOfBytesAvailable)
  end if
  free(lpdwNumberOfBytesAvailable)
  if result = TRUE then
    return BytesAvailable
  else
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
    return 0
  end if
end function

--------------------------------------------------------------------------------

integer xInternetQueryOption
xInternetQueryOption = 0

global function InternetQueryOption(atom hInternet,integer dwOption)
--<function>
--<name>InternetQueryOption</name>
--<digest>
-- Queries an Internet option on the specified handle.
--</digest>
--<desc>
-- Remarks
--   GetLastError will return the ERROR_INVALID_PARAMETER if an option flag that
--   is invalid for the specified handle type is passed to the dwOption
--   parameter.
--</desc>
--<param>
--<type>atom</type>
--<name>hInternet</name>
--<desc>HINTERNET handle on which to query information.</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwOption</name>
--<desc>
--     Unsigned long integer value that contains the Internet option to query.
--     This can be one of the Option Flags values.
--</desc>
--</param>
--<return>
--   Returns TRUE if successful, or FALSE otherwise. To get a specific error
--   message, call GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpBuffer, lpdwBufferLength
  integer result

  lpBuffer = allocate(128 + 1)  -- 128 Chars initially
  lpdwBufferLength = allocate(4)
  poke4(lpdwBufferLength, 128)    -- set size of message

  result =  exec_w32func(xInternetQueryOption, {hInternet, dwOption, lpBuffer,
                      lpdwBufferLength}, "InternetQueryOptionA")
  if result = FALSE then
    if GetLastError() = ERROR_INSUFFICIENT_BUFFER  then
     -- the buffer was smaller than needed
      free(lpBuffer)
      lpBuffer = allocate(peek4u(lpdwBufferLength))

      result =  exec_w32func(xInternetQueryOption, {hInternet, dwOption, lpBuffer,
                          lpdwBufferLength}, "InternetQueryOptionA")
    end if
  end if
  if result = TRUE then
    result = peek({lpBuffer, peek4u(lpdwBufferLength)})
  else
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if

  free(lpBuffer)
  free(lpdwBufferLength)
  return result
end function

--------------------------------------------------------------------------------

integer xInternetReadFile
xInternetReadFile = 0

global function InternetReadFile(atom hFile, integer dwNumberOfBytesToRead)
--<function>
--<name>InternetReadFile</name>
--<digest>
-- Reads data from a handle opened by the InternetOpenUrl, FtpOpenFile,
-- GopherOpenFile, or HttpOpenRequest function.
--</digest>
--<desc>
-- Remarks
--   If the return value is an empty string, the transfer has been completed and
--   there are no more bytes to read on the handle.
--   This is analogous to reaching end-of-file in a local file.
--
--   The buffer is not always filled by calls to InternetReadFile (sufficient data
--   might not have arrived from the server). When reading HTML data, for the
--   first read posted, the buffer must be large enough to hold the HTML headers.
--   When reading HTML-encoded directory entries, the buffer must be large enough
--   to hold at least one complete entry.
--
--   When the item being read is also being cached by a Win32(r) Internet
--   function, the application must ensure that a read is made for end-of-file
--   so the cache file is committed correctly.
--
--   This function always fulfills the user's request. If more data is requested
--   than is available, the function waits until enough data to complete the
--   request is available. The only time that less data is returned than
--   requested is when the end of the file has been reached.
--
--   This function can also be used to retrieve FTP and Gopher directory
--   listings as an HTML document on a handle opened by InternetOpenUrl. The
--   INTERNET_FLAG_RAW_DATA value must not have been specified in the call to
--   InternetOpenUrl.
--
--   If the HINTERNET handle stored in hFile was created by HttpOpenRequest and
--   sent by HttpSendRequestEx, a call to HttpEndRequest must be made on the
--   handle before InternetReadFile is used.
--</desc>
--<param>
--<type>atom</type>
--<name>hFile</name>
--<desc>
-- Valid HINTERNET handle returned from a previous call to InternetOpenUrl,
-- FtpOpenFile, GopherOpenFile, or HttpOpenRequest.
--</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwNumberOfBytesToRead</name>
--<desc>
-- Number of bytes to read.
--</desc>
--</param>
--<return>
--   Returns data read if successful, or FALSE otherwise. To get extended error
--   information, call GetLastError. An application can also use
--   InternetGetLastResponseInfo when necessary.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpBuffer, lpdwNumberOfBytesRead
  sequence Buffer
  integer result, NumberOfBytesRead

  Buffer = {}
  lpdwNumberOfBytesRead = allocate(4)
  lpBuffer = allocate(dwNumberOfBytesToRead)

  result = exec_w32func(xInternetReadFile, {hFile, lpBuffer, dwNumberOfBytesToRead,
                                      lpdwNumberOfBytesRead}, "InternetReadFile")

  if result = TRUE then
    NumberOfBytesRead = peek4u(lpdwNumberOfBytesRead)
    if NumberOfBytesRead > 0 then
      Buffer = peek({lpBuffer, NumberOfBytesRead})
    end if
  else
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
    free(lpdwNumberOfBytesRead)
    free(lpBuffer)
    return 0
  end if

  free(lpdwNumberOfBytesRead)
  free(lpBuffer)
  return Buffer -- Empty is ok. Validate comparing size
end function

--------------------------------------------------------------------------------

integer xInternetReadFileEx
xInternetReadFileEx = 0

global function InternetReadFileEx(atom hFile, atom lpBuffersOut,
                                   integer dwFlags, integer dwContext)
--<function>
--<name>InternetReadFileEx</name>
--<digest>
-- Reads data from a handle opened by the InternetOpenUrl, FtpOpenFile,
-- GopherOpenFile, or HttpOpenRequest function.
--</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>hFile</name>
--<desc>
--     HINTERNET handle returned by the InternetOpenUrl, FtpOpenFile,
--     GopherOpenFile, or HttpOpenRequest function.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lpBuffersOut</name>
--<desc>
--     Address of an INTERNET_BUFFERS structure that contains the data downloaded.
--</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwFlags</name>
--<desc>
--     Unsigned long integer variable that contains the flags controlling the
--     download. This can be one of the following values:
--     IRF_ASYNC         Identical to WININET_API_FLAG_ASYNC.
--     IRF_SYNC          Identical to WININET_API_FLAG_SYNC.
--     IRF_USE_CONTEXT   Identical to WININET_API_FLAG_USE_CONTEXT.
--     IRF_NO_WAIT       Do not wait for data. If there is data available, the
--                       function returns either the amount of data requested or
--                       the amount of data available (whichever is smaller).
--</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwContext</name>
--<desc>
--     Unsigned long integer variable that contains the context value used for
--     asynchronous operations.
--</desc>
--</param>
--<return>
--     Returns TRUE if successful, or FALSE otherwise. To get extended error
--     information, call GetLastError. An application can also use
--     InternetGetLastResponseInfo when necessary.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- TODO: changer les paramètres
  integer result

  result =  exec_w32func(xInternetReadFileEx, {hFile, lpBuffersOut, dwFlags,
                                         dwContext}, "InternetReadFileExA")
  if result = TRUE then
    return lpBuffersOut
  else
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
    return 0
  end if
end function

--------------------------------------------------------------------------------
integer xInternetSetFilePointer
xInternetSetFilePointer = 0

global function InternetSetFilePointer(atom hFile, atom lDistanceToMove,
                                       integer dwMoveMethod)
--<function>
--<name>InternetSetFilePointer</name>
--<digest>Sets a file position for InternetReadFile.</digest>
--<desc>
-- This is a synchronous call; however, subsequent calls to InternetReadFile might
-- block or return pending if the data is not available from the cache and the
-- server does not support random access.
--
-- Remarks
--   This function cannot be used once the end of the file has been reached by
--   InternetReadFile.
--
--   For HINTERNET handles created by HttpOpenRequest and sent by
--   HttpSendRequestEx, a call to HttpEndRequest must be made on the handle
--   before InternetSetFilePointer is used.
--
--   InternetSetFilePointer cannot be used reliably if the content length is
--   unknown, and it cannot be used reliably with chunked transfers.
--</desc>
--<param>
--<type>atom</type>
--<name>hFile</name>
--<desc>
--     Valid HINTERNET handle returned from a previous call to InternetOpenUrl
--     (on an HTTP or HTTPS URL) or HttpOpenRequest (using the GET or HEAD
--     method and passed to HttpSendRequest or HttpSendRequestEx). This handle
--     must not have been created with the INTERNET_FLAG_DONT_CACHE or
--     INTERNET_FLAG_NO_CACHE_WRITE value set.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lDistanceToMove</name>
--<desc>
--     Long integer value that contains the number of bytes to move the file
--     pointer. A positive value moves the pointer forward in the file; a
--     negative value moves it backward.
--</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwMoveMethod</name>
--<desc>
--     Unsigned long integer value that indicates the starting point for the
--     file pointer move. This can be one of the following values:
--     FILE_BEGIN     Starting point is zero or the beginning of the file. If
--                    FILE_BEGIN is specified, lDistanceToMove is interpreted as
--                    an unsigned location for the new file pointer.
--     FILE_CURRENT   Current value of the file pointer is the starting point.
--     FILE_END       Current end-of-file position is the starting point. This
--                    method fails if the content length is unknown.
--</desc>
--</param>
--<return>
--   Returns the current file position if the function succeeds, or -1
--   otherwise.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer result

  result =  exec_w32func(xInternetSetFilePointer, {hFile, lDistanceToMove, NULL,
                                             dwMoveMethod, 0}, "InternetSetFilePointer")
  if result = -1 then
    error_msg = "InternetSetFilePointer failed"
  end if
  return result
end function

--------------------------------------------------------------------------------
integer xInternetSetOption
xInternetSetOption = 0

global function InternetSetOption(atom hInternet, integer dwOption,
                                    atom lpBuffer, integer dwBufferLength)
--<function>
--<name>InternetSetOption</name>
--<digest>Sets an Internet option.</digest>
--<desc>
-- Remarks
--   GetLastError will return the error ERROR_INVALID_PARAMETER if an option
--   flag that cannot be set is specified.
--</desc>
--<param>
--<type>atom</type>
--<name>hInternet</name>
--<desc>HINTERNET handle on which to set information.</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwOption</name>
--<desc>
-- Contains the Internet option to set.
-- This can be one of the Option Flags values.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lpBuffer</name>
--<desc>
--     Address of a buffer that contains the option setting.
--</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwBufferLength</name>
--<desc>
--     Unsigned long integer value that contains the length of the lpBuffer
--     buffer in TCHAR.
--</desc>
--</param>
--<return>
--   Returns TRUE if successful, or FALSE otherwise. To get a specific error
--   message, call GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer result

  result =  exec_w32func(xInternetSetOption, {hInternet, dwOption, lpBuffer,
                                        dwBufferLength}, "InternetSetOptionA")
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

-- InternetSetOptionEx
--
-- Not currently implemented.

--------------------------------------------------------------------------------

integer xInternetSetStatusCallback
xInternetSetStatusCallback = 0

global function InternetSetStatusCallback(atom hInternet,
                                    atom lpfnInternetCallback)
--<function>
--<name>InternetSetStatusCallback</name>
--<digest>
-- Sets up a callback function that Win32(r) Internet functions can call as
-- progress is made during an operation.
--</digest>
--<desc>
-- Remarks
--   Both synchronous and asynchronous functions use the callback function to
--   indicate the progress of the request, such as resolving a name, connecting
--   to a server, and so on. The callback function is required for an
--   asynchronous operation. The asynchronous request will call back to the
--   application with INTERNET_STATUS_REQUEST_COMPLETE to indicate the request
--   has been completed.
--
--   A callback function can be set on any handle, and is inherited by derived
--   handles. A callback function can be changed using
--   InternetSetStatusCallback, providing there are no pending requests that
--   need to use the previous callback value. Note, however, that changing the
--   callback function on a handle does not change the callbacks on derived
--   handles, such as that returned by InternetConnect. You must change the
--   callback function at each level.
--
--   Many of the Win32 Internet functions perform several operations on the
--   network. Each operation can take time to complete, and each can fail.
--
--   It is sometimes desirable to display status information during a long-term
--   operation. You can display status information by setting up an Internet
--   status callback function that cannot be removed as long as any callbacks or
--   any asynchronous functions are pending.
--
--   After initiating InternetSetStatusCallback, the callback function can be
--   accessed from within any Win32 Internet function for monitoring
--   time-intensive network operations.
--</desc>
--<param>
--<type>atom</type>
--<name>hInternet</name>
--<desc>
--     HINTERNET handle for which the callback is to be set.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lpfnInternetCallback</name>
--<desc>
--     Address of the callback function to call when progress is made, or to
--     return NULL to remove the existing callback function. For more
--     information about the callback function, see INTERNET_STATUS_CALLBACK.
--</desc>
--</param>
--<return>
--   Returns the previously defined status callback function if successful, NULL
--   if there was no previously defined status callback function, or
--   INTERNET_INVALID_STATUS_CALLBACK if the callback function is not valid.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom result

  result =  exec_w32func(xInternetSetStatusCallback, {hInternet,
                      lpfnInternetCallback}, "InternetSetStatusCallback")
  return result
end function

--------------------------------------------------------------------------------

integer xInternetTimeToSystemTime
xInternetTimeToSystemTime = 0

global function InternetTimeToSystemTime(sequence InternetTime, atom pst)
--<function>
--<name>InternetTimeToSystemTime</name>
--<digest>
-- Takes an HTTP time/date string and converts it to a SYSTEMTIME structure.
--</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>InternetTime</name>
--<desc>
-- Date/time string to convert.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>pst</name>
--<desc>
--     Address of SYSTEMTIME structure that receives the converted time.
--</desc>
--</param>
--<return>
-- Returns Address of SYSTEMTIME structure if the string was converted, or FALSE
-- otherwise. To get extended error information, call GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszTime
  atom result

  lpszTime = allocate_string(InternetTime)
  result =  exec_w32func(xInternetTimeToSystemTime, {lpszTime, pst, 0},
                      "InternetTimeToSystemTime")
  free(lpszTime)
  if result = TRUE then
    return pst
  else
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
    return 0
  end if
end function

--------------------------------------------------------------------------------
integer xInternetUnlockRequestFile
xInternetUnlockRequestFile = 0

global function InternetUnlockRequestFile(atom hLockHandle)
--<function>
--<name>InternetUnlockRequestFile</name>
--<digest>
-- Unlocks a file that was locked using InternetLockRequestFile.
--</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>hLockHandle</name>
--<desc>
--     Lock request handle that was returned by InternetLockRequestFile.
--</desc>
--</param>
--<return>
--   Returns TRUE if successful, or FALSE otherwise. To get a specific error
--   message, call GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer result

  result =  exec_w32func(xInternetUnlockRequestFile, {hLockHandle},
                      "InternetUnlockRequestFile")
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------
integer xInternetWriteFile
xInternetWriteFile = 0

global function InternetWriteFile(atom hFile, sequence Data)
--<function>
--<name>InternetWriteFile</name>
--<digest>Writes data to an open Internet file.</digest>
--<desc>
-- Remarks
--   When the application is sending data, it must call InternetCloseHandle to
--   end the data transfer.
--</desc>
--<param>
--<type>atom</type>
--<name>hFile</name>
--<desc>
-- Valid HINTERNET handle returned from a previous call to FtpOpenFile or an
-- HINTERNET handle sent by HttpSendRequestEx.
--</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>Data</name>
--<desc>Contains the data to be written to the file.</desc>
--</param>
--<return>
-- Returns TRUE if the function succeeds, or FALSE otherwise. To get extended
-- error information, call GetLastError. An application can also use
-- InternetGetLastResponseInfo when necessary.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpBuffer, dwNumberOfBytesToWrite, lpdwNumberOfBytesWritten
  integer result

  dwNumberOfBytesToWrite = length(Data)

  -- it can be zero if reading in a loop and an exact divisor is used
  if dwNumberOfBytesToWrite = 0 then
    return TRUE -- if an empty sequence is passed
  end if

  lpdwNumberOfBytesWritten = allocate(4)
  lpBuffer = allocate_string(Data)

  result = exec_w32func(xInternetWriteFile, {hFile, lpBuffer,
                     dwNumberOfBytesToWrite, lpdwNumberOfBytesWritten},
                     "InternetWriteFile")
  free(lpBuffer)
  free(lpdwNumberOfBytesWritten)
  
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if

  return result
end function


-- =============================================================================
--                        Automatic Dialing Functions
-- =============================================================================
--
-- The functions described in this section handle dial-up access to the Internet.
--

--------------------------------------------------------------------------------

integer xInternetAutodial
xInternetAutodial = 0

global function InternetAutodial(integer dwFlags, atom hwndParent)
--<function>
--<name>InternetAutodial</name>
--<digest>
-- Causes the modem to automatically dial the default Internet connection.
--</digest>
--<desc>
--</desc>
--<param>
--<type>integer</type>
--<name>dwFlags</name>
--<desc>
--     Unsigned long integer value that contains the flags controlling this
--     operation. This can be one of the following values:
--     INTERNET_AUTODIAL_FAILIFSECURITYCHECK   Causes InternetAutodial to fail
--                                             if file and printer sharing is
--                                             disabled for Microsoft(r)
--                                             Windows(r) 95/98.
--     INTERNET_AUTODIAL_FORCE_ONLINE          Forces an online Internet
--                                             connection.
--     INTERNET_AUTODIAL_FORCE_UNATTENDED      Forces an unattended Internet
--                                             dial-up.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>hwndParent</name>
--<desc>
--     Handle to the parent window.
--</desc>
--</param>
--<return>
--   Returns TRUE if successful, or FALSE otherwise.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer result
  
  result = exec_w32func(xInternetAutodial, {dwFlags, hwndParent}, "InternetAutodial")
  if result = FALSE then
    error_msg = "InternetAutodial failed"
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xInternetAutodialHangup
xInternetAutodialHangup = 0

global function InternetAutodialHangup()
--<function>
--<name>InternetAutodialHangup</name>
--<digest>
-- Disconnects an automatic dial-up connection.
--</digest>
--<desc>
--</desc>
--<return>
--   Returns TRUE if successful, or FALSE otherwise.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer result

  result = exec_w32func(xInternetAutodialHangup, {0}, "InternetAutodialHangup")
  if result = FALSE then
    error_msg = "InternetAutodialHangup failed"
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xInternetDial
xInternetDial = 0

global function InternetDial(atom hWnd, sequence Connectoid, atom dwFlags, atom lpdwConnection)
--<function>
--<name>InternetDial</name>
--<digest>Initiates a connection to the Internet using a modem.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>hWnd</name>
--<desc>Handle to the parent window.</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>Connectoid</name>
--<desc>
-- String variable containing the name of the dial-up connection to use.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwFlags</name>
--<desc>
-- Contains the flags to use. This can be one of the following values:
-- INTERNET_AUTODIAL_FORCE_ONLINE       Forces an online connection.
-- INTERNET_AUTODIAL_FORCE_UNATTENDED   Forces an unattended Internet dial-up. If user
--                                      intervention is required, the function will fail.
-- INTERNET_DIAL_FORCE_PROMPT           Ignores the "dial automatically" setting and
--                                      forces the dialing user interface to be displayed.
-- INTERNET_DIAL_UNATTENDED             Connects to the Internet through a modem,
--                                      without displaying a user interface, if possible.
--                                      Otherwise, the function will wait for user input.
-- INTERNET_DIAL_SHOW_OFFLINE           Shows the Work Offline button instead of Cancel
--                                      button in the dialing user interface.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lpdwConnection</name>
--<desc>
-- Address of an unsigned long integer value containing the number associated to the connection.
--</desc>
--</param>
--<return>
-- Returns ERROR_SUCCESS if successful, or an error code otherwise. The error code can be one of
-- the following:
-- ERROR_INVALID_PARAMETER    One or more of the parameters are incorrect.
-- ERROR_NO_CONNECTION        There is a problem with the dial-up connection.
-- ERROR_USER_DISCONNECTION   The user clicked either the Work Offline or Cancel
--                            button on the Internet connection dialog box.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom sConnectoid
  atom result

  sConnectoid = allocate_string(Connectoid)
  result = exec_w32func(xInternetDial, {hWnd, sConnectoid, dwFlags, lpdwConnection, 0}, "InternetDial")
  free(sConnectoid)
  return result
end function

--------------------------------------------------------------------------------

integer xInternetGetConnectedState
xInternetGetConnectedState = 0

global function InternetGetConnectedState()
--<function>
--<name>InternetGetConnectedState</name>
--<digest>Retrieves the connected state of the local system.</digest>
--<desc>
--</desc>
--<return>
-- integer: can be a combination of the following values:
-- INTERNET_CONNECTION_CONFIGURED   Local system has a valid connection to the
--                                  Internet, but it may or may not be currently
--                                  connected.
-- INTERNET_CONNECTION_LAN          Local system uses a local area network to
--                                  connect to the Internet.
-- INTERNET_CONNECTION_MODEM        Local system uses a modem to connect to the
--                                  Internet.
-- INTERNET_CONNECTION_MODEM_BUSY   No longer used.
-- INTERNET_CONNECTION_OFFLINE      Local system is in offline mode.
-- INTERNET_CONNECTION_PROXY        Local system uses a proxy server to connect to
--                                  the Internet.
-- INTERNET_RAS_INSTALLED           Local system has RAS installed.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpdwFlags
  integer result

  lpdwFlags = allocate(4) 
  result = 0
  if exec_w32func(xInternetGetConnectedState, {lpdwFlags, 0}, "InternetGetConnectedState") then
    result = peek4u(lpdwFlags)
  else
    error_msg = "InternetGetConnectedState failed"
  end if
  free(lpdwFlags)
  return result
end function

--------------------------------------------------------------------------------

integer xInternetGetConnectedStateEx
xInternetGetConnectedStateEx = 0
-- InternetGetConnectedStateEx
--
-- Retrieves the connected state of the specified Internet connection.
--
-- Syntax
-- BOOL InternetGetConnectedStateEx(
-- OUT LPDWORD lpdwFlags,
-- OUT LPTSTR lpszConnectionName,
-- IN DWORD dwNameLen,
-- IN DWORD dwReserved
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpdwFlags
-- Address of an unsigned long integer variable where the connection description should be
-- returned. This can be a combination of the following values:
-- INTERNET_CONNECTION_CONFIGURED    Local system has a valid connection to the
-- Internet, but it may or may not be currently
-- connected.
--
-- INTERNET_CONNECTION_LAN    Local system uses a local area network to
-- connect to the Internet.
--
-- INTERNET_CONNECTION_MODEM    Local system uses a modem to connect to
-- the Internet.
--
-- INTERNET_CONNECTION_MODEM_BUSY    No longer used.
--
-- INTERNET_CONNECTION_OFFLINE    Local system is in offline mode.
--
-- INTERNET_CONNECTION_PROXY    Local system uses a proxy server to connect
-- to the Internet.
--
-- INTERNET_RAS_INSTALLED    Local system has RAS installed.
--
-- lpszConnectionName
-- Address of a string value that receives the connection name.
--
-- dwNameLen
-- Unsigned long integer value that contains the length of the lpszConnectionName string in
-- TCHAR.
--
-- Return Value
-- Returns TRUE if there is an Internet connection, or FALSE otherwise.
--global function ()
--  integer result
--
--  result = exec_w32func(xInternetGetConnectedStateEx, {lpdwFlags, lpszConnectionName,
--                     dwNameLen, 0}, "InternetGetConnectedStateEx")
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------

integer xInternetGoOnline
xInternetGoOnline = 0

-- InternetGoOnline
--
-- Prompts the user for permission to initiate connection to a URL.
--
-- Parameters
-- lpszURL
-- Address of a string variable containing the URL of the Web site to connect to.
--
-- hwndParent
-- Handle to the parent window.
--
-- dwReserved
-- Reserved. Must be set to zero.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise.
--global function ()
--  integer result
--
--  result = exec_w32func(xInternetGoOnline, {lpszURL, hwndParent, 0}, "InternetGoOnline")
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------

integer xInternetHangUp
xInternetHangUp = 0

global function InternetHangUp(integer dwConnection)
--<function>
--<name>InternetHangUp</name>
--<digest>Instructs the modem to disconnect from the Internet.</digest>
--<desc></desc>
--<param>
--<type>integer</type>
--<name>dwConnection</name>
--<desc>Number assigned to the connection to be disconnected.</desc>
--</param>
--<return>
--   Returns ERROR_SUCCESS if successful, or an error code otherwise.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer result

  result = exec_w32func(xInternetHangUp, {dwConnection, 0}, "InternetHangUp")
  if result != ERROR_SUCCESS then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

-- InternetSetDialState        -- Obsolete. Do not use.

--------------------------------------------------------------------------------


-- =============================================================================
--               Uniform Resource Locator (URL) Functions
-- =============================================================================
--
-- The URL functions deal with URL manipulation and access. These functions operate in a task-oriented
-- manner. The content and format of the URL that is being used by the function is not verified. Usage of
-- these functions should be tracked by the calling application to ensure that the data is in the format
-- intended.

--------------------------------------------------------------------------------

-- integer xInternetCanonicalizeUrl
-- xInternetCanonicalizeUrl = 0

-- InternetCanonicalizeUrl
-- Canonicalizes a URL, which includes converting unsafe characters and spaces into escape sequences.
--
-- Syntax
-- BOOL InternetCanonicalizeUrl(
-- IN LPCTSTR lpszUrl,
-- OUT LPTSTR lpszBuffer,
-- IN OUT LPDWORD lpdwBufferLength,
-- IN DWORD dwFlags
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrl
-- Address of the string that contains the URL to canonicalize.
--
-- lpszBuffer
-- Address of the buffer that receives the resulting canonicalized URL.
--
-- lpdwBufferLength
-- Address of an unsigned long integer value that contains the length, in TCHARs, of the lpszBuffer
-- buffer. If the function succeeds, this parameter receives the length of the lpszBuffer buffer-the
-- length does not include the terminating null character. If the function fails, this parameter
-- receives the required length, in bytes, of the lpszBuffer buffer-the required length includes the
-- terminating null character.
--
-- dwFlags
-- Unsigned long integer value that contains the flags that control canonicalization. This can be one
-- of the following values:
-- ICU_BROWSER_MODE    Does not encode or decode characters after "#" or "?",
-- and does not remove trailing white space after "?". If
-- this value is not specified, the entire URL is encoded
-- and trailing white space is removed.
--
-- ICU_DECODE    Converts all %XX sequences to characters, including
-- escape sequences, before the URL is parsed.
--
-- ICU_ENCODE_PERCENT    Encodes any percent signs encountered. By default,
-- percent signs are not encoded. This value is available in
-- Microsoft(r) Internet Explorer 5 and later versions of the
-- Win32(r) Internet functions.
--
-- ICU_ENCODE_SPACES_ONLY    Encodes spaces only.
--
-- ICU_NO_ENCODE    Does not convert unsafe characters to escape
-- sequences.
--
-- ICU_NO_META    Does not remove meta sequences (such as "." and "..")
-- from the URL.
--
-- If no flags are specified (dwFlags = 0), the function converts all unsafe characters and meta
-- sequences (such as \.,\ .., and \...) to escape sequences.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError. Possible errors include:
-- ERROR_BAD_PATHNAME    The URL could not be canonicalized. This flag is
-- valid for Internet Explorer 5 and later versions of the
-- Win32 Internet API.
--
-- ERROR_INSUFFICIENT_BUFFER    The canonicalized URL is too large to fit in the
-- buffer provided. The lpdwBufferLength parameter is
-- set to the size, in bytes, of the buffer required to hold
-- the canonicalized URL.
--
-- ERROR_INTERNET_INVALID_URL    The format of the URL is invalid.
--
-- ERROR_INVALID_PARAMETER    There is a bad string, buffer, buffer size, or flags
-- parameter.
--
-- Remarks
-- In Internet Explorer 4.0 and later, InternetCanonicalizeUrl always functions as if the
-- ICU_BROWSER_MODE flag is set. Client applications that need to canonicalize the entire URL
-- should use either CoInternetParseUrl (with the action PARSE_CANONICALIZE and the flag
-- URL_ESCAPE_UNSAFE) or UrlCanonicalize.
--
-- InternetCanonicalizeUrl always encodes by default, even if the ICU_DECODE flag has been
-- specified. To decode without re-encoding, use ICU_DECODE | ICU_NO_ENCODE. If the
-- ICU_DECODE flag is used without ICU_NO_ENCODE, the URL is decoded before being parsed;
-- unsafe characters then are re-encoded after parsing. This function will handle arbitrary protocol
-- schemes, but to do so it must make inferences from the unsafe character set.
--
-- Applications calling InternetCanonicalizeUrl when using the Internet Explorer 3.0 version of the
-- Win32 Internet API (or when setting the ICU_ENCODE_PERCENT flag for Internet Explorer 5
-- and later) should track the usage of this function on a particular URL. If unsafe characters in a URL
-- have been converted to escape sequences, using InternetCanonicalizeUrl again on the URL (with
-- no flags) will cause the escape sequences to be converted to another escape sequence. For example,
-- a blank space in a URL would be converted to the escape sequence %20. Calling
-- InternetCanonicalizeUrl again on the URL would cause the escape sequence %20 to be converted
-- to the escape sequence %2520, because the % sign is an unsafe character that is reserved for escape
-- sequences and is replaced by the function with the escape sequence %25.
--global function ()
--  integer result

--  result = exec_w32func(xInternetCanonicalizeUrl, {}, "InternetCanonicalizeUrlA")
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- integer xInternetCombineUrl
-- xInternetCombineUrl = 0

-- InternetCombineUrl
-- Combines a base and relative URL into a single URL. The resultant URL will be canonicalized (see
-- InternetCanonicalizeUrl).
--
-- Syntax
-- BOOL InternetCombineUrl(
-- IN LPCTSTR lpszBaseUrl,
-- IN LPCTSTR lpszRelativeUrl,
-- OUT LPTSTR lpszBuffer,
-- IN OUT LPDWORD lpdwBufferLength,
-- IN DWORD dwFlags
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszBaseUrl
-- Address of a string variable that contains the base URL.
--
-- lpszRelativeUrl
-- Address of a string variable that contains the relative URL.
--
-- lpszBuffer
-- Address of a buffer that receives the combined URL.
--
-- lpdwBufferLength
-- Address of an unsigned long integer value that contains the size, in TCHARs, of the lpszBuffer
-- buffer. If the function succeeds, this parameter receives the length, in TCHARs, of the combined
-- URL-this length does not include the null terminator. If the function fails, this parameter
-- receives the length, in bytes, of the required buffer-this length includes the null terminator.
--
-- dwFlags
-- Unsigned long integer value that contains the flags controlling the operation of the function. This
-- can be one of the following values:
-- ICU_BROWSER_MODE    Does not encode or decode characters after "#" or "?",
-- and does not remove trailing white space after "?". If
-- this value is not specified, the entire URL is encoded
-- and trailing white space is removed.
--
-- ICU_DECODE    Converts all %XX sequences to characters, including
-- escape sequences, before the URL is parsed.
--
-- ICU_ENCODE_PERCENT    Encodes any percent signs encountered. By default,
-- percent signs are not encoded. This value is available in
-- Microsoft(r) Internet Explorer 5 and later versions of the
-- Win32(r) Internet functions.
--
-- ICU_ENCODE_SPACES_ONLY    Encodes spaces only.
--
-- ICU_NO_ENCODE    Does not convert unsafe characters to escape
-- sequences.
--
-- ICU_NO_META    Does not remove meta sequences (such as "." and "..")
-- from the URL.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError. Possible errors include:
-- ERROR_BAD_PATHNAME    The URLs could not be combined.
--
-- ERROR_INSUFFICIENT_BUFFER    The buffer supplied to the function was insufficient
-- or NULL. The value indicated by the
-- lpdwBufferLength parameter will contain the
-- number of bytes required to hold the combined
-- URL.
--
-- ERROR_INTERNET_INVALID_URL    The format of the URL is invalid.
--
-- ERROR_INVALID_PARAMETER    There is a bad string, buffer, buffer size, or flags
-- parameter.
--global function ()
--  integer result

--  result = exec_w32func(xInternetCombineUrl, {}, "InternetCombineUrlA")
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- integer xInternetCrackUrl
-- xInternetCrackUrl = 0
-- InternetCrackUrl
-- Cracks a URL into its component parts.
--
-- Syntax
-- BOOL InternetCrackUrl(
-- IN LPCTSTR lpszUrl,
-- IN DWORD dwUrlLength,
-- IN DWORD dwFlags,
-- IN OUT LPURL_COMPONENTS lpUrlComponents
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrl
-- Address of a string that contains the canonical URL to crack.
--
-- dwUrlLength
-- Unsigned long integer value that contains the length of the lpszUrl string in TCHAR, or zero if
-- lpszUrl is an ASCIIZ string.
--
-- dwFlags
-- Unsigned long integer value that contains the flags controlling the operation. This can be one of
-- the following values:
-- ICU_DECODE    Converts encoded characters back to their normal form. This can be used
-- only if the user provides buffers in the URL_COMPONENTS structure
-- to copy the components into.
--
-- ICU_ESCAPE    Converts all escape sequences (%xx) to their corresponding characters.
-- This can be used only if the user provides buffers in the
-- URL_COMPONENTS structure to copy the components into.
--
-- lpUrlComponents
-- Address of a URL_COMPONENTS structure that receives the URL components.
--
-- Return Value
-- Returns TRUE if the function succeeds, or FALSE otherwise. To get extended error information,
-- call GetLastError.
--
-- Remarks
-- The required components are indicated by members of the URL_COMPONENTS structure. Each
-- component has a pointer to the value and has a member that stores the length of the stored value. If
-- both the value and the length for a component are equal to zero, that component is not returned. If
-- the pointer to the value of the component is NULL and the value of its corresponding length
-- member is nonzero, the address of the first character of the corresponding component in the lpszUrl
-- string is stored in the pointer, and the length of the component is stored in the length member.
--
-- If the pointer contains the address of the user-supplied buffer, the length member must contain the
-- size of the buffer. InternetCrackUrl copies the component into the buffer, and the length member
-- is set to the length of the copied component, minus 1 for the trailing string terminator.
--
-- For InternetCrackUrl to work properly, the size of the URL_COMPONENTS structure must be
-- stored in the dwStructSize member.
--global function ()
--  integer result

--  result = exec_w32func(xInternetCrackUrl, {}, "InternetCrackUrlA")
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------

-- integer xInternetCreateUrl
-- xInternetCreateUrl = 0

-- InternetCreateUrl
-- Creates a URL from its component parts.
--
-- Syntax
-- BOOL InternetCreateUrl(
-- IN LPURL_COMPONENTS lpUrlComponents,
-- IN DWORD dwFlags,
-- OUT LPTSTR lpszUrl,
-- IN OUT LPDWORD lpdwUrlLength
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpUrlComponents
-- Address of a URL_COMPONENTS structure that contains the components from which to create
-- the URL.
--
-- dwFlags
-- Unsigned long integer value that contains the flags that control the operation of this function.
-- This can be a combination of these values:
-- ICU_ESCAPE    Converts all escape sequences (%xx) to their corresponding
-- characters.
--
-- ICU_USERNAME    When adding the user name, uses the name that was specified at logon
-- time.
--
-- lpszUrl
-- Address of a buffer that receives the URL.
--
-- lpdwUrlLength
-- Address of an unsigned long integer value that contains the length, in TCHARs, of the lpszUrl
-- buffer. When the function returns, this parameter receives the length, in TCHARs, of the URL
-- string, minus 1 for the terminating character. If GetLastError returns
-- ERROR_INSUFFICIENT_BUFFER, this parameter receives the number of bytes required to
-- hold the created URL.
--
-- Return Value
-- Returns TRUE if the function succeeds, or FALSE otherwise. To get extended error information,
-- call GetLastError.
--global function ()
--  integer result

--  result = exec_w32func(xInternetCreateUrl, {lpUrlComponents, dwFlags, lpszUrl, lpdwUrlLength},
--                     "InternetCreateUrlA")
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
integer xInternetOpenUrl
xInternetOpenUrl = 0

global function InternetOpenUrl(atom hInternet, sequence Url, sequence Headers,
                                atom dwFlags, atom dwContext)
--<function>
--<name>InternetOpenUrl</name>
--<digest>
-- Begins reading a complete FTP, Gopher, or HTTP URL. Use
-- InternetCanonicalizeUrl first if the URL being used contains a relative URL
-- and a base URL separated by blank spaces.
--</digest>
--<desc>
-- Remarks
--   This is a general function that an application can use to retrieve data
--   over any of the protocols that the Win32(r) Internet functions support.
--   This function is especially useful when the application does not need to
--   access the particulars of a protocol, but only requires the data
--   corresponding to a URL. The InternetOpenUrl function parses the URL string,
--   establishes a connection to the server, and prepares to download the data
--   identified by the URL. The application can then use InternetReadFile (for
--   files) or InternetFindNextFile (for directories) to retrieve the URL data.
--   It is not necessary to call InternetConnect before InternetOpenUrl.
--
--   InternetOpenUrl disables Gopher on ports less than 1024, except for port 70
--   (the standard Gopher port) and port 105 (typically used for Central
--   Services Organization [CSO] name searches).
--</desc>
--<param>
--<type>atom</type>
--<name>hInternet</name>
--<desc>
--     HINTERNET handle to the current Internet session. The handle must have
--     been returned by a previous call to InternetOpen.
--</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>Url</name>
--<desc>
--     String variable that contains the URL to begin reading. Only
--     URLs beginning with ftp:, gopher:, http:, or https: are supported.
--</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>Headers</name>
--<desc>
--     String variable that contains the headers to be sent to the
--     HTTP server. (For more information, see the description of the
--     lpszHeaders parameter in the HttpSendRequest function.)
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwFlags</name>
--<desc>
--     Unsigned long integer value that contains the API flags. This can be one
--     of the following values:
--     INTERNET_FLAG_EXISTING_CONNECT
--     INTERNET_FLAG_HYPERLINK
--     INTERNET_FLAG_IGNORE_CERT_CN_INVALID
--     INTERNET_FLAG_IGNORE_CERT_DATE_INVALID
--     INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP
--     INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS
--     INTERNET_FLAG_KEEP_CONNECTION
--     INTERNET_FLAG_NEED_FILE
--     INTERNET_FLAG_NO_AUTH
--     INTERNET_FLAG_NO_AUTO_REDIRECT
--     INTERNET_FLAG_NO_CACHE_WRITE
--     INTERNET_FLAG_NO_COOKIES
--     INTERNET_FLAG_NO_UI
--     INTERNET_FLAG_PASSIVE
--     INTERNET_FLAG_PRAGMA_NOCACHE
--     INTERNET_FLAG_RAW_DATA
--     INTERNET_FLAG_RELOAD
--     INTERNET_FLAG_RESYNCHRONIZE
--     INTERNET_FLAG_SECURE
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwContext</name>
--<desc>
--     Address of an unsigned long integer value that contains the application-
--     defined value that is passed, along with the returned handle, to any
--     callback functions.
--</desc>
--</param>
--<return>
--   Returns a valid handle to the FTP, Gopher, or HTTP URL if the connection is
--   successfully established, or NULL if the connection fails. To get a
--   specific error message, call GetLastError. To determine why access to the
--   service was denied, call InternetGetLastResponseInfo.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom result,lpszUrl, lpszHeaders
  integer dwHeadersLength

  if sequence(Url) then
    lpszUrl = allocate_string(Url)
  else
    error_msg = "InternetOpenUrl: Url should be a sequence"
    return NULL
  end if
  if sequence(Headers) then
    lpszHeaders = allocate_string(Headers)
    dwHeadersLength = length(Headers)
  elsif Headers = NULL then
    lpszHeaders = NULL
    dwHeadersLength = 0
  else
    error_msg = "InternetOpenUrl: Headers should be a sequence or NULL"
    return NULL
  end if

  result = exec_w32func(xInternetOpenUrl, {hInternet, lpszUrl, lpszHeaders,
                                     dwHeadersLength, dwFlags, dwContext}, "InternetOpenUrlA")

  free(lpszUrl)
  if lpszHeaders != NULL then free(lpszHeaders) end if
  if result = NULL then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

-- =========================================================================================
--                                       FTP Functions
-- =========================================================================================
--
-- The FTP functions deal with FTP file and directory manipulation and navigation. Applications that use
-- a CERN proxy exclusively must use the InternetOpenUrl function because CERN proxies do not
-- support FTP. For more information on how to use InternetOpenUrl, see Accessing URLs Directly.

--------------------------------------------------------------------------------
-- integer xFtpCommand
-- xFtpCommand = 0
-- FtpCommand
-- Allows an application to send commands directly to an FTP server.
--
-- Syntax
-- BOOL FtpCommand(
-- IN HINTERNET hConnect,
-- IN BOOL fExpectResponse,
-- IN DWORD dwFlags,
-- IN LPCTSTR lpszCommand,
-- IN DWORD_PTR dwContext,
-- OUT HINTERNET *phFtpCommand
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- hConnect
-- HINTERNET handle returned from a call to InternetConnect.
--
-- fExpectResponse
-- Boolean value that indicates whether or not the application expects a response from the FTP
-- server. This must be set to TRUE if a response is expected, or FALSE otherwise.
--
-- dwFlags
-- Unsigned long integer value that contains the flags that control this function. This can be set to
-- one of the following values:
-- FTP_TRANSFER_TYPE_ASCII    Transfers the file using FTP's ASCII (Type A)
-- transfer method. Control and formatting information
-- is converted to local equivalents.
--
-- FTP_TRANSFER_TYPE_BINARY    Transfers the file using FTP's Image (Type I) transfer
-- method. The file is transferred exactly as it exists with
-- no changes. This is the default transfer method.
--
-- lpszCommand
-- Address of a string value that contains the command to send to the FTP server.
--
-- dwContext
-- Address of an unsigned long integer value that contains an application-defined value that is used
-- to identify the application context in callbacks.
--
-- phFtpCommand
-- Address of an HINTERNET handle that will be created if a valid data socket is opened. The
-- fExpectResponse parameter must be set to TRUE for phFtpCommand to be filled.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get a specific error message, call
-- GetLastError.
--
-- Remarks
-- GetLastError can return ERROR_INTERNET_NO_DIRECT_ACCESS if the client application is
-- offline. If one or more of the parameters are invalid, GetLastError will return
-- ERROR_INVALID_PARAMETER.
--global function ()
--  integer result

--  result = exec_w32func(x, {}, "FtpCommandA")
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- integer xFtpCreateDirectory
-- xFtpCreateDirectory = 0
-- FtpCreateDirectory
-- Creates a new directory on the FTP server.
--
-- Syntax
-- BOOL FtpCreateDirectory(
-- IN HINTERNET hConnect,
-- IN LPCTSTR lpszDirectory
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- hConnect
-- Valid HINTERNET handle returned by a previous call to InternetConnect using
-- INTERNET_SERVICE_FTP.
--
-- lpszDirectory
-- Address of a null-terminated string that contains the name of the directory to create on the
-- remote system. This can be either a fully qualified path or a name relative to the current
-- directory.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get a specific error message, call
-- GetLastError. If the error message indicates that the FTP server denied the request to create a
-- directory, use InternetGetLastResponseInfo to determine why.
--
-- Remarks
-- An application should use FtpGetCurrentDirectory to determine the remote site's current working
-- directory instead of assuming that the remote system uses a hierarchical naming scheme for
-- directories.
--
-- The lpszDirectory parameter can be either partially or fully qualified file names relative to the
-- current directory.
--global function ()
--  integer result

--  result = exec_w32func(x, {}, "FtpCreateDirectoryA")
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------

integer xFtpDeleteFile
xFtpDeleteFile = 0

global function FtpDeleteFile(atom hFtpSession, sequence FileName)
--<function>
--<name>FtpDeleteFile</name>
--<digest>Deletes a file stored on the FTP server.</digest>
--<desc>
-- Remarks
-- The FileName parameter can be either partially or fully qualified file names relative to the
-- current directory.
--</desc>
--<param>
--<type>atom</type>
--<name>hFtpSession</name>
-- Valid HINTERNET handle returned by a previous call to InternetConnect using
-- INTERNET_SERVICE_FTP.
--<desc>
--</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>FileName</name>
--<desc>
-- Name of the file to delete on the remote system.
--</desc>
--</param>
--<return>
-- Returns TRUE if successful, or FALSE otherwise. To get a specific error message, call
-- GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszFileName
  atom result

  lpszFileName = allocate_string(FileName)
  result = exec_w32func(xFtpDeleteFile, {hFtpSession, lpszFileName}, "FtpDeleteFileA")
  free(lpszFileName)
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function


--------------------------------------------------------------------------------

integer xFtpFindFirstFile
xFtpFindFirstFile = 0

global function FtpFindFirstFile(atom hFtpSession, sequence SearchFile, atom lpFindFileData, atom dwFlags, atom dwContent)  -- lpFindFileData As WIN32_FIND_DATA
--<function>
--<name>FtpFindFirstFile</name>
--<digest>
-- Searches the specified directory of the given FTP session. File and directory entries are returned to the
-- application in the WIN32_FIND_DATA structure.
--</digest>
--<desc>
-- Remarks
-- For FtpFindFirstFile, file times returned in the WIN32_FIND_DATA structure are in the local
-- time zone, not in a time zone independent (UTC) format.
--
-- FtpFindFirstFile is similar to the Win32(r) FindFirstFile function. Note, however, that only one
-- FtpFindFirstFile can occur at a time within a given FTP session. The enumerations, therefore, are
-- correlated with the FTP session handle. This is because the FTP protocol allows only a single
-- directory enumeration per session.
--
-- After calling FtpFindFirstFile and until calling InternetCloseHandle, the application cannot call
-- FtpFindFirstFile again on the given FTP session handle. If a call is made to FtpFindFirstFile on
-- that handle, the function will fail with ERROR_FTP_TRANSFER_IN_PROGRESS.
--
-- After beginning a directory enumeration with FtpFindFirstFile, the InternetFindNextFile
-- function can be used to continue the enumeration.
--
-- Because the FTP protocol provides no standard means of enumerating, some of the common
-- information about files, such as file creation date and time, is not always available or correct. When
-- this happens, FtpFindFirstFile and InternetFindNextFile fill in unavailable information with a
-- best guess based on available information. For example, creation and last access dates will often be
-- the same as the file's modification date.
--
-- The application cannot call FtpFindFirstFile between calls to FtpOpenFile and
-- InternetCloseHandle.
--</desc>
--<param>
--<type>atom</type>
--<name>hFtpSession</name>
--<desc>Valid handle to an FTP session returned from InternetConnect.</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>SearchFile</name>
--<desc>
-- Valid directory path or file name for the FTP server's file system. The string can contain
-- wildcards, but no blank spaces are allowed. If SearchFile is an empty string, it will
-- find the first file in the current directory on the server.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lpFindFileData</name>
--<desc>
-- Address of a WIN32_FIND_DATA structure that receives information about the found file or
-- directory.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwFlags</name>
--<desc>
-- Unsigned long integer value that contains the flags that control the behavior of this function.
-- This can be a combination of the following values:
-- INTERNET_FLAG_HYPERLINK
-- INTERNET_FLAG_NEED_FILE
-- INTERNET_FLAG_NO_CACHE_WRITE
-- INTERNET_FLAG_RELOAD
-- INTERNET_FLAG_RESYNCHRONIZE
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwContent</name>
--<desc>
-- Address of an unsigned long integer value that contains the application-defined value that
-- associates this search with any application data. This parameter is used only if the application
-- has already called InternetSetStatusCallback to set up a status callback function.
--</desc>
--</param>
--<return>
-- Returns a valid handle for the request if the directory enumeration was started successfully, or
-- returns NULL otherwise. To get a specific error message, call GetLastError. If the function finds
-- no matching files, GetLastError returns ERROR_NO_MORE_FILES.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszSearchFile
  atom result
  
  lpszSearchFile = allocate_string(SearchFile)
  result = exec_w32func(xFtpFindFirstFile, {hFtpSession, lpszSearchFile,
                     lpFindFileData, dwFlags, dwContent}, "FtpFindFirstFileA")
  free(lpszSearchFile)
  if result = NULL then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xFtpGetCurrentDirectory
xFtpGetCurrentDirectory = 0

global function FtpGetCurrentDirectory(atom hFtpSession, sequence Directory, atom lpdwCurrentDirectory)
--<function>
--<name>FtpGetCurrentDirectory</name>
--<digest>
-- Retrieves the current directory for the specified FTP session.
--</digest>
--<desc>
-- Remarks
-- If the lpszCurrentDirectory buffer is not large enough, lpdwCurrentDirectory receives the number
-- of bytes required to retrieve the full, current directory name.
--</desc>
--<param>
--<type>atom</type>
--<name>hFtpSession</name>
--<desc>
-- Valid handle to an FTP session.
--</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>Directory</name>
--<desc>
-- Absolute path to the current directory.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lpdwCurrentDirectory</name>
--<desc>
-- Address of a variable that specifies the length, in characters, of the buffer for the current
-- directory string. The buffer length must include room for a terminating null character. Using a
-- length of MAX_PATH is sufficient for all paths. When the function returns, the variable receives
-- the number of characters copied into the buffer.
--</desc>
--</param>
--<return>
-- Returns TRUE if successful, or FALSE otherwise. To get a specific error message, call
-- GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszDirectory
  atom result
  
  lpszDirectory = allocate_string(Directory)
  result = exec_w32func(xFtpGetCurrentDirectory, {hFtpSession, lpszDirectory, lpdwCurrentDirectory}, "FtpGetCurrentDirectoryA")
  free(lpszDirectory)
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xFtpGetFile
xFtpGetFile = 0

global function FtpGetFile(atom hFtpSession, sequence RemoteFile, sequence NewFile,
                             integer fFailIfExists, atom dwFlagsAndAttributes, atom dwFlags,
							 atom dwContext)
--<function>
--<name>FtpGetFile</name>
--<digest>
-- Retrieves a file from the FTP server and stores it under the specified file name, creating a new local
-- file in the process.
--</digest>
--<desc>
-- Remarks
-- FtpGetFile is a high-level routine that handles all the bookkeeping and overhead associated with
-- reading a file from an FTP server and storing it locally. An application that needs to retrieve file
-- data only or that requires close control over the file transfer should use the FtpOpenFile and
-- InternetReadFile functions.
--
-- If the dwFlags parameter specifies FILE_TRANSFER_TYPE_ASCII, translation of the file data
-- converts control and formatting characters to local equivalents. The default transfer is binary mode,
-- where the file is downloaded in the same format as it is stored on the server.
--
-- Both lpszRemoteFile and lpszNewFile can be either partially or fully qualified file names relative to
-- the current directory.
--</desc>
--<param>
--<type>atom</type>
--<name>hFtpSession</name>
--<desc>Valid handle to an FTP session.</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>RemoteFile</name>
--<desc>Name of the file to retrieve from the remote system.</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>NewFile</name>
--<desc>Name of the file to create on the local system.</desc>
--</param>
--<param>
--<type>integer</type>
--<name>fFailIfExists</name>
--<desc>
-- Boolean flag that indicates whether the function should proceed if a local file of the specified
-- name already exists. If fFailIfExists is TRUE and the local file exists, FtpGetFile fails.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwFlagsAndAttributes</name>
--<desc>
-- Unsigned long integer value that contains the file attributes for the new file. This can be any
-- combination of the FILE_ATTRIBUTE_* flags used by the CreateFile function. For more
-- information on FILE_ATTRIBUTE_* attributes, see CreateFile in the Platform SDK.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwFlags</name>
--<desc>
-- Unsigned long integer value that contains the flags that control how the function will handle the
-- file download. The first set of flag values indicates the conditions under which the transfer
-- occurs. These transfer type flags can be used in combination with the second set of flags that
-- control caching. The application can select one of these transfer type values:
-- FTP_TRANSFER_TYPE_ASCII         Transfers the file using FTP's ASCII (Type A) transfer method.
--                                 Control and formatting information is converted to local equivalents.
-- FTP_TRANSFER_TYPE_BINARY        Transfers the file using FTP's Image (Type I) transfer method.
--                                 The file is transferred exactly as it exists with no changes.
--                                 This is the default transfer method.
-- FTP_TRANSFER_TYPE_UNKNOWN       Defaults to FTP_TRANSFER_TYPE_BINARY.
-- INTERNET_FLAG_TRANSFER_ASCII    Transfers the file as ASCII.
-- INTERNET_FLAG_TRANSFER_BINARY   Transfers the file as binary.
--
-- The following flags determine how the caching of this file will be done. Any combination of the
-- following flags can be used with the transfer type flag. The possible values are:
-- INTERNET_FLAG_HYPERLINK
-- INTERNET_FLAG_NEED_FILE
-- INTERNET_FLAG_RELOAD
-- INTERNET_FLAG_RESYNCHRONIZE
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwContext</name>
--<desc>
-- Address of an unsigned long integer value that contains the application-defined value that
-- associates this search with any application data. This is used only if the application has already
-- called InternetSetStatusCallback to set up a status callback function.
--</desc>
--</param>
--<return>
-- Returns TRUE if successful, or FALSE otherwise. To get a specific error message, call
-- GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszRemoteFile, lpszNewFile
  atom result

  lpszRemoteFile = allocate_string(RemoteFile)
  lpszNewFile = allocate_string(NewFile)
  result = exec_w32func(xFtpGetFile, {hFtpSession, lpszRemoteFile, lpszNewFile,
                     fFailIfExists, dwFlagsAndAttributes, dwFlags, dwContext},
					 "FtpGetFileA")
  free(lpszRemoteFile)
  free(lpszNewFile)
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xFtpGetFileSize
xFtpGetFileSize = 0

global function FtpGetFileSize(atom hFile)
--<function>
--<name>FtpGetFileSize</name>
--<digest>
-- Retrieves the file size of the requested FTP resource.
--</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>hFile</name>
--<desc>
--     HINTERNET handle returned from a call to FtpOpenFile.
--</desc>
--</param>
--<return>
--   Returns the low-order unsigned long integer of the file size, in TCHAR,
--   of the requested FTP resource.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpdwFileSizeHigh
  integer result, FileSize

  lpdwFileSizeHigh = allocate(4)
  result = exec_w32func(xFtpGetFileSize, {hFile, lpdwFileSizeHigh}, "FtpGetFileSize")
  FileSize = peek4u(lpdwFileSizeHigh)
  free(lpdwFileSizeHigh)
  return FileSize
end function

--------------------------------------------------------------------------------

integer xFtpOpenFile
xFtpOpenFile = 0

global function FtpOpenFile(atom hFtpSession, sequence FileName,
                              atom lAccess, atom lFlags, atom lContext)
--<function>
--<name>FtpOpenFile</name>
--<digest>
-- Initiates access to a remote file on an FTP server for reading or writing.
--</digest>
--<desc>
-- Remarks
-- After calling FtpOpenFile and until calling InternetCloseHandle, all other calls to FTP functions
-- on the same FTP session handle will fail and set the error message to
-- ERROR_FTP_TRANSFER_IN_PROGRESS.
--
-- Only one file can be open in a single FTP session. Therefore, no file handle is returned and the
-- application simply uses the FTP session handle when necessary.
--
-- The lpszFileName parameter can be either partially or fully qualified file names relative to the
-- current directory.
--</desc>
--<param>
--<type>atom</type>
--<name>hFtpSession</name>
--<desc>
-- Valid HINTERNET handle to an FTP session.
--</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>FileName</name>
--<desc>
-- Name of the file to access on the remote system.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lAccess</name>
--<desc>
-- Determines how the file will be accessed. This can be GENERIC_READ or
-- GENERIC_WRITE, but not both.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lFlags</name>
--<desc>
-- Conditions under which the transfers occur. The application should select one
-- transfer type and any of the flags that indicate how the caching of the file
-- will be controlled. The transfer type can be any one of the following values:
--
-- FTP_TRANSFER_TYPE_ASCII         Transfers the file using FTP's ASCII (Type A)
--                                 transfer method. Control and formatting
--                                 information is converted to local equivalents.
-- FTP_TRANSFER_TYPE_BINARY        Transfers the file using FTP's Image (Type I)
--                                 transfer method. The file is transferred
--                                 exactly as it exists with no changes.
--                                 This is the default transfer method.
-- FTP_TRANSFER_TYPE_UNKNOWN       Defaults to FTP_TRANSFER_TYPE_BINARY.
-- INTERNET_FLAG_TRANSFER_ASCII    Transfers the file as ASCII.
-- INTERNET_FLAG_TRANSFER_BINARY   Transfers the file as binary.
--
-- The following values are used to control the caching of the file.
-- The application can use one or more of these.
--
-- INTERNET_FLAG_HYPERLINK
-- INTERNET_FLAG_NEED_FILE
-- INTERNET_FLAG_RELOAD
-- INTERNET_FLAG_RESYNCHRONIZE
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lContext</name>
--<desc>
-- Address of the application-defined value that associates this search with
-- any application data. This is only used if the application has already
-- called InternetSetStatusCallback to set up a status callback function.
--</desc>
--</param>
--<return>
-- Returns a handle if successful, or NULL otherwise. To get a specific error message, call
-- GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom sFileName
  atom result

  sFileName = allocate_string(FileName)
  result = exec_w32func(xFtpOpenFile, {hFtpSession, sFileName, lAccess,
                     lFlags, lContext}, "FtpOpenFileA")
  free(sFileName)
  if result = NULL then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xFtpPutFile
xFtpPutFile = 0

global function FtpPutFile(atom hFtpSession, sequence LocalFile,
                             sequence RemoteFile, atom dwFlags, atom dwContext)
--<function>
--<name>FtpPutFile</name>
--<digest>
-- Stores a file on the FTP server.
--</digest>
--<desc>
-- Remarks
-- FtpPutFile is a high-level routine that handles all the bookkeeping and overhead associated with
-- reading a file locally and storing it on an FTP server. An application that needs to send file data
-- only, or that requires close control over the file transfer, should use the FtpOpenFile and
-- InternetWriteFile functions.
--
-- If the dwFlags parameter specifies FILE_TRANSFER_TYPE_ASCII, translation of the file data
-- converts control and formatting characters to local equivalents.
--
-- Both lpszNewRemoteFile and lpszLocalFile can be either partially or fully qualified file names
-- relative to the current directory.
--</desc>
--<param>
--<type>atom</type>
--<name>hFtpSession</name>
--<desc>
-- Valid HINTERNET handle to an FTP session.
--</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>LocalFile</name>
--<desc>
-- Name of the file to send from the local system.
--</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>RemoteFile</name>
--<desc>
-- Name of the file to create on the remote system.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwFlags</name>
--<desc>
-- Conditions under which the transfers occur. The application should select one
-- transfer type and any of the flags that control how the caching of the file
-- will be controlled. The transfer type can be any one of the following values:
--
-- FTP_TRANSFER_TYPE_ASCII         Transfers the file using FTP's ASCII (Type A)
--                                 transfer method. Control and formatting
--                                 information is converted to local equivalents.
-- FTP_TRANSFER_TYPE_BINARY        Transfers the file using FTP's Image (Type I)
--                                 transfer method. The file is transferred
--                                 exactly as it exists, with no changes.
--                                 This is the default transfer method.
-- FTP_TRANSFER_TYPE_UNKNOWN       Defaults to FTP_TRANSFER_TYPE_BINARY.
-- INTERNET_FLAG_TRANSFER_ASCII    Transfers the file as ASCII.
-- INTERNET_FLAG_TRANSFER_BINARY   Transfers the file as binary.
--
-- The following values are used to control the caching of the file. The application can use one or
-- more of the following values:
-- INTERNET_FLAG_HYPERLINK
-- INTERNET_FLAG_NEED_FILE
-- INTERNET_FLAG_RELOAD
-- INTERNET_FLAG_RESYNCHRONIZE
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwContext</name>
--<desc>
-- Address of the application-defined value that associates this search with any
-- application data. This parameter is used only if the application has
-- already called InternetSetStatusCallback to set up a status callback.
--</desc>
--</param>
--<return>
-- Returns TRUE if successful, or FALSE otherwise. To get a specific error message, call
-- GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszLocalFile, lpszRemoteFile
  atom result

  lpszLocalFile = allocate_string(LocalFile)
  lpszRemoteFile = allocate_string(RemoteFile)
  result = exec_w32func(xFtpPutFile, {hFtpSession, lpszLocalFile, lpszRemoteFile, dwFlags, dwContext}, "FtpPutFileA")
  free(lpszLocalFile)
  free(lpszRemoteFile)
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xFtpRemoveDirectory
xFtpRemoveDirectory = 0

-- FtpRemoveDirectory
-- Removes the specified directory on the FTP server.
--
-- Syntax
-- BOOL FtpRemoveDirectory(
-- IN HINTERNET hConnect,
-- IN LPCTSTR lpszDirectory
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- hConnect
-- Valid HINTERNET handle to an FTP session.
--
-- lpszDirectory
-- Address of a null-terminated string that contains the name of the directory to remove on the
-- remote system. This can be either a fully qualified path or a name relative to the current
-- directory.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get a specific error message, call
-- GetLastError. If the error message indicates that the FTP server denied the request to remove a
-- directory, use InternetGetLastResponseInfo to determine why.
--
-- Remarks
-- An application should use FtpGetCurrentDirectory to determine the remote site's current working
-- directory, instead of assuming that the remote system uses a hierarchical naming scheme for
-- directories.
--
-- The lpszDirectory parameter can be either partially or fully qualified file names relative to the
-- current directory.
--global function ()
--  integer result

--  result = exec_w32func(xFtpRemoveDirectory, {}, "FtpRemoveDirectoryA")
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------

-- integer xFtpRenameFile
-- xFtpRenameFile = 0

-- FtpRenameFile
-- Renames a file stored on the FTP server.
--
-- Syntax
-- BOOL FtpRenameFile(
-- IN HINTERNET hConnect,
-- IN LPCTSTR lpszExisting,
-- IN LPCTSTR lpszNew
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- hConnect
-- Valid HINTERNET handle to an FTP session.
--
-- lpszExisting
-- Address of a null-terminated string that contains the name of the file that will have its name
-- changed on the remote FTP server.
--
-- lpszNew
-- Address of a null-terminated string that contains the new name for the remote file.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get a specific error message, call
-- GetLastError.
--
-- Remarks
-- The lpszExisting and lpszNew parameters can be either partially or fully qualified file names relative
-- to the current directory.
--global function ()
--  integer result

--  result = exec_w32func(x, {}, "FtpRenameFileA")
--  if result = FALSE then
--    error_code = GetLastError()
--    error_msg  = FormatSystemError(error_code)
--  end if
--  return result
--end function

--------------------------------------------------------------------------------

integer xFtpSetCurrentDirectory
xFtpSetCurrentDirectory = 0

global function FtpSetCurrentDirectory(atom hConnect, sequence Directory)
--<function>
--<name>FtpSetCurrentDirectory</name>
--<digest>
-- Changes to a different working directory on the FTP server.
--</digest>
--<desc>
-- Remarks
--   An application should use FtpGetCurrentDirectory to determine the remote
--   site's current working directory, instead of assuming that the remote
--   system uses a hierarchical naming scheme for directories.
--
--   The lpszDirectory parameter can be either partially or fully qualified file
--   names relative to the current directory.
--</desc>
--<param>
--<type>atom</type>
--<name>hConnect</name>
--<desc>
-- Valid HINTERNET handle to an FTP session.
--</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>Directory</name>
--<desc>
-- Name of the directory to change to on the remote system. This can be either a
-- fully qualified path or a name relative to the current directory.
--</desc>
--</param>
--<return>
-- Returns TRUE if successful, or FALSE otherwise. To get a specific error
-- message, call GetLastError. If the error message indicates that the FTP
-- server denied the request to change a directory, use
-- InternetGetLastResponseInfo to determine why.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszDirectory
  integer result

  lpszDirectory = allocate_string(Directory)
  result =  exec_w32func(xFtpSetCurrentDirectory, {hConnect, lpszDirectory}, "FtpSetCurrentDirectoryA")
  free(lpszDirectory)
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

-- =========================================================================================
--                                      Gopher Functions
-- =========================================================================================
--
-- The Gopher functions described in this section control the creation and use of the Internet Gopher
-- utilities.

--------------------------------------------------------------------------------
-- GopherCreateLocator
-- Creates a Gopher or Gopher+ locator string from its component parts.
--
-- Syntax
-- BOOL GopherCreateLocator(
-- IN LPCSTR lpszHost,
-- IN INTERNET_PORT nServerPort,
-- IN LPCTSTR lpszDisplayString,
-- IN LPCTSTR lpszSelectorString,
-- IN DWORD dwGopherType,
-- OUT LPTSTR lpszLocator,
-- IN OUT LPDWORD lpdwBufferLength
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszHost
-- Address of a string that contains the name of the host, or a dotted-decimal IP address (such as
-- 198.105.232.1).
--
-- nServerPort
-- Port number on which the Gopher server at lpszHost lives, in host byte order. If nServerPort is
-- INTERNET_INVALID_PORT_NUMBER, the default Gopher port is used.
--
-- lpszDisplayString
-- Address of a string that contains the Gopher document or directory to be displayed. If this
-- parameter is NULL, the function returns the default directory for the Gopher server.
--
-- lpszSelectorString
-- Address of the selector string to send to the Gopher server in order to retrieve information. This
-- parameter can be NULL.
--
-- dwGopherType
-- Unsigned long integer value that specifies whether lpszSelectorString refers to a directory or
-- document, and whether the request is Gopher+ or Gopher. The default value,
-- GOPHER_TYPE_DIRECTORY, is used if the value of dwGopherType is zero. This can be one
-- of the Gopher Type Values.
--
-- lpszLocator
-- Address of a buffer that receives the locator string. If lpszLocator is NULL, lpdwBufferLength
-- receives the necessary buffer length, but the function performs no other processing.
--
-- lpdwBufferLength
-- Address of an unsigned long integer value that contains the length of the lpszLocator buffer, in
-- TCHARs. When the function returns, this parameter receives the number of TCHARs written to
-- the lpszLocator buffer. If GetLastError returns ERROR_INSUFFICIENT_BUFFER, this
-- parameter receives the number of bytes required to form the locator successfully.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError or InternetGetLastResponseInfo.
--
-- Remarks
-- To retrieve information from a Gopher server, an application must first get a Gopher "locator" from
-- the Gopher server.
--
-- The locator, which the application should treat as an opaque token, is normally used for calls to the
-- GopherFindFirstFile function to retrieve a specific piece of information.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- GopherFindFirstFile
--
-- Uses a Gopher locator and some search criteria to create a session with the server and locate the
-- requested documents, binary files, index servers, or directory trees.
--
-- Syntax
-- HINTERNET GopherFindFirstFile(
-- IN HINTERNET hConnect,
-- IN LPCTSTR lpszLocator,
-- IN LPCTSTR lpszSearchString,
-- OUT LPGOPHER_FIND_DATA lpFindData,
-- IN DWORD dwFlags,
-- IN DWORD_PTR dwContext
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- hConnect
-- Handle to a Gopher session returned by InternetConnect.
--
-- lpszLocator
-- Address of a string containing the name of the item to locate. This can be one of the following
-- items:
-- *    Gopher locator returned by a previous call to this function or the InternetFindNextFile
-- function.
-- *    NULL pointer or zero-length string indicating that the topmost information from a Gopher
-- server is being returned.
-- *    Locator created by the GopherCreateLocator function.
-- lpszSearchString
-- Address of a buffer that contains the strings to search, if this request is to an index server.
-- Otherwise, this parameter should be NULL.
--
-- lpFindData
-- Address of a GOPHER_FIND_DATA structure that receives the information retrieved by this
-- function.
--
-- dwFlags
-- Unsigned long integer value containing the flags controlling the function behavior. This can be a
-- combination of the following values:
-- INTERNET_FLAG_HYPERLINK
-- INTERNET_FLAG_NEED_FILE
-- INTERNET_FLAG_NO_CACHE_WRITE
-- INTERNET_FLAG_RELOAD
-- INTERNET_FLAG_RESYNCHRONIZE
-- dwContext
-- Address of an unsigned long integer value containing the application-defined value that
-- associates this search with any application data.
--
-- Return Value
-- Returns a valid search handle if successful, or NULL otherwise. To get extended error information,
-- call GetLastError or InternetGetLastResponseInfo.
--
-- Remarks
-- GopherFindFirstFile closely resembles the Win32(r) FindFirstFile function. It creates a
-- connection with a Gopher server, and then returns a single structure containing information about
-- the first Gopher object referenced by the locator string.
--
-- After calling GopherFindFirstFile to retrieve the first Gopher object in an enumeration, an
-- application can use the InternetFindNextFile function to retrieve subsequent Gopher objects.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- GopherGetAttribute
--
-- Retrieves the specific attribute information from the server.
--
-- Syntax
-- BOOL GopherGetAttribute(
-- IN HINTERNET hConnect,
-- IN LPCTSTR lpszLocator,
-- IN LPCTSTR lpszAttributeName,
-- OUT LPBYTE lpBuffer,
-- IN DWORD dwBufferLength,
-- OUT LPDWORD lpdwCharactersReturned,
-- IN GOPHER_ATTRIBUTE_ENUMERATOR lpfnEnumerator,
-- IN DWORD dwContext
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- hConnect
-- Handle to a Gopher session returned by InternetConnect.
--
-- lpszLocator
-- Address of a string that identifies the item at the Gopher server on which to return attribute
-- information.
--
-- lpszAttributeName
-- Address of a space-delimited string specifying the names of attributes to return. If
-- lpszAttributeName is NULL, GopherGetAttribute returns information about all attributes.
--
-- lpBuffer
-- Address of an application-defined buffer from which attribute information is retrieved.
--
-- dwBufferLength
-- Unsigned long integer value containing the size, in TCHAR, of the lpBuffer buffer.
--
-- lpdwCharactersReturned
-- Address of an unsigned long integer value that contains the number of characters read into the
-- lpBuffer buffer.
--
-- lpfnEnumerator
-- Address of a callback function that enumerates each attribute of the locator. This parameter is
-- optional. If it is NULL, all the Gopher attribute information is placed into lpBuffer. If
-- lpfnEnumerator is specified, the callback function is called once for each attribute of the object.
-- The callback function receives the address of a single GOPHER_ATTRIBUTE_TYPE structure
-- with each call. The enumeration callback function allows the application to avoid having to parse
-- the Gopher attribute information.
--
-- dwContext
-- Unsigned long integer value that contains the application-defined value that associates this
-- operation with any application data.
--
-- Return Value
-- Returns TRUE if the request is satisfied, or FALSE otherwise. To get extended error information,
-- call GetLastError or InternetGetLastResponseInfo.
--
-- Remarks
-- Generally, applications call this function after calling GopherFindFirstFile or
-- InternetFindNextFile.
--
-- The size of the lpBuffer parameter must be equal to or greater than the value of
-- MIN_GOPHER_ATTRIBUTE_LENGTH (currently defined in Wininet.h as 256 bytes).
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- GopherGetLocatorType
--
-- Parses a Gopher locator and determines its attributes.
--
-- Syntax
-- BOOL GopherGetLocatorType(
-- IN LPCTSTR lpszLocator,
-- OUT LPDWORD lpdwGopherType
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszLocator
-- Address of the Gopher locator string to parse.
--
-- lpdwGopherType
-- Address of an unsigned long integer variable that receives the type of the locator. The type is a
-- bitmask that consists of a combination of the Gopher Type Values.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError.
--
-- Remarks
-- GopherGetLocatorType returns information about the item referenced by a Gopher locator. Note
-- that it is possible for multiple attributes to be set on a file. For example, both
-- GOPHER_TYPE_TEXT_FILE and GOPHER_TYPE_GOPHER_PLUS are set for a text file stored
-- on a Gopher+ server.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- GopherOpenFile
--
-- Begins reading a Gopher data file from a Gopher server.
--
-- Syntax
-- HINTERNET GopherOpenFile(
-- IN HINTERNET hConnect,
-- IN LPCTSTR lpszLocator,
-- IN LPCTSTR lpszView,
-- IN DWORD dwFlags,
-- IN DWORD_PTR dwContext
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- hConnect
-- Handle to a Gopher session returned by InternetConnect.
--
-- lpszLocator
-- Address of a string that identifies the file to open. Generally, this locator is returned from a call
-- to GopherFindFirstFile or InternetFindNextFile. Because the Gopher protocol has no concept
-- of a current directory, the locator is always fully qualified.
--
-- lpszView
-- Address of a string that describes the view to open if several views of the file exist at the server.
-- If lpszView is NULL, the function uses the default file view.
--
-- dwFlags
-- Unsigned long integer value that contains the conditions under which subsequent transfers occur.
-- This can be any combination of the following values:
-- INTERNET_FLAG_HYPERLINK
-- INTERNET_FLAG_NEED_FILE
-- INTERNET_FLAG_NO_CACHE_WRITE
-- INTERNET_FLAG_RELOAD
-- INTERNET_FLAG_RESYNCHRONIZE
-- dwContext
-- Address of an unsigned long integer value that contains an application-defined value that
-- associates this operation with any application data.
--
-- Return Value
-- Returns a handle if successful, or NULL if the file cannot be opened. To get extended error
-- information, call GetLastError or InternetGetLastResponseInfo.
--
-- Remarks
-- GopherOpenFile opens a file at a Gopher server. Because a file cannot actually be opened or
-- locked at a server, this function simply associates location information with a handle that an
-- application can use for file-based operations such as InternetReadFile or GopherGetAttribute.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

-- =============================================================================
--                                HTTP Functions
-- =============================================================================
--
-- The HTTP functions described in this section control the transmission and
-- content of HTTP requests.

--------------------------------------------------------------------------------

integer xHttpAddRequestHeaders
xHttpAddRequestHeaders = 0

global function HttpAddRequestHeaders(atom hConnect, object Headers,
                                      atom dwModifiers)
--<function>
--<name>HttpAddRequestHeaders</name>
--<digest>Adds one or more HTTP request headers to the HTTP request handle.</digest>
--<desc>
-- Remarks
-- HttpAddRequestHeaders appends additional, free-format headers to the HTTP
-- request handle and is intended for use by sophisticated clients that need
-- detailed control over the exact request sent to the HTTP server.
--
-- Note that for basic HttpAddRequestHeaders, the application can pass in multiple
-- headers in a single buffer. If the application is trying to remove or replace a
-- header, only one header can be supplied in lpszHeaders.
--</desc>
--<param>
--<type>atom</type>
--<name>hConnect</name>
--<desc>HINTERNET handle returned by a call to the HttpOpenRequest function.</desc>
--</param>
--<param>
--<type>object</type>
--<name>Headers</name>
--<desc>
-- Headers to append to the request. Each header must be terminated by a CR/LF
-- (carriage return/line feed) pair.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwModifiers</name>
--<desc>
-- Flags used to modify the semantics of this function.
-- Can be a combination of the following values:
--
-- HTTP_ADDREQ_FLAG_ADD                       Adds the header if it does not exist. Used with
--                                            HTTP_ADDREQ_FLAG_REPLACE.
-- HTTP_ADDREQ_FLAG_ADD_IF_NEW                Adds the header only if it does not already exist;
--                                            otherwise, an error is returned.
-- HTTP_ADDREQ_FLAG_COALESCE                  Coalesces headers of the same name.
-- HTTP_ADDREQ_FLAG_COALESCE_WITH_COMMA       Coalesces headers of the same name. For example, adding
--                                            "Accept: text/*" followed by "Accept: audio/*" with this
--                                            flag results in the formation of the single header
--                                            "Accept: text/*, audio/*". This causes the first header
--                                            found to be coalesced. It is up to the calling application
--                                            to ensure a cohesive scheme with respect to coalesced/
--                                            separate headers.
-- HTTP_ADDREQ_FLAG_COALESCE_WITH_SEMICOLON   Coalesces headers of the same name using a semicolon.
-- HTTP_ADDREQ_FLAG_REPLACE                   Replaces or removes a header. If the header value is empty
--                                            and the header is found, it is removed. If not empty, the
--                                            header value is replaced.
--</desc>
--</param>
--<return>
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszHeaders, dwHeadersLength
  integer result

  if sequence(Headers) then
    lpszHeaders = allocate_string(Headers)
    dwHeadersLength = length(Headers)
  else
    error_msg = "HttpAddRequestHeaders: Headers should be a sequence"
    return FALSE
  end if

  result = exec_w32func(xHttpAddRequestHeaders, {hConnect, lpszHeaders,
                     dwHeadersLength, dwModifiers}, "HttpAddRequestHeadersA")

  if lpszHeaders != NULL then free(lpszHeaders) end if
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xHttpEndRequest
xHttpEndRequest = 0

global function HttpEndRequest(atom hRequest, integer dwFlags, integer dwContext)
--<function>
--<name>HttpEndRequest</name>
--<digest>Ends an HTTP request that was initiated by HttpSendRequestEx.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>hRequest</name>
--<desc>HINTERNET handle returned by HttpOpenRequest and sent by HttpSendRequestEx.</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwFlags</name>
--<desc>
--     Unsigned long integer value that contains the flags that control this
--     function. Can be one of the following values:
--     HSR_ASYNC         Identical to WININET_API_FLAG_ASYNC.
--     HSR_SYNC          Identical to WININET_API_FLAG_SYNC.
--     HSR_USE_CONTEXT   Identical to WININET_API_FLAG_USE_CONTEXT.
--     HSR_INITIATE      Iterative operation (completed by HttpEndRequest).
--     HSR_DOWNLOAD      Download resource to file.
--     HSR_CHUNKED       Send chunked data.
--</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwContext</name>
--<desc>
--   Unsigned long integer variable that contains the application-defined context
--   value for applications that register a status callback function.
--</desc>
--</param>
--<return>
--   Returns TRUE if successful, or FALSE otherwise. To get extended error
--   information, call GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer result

  result = exec_w32func(xHttpEndRequest, {hRequest, NULL, dwFlags, dwContext},
                     "HttpEndRequestA")
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------
integer xHttpOpenRequest
xHttpOpenRequest = 0

global function HttpOpenRequest(atom hConnect, sequence Verb, object ObjectName,
                  object Version, object Referer, object AcceptTypes,
                  atom dwFlags, atom dwContext)
--<function>
--<name>HttpOpenRequest</name>
--<digest>Creates an HTTP request handle.</digest>
--<desc>
-- Remarks
--   HttpOpenRequest creates a new HTTP request handle and stores the specified parameters in that
--   handle. An HTTP request handle holds a request to be sent to an HTTP server and contains all
--   RFC822/MIME/HTTP headers to be sent as part of the request.
--
--   Beginning with Microsoft(r) Internet Explorer 5, if lpszVerb is set to "HEAD", the Content-Length
--   header is ignored on responses from HTTP/1.1 servers.
--</desc>
--<param>
--<type>atom</type>
--<name>hConnect</name>
--<desc>HINTERNET handle to an HTTP session returned by InternetConnect.</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>Verb</name>
--<desc>Verb to use in the request. If this parameter is NULL, the function uses GET as the verb.</desc>
--</param>
--<param>
--<type>object</type>
--<name>ObjectName</name>
--<desc>
-- Name of the target object of the specified verb. This is generally a file name,
-- an executable module, or a search specifier.
--</desc>
--</param>
--<param>
--<type>object</type>
--<name>Version</name>
--<desc>
-- HTTP version. If this parameter is NULL, the function uses HTTP/1.0 as the version.
--</desc>
--</param>
--<param>
--<type>object</type>
--<name>Referer</name>
--<desc>
-- URL of the document from which the URL in the request (lpszObjectName) was obtained.
-- If this parameter is NULL, no "referrer" is specified.
--</desc>
--</param>
--<param>
--<type>object</type>
--<name>AcceptTypes</name>
--<desc>
-- Media types accepted by the client. If NULL, no types are accepted by the client.
-- Servers interpret a lack of accept types to indicate that the client accepts only documents
-- of type "text/*" (that is, only text documents, and not pictures or other binary files).
-- For a listing of valid media types, see Media Types at
-- ftp.isi.edu/in-notes/iana/assignments/media-types/media-types.
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwFlags</name>
--<desc>
--     Unsigned long integer value that contains the Internet flag values. This can be any of the
--     following values:
--     INTERNET_FLAG_CACHE_IF_NET_FAIL
--     INTERNET_FLAG_HYPERLINK
--     INTERNET_FLAG_IGNORE_CERT_CN_INVALID
--     INTERNET_FLAG_IGNORE_CERT_DATE_INVALID
--     INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP
--     INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS
--     INTERNET_FLAG_KEEP_CONNECTION
--     INTERNET_FLAG_NEED_FILE
--     INTERNET_FLAG_NO_AUTH
--     INTERNET_FLAG_NO_AUTO_REDIRECT
--     INTERNET_FLAG_NO_CACHE_WRITE
--     INTERNET_FLAG_NO_COOKIES
--     INTERNET_FLAG_NO_UI
--     INTERNET_FLAG_PRAGMA_NOCACHE
--     INTERNET_FLAG_RELOAD
--     INTERNET_FLAG_RESYNCHRONIZE
--     INTERNET_FLAG_SECURE
--</desc>
--</param>
--<param>
--<type>atom</type>
--<name>dwContext</name>
--<desc>
--     Address of an unsigned long integer value that contains the application-defined value that
--     associates this operation with any application data.
--</desc>
--</param>
--<return>
--   Returns a valid (non-NULL) HTTP request handle if successful, or NULL otherwise. To get
--   extended error information, call GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszVerb, lpszObjectName, lpszVersion, lpszReferer, lpszAcceptTypes
  atom result

  if sequence(Verb) then
    lpszVerb = allocate_string(Verb)
  else
    error_msg = "HttpOpenRequest: Verb should be a sequence"
    return NULL
  end if
  if sequence(ObjectName) then
    lpszObjectName = allocate_string(ObjectName)
  elsif ObjectName = NULL then
    lpszObjectName = NULL
  else
    error_msg = "HttpOpenRequest: ObjectName should be a sequence or NULL"
    return NULL
  end if
  if sequence(Version) then
    lpszVersion = allocate_string(Version)
  elsif Version = NULL then
    lpszVersion = NULL
  else
    error_msg = "HttpOpenRequest: Version should be a sequence or NULL"
    return NULL
  end if
  if sequence(Referer) then
    lpszReferer = allocate_string(Referer)
  elsif Referer = NULL then
    lpszReferer = NULL
  else
    error_msg = "HttpOpenRequest: Referer should be a sequence or NULL"
    return NULL
  end if
  if sequence(AcceptTypes) then
    lpszAcceptTypes = allocate_string(AcceptTypes)
  elsif AcceptTypes = NULL then
    lpszAcceptTypes = NULL
  else
    error_msg = "HttpOpenRequest: AcceptTypes should be a sequence or NULL"
    return NULL
  end if

  result = exec_w32func(xHttpOpenRequest, {hConnect, lpszVerb, lpszObjectName,
                                     lpszVersion, lpszReferer, lpszAcceptTypes,
                                     dwFlags, dwContext}, "HttpOpenRequestA")

  free(lpszVerb)
  if lpszObjectName != NULL then free(lpszObjectName) end if
  if lpszVersion != NULL then free(lpszVersion) end if
  if lpszReferer != NULL then free(lpszReferer) end if
  if lpszAcceptTypes != NULL then free(lpszAcceptTypes) end if
  if result = NULL then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------

integer xHttpQueryInfo
xHttpQueryInfo = 0

global function HttpQueryInfo(atom hRequest, integer dwInfoLevel,
                                object Headers)
--<function>
--<name>HttpQueryInfo</name>
--<digest>Retrieves header information associated with an HTTP request.</digest>
--<desc>
-- Remarks
--   You can retrieve different types of data from HttpQueryInfo:
--   *    Strings (default)
--   *    SYSTEMTIME (for Data: Expires:, headers)
--   *    DWORD (for STATUS_CODE, CONTENT_LENGTH, and so on, if HTTP_QUERY_FLAG_NUMBER has been used)
--
--   If your application needs the data returned as a data type other than a string, you must include the
--   appropriate modifier with the attribute passed to dwInfoLevel.
--</desc>
--<param>
--<type>atom</type>
--<name>hRequest</name>
--<desc>HINTERNET request handle returned by HttpOpenRequest or InternetOpenUrl.</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwInfoLevel</name>
--<desc>
--     Unsigned long integer value that contains a combination of an attribute to retrieve and the flags
--     that modify the request. The attribute can be any one of the Attributes, and the flag can be any
--     one of the Modifiers on the Query Info Flags page.
--</desc>
--</param>
--<param>
--<type>object</type>
--<name>Headers</name>
--<desc>Buffer that receives the information.</desc>
--</param>
--<return>
--   Returns data if successful, or FALSE otherwise. To get extended error information, call
--   GetLastError.
--</return>
--<example>
--     bRet = HttpQueryInfo(hResource, HTTP_QUERY_RAW_HEADERS_CRLF, lpvSomeBuffer)
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpdwIndex, lpvBuffer, lpdwBufferLength
  integer result
  object ExtMessage

  ExtMessage = 0

  lpdwIndex = allocate(4)
  lpdwBufferLength = allocate(4)

  if sequence(Headers) then
    lpvBuffer = allocate_string(Headers)
    poke4(lpdwBufferLength, length(Headers)+1)
  elsif Headers = NULL then
    lpvBuffer = NULL
    poke4(lpdwBufferLength, 0)
  else
    error_msg = "HttpQueryInfo: Headers should be a sequence or NULL"
    return NULL
  end if

  result = exec_w32func(xHttpQueryInfo, {hRequest, dwInfoLevel, lpvBuffer,
                     lpdwBufferLength, lpdwIndex}, "HttpQueryInfoA")

  if result = FALSE then
    error_code = GetLastError()
    if error_code = ERROR_HTTP_HEADER_NOT_FOUND then
      -- Code to handle the case where the header isn't available.
      ExtMessage = {}
    else
      -- Check for an insufficient buffer.
      if error_code = ERROR_INSUFFICIENT_BUFFER  then
       -- the buffer was smaller than needed
        if lpvBuffer != NULL then free(lpvBuffer) end if
        lpvBuffer = allocate(peek4u(lpdwBufferLength))

        result = exec_w32func(xHttpQueryInfo, {hRequest, dwInfoLevel, lpvBuffer,
                           lpdwBufferLength, lpdwIndex}, "HttpQueryInfoA")
        if result = TRUE then
          ExtMessage =  peek({lpvBuffer, peek4u(lpdwBufferLength)})
        else
          error_code = GetLastError()
          error_msg  = FormatSystemError(error_code)
          log_printf("Error %d: %s\n", {error_code, error_msg})
        end if
      end if
    end if
  else
    ExtMessage =  peek({lpvBuffer, peek4u(lpdwBufferLength)})
  end if

  free(lpdwIndex)
  if lpvBuffer != NULL then free(lpvBuffer) end if
  free(lpdwBufferLength)

  return ExtMessage
end function

--------------------------------------------------------------------------------
integer xHttpSendRequest
xHttpSendRequest = 0

global function HttpSendRequest(atom hRequest, object Headers, object Optional)
--<function>
--<name>HttpSendRequest</name>
--<digest>Sends the specified request to the HTTP server.</digest>
--<desc>
-- Remarks
--   HttpSendRequest sends the specified request to the HTTP server and allows the client to specify
--   additional headers to send along with the request.
--
--   The function also lets the client specify optional data to send to the HTTP server immediately
--   following the request headers. This feature is generally used for "write" operations such as PUT and
--   POST.
--
--   After the request is sent, the status code and response headers from the HTTP server are read. These
--   headers are maintained internally and are available to client applications through the
--   HttpQueryInfo function.
--
--   An application can use the same HTTP request handle in multiple calls to HttpSendRequest, but
--   the application must read all data returned from the previous call before calling the function again.
--
--   In offline mode, HttpSendRequest will return ERROR_FILE_NOT_FOUND if the resource is
--   not found in the Internet cache.
--</desc>
--<param>
--<type>atom</type>
--<name>hRequest</name>
--<desc>HINTERNET handle returned by HttpOpenRequest.</desc>
--</param>
--<param>
--<type>object</type>
--<name>Headers</name>
--<desc>
--     String variable that contains the additional headers to be appended to the request.
--     This parameter can be NULL if there are no additional headers to append.
--</desc>
--</param>
--<param>
--<type>object</type>
--<name>Optional</name>
--<desc>
--     Optional data to send immediately after the request headers.
--     This parameter is generally used for POST and PUT operations. The optional data can be the
--     resource or information being posted to the server. This parameter can be NULL if there is no
--     optional data to send.
--</desc>
--</param>
--<return>
--   Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
--   GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszHeaders, lpOptional
  integer result, dwHeadersLength, dwOptionalLength

  if sequence(Headers) then
    lpszHeaders = allocate_string(Headers)
    dwHeadersLength = length(Headers)
  elsif Headers = NULL then
    lpszHeaders = NULL
    dwHeadersLength = 0
  else
    error_msg = "HttpSendRequest: Headers should be a sequence or NULL"
    return FALSE
  end if
  if sequence(Optional) then
    lpOptional = allocate_string(Optional)
    dwOptionalLength = length(Optional)
  elsif Optional = NULL then
    lpOptional = NULL
    dwOptionalLength = 0
  else
    error_msg = "HttpSendRequest: Optional should be a sequence or NULL"
    return FALSE
  end if

  result = exec_w32func(xHttpSendRequest, {hRequest, lpszHeaders, dwHeadersLength,
                                     lpOptional, dwOptionalLength}, "HttpSendRequestA")

  if lpszHeaders != NULL then free(lpszHeaders) end if
  if lpOptional != NULL then free(lpOptional) end if
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

--------------------------------------------------------------------------------
integer xHttpSendRequestEx
xHttpSendRequestEx = 0

global function HttpSendRequestEx(atom hRequest, atom lpBuffersIn,
                  atom lpBuffersOut, integer dwFlags, integer dwContext)
--<function>
--<name>HttpSendRequestEx</name>
--<digest>Sends the specified request to the HTTP server and allows chunked transfers.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>hRequest</name>
--<desc>HINTERNET handle returned by HttpOpenRequest.</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lpBuffersIn</name>
--<desc>Optional. Address of an INTERNET_BUFFERS structure.</desc>
--</param>
--<param>
--<type>atom</type>
--<name>lpBuffersOut</name>
--<desc>Optional. Address of an INTERNET_BUFFERS structure.</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwFlags</name>
--<desc>
--     One of the following values:
--     HSR_ASYNC         Identical to WININET_API_FLAG_ASYNC.
--     HSR_SYNC          Identical to WININET_API_FLAG_SYNC.
--     HSR_USE_CONTEXT   Identical to WININET_API_FLAG_USE_CONTEXT.
--     HSR_INITIATE      Iterative operation (completed by HttpEndRequest).
--     HSR_DOWNLOAD      Download resource to file.
--     HSR_CHUNKED       Send chunked data.
--</desc>
--</param>
--<param>
--<type>integer</type>
--<name>dwContext</name>
--<desc>
--     Unsigned long integer variable that contains the application-defined context value, if a status
--     callback function has been registered.
--</desc>
--</param>
--<return>
--   Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
--   GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  integer result

  result = exec_w32func(xHttpSendRequestEx, {hRequest, lpBuffersIn, lpBuffersOut,
                     dwFlags, dwContext}, "HttpSendRequestExA")
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function

-- =============================================================================
--                                Cookie Functions
-- =============================================================================
--
-- Cookies are the means by which, under HTTP protocol, a server or a script can maintain state
-- information on the client workstation. The Microsoft(r) Win32(r) Internet functions maintain a
-- persistent cookie database for this purpose. Cookie functions are provided for users of Win32 Internet
-- functions to set cookies into, and access them from, the cookie database. For more information about
-- cookies, see Cookie Functions in the Win32 Internet Functions Overview.

--------------------------------------------------------------------------------

integer xInternetGetCookie
xInternetGetCookie = 0

global function InternetGetCookie(sequence UrlName, sequence CookieName)
--<function>
--<name>InternetGetCookie</name>
--<digest></digest>
--<desc>
-- Remarks
--   InternetGetCookie does not require a call to InternetOpen.
--   InternetGetCookie checks in the windows\cookies directory for persistent
--   cookies that have an expiration date set sometime in the future.
--   InternetGetCookie also searches memory for any session cookies (cookies
--   that do not have an expiration date) that were created in the same process
--   by InternetSetCookie, since these cookies are not written to any files.
--   Rules for creating cookie files are internal to Win32(r) Internet functions
--   and might change in the future.
--</desc>
--<param>
--<type>sequence</type>
--<name>UrlName</name>
--<desc>URL to get cookies for.</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>CookieName</name>
--<desc>Name of the cookie to get for the specified URL. This has not been implemented in this release.</desc>
--</param>
--<return>
--   Returns TRUE if successful, or FALSE otherwise. To get a specific error
--   value, call GetLastError. The following error values apply to
--   InternetGetCookie:
--   ERROR_NO_MORE_ITEMS         There is no cookie for the specified URL and
--                               all its parents.
--   ERROR_INSUFFICIENT_BUFFER   The value passed in lpdwSize is insufficient
--                               to copy all the cookie data. The value returned
--                               in lpdwSize is the size of the buffer
--                               necessary to get all the data.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
-- Retrieves the cookie for the specified URL.
  atom lpszUrlName, lpszCookieName, lpszCookieData, lpdwSize
  integer result
  sequence ExtMessage

  if sequence(UrlName) then
    lpszUrlName = allocate_string(UrlName)
  else
    error_msg = "InternetGetCookie: UrlName should be a sequence"
    return FALSE
  end if
  if sequence(CookieName) then
    lpszCookieName = allocate_string(CookieName)
  elsif CookieName = NULL then
    lpszCookieName = NULL
  else
    error_msg = "InternetGetCookie: CookieName should be a sequence or NULL"
    return FALSE
  end if

  ExtMessage = {}

  lpszCookieData = allocate(128 + 1) -- 128 Chars initially
  lpdwSize = allocate(4)
  poke4(lpdwSize, 128) -- set size of message

  result = exec_w32func(xInternetGetCookie, {lpszUrlName, lpszCookieName,
                     lpszCookieData, lpdwSize}, "InternetGetCookieA")

  if result = FALSE then
    if GetLastError() = ERROR_INSUFFICIENT_BUFFER  then
     -- the buffer was smaller than needed
      free(lpszCookieData)
      lpszCookieData = allocate(peek4u(lpdwSize))

      result = exec_w32func(xInternetGetCookie, {lpszUrlName, lpszCookieName,
                         lpszCookieData, lpdwSize}, "InternetGetCookieA")
    end if
  end if
  if result = TRUE then
    ExtMessage =  peek({lpszCookieData, peek4u(lpdwSize)})
  else
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if

  free(lpszUrlName)
  if lpszCookieName != NULL then free(lpszCookieName) end if
  free(lpszCookieData)
  free(lpdwSize)

  if result = TRUE then
    return ExtMessage
  else
    return 0
  end if
end function

--------------------------------------------------------------------------------

integer xInternetSetCookie
xInternetSetCookie = 0

global function InternetSetCookie(sequence UrlName, sequence CookieName,
                                  sequence CookieData)
--<function>
--<name>InternetSetCookie</name>
--<digest>Creates a cookie associated with the specified URL.</digest>
--<desc>
-- Remarks
--   Cookies created by InternetSetCookie without an expiration date are stored
--   in memory and are available only in the same process that created them.
--   Cookies that include an expiration date are stored in the windows\cookies
--   directory.
--
--   Creating a new cookie might cause a dialog box to appear on the screen if
--   the appropriate registry value, AllowCookies, is set. There is no way to
--   change the registry value from a Win32(r) Internet function.
--</desc>
--<param>
--<type>sequence</type>
--<name>UrlName</name>
--<desc>URL for which the cookie should be set.</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>CookieName</name>
--<desc>Name to associate with the cookie data. If this parameter is NULL, no name is associated with the cookie.</desc>
--</param>
--<param>
--<type>sequence</type>
--<name>CookieData</name>
--<desc>Actual data to associate with the URL.</desc>
--</param>
--<return>
--   Returns TRUE if successful, or FALSE otherwise. To get a specific error
--   message, call GetLastError.
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom lpszUrlName, lpszCookieName, lpszCookieData
  integer result

  if sequence(UrlName) then
    lpszUrlName = allocate_string(UrlName)
  else
    error_msg = "InternetSetCookie: UrlName should be a sequence"
    return FALSE
  end if
  if sequence(CookieName) then
    lpszCookieName = allocate_string(CookieName)
  elsif CookieName = NULL then
    lpszCookieName = NULL
  else
    error_msg = "InternetSetCookie: CookieName should be a sequence or NULL"
    return FALSE
  end if
  if sequence(CookieData) then
    lpszCookieData = allocate_string(CookieData)
  else
    error_msg = "InternetSetCookie: CookieData should be a sequence"
    return FALSE
  end if

  result = exec_w32func(xInternetSetCookie, {lpszUrlName, lpszCookieName,
                                       lpszCookieData}, "InternetSetCookieA")

  free(lpszUrlName)
  if lpszCookieName != NULL then free(lpszCookieName) end if
  free(lpszCookieData)
  if result = FALSE then
    error_code = GetLastError()
    error_msg  = FormatSystemError(error_code)
    log_printf("Error %d: %s\n", {error_code, error_msg})
  end if
  return result
end function


-- =============================================================================
--                      Persistent URL Cache Functions
-- =============================================================================

-- This section describes the functions used by applications that need persistent caching services. These
-- functions allow an application to save data in the local file system for subsequent use, such as in
-- situations where access to the data is over a low bandwidth link, or the access is not available at all.
-- The calling program that inserts data into the persistent cache assigns a source name that is used to do
-- operations such as retrieve the data, set and get some properties on the data, and delete data.
-- The protocols implemented in Microsoft(r) Win32(r) Internet functions use the cache functions to
-- provide persistent caching and offline browsing. Unless explicitly specified not to cache through the
-- INTERNET_FLAG_NO_CACHE_WRITE flag, Win32 Internet functions cache all data downloaded
-- from the network. The responses to POST data are not cached.

--------------------------------------------------------------------------------
-- CommitUrlCacheEntry
-- Stores data in the specified file in the Internet cache and associates it with the given URL.
--
-- Syntax
-- BOOL CommitUrlCacheEntry(
-- IN LPCTSTR lpszUrlName,
-- IN LPCTSTR lpszLocalFileName,
-- IN FILETIME ExpireTime,
-- IN FILETIME LastModifiedTime,
-- IN DWORD CacheEntryType,
-- IN LPHACK lpHeaderInfo,
-- IN DWORD dwHeaderSize,
-- IN LPCTSTR lpszFileExtension,
-- IN LPCTSTR lpszOriginalUrl
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrlName
-- Address of a string variable that contains the source name of the cache entry. The name string
-- must be unique and should not contain any escape characters.
--
-- lpszLocalFileName
-- Address of a string variable that contains the name of the local file that is being cached. This
-- should be the same name as that returned by CreateUrlCacheEntry.
--
-- ExpireTime
-- FILETIME structure that contains the expire date and time (in Greenwich mean time) of the file
-- that is being cached. If the expire date and time is unknown, set this parameter to zero.
--
-- LastModifiedTime
-- FILETIME structure that contains the last modified date and time (in Greenwich mean time) of
-- the URL that is being cached. If the last modified date and time is unknown, set this parameter to
-- zero.
--
-- CacheEntryType
-- Unsigned long integer value that contains the cache type bitmask. This can be a combination of
-- the following values:
-- COOKIE_CACHE_ENTRY
-- EDITED_CACHE_ENTRY
-- NORMAL_CACHE_ENTRY
-- SPARCE_CACHE_ENTRY
-- STICKY_CACHE_ENTRY
-- TRACK_CACHE_ENTRY
-- TRACK_OFFLINE_CACHE_ENTRY
-- TRACK_ONLINE_CACHE_ENTRY
-- URLHISTORY_CACHE_ENTRY
-- The STICKY_CACHE_ENTRY type is used to make cache entries exempt from scavenging.
-- The default exempt time for entries set using CommitUrlCacheEntry is one day. The exempt
-- time can be changed using the SetUrlCacheEntryInfo function.
-- lpHeaderInfo
-- Address of the buffer containing the header information. If this parameter is not NULL, the
-- header information is treated as extended attributes of the URL that are returned in the
-- INTERNET_CACHE_ENTRY_INFO structure.
--
-- dwHeaderSize
-- Unsigned long integer value that contains the size of the header information in TCHAR. If
-- lpHeaderInfo is not NULL, this value is assumed to indicate the size of the buffer that will store
-- the header information. An application can maintain headers as part of the data and provide
-- dwHeaderSize together with a NULL value for lpHeaderInfo.
--
-- lpszFileExtension
-- Reserved. Must be set to NULL.
--
-- lpszOriginalUrl
-- Address of a string variable that contains the original URL if redirection has occurred.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError Possible error values include:
-- ERROR_DISK_FULL    The cache storage is full.
--
-- ERROR_FILE_NOT_FOUND    The specified local file is not found.
--
-- Remarks
-- If the cache storage is full, CommitUrlCacheEntry invokes cache cleanup to make space for this
-- new file. If the cache entry already exists, the function overwrites the entry if it is not in use. An
-- entry is in use when it has been retrieved with either RetrieveUrlCacheEntryStream or
-- RetrieveUrlCacheEntryFile.
--
-- Clients that add entries to the cache should set the headers to at least "HTTP/1.0 200 OK\r\n\r\n";
-- otherwise, Microsoft Internet Explorer and other client applications that are utilizing the Win32
-- Internet functions will disregard the entry.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- CreateUrlCacheEntry
-- Creates a local file name for saving the cache entry based on the specified URL and the file extension.
--
-- Syntax
-- BOOL CreateUrlCacheEntry(
-- IN LPCTSTR lpszUrlName,
-- IN DWORD dwExpectedFileSize,
-- IN LPCTSTR lpszFileExtension,
-- OUT LPTSTR lpszFileName,
-- IN DWORD dwReserved
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrlName
-- Address of a string value that contains the name of the URL. The string should not contain any
-- escape characters.
--
-- dwExpectedFileSize
-- Unsigned long integer value that contains the expected size of the file needed to store the data
-- corresponding to the source entity in TCHAR. If the expected size is unknown, set this value to
-- zero.
--
-- lpszFileExtension
-- Address of a string value that contains an extension name of the file in the local storage.
-- lpszFileName
-- Address of a buffer that receives the file name. The buffer should be large enough
-- (MAX_PATH) to store the path of the created file.
--
-- dwReserved
-- Reserved. Must be set to zero.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError.
--
-- Remarks
-- After CreateUrlCacheEntry is called, the application can write directly into the file in local
-- storage. When the file is completely received, the caller should call CommitUrlCacheEntry to
-- commit the entry in the cache.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- CreateUrlCacheGroup
-- Generates cache group identifications.
--
-- Syntax
-- GROUPID CreateUrlCacheGroup(
-- IN DWORD dwFlags,
-- IN LPVOID lpReserved
-- );
-- Parameters
-- dwFlags
-- Unsigned long integer value that contains the flags to control the creation of the cache group.
-- This can be set to CACHEGROUP_FLAG_GIDONLY, which causes CreateUrlCacheGroup
-- to generate a unique GROUPID, but does not create a physical group.
--
-- lpReserved
-- Reserved. Must be set to NULL.
--
-- Return Value
-- Returns a valid GROUPID if successful, or FALSE otherwise. To get specific error information,
-- call GetLastError.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- DeleteUrlCacheEntry
-- Removes the file associated with the source name from the cache, if the file exists.
--
-- Syntax
-- BOOL DeleteUrlCacheEntry(
-- IN LPCTSTR lpszUrlName
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrlName
-- Address of a string that contains the name of the source corresponding to the cache entry.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError. Possible error values include:
-- ERROR_ACCESS_DENIED    The file is locked or in use. The entry will be marked and
-- will be deleted when the file is unlocked.
--
-- ERROR_FILE_NOT_FOUND    The file is not in the cache.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- DeleteUrlCacheGroup
-- Releases the specified GROUPID and any associated state in the cache index file.
--
-- Syntax
-- BOOL DeleteUrlCacheGroup(
-- IN GROUPID GroupId,
-- IN DWORD dwFlags,
-- IN LPVOID lpReserved
-- );
-- Parameters
-- GroupId
-- GROUPID value that is associated with the cache group to be released.
--
-- dwFlags
-- Unsigned long integer value containing the flags to control the cache group deletion. This can be
-- set to CACHEGROUP_FLAG_FLUSHURL_ONDELETE, which causes
-- DeleteUrlCacheGroup to delete all of the cache entries associated with this group, unless the
-- entry belongs to another group.
--
-- lpReserved
-- Reserved. Must be set to NULL.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get specific error information, call
-- GetLastError.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- FindCloseUrlCache
-- Closes the specified cache enumeration handle.
--
-- Syntax
-- BOOL FindCloseUrlCache(
-- IN HANDLE hEnumHandle
-- );
-- Parameters
-- hEnumHandle
-- Handle returned by a previous call to the FindFirstUrlCacheEntry function.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- FindFirstUrlCacheEntry
-- Begins the enumeration of the Internet cache.
--
-- Syntax
-- HANDLE FindFirstUrlCacheEntry(
-- IN LPCTSTR lpszUrlSearchPattern,
-- OUT LPINTERNET_CACHE_ENTRY_INFO lpFirstCacheEntryInfo,
-- IN OUT LPDWORD lpdwFirstCacheEntryInfoBufferSize
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrlSearchPattern
-- Address of a string that contains the source name pattern to search for. This can be set to
-- "cookie:" or "visited:" to enumerate the cookies and URL History entries in the cache. If this
-- parameter is NULL, the function uses *.*.
--
-- lpFirstCacheEntryInfo
-- Address of an INTERNET_CACHE_ENTRY_INFO structure.
--
-- lpdwFirstCacheEntryInfoBufferSize
-- Address of an unsigned long integer variable that specifies the size of the lpFirstCacheEntryInfo
-- buffer, in TCHARs. When the function returns, the variable contains the number of TCHARs
-- copied to the buffer, or the required size, in bytes, needed to retrieve the cache entry.
--
-- Return Value
-- Returns a handle that the application can use in the FindNextUrlCacheEntry function to retrieve
-- subsequent entries in the cache. If the function fails, the return value is NULL. To get extended
-- error information, call GetLastError.
--
-- ERROR_INSUFFICIENT_BUFFER indicates that the size of lpFirstCacheEntryInfo as specified
-- by lpdwFirstCacheEntryInfoBufferSize is not sufficient to contain all the information. The value
-- returned in lpdwFirstCacheEntryInfoBufferSize indicates the buffer size necessary to contain all the
-- information.
--
-- Remarks
-- FindFirstUrlCacheEntry and FindNextUrlCacheEntry return variable size information. If
-- ERROR_INSUFFICIENT_BUFFER is returned, the application should allocate a buffer of the size
-- specified by lpdwFirstCacheEntryInfoBufferSize. For more information, see Appendix B: Using
-- Buffers in the Win32 Internet Functions Overview.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- FindFirstUrlCacheEntryEx
-- Starts a filtered enumeration of the Internet cache.
--
-- Syntax
-- HANDLE FindFirstUrlCacheEntryEx(
-- IN LPCSTR lpszUrlSearchPattern,
-- IN DWORD dwFlags,
-- IN DWORD dwFilter,
-- IN GROUPID GroupId
-- OUT LPINTERNET_CACHE_ENTRY_INFO  lpFirstCacheEntryInfo,
-- IN OUT LPDWORD lpdwFirstCacheEntryInfoBufferSize,
-- OUT LPVOID lpReserved,
-- IN OUT LPDWORD pcbReserved2,
-- IN LPVOID lpReserved3
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrlSearchPattern
-- String that contains the search pattern. Search patterns are currently not supported, so the value
-- must be set to NULL to indicate all entries with the matching GROUPID.
--
-- dwFlags
-- Unsigned long integer value that contains the flags controlling the enumeration. No flags are
-- currently implemented; this must be set to zero.
--
-- dwFilter
-- Unsigned long integer value that indicates the cache entry types that are allowed. This can be any
-- combination of cache entry types:
-- COOKIE_CACHE_ENTRY    Cookie cache entry.
--
-- NORMAL_CACHE_ENTRY    Normal cache entry; can be deleted to recover
-- space for new entries.
--
-- STICKY_CACHE_ENTRY    Sticky cache entry; exempt from scavenging.
--
-- TRACK_OFFLINE_CACHE_ENTRY    Not currently implemented.
--
-- TRACK_ONLINE_CACHE_ENTRY    Not currently implemented.
--
-- URLHISTORY_CACHE_ENTRY    Visited link cache entry.
--
-- GroupId
-- GROUPID value that indicates the cache group to enumerate. Set the value to zero to enumerate
-- all entries that are not grouped.
--
-- lpFirstCacheEntryInfo
-- Address of the buffer to hold the INTERNET_CACHE_ENTRY_INFO structure in which the
-- cache entry information will be stored.
--
-- lpdwFirstCacheEntryInfoBufferSize
-- Address of an unsigned long integer variable that indicates the size of lpFirstCacheEntryInfo, in
-- TCHARs.
--
-- lpReserved
-- Reserved. Must be set to NULL.
--
-- pcbReserved2
-- Reserved. Must be set to NULL.
--
-- lpReserved3
-- Reserved. Must be set to NULL.
--
-- Return Value
-- Returns a valid handle if successful, or NULL otherwise. To get specific error information, call
-- GetLastError. If the function finds no matching files, GetLastError returns
-- ERROR_NO_MORE_FILES.
--
-- Remarks
-- At the end of the enumeration, the application should call FindCloseUrlCache.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- FindFirstUrlCacheGroup
-- Initiates the enumeration of the cache groups in the Internet cache.
--
-- Syntax
-- HANDLE FindFirstUrlCacheGroup(
-- IN DWORD dwFlags,
-- IN DWORD dwFilter,
-- IN LPVOID lpSearchCondition,
-- IN DWORD dwSearchCondition,
-- OUT GROUPID *lpGroupId,
-- IN OUT LPVOID lpReserved
-- );
-- Parameters
-- dwFlags
-- Reserved. Must be set to zero.
--
-- dwFilter
-- Unsigned long integer value that indicates what filters to use. This can be one of the following
-- values:
-- CACHEGROUP_SEARCH_ALL    Search all of the cache groups.
--
-- CACHEGROUP_SEARCH_BYURL    Not currently implemented.
--
-- lpSearchCondition
-- Reserved. Must be set to NULL.
--
-- dwSearchCondition
-- Reserved. Must be set to zero.
--
-- lpGroupId
-- Address of a GROUPID variable that contains the identification of the first cache group that
-- matches the search criteria.
--
-- lpReserved
-- Reserved. Must be set to NULL.
--
-- Return Value
-- Returns a valid handle if successful, or NULL otherwise. To get specific error information, call
-- GetLastError. If the function finds no matching files, GetLastError returns
-- ERROR_NO_MORE_FILES.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- FindNextUrlCacheEntry
-- Retrieves the next entry in the Internet cache.
--
-- Syntax
-- BOOL FindNextUrlCacheEntry(
-- IN HANDLE hEnumHandle,
-- OUT LPINTERNET_CACHE_ENTRY_INFO lpNextCacheEntryInfo,
-- IN OUT LPWORD lpdwNextCacheEntryInfoBufferSize
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- hEnumHandle
-- Enumeration handle obtained from a previous call to FindFirstUrlCacheEntry.
--
-- lpNextCacheEntryInfo
-- Address of an INTERNET_CACHE_ENTRY_INFO structure that receives information about
-- the cache entry.
--
-- lpdwNextCacheEntryInfoBufferSize
-- Address of an unsigned long integer variable that specifies the size of the lpNextCacheEntryInfo
-- buffer, in TCHARs. When the function returns, the variable contains the number of TCHARs
-- copied to the buffer, or the size of the buffer (in bytes) required to retrieve the cache entry.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError. Possible error values include:
-- ERROR_INSUFFICIENT_BUFFER    The size of lpNextCacheEntryInfo as specified by
-- lpdwNextCacheEntryInfoBufferSize is not sufficient to
-- contain all the information. The value returned in
-- lpdwNextCacheEntryInfoBufferSize indicates the
-- buffer size necessary to contain all the information.
--
-- ERROR_NO_MORE_ITEMS    The enumeration completed.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- FindNextUrlCacheEntryEx
-- Finds the next cache entry in a cache enumeration started by the FindFirstUrlCacheEntryEx
-- function.
--
-- Syntax
-- BOOL FindNextUrlCacheEntryEx(
-- IN HANDLE hEnumHandle,
-- OUT LPINTERNET_CACHE_ENTRY_INFO lpFirstCacheEntryInfo,
-- IN OUT LPDWORD lpdwFirstCacheEntryInfoBufferSize,
-- OUT LPVOID lpReserved,
-- IN OUT LPDWORD pcbReserved2,
-- IN LPVOID lpReserved3
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- hEnumHandle
-- Handle returned by FindFirstUrlCacheEntryEx, which started a cache enumeration.
-- lpFirstCacheEntryInfo
-- Address of the buffer to hold the INTERNET_CACHE_ENTRY_INFO structure in which the
-- cache entry information will be stored.
--
-- lpdwFirstCacheEntryInfoBufferSize
-- Address of an unsigned long integer value that indicates the size of the buffer in TCHAR.
--
-- lpReserved
-- Reserved. Must be set to NULL.
--
-- pcbReserved2
-- Reserved. Must be set to NULL.
--
-- lpReserved3
-- Reserved. Must be set to NULL.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get specific error information, call
-- GetLastError.
-- Remarks
-- At the end of the enumeration, the application should call FindCloseUrlCache.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- FindNextUrlCacheGroup
-- Retrieves the next cache group in a cache group enumeration started by FindFirstUrlCacheGroup.
--
-- Syntax
-- BOOL FindNextUrlCacheGroup(
-- IN HANDLE hFind,
-- OUT GROUPID *lpGroupId,
-- IN OUT LPVOID lpReserved
-- );
-- Parameters
-- hFind
-- Valid cache group enumeration handle returned by FindFirstUrlCacheGroup.
--
-- lpGroupId
-- Address of a GROUPID variable that contains the cache group identification.
--
-- lpReserved
-- Reserved. Must be set to NULL.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get specific error information, call
-- GetLastError.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- GetUrlCacheEntryInfo
-- Retrieves information about a cache entry.
--
-- Syntax
-- BOOL GetUrlCacheEntryInfo(
-- IN LPCTSTR lpszUrlName,
-- IN LPINTERNET_CACHE_ENTRY_INFO lpCacheEntryInfo,
-- IN OUT LPDWORD lpdwCacheEntryInfoBufferSize
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrlName
-- Address of a string that contains the name of the cache entry. The name string should not contain
-- any escape characters.
--
-- lpCacheEntryInfo
-- Address of an INTERNET_CACHE_ENTRY_INFO structure that receives information about
-- the cache entry.
--
-- lpdwCacheEntryInfoBufferSize
-- Address of an unsigned long integer variable that specifies the size of the lpCacheEntryInfo
-- buffer, in TCHARs. When the function returns, the variable contains the number of TCHARs
-- copied to the buffer, or the required size of the buffer, in bytes.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError. Possible error values include:
-- ERROR_FILE_NOT_FOUND    The specified cache entry is not found in the cache.
--
-- ERROR_INSUFFICIENT_BUFFER    The size of lpCacheEntryInfo as specified by
-- lpdwCacheEntryInfoBufferSize is not sufficient to
-- contain all the information. The value returned in
-- lpdwCacheEntryInfoBufferSize indicates the buffer
-- size necessary to contain all the information.
--
-- Remarks
-- GetUrlCacheEntryInfo does not do any URL parsing, so a URL containing an anchor (#) will not
-- be found in the cache, even if the resource is cached. For example, if the URL
-- http://example.com/example.htm#sample is passed, the function returns
-- ERROR_FILE_NOT_FOUND even if http://example.com/example.htm is in the cache.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- GetUrlCacheEntryInfoEx
-- Searches for the URL after translating any cached redirections that would be applied in offline mode
-- by HttpSendRequest.
--
-- Syntax
-- BOOL GetUrlCacheEntryInfoEx(
-- IN LPCTSTR lpszUrl,
-- OUT LPINTERNET_CACHE_ENTRY_INFO lpCacheEntryInfo,
-- IN OUT LPDWORD lpdwCacheEntryInfoBufSize,
-- OUT LPTSTR lpszReserved,
-- IN OUT LPDWORD lpdwReserved,
-- LPVOID lpReserved,
-- DWORD dwFlags
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrl
-- Address of a string that contains the name of the cache entry. The name string should not contain
-- any escape characters.
--
-- lpCacheEntryInfo
-- Address of an INTERNET_CACHE_ENTRY_INFO structure that receives information about
-- the cache entry.
--
-- lpdwCacheEntryInfoBufSize
-- Address of an unsigned long integer variable that specifies the size of the lpCacheEntryInfo
-- buffer, in TCHARs. When the function returns, the variable contains the number of TCHARs
-- copied to the buffer, or the required size of the buffer in bytes.
--
-- lpszReserved
-- Reserved. Must be set to NULL.
--
-- lpdwReserved
-- Reserved. Must be set to NULL.
--
-- lpReserved
-- Reserved. Must be set to NULL.
--
-- dwFlags
-- Reserved. Must be set to zero.
--
-- Return Value
-- Returns TRUE if the URL was located, or FALSE otherwise. Call GetLastError for specific error
-- information. Possible errors include:
-- ERROR_FILE_NOT_FOUND    The URL, after taking any cached redirections into
-- account, was not found in the cache index.
--
-- ERROR_INSUFFICIENT_BUFFER    The buffer referenced by lpCacheEntryInfo was not
-- large enough to hold the requested information. The
-- size of the buffer needed will be returned to
-- lpdwCacheEntryInfoBufSize.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- GetUrlCacheGroupAttribute
-- Retrieves the attribute information of the specified cache group.
--
-- Syntax
-- BOOL GetUrlCacheGroupAttribute(
-- IN GROUPID gid,
-- IN DWORD dwFlags,
-- IN DWORD dwAttributes,
-- OUT LPINTERNET_CACHE_GROUP_INFO lpGroupInfo,
-- IN OUT LPDWORD lpdwGroupInfo,
-- IN OUT LPVOID lpReserved
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- gid
-- GROUPID of the cache group.
--
-- dwFlags
-- Reserved. Must be set to zero.
--
-- dwAttributes
-- Unsigned long integer value that contains the attributes to retrieve. This can be one of the
-- following values:
-- CACHEGROUP_ATTRIBUTE_BASIC
-- CACHEGROUP_ATTRIBUTE_FLAG
-- CACHEGROUP_ATTRIBUTE_GET_ALL
-- CACHEGROUP_ATTRIBUTE_GROUPNAME
-- CACHEGROUP_ATTRIBUTE_QUOTA
-- CACHEGROUP_ATTRIBUTE_STORAGE
-- CACHEGROUP_ATTRIBUTE_TYPE
-- lpGroupInfo
-- Address of buffer that contains an INTERNET_CACHE_GROUP_INFO structure to store the
-- requested information.
--
-- lpdwGroupInfo
-- Address of an unsigned long integer value that contains the size of the lpGroupInfo buffer.
--
-- lpReserved
-- Reserved. Must be set to NULL.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get specific error information, call
-- GetLastError.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- ReadUrlCacheEntryStream
-- Reads the cached data from a stream that has been opened using the RetrieveUrlCacheEntryStream
-- function.
--
-- Syntax
-- BOOL ReadUrlCacheEntryStream(
-- IN HANDLE hUrlCacheStream,
-- IN  DWORD dwLocation,
-- IN OUT LPVOID lpBuffer,
-- IN OUT LPDWORD lpdwLen,
-- IN DWORD dwReserved
-- );
-- Parameters
-- hUrlCacheStream
-- HINTERNET handle that was returned by the RetrieveUrlCacheEntryStream function.
--
-- dwLocation
-- Unsigned long integer value that contains the offset to read from.
--
-- lpBuffer
-- Address of a buffer that receives the data.
--
-- lpdwLen
-- Address of an unsigned long integer variable that specifies the length of the lpBuffer buffer, in
-- TCHARs. When the function returns, the variable contains the number of TCHARs copied to the
-- buffer, or the required size of the buffer, in bytes.
--
-- dwReserved
-- Reserved. Must be set to zero.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError.
--
-- Remarks
-- If the buffer size is not sufficient, GetLastError returns ERROR_INSUFFICIENT_BUFFER and
-- sets lpdwLen to the size necessary to contain all the information.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- RetrieveUrlCacheEntryStream
-- Provides the most efficient and implementation-independent way of accessing the cache data.
--
-- Syntax
-- HANDLE RetrieveUrlCacheEntryStream(
-- IN LPCTSTR lpszUrlName,
-- OUT LPINTERNET_CACHE_ENTRY_INFO lpCacheEntryInfo,
-- IN OUT LPDWORD lpdwCacheEntryInfoBufferSize,
-- IN BOOL fRandomRead,
-- IN DWORD dwReserved
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrlName
-- Address of a string that contains the source name of the cache entry. This must be a unique
-- name. The name string should not contain any escape characters.
--
-- lpCacheEntryInfo
-- Address of an INTERNET_CACHE_ENTRY_INFO structure that receives information about
-- the cache entry.
--
-- lpdwCacheEntryInfoBufferSize
-- Address of an unsigned long integer variable that specifies the size of the lpCacheEntryInfo
-- buffer, in TCHARs. When the function returns, the variable receives the number of TCHARs
-- copied to the buffer, or the required size of the buffer, in bytes.
--
-- fRandomRead
-- Boolean value that indicates whether the stream is open for random access. Set the flag to TRUE
-- to open the stream for random access.
--
-- dwReserved
-- Reserved. Must be set to zero.
--
-- Return Value
-- Returns a valid handle for use in the ReadUrlCacheEntryStream and
-- UnlockUrlCacheEntryStream functions if successful, or NULL otherwise. To get extended error
-- information, call GetLastError. Possible error values include:
-- ERROR_FILE_NOT_FOUND    The cache entry specified by the source name is not
-- found in the cache storage.
--
-- ERROR_INSUFFICIENT_BUFFER    The size of lpCacheEntryInfo as specified by
-- lpdwCacheEntryInfoBufferSize is not sufficient to
-- contain all the information. The value returned in
-- lpdwCacheEntryInfoBufferSize indicates the buffer
-- size necessary to contain all the information.
--
-- Remarks
-- RetrieveUrlCacheEntryStream does not do any URL parsing, so a URL containing an anchor (#)
-- will not be found in the cache, even if the resource is cached. For example, if the URL
-- http://example.com/example.htm#sample is passed, the function returns
-- ERROR_FILE_NOT_FOUND even if http://example.com/example.htm is in the cache.
--
-- Cache clients that do not need URL data in the form of a file should use this function to access the
-- data for a particular URL.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- RetrieveUrlCacheEntryFile
-- Locks the cache entry file associated with the specified URL.
--
-- Syntax
-- BOOL RetrieveUrlCacheEntryFile(
-- IN LPCTSTR lpszUrlName,
-- OUT LPINTERNET_CACHE_ENTRY_INFO lpCacheEntryInfo,
-- IN OUT LPDWORD lpdwCacheEntryInfoBufferSize,
-- IN DWORD dwReserved
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrlName
-- Address of a string that contains the URL of the resource associated with the cache entry. This
-- must be a unique name. The name string should not contain any escape characters.
--
-- lpCacheEntryInfo
-- Address of a cache entry information buffer. If the buffer is not sufficient, this function returns
-- ERROR_INSUFFICIENT_BUFFER and sets lpdwCacheEntryInfoBufferSize to the number of
-- bytes required.
--
-- lpdwCacheEntryInfoBufferSize
-- Address of an unsigned long integer variable that specifies the size of the lpCacheEntryInfo
-- buffer, in TCHARs. When the function returns, the variable contains the size, in TCHARs, of the
-- actual buffer used or the number of bytes required to retrieve the cache entry file. The caller
-- should check the return value in this parameter. If the return size is less than or equal to the size
-- passed in, all the relevant data has been returned.
--
-- dwReserved
-- Reserved. Must be set to zero.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError. Possible error values include:
-- ERROR_FILE_NOT_FOUND    The cache entry specified by the source name is not
-- found in the cache storage.
--
-- ERROR_INSUFFICIENT_BUFFER    The size of the lpCacheEntryInfo buffer as specified
-- by lpdwCacheEntryInfoBufferSize is not sufficient to
-- contain all the information. The value returned in
-- lpdwCacheEntryInfoBufferSize indicates the buffer
-- size necessary to get all the information.
--
-- Remarks
-- RetrieveUrlCacheEntryFile does not do any URL parsing, so a URL containing an anchor (#) will
-- not be found in the cache, even if the resource is cached. For example, if the URL
-- http://example.com/example.htm#sample was passed, the function would return
-- ERROR_FILE_NOT_FOUND even if http://example.com/example.htm is in the cache.
--
-- The file is locked for the caller when it is retrieved; the caller should unlock the file after it has been
-- used up. The cache manager automatically unlocks the files after a certain interval. While the file is
-- locked, the cache manager will not remove the file from the cache. It is important to note that this
-- function can be efficient or expensive, depending on the internal implementation of the cache. For
-- instance, if the URL data is stored in a packed file that contains data for other URLs, the cache will
-- make a copy of the data to a file in a temporary directory maintained by the cache. The cache will
-- eventually delete the copy. It is recommended that this function be used only in situations where a
-- file name is needed to launch an application. RetrieveUrlCacheEntryStream and associated
-- stream functions should be used in most cases.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- SetUrlCacheEntryGroup
-- Adds entries to or removes entries from a cache group.
--
-- Syntax
-- BOOL SetUrlCacheEntryGroup(
-- IN LPCTSTR lpszUrlName,
-- IN DWORD dwFlags,
-- IN GROUPID GroupId,
-- IN LPBYTE pbGroupAttributes,
-- IN DWORD cbGroupAttributes,
-- IN LPVOID lpReserved
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrlName
-- Address of a string value that contains the URL of the cached resource.
--
-- dwFlags
-- Unsigned long integer value that determines whether the entry is added to or removed from a
-- cache group. This can be one of the following values:
-- INTERNET_CACHE_GROUP_ADD    Adds the cache entry to the cache group.
--
-- INTERNET_CACHE_GROUP_REMOVE    Removes the entry from the cache group.
--
-- GroupId
-- GROUPID value that indicates the cache group that the entry will be added to or removed from.
--
-- pbGroupAttributes
-- Reserved. Must be set to NULL.
--
-- cbGroupAttributes
-- Reserved. Must be set to zero.
--
-- lpReserved
-- Reserved. Must be set to NULL.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise.
--
-- Remarks
-- A cache entry can belong to more than one cache group.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- SetUrlCacheEntryInfo
-- Sets the specified members of the INTERNET_CACHE_ENTRY_INFO structure.
--
-- Syntax
-- BOOL SetUrlCacheEntryInfo(
-- IN LPCTSTR lpszUrlName,
-- IN LPINTERNET_CACHE_ENTRY_INFO lpCacheEntryInfo,
-- IN DWORD dwFieldControl
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrlName
-- Address of a string that contains the name of the cache entry. The name string should not contain
-- any escape characters.
--
-- lpCacheEntryInfo
-- Address of an INTERNET_CACHE_ENTRY_INFO structure containing the values to be
-- assigned to the cache entry designated by lpszUrlName.
--
-- dwFieldControl
-- Unsigned long integer value that contains a bitmask that indicates the members that are to be set.
-- This can be a combination of the following values:
-- CACHE_ENTRY_ACCTIME_FC    Sets the last access time.
--
-- CACHE_ENTRY_ATTRIBUTE_FC    Sets the cache entry type.
--
-- CACHE_ENTRY_EXEMPT_DELTA_FC    Sets the exempt delta.
--
-- CACHE_ENTRY_EXPTIME_FC    Sets the expire time.
--
-- CACHE_ENTRY_HEADERINFO_FC    Not currently implemented.
--
-- CACHE_ENTRY_HITRATE_FC    Sets the hit rate.
--
-- CACHE_ENTRY_MODTIME_FC    Sets the last modified time.
--
-- CACHE_ENTRY_SYNCTIME_FC    Sets the last sync time.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError. Possible error values include:
-- ERROR_FILE_NOT_FOUND    The specified cache entry is not found in the cache.
--
-- ERROR_INVALID_PARAMETER    The value(s) to be set is invalid.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- SetUrlCacheGroupAttribute
-- Sets the attribute information of the specified cache group.
--
-- Syntax
-- BOOL SetUrlCacheGroupAttribute(
-- IN GROUPID gid,
-- IN DWORD dwFlags,
-- IN DWORD dwAttributes,
-- IN LPINTERNET_CACHE_GROUP_INFO lpGroupInfo,
-- IN OUT LPVOID lpReserved
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- gid
-- GROUPID of the cache group.
--
-- dwFlags
-- Reserved. Must be set to zero.
--
-- dwAttributes
-- Unsigned long integer value that indicates what attributes to set. This can be one of the following
-- values:
-- CACHEGROUP_ATTRIBUTE_FLAG
-- CACHEGROUP_ATTRIBUTE_GROUPNAME
-- CACHEGROUP_ATTRIBUTE_QUOTA
-- CACHEGROUP_ATTRIBUTE_STORAGE
-- CACHEGROUP_ATTRIBUTE_TYPE
-- CACHEGROUP_READWRITE_MASK
-- lpGroupInfo
-- Address of an INTERNET_CACHE_GROUP_INFO structure that contains the attribute
-- information to store.
--
-- lpReserved
-- Reserved. Must be set to NULL.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get specific error information, call
-- GetLastError.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- UnlockUrlCacheEntryFile
-- Unlocks the cache entry that was locked while the file was retrieved for use from the cache.
--
-- Syntax
-- BOOL UnlockUrlCacheEntryFile(
-- IN LPCTSTR lpszUrlName,
-- IN DWORD dwReserved
--   );
--
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Parameters
-- lpszUrlName
-- Address of a string that contains the source name of the cache entry that is being unlocked. The
-- name string should not contain any escape characters.
--
-- dwReserved
-- Reserved. Must be set to zero.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError. ERROR_FILE_NOT_FOUND indicates that the cache entry specified by the
-- source name is not found in the cache storage.
--
-- Remarks
-- The application should not access the file after calling this function.
--
-- When this function returns, the cache manager is free to delete the cache entry.
--global function ()
--  integer result

--  result = exec_w32func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- UnlockUrlCacheEntryStream
-- Closes the stream that has been retrieved using the RetrieveUrlCacheEntryStream function.
--
-- Syntax
-- BOOL UnlockUrlCacheEntryStream(
-- IN HANDLE hUrlCacheStream,
-- IN DWORD dwReserved
-- );
-- Parameters
-- hUrlCacheStream
-- Handle that was returned by the RetrieveUrlCacheEntryStream function.
--
-- dwReserved
-- Reserved. Must be set to zero.
--
-- Return Value
-- Returns TRUE if successful, or FALSE otherwise. To get extended error information, call
-- GetLastError.

