include _common_.e  -- TRUE, FALSE
include _machine_.e  -- or_all()

-- =============================================================================
--                   Win32 Internet API Enumerated Types
-- =============================================================================
--
-- The following list contains a link to the enumerated type used by the Win32(r) Internet functions.
--
-- INTERNET_SCHEME Enumerated Type
global constant
  INTERNET_SCHEME_PARTIAL    = -2,  -- Partial URL.
  INTERNET_SCHEME_UNKNOWN    = -1,  -- Unknown URL scheme.
  INTERNET_SCHEME_DEFAULT    =  0,  -- Default URL scheme.
  INTERNET_SCHEME_FTP        =  1,  -- FTP URL scheme (ftp:).
  INTERNET_SCHEME_GOPHER     =  2,  -- Gopher URL scheme (gopher:).
  INTERNET_SCHEME_HTTP       =  3,  -- HTTP URL scheme (http:).
  INTERNET_SCHEME_HTTPS      =  4,  -- HTTPS URL scheme (https:).
  INTERNET_SCHEME_FILE       =  5,  -- File URL scheme (file:).
  INTERNET_SCHEME_NEWS       =  6,  -- News URL scheme (news:).
  INTERNET_SCHEME_MAILTO     =  7,  -- Mail URL scheme (mailto:).
  INTERNET_SCHEME_SOCKS      =  8,  -- Socks URL scheme (socks:).
  INTERNET_SCHEME_JAVASCRIPT =  9,  -- JScript URL scheme (javascript:).
  INTERNET_SCHEME_VBSCRIPT   = 10,  -- VBScript URL scheme (vbscript:).
  INTERNET_SCHEME_FIRST      = INTERNET_SCHEME_FTP,      -- Lowest known scheme value.
  INTERNET_SCHEME_LAST       = INTERNET_SCHEME_VBSCRIPT  -- Highest known scheme value.
--<constant>
--<name>INTERNET_SCHEME_PARTIAL</name>
--<value>-2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_UNKNOWN</name>
--<value>-1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_DEFAULT</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_FTP</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_GOPHER</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_HTTP</name>
--<value>3</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_HTTPS</name>
--<value>4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_FILE</name>
--<value>5</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_NEWS</name>
--<value>6</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_MAILTO</name>
--<value>7</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_SOCKS</name>
--<value>8</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_JAVASCRIPT</name>
--<value>9</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_VBSCRIPT</name>
--<value>10</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_FIRST</name>
--<value>INTERNET_SCHEME_FTP</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SCHEME_LAST</name>
--<value>INTERNET_SCHEME_VBSCRIPT</value>
--<desc></desc>
--</constant>

global type INTERNET_SCHEME(integer n)
--<eu_type>
--<name>INTERNET_SCHEME</name>
--<desc></desc>
--<param>
--<type>integer</type>
--<name>n</name>
--<desc></desc>
--</param>
--</eu_type>
  return (n >= INTERNET_SCHEME_FIRST) and (n <= INTERNET_SCHEME_LAST)
end type


-- =============================================================================
--                         Win32 Internet API Constants
-- =============================================================================
--
-- The Win32(r) Internet functions use the following constants:
-- *    API Flags
-- *    HTTP Status Codes
-- *    Option Flags
-- *    Query Info Flags
-- *    Cache Group Constants
-- *    Gopher Type Values
-- *    Error Messages
-- API Flags
-- Many of the Microsoft(r) Win32(r) Internet functions accept an array of unsigned long integer flags as a
-- parameter. The following is a brief description of the defined flags.

global constant
  INTERNET_COOKIE_THIRD_PARTY  = #10,
  -- Indicates that a third-party cookie is being set or retrieved.
  INTERNET_COOKIE_EVALUATE_P3P = #80,
  -- Indicates that a Platform for Privacy Protection (P3P) header is to be associated with a cookie.

  INTERNET_FLAG_TRANSFER_ASCII           = #00000001,
  -- Transfers file as ASCII (FTP only).
  -- This flag can be used by FtpOpenFile, FtpGetFile, and FtpPutFile.
  INTERNET_FLAG_TRANSFER_BINARY          = #00000002,
  -- Transfers file as binary (FTP only).
  -- This flag can be used by FtpOpenFile, FtpGetFile, and FtpPutFile.
  INTERNET_FLAG_NEED_FILE                = #00000010,
  -- Causes a temporary file to be created if the file cannot be cached.
  -- This flag can be used by FtpFindFirstFile, FtpGetFile, FtpOpenFile, FtpPutFile, HttpOpenRequest,
  -- and InternetOpenUrl.
  -- Windows XP and Windows Server 2003 R2 and earlier:  Also used by GopherFindFirstFile and GopherOpenFile.
  INTERNET_FLAG_MUST_CACHE_REQUEST       = #00000010,
  -- Identical to the preferred value, INTERNET_FLAG_NEED_FILE.
  -- Causes a temporary file to be created if the file cannot be cached.
  -- This flag can be used by FtpFindFirstFile, FtpGetFile, FtpOpenFile, FtpPutFile, HttpOpenRequest,
  -- and InternetOpenUrl.
  -- Windows XP and Windows Server 2003 R2 and earlier:  Also used by GopherFindFirstFile and GopherOpenFile.
  INTERNET_FLAG_FWD_BACK                 = #00000020,
  -- Indicates that the function should use the copy of the resource that is currently in the Internet
  -- cache. The expiration date and other information about the resource is not checked. If the
  -- requested item is not found in the Internet cache, the system will attempt to locate the resource
  -- on the network. This value was introduced in Microsoft Internet Explorer 5 and is associated
  -- with the Forward and Back button operations of Internet Explorer.
  INTERNET_FLAG_FORMS_SUBMIT             = #00000040,
  -- Indicates that this is a Forms submission.
  INTERNET_FLAG_CACHE_ASYNC              = #00000080,
  -- Allows a lazy cache write.
  INTERNET_FLAG_PRAGMA_NOCACHE           = #00000100,
  -- Forces the request to be resolved by the origin server, even if a cached copy exists on the proxy.
  -- The InternetOpenUrl function (on HTTP and HTTPS requests only) and HttpOpenRequest function use
  -- this flag.
  INTERNET_FLAG_NO_UI                    = #00000200,
  -- Disables the cookie dialog box.
  -- This flag can be used by HttpOpenRequest and InternetOpenUrl (HTTP requests only).
  INTERNET_FLAG_HYPERLINK                = #00000400,
  -- Forces a reload if there was no Expires time and no LastModified time returned from the server
  -- when determining whether to reload the item from the network. This flag can be used by
  -- GopherFindFirstFile, GopherOpenFile, FtpFindFirstFile, FtpGetFile, FtpOpenFile,
  -- FtpPutFile, HttpOpenRequest, and InternetOpenUrl.
  INTERNET_FLAG_RESYNCHRONIZE            = #00000800,
  -- Reloads HTTP resources if the resource has been modified since the last time it was downloaded.
  -- All FTP resources are reloaded.
  -- This flag can be used by FtpFindFirstFile, FtpGetFile, FtpOpenFile, FtpPutFile, HttpOpenRequest, and InternetOpenUrl.,
  -- Windows XP and Windows Server 2003 R2 and earlier:  Also used by GopherFindFirstFile and GopherOpenFile, and Gopher resources are reloaded.
  INTERNET_FLAG_IGNORE_CERT_CN_INVALID   = #00001000,
  -- Disables Win32 Internet function checking of SSL/PCT-based certificates that are returned from
  -- the server against the host name given in the request. Win32 Internet functions use a simple
  -- check against certificates by comparing for matching host names and simple wildcarding rules.
  -- This flag can be used by HttpOpenRequest and InternetOpenUrl (for HTTP requests).
  INTERNET_FLAG_IGNORE_CERT_DATE_INVALID = #00002000,
  -- Disables Win32 Internet function checking of SSL/PCT-based certificates for proper validity
  -- dates. This flag can be used by HttpOpenRequest and InternetOpenUrl (for HTTP requests).
  INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS = #00004000,
  -- Disables the ability of the Win32 Internet functions to detect this special type of redirect. When
  -- this flag is used, Win32 Internet functions transparently allow redirects from HTTP to HTTPS
  -- URLs. This flag can be used by HttpOpenRequest and InternetOpenUrl (for HTTP requests).
  -- Disables the ability of the Win32 Internet functions to detect this special type of redirect. When
  -- this flag is used, Win32 Internet functions transparently allow redirects from HTTP to HTTPS
  -- URLs. This flag can be used by HttpOpenRequest and InternetOpenUrl (for HTTP requests).
  INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP  = #00008000,
  -- Disables the ability of the Win32 Internet functions to detect this special type of redirect. When
  -- this flag is used, Win32 Internet functions transparently allow redirects from HTTPS to HTTP
  -- URLs. This flag can be used by HttpOpenRequest and InternetOpenUrl (for HTTP requests).
  INTERNET_FLAG_CACHE_IF_NET_FAIL        = #00010000,
  -- Returns the resource from the cache if the network request for the resource fails due to an
  -- ERROR_INTERNET_CONNECTION_RESET or ERROR_INTERNET_CANNOT_CONNECT error.
  -- This flag is used by HttpOpenRequest.
  INTERNET_FLAG_RESTRICTED_ZONE          = #00020000,
  -- Indicates that the cookie being set is associated with an untrusted site.
  INTERNET_FLAG_NO_AUTH                  = #00040000,
  -- Does not attempt authentication automatically.
  -- This flag can be used by HttpOpenRequest and InternetOpenUrl (for HTTP requests).
  INTERNET_FLAG_NO_COOKIES               = #00080000,
  -- Does not automatically add cookie headers to requests, and does not automatically add returned cookies
  -- to the cookie database.
  -- This flag can be used by HttpOpenRequest and InternetOpenUrl (for HTTP requests).
  INTERNET_FLAG_READ_PREFETCH            = #00100000,
  -- This flag is currently disabled.
  INTERNET_FLAG_NO_AUTO_REDIRECT         = #00200000,
  -- Does not automatically handle redirection in HttpSendRequest.
  -- This flag can also be used by InternetOpenUrl for HTTP requests.
  INTERNET_FLAG_KEEP_CONNECTION          = #00400000,
  -- Uses keep-alive semantics, if available, for the connection.
  -- This flag is used by HttpOpenRequest and InternetOpenUrl (for HTTP requests).
  -- This flag is required for Microsoft Network (MSN), NTLM, and other types of authentication.
  INTERNET_FLAG_SECURE                   = #00800000,
  -- Uses secure transaction semantics.
  -- This translates to using Secure Sockets Layer/Private Communications Technology (SSL/PCT) and is only meaningful in HTTP requests.
  -- This flag is used by HttpOpenRequest and InternetOpenUrl, but this is redundant if https:// appears in the URL.The InternetConnect function uses this flag for HTTP connections; all the request handles created under this connection will inherit this flag.
  INTERNET_FLAG_FROM_CACHE               = #01000000,
  -- Does not make network requests. All entities are returned from the cache. If the requested item is
  -- not in the cache, a suitable error, such as ERROR_FILE_NOT_FOUND, is returned. Only the
  -- InternetOpen function uses this flag.
  INTERNET_FLAG_OFFLINE                  = #01000000,
  -- Identical to INTERNET_FLAG_FROM_CACHE.
  -- Does not make network requests.
  -- All entities are returned from the cache.
  -- If the requested item is not in the cache, a suitable error, such as ERROR_FILE_NOT_FOUND, is returned.
  -- Only the InternetOpen function uses this flag.
  INTERNET_FLAG_MAKE_PERSISTENT          = #02000000,
  -- No longer supported.
  INTERNET_FLAG_NO_CACHE_WRITE           = #04000000,
  -- Does not add the returned entity to the cache.
  -- This flag is used by , HttpOpenRequest, and InternetOpenUrl.,
  -- Windows XP and Windows Server 2003 R2 and earlier:  Also used by GopherFindFirstFile and GopherOpenFile.
  INTERNET_FLAG_DONT_CACHE               = #04000000,
  -- Does not add the returned entity to the cache.
  -- This is identical to the preferred value, INTERNET_FLAG_NO_CACHE_WRITE.
  INTERNET_FLAG_PASSIVE                  = #08000000,
  -- Uses passive FTP semantics.
  -- Only InternetConnect and InternetOpenUrl use this flag.
  -- InternetConnect uses this flag for FTP requests, and InternetOpenUrl uses this flag for FTP files and
  -- directories.
  INTERNET_FLAG_ASYNC                    = #10000000,
  -- Makes only asynchronous requests on handles descended from the handle returned from this
  -- function. Only the InternetOpen function uses this flag.
  INTERNET_FLAG_EXISTING_CONNECT         = #20000000,
  -- Attempts to use an existing InternetConnect object if one exists with the same attributes
  -- required to make the request. This is useful only with FTP operations, since FTP is the only
  -- protocol that typically performs multiple operations during the same session. The Win32 Internet
  -- API caches a single connection handle for each HINTERNET handle generated by
  -- InternetOpen. Only the InternetOpenUrl function uses this flag.
  INTERNET_FLAG_RAW_DATA                 = #40000000,
  -- Returns the data as a WIN32_FIND_DATA structure when retrieving FTP directory information.
  -- If this flag is not specified or if the call is made through a CERN proxy, InternetOpenUrl returns
  -- the HTML version of the directory. Only the InternetOpenUrl function uses this flag.,
  -- Windows XP and Windows Server 2003 R2 and earlier:  Also returns a GOPHER_FIND_DATA structure when
  -- retrieving Gopher directory information.
  INTERNET_FLAG_RELOAD                   = #80000000,
  -- Forces a download of the requested file, object, or directory listing from the origin server, not from the cache.
  -- The FtpFindFirstFile, FtpGetFile, FtpOpenFile, FtpPutFile, HttpOpenRequest, and InternetOpenUrl functions use this flag.,
  -- Windows XP and Windows Server 2003 R2 and earlier:  Also used by GopherFindFirstFile and GopherOpenFile.

  INTERNET_NO_CALLBACK         = #00000000,
  -- Indicates that no callbacks should be made for that API. This is used for the dxContext parameter
  -- of the Win32 Internet functions that allow asynchronous operations.
  WININET_API_FLAG_ASYNC       = #00000001,
  -- Forces asynchronous operations.
  WININET_API_FLAG_SYNC        = #00000004,
  -- Forces synchronous operations.
  WININET_API_FLAG_USE_CONTEXT = #00000008
--<constant>
--<name>INTERNET_COOKIE_THIRD_PARTY</name>
--<value>#10</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_COOKIE_EVALUATE_P3P</name>
--<value>#80</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_TRANSFER_ASCII</name>
--<value>#00000001</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_TRANSFER_BINARY</name>
--<value>#00000002</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_NEED_FILE</name>
--<value>#00000010</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_MUST_CACHE_REQUEST</name>
--<value>#00000010</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_FWD_BACK</name>
--<value>#00000020</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_FORMS_SUBMIT</name>
--<value>#00000040</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_CACHE_ASYNC</name>
--<value>#00000080</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_PRAGMA_NOCACHE</name>
--<value>#00000100</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_NO_UI</name>
--<value>#00000200</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_HYPERLINK</name>
--<value>#00000400</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_RESYNCHRONIZE</name>
--<value>#00000800</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_IGNORE_CERT_CN_INVALID</name>
--<value>#00001000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_IGNORE_CERT_DATE_INVALID</name>
--<value>#00002000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS</name>
--<value>#00004000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP</name>
--<value>#00008000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_CACHE_IF_NET_FAIL</name>
--<value>#00010000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_RESTRICTED_ZONE</name>
--<value>#00020000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_NO_AUTH</name>
--<value>#00040000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_NO_COOKIES</name>
--<value>#00080000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_READ_PREFETCH</name>
--<value>#00100000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_NO_AUTO_REDIRECT</name>
--<value>#00200000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_KEEP_CONNECTION</name>
--<value>#00400000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_SECURE</name>
--<value>#00800000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_FROM_CACHE</name>
--<value>#01000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_OFFLINE</name>
--<value>#01000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_MAKE_PERSISTENT</name>
--<value>#02000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_NO_CACHE_WRITE</name>
--<value>#04000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_DONT_CACHE</name>
--<value>#04000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_PASSIVE</name>
--<value>#08000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_ASYNC</name>
--<value>#10000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_EXISTING_CONNECT</name>
--<value>#20000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_RAW_DATA</name>
--<value>#40000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_FLAG_RELOAD</name>
--<value>#80000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_NO_CALLBACK</name>
--<value>#00000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>WININET_API_FLAG_ASYNC</name>
--<value>#00000001</value>
--<desc></desc>
--</constant>
--<constant>
--<name>WININET_API_FLAG_SYNC</name>
--<value>#00000004</value>
--<desc></desc>
--</constant>
--<constant>
--<name>WININET_API_FLAG_USE_CONTEXT</name>
--<value>#00000008</value>
--<desc></desc>
--</constant>
  -- Forces the API to use the context value, even if it is set to zero.


-- =============================================================================
--                                HTTP Status Codes
-- =============================================================================

-- The following table contains the constants and corresponding values for the HTTP status codes
-- returned by servers on the Internet.
--
global constant
  HTTP_STATUS_CONTINUE           = 100,  -- The request can be continued.
  HTTP_STATUS_SWITCH_PROTOCOLS   = 101,  -- The server has switched protocols in
                                         -- an upgrade header.
  HTTP_STATUS_OK                 = 200,  -- The request completed successfully.
  HTTP_STATUS_CREATED            = 201,  -- The request has been fulfilled and
                                         -- resulted in the creation of a new
                                         -- resource.
  HTTP_STATUS_ACCEPTED           = 202,  -- The request has been accepted for
                                         -- processing, but the processing has
                                         -- not been completed.
  HTTP_STATUS_PARTIAL            = 203,  -- The returned meta information in the
                                         -- entity-header is not the definitive
                                         -- set available from the origin server.
  HTTP_STATUS_NO_CONTENT         = 204,  -- The server has fulfilled the request,
                                         -- but there is no new information to
                                         -- send back.
  HTTP_STATUS_RESET_CONTENT      = 205,  -- The request has been completed, and
                                         -- the client program should reset the
                                         -- document view that caused the request
                                         -- to be sent to allow the user to
                                         -- easily initiate another input action.
  HTTP_STATUS_PARTIAL_CONTENT    = 206,  -- The server has fulfilled the partial
                                         -- GET request for the resource.
  HTTP_STATUS_AMBIGUOUS          = 300,  -- The server couldn't decide what to
                                         -- return.
  HTTP_STATUS_MOVED              = 301,  -- The requested resource has been
                                         -- assigned to a new permanent URI
                                         -- (Uniform Resource Identifier), and
                                         -- any future references to this
                                         -- resource should be done using one of
                                         -- the returned URIs.
  HTTP_STATUS_REDIRECT           = 302,  -- The requested resource resides
                                         -- temporarily under a different URI
                                         -- (Uniform Resource Identifier).
  HTTP_STATUS_REDIRECT_METHOD    = 303,  -- The response to the request can be
                                         -- found under a different URI (Uniform
                                         -- Resource Identifier) and should be
                                         -- retrieved using a GET method on that
                                         -- resource.
  HTTP_STATUS_NOT_MODIFIED       = 304,  -- The requested resource has not been
                                         -- modified.
  HTTP_STATUS_USE_PROXY          = 305,  -- The requested resource must be
                                         -- accessed through the proxy given by
                                         -- the location field.
  HTTP_STATUS_REDIRECT_KEEP_VERB = 307,  -- The redirected request keeps the same
                                         -- verb. HTTP/1.1 behavior.
  HTTP_STATUS_BAD_REQUEST        = 400,  -- The request could not be processed by
                                         -- the server due to invalid syntax.
  HTTP_STATUS_DENIED             = 401,  -- The requested resource requires user
                                         -- authentication.
  HTTP_STATUS_PAYMENT_REQ        = 402,  -- Not currently implemented in the HTTP
                                         -- protocol.
  HTTP_STATUS_FORBIDDEN          = 403,  -- The server understood the request,
                                         -- but is refusing to fulfill it.
  HTTP_STATUS_NOT_FOUND          = 404,  -- The server has not found anything
                                         -- matching the requested URI (Uniform
                                         -- Resource Identifier).
  HTTP_STATUS_BAD_METHOD         = 405,  -- The method used is not allowed.
  HTTP_STATUS_NONE_ACCEPTABLE    = 406,  -- No responses acceptable to the client
                                         -- were found.
  HTTP_STATUS_PROXY_AUTH_REQ     = 407,  -- Proxy authentication required.
  HTTP_STATUS_REQUEST_TIMEOUT    = 408,  -- The server timed out waiting for the
                                         -- request.
  HTTP_STATUS_CONFLICT           = 409,  -- The request could not be completed
                                         -- due to a conflict with the current
                                         -- state of the resource. The user
                                         -- should resubmit with more information.
  HTTP_STATUS_GONE               = 410,  -- The requested resource is no longer
                                         -- available at the server, and no
                                         -- forwarding address is known.
  HTTP_STATUS_LENGTH_REQUIRED    = 411,  -- The server refuses to accept the
                                         -- request without a defined content
                                         -- length.
  HTTP_STATUS_PRECOND_FAILED     = 412,  -- The precondition given in one or more
                                         -- of the request header fields evaluated
                                         -- to false when it was tested on the
                                         -- server.
  HTTP_STATUS_REQUEST_TOO_LARGE  = 413,  -- The server is refusing to process a
                                         -- request because the request entity is
                                         -- larger than the server is willing or
                                         -- able to process.
  HTTP_STATUS_URI_TOO_LONG       = 414,  -- The server is refusing to service the
                                         -- request because the request URI
                                         -- (Uniform Resource Identifier) is
                                         -- longer than the server is willing to
                                         -- interpret.
  HTTP_STATUS_UNSUPPORTED_MEDIA  = 415,  -- The server is refusing to service the
                                         -- request because the entity of the
                                         -- request is in a format not supported
                                         -- by the requested resource for the
                                         -- requested method.
  HTTP_STATUS_RETRY_WITH         = 449,  -- The request should be retried after
                                         -- doing the appropriate action.
  HTTP_STATUS_SERVER_ERROR       = 500,  -- The server encountered an unexpected
                                         -- condition that prevented it from
                                         -- fulfilling the request.
  HTTP_STATUS_NOT_SUPPORTED      = 501,  -- The server does not support the
                                         -- functionality required to fulfill
                                         -- the request.
  HTTP_STATUS_BAD_GATEWAY        = 502,  -- The server, while acting as a gateway
                                         -- or proxy, received an invalid response
                                         -- from the upstream server it accessed
                                         -- in attempting to fulfill the request.
  HTTP_STATUS_SERVICE_UNAVAIL    = 503,  -- The service is temporarily overloaded.
  HTTP_STATUS_GATEWAY_TIMEOUT    = 504,  -- The request was timed out waiting for
                                         -- a gateway.
  HTTP_STATUS_VERSION_NOT_SUP    = 505   -- The server does not support, or refuses to
--<constant>
--<name>HTTP_STATUS_CONTINUE</name>
--<value>100</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_SWITCH_PROTOCOLS</name>
--<value>101</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_OK</name>
--<value>200</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_CREATED</name>
--<value>201</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_ACCEPTED</name>
--<value>202</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_PARTIAL</name>
--<value>203</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_NO_CONTENT</name>
--<value>204</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_RESET_CONTENT</name>
--<value>205</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_PARTIAL_CONTENT</name>
--<value>206</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_AMBIGUOUS</name>
--<value>300</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_MOVED</name>
--<value>301</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_REDIRECT</name>
--<value>302</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_REDIRECT_METHOD</name>
--<value>303</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_NOT_MODIFIED</name>
--<value>304</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_USE_PROXY</name>
--<value>305</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_REDIRECT_KEEP_VERB</name>
--<value>307</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_BAD_REQUEST</name>
--<value>400</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_DENIED</name>
--<value>401</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_PAYMENT_REQ</name>
--<value>402</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_FORBIDDEN</name>
--<value>403</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_NOT_FOUND</name>
--<value>404</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_BAD_METHOD</name>
--<value>405</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_NONE_ACCEPTABLE</name>
--<value>406</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_PROXY_AUTH_REQ</name>
--<value>407</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_REQUEST_TIMEOUT</name>
--<value>408</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_CONFLICT</name>
--<value>409</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_GONE</name>
--<value>410</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_LENGTH_REQUIRED</name>
--<value>411</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_PRECOND_FAILED</name>
--<value>412</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_REQUEST_TOO_LARGE</name>
--<value>413</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_URI_TOO_LONG</name>
--<value>414</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_UNSUPPORTED_MEDIA</name>
--<value>415</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_RETRY_WITH</name>
--<value>449</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_SERVER_ERROR</name>
--<value>500</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_NOT_SUPPORTED</name>
--<value>501</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_BAD_GATEWAY</name>
--<value>502</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_SERVICE_UNAVAIL</name>
--<value>503</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_GATEWAY_TIMEOUT</name>
--<value>504</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_STATUS_VERSION_NOT_SUP</name>
--<value>505</value>
--<desc></desc>
--</constant>
                                         -- support, the HTTP protocol version that was
                                         -- used in the request message.


-- =============================================================================
--                                  Option Flags
-- =============================================================================
--
-- The following list contains the options supported by InternetQueryOption and InternetSetOption.
-- All valid option flags have a value greater than or equal to INTERNET_FIRST_OPTION and less than
-- or equal to INTERNET_LAST_OPTION.
global constant
  INTERNET_OPTION_CALLBACK                = 1,
  -- Sets or retrieves the address of the callback function defined for this handle. This option can be used on all HINTERNET handles. Used by InternetQueryOption and InternetSetOption.
  INTERNET_OPTION_CONNECT_TIMEOUT         = 2,
  -- Sets or retrieves an unsigned long integer value that contains the time-out value, in milliseconds, to use for Internet connection requests. Setting this option to infinite (0xFFFFFFFF) will disable this timer.
  -- If a connection request takes longer than this time-out value, the request is canceled. When attempting to connect to multiple IP addresses for a single host (a multihome host), the timeout limit is cumulative for all of the IP addresses. This option can be used on any HINTERNET handle, including a NULL handle. It is used by InternetQueryOption and InternetSetOption.
  INTERNET_OPTION_CONNECT_RETRIES         = 3,
  -- Sets or retrieves an unsigned long integer value that contains the number of times WinINet attempts to resolve and connect to a host. It only attempts once per IP address. For example, if you attempt to connect to a multihome host that has ten IP addresses and INTERNET_OPTION_CONNECT_RETRIES is set to seven, WinINet only attempts to resolve and connect to the first seven IP addresses. Conversely, given the same set of ten IP addresses, if INTERNET_OPTION_CONNECT_RETRIES is set to 20, WinINet attempts each of the ten only once. If a host has only one IP address and the first connection attempt fails, there are no further attempts. If a connection attempt still fails after the specified number of attempts, the request is canceled. The default value for INTERNET_OPTION_CONNECT_RETRIES is five attempts. This option can be used on any HINTERNET handle, including a NULL handle. It is used by InternetQueryOption and InternetSetOption.
  INTERNET_OPTION_CONNECT_BACKOFF         = 4,
  -- Not implemented.
  INTERNET_OPTION_SEND_TIMEOUT            = 5,
  -- Sets or retrieves an unsigned long integer value, in milliseconds, that contains the time-out value to send a request. If the send takes longer than this time-out value, the send is canceled. This option can be used on any HINTERNET handle, including a NULL handle. It is used by InternetQueryOption and InternetSetOption.
  -- When used in reference to an FTP transaction, this option refers to the control channel.
  INTERNET_OPTION_CONTROL_SEND_TIMEOUT    = 5,
  -- Identical to INTERNET_OPTION_SEND_TIMEOUT. This is used by InternetQueryOption and InternetSetOption.
  INTERNET_OPTION_RECEIVE_TIMEOUT         = 6,
  -- Sets or retrieves an unsigned long integer value that contains the time-out value, in milliseconds, to receive a response to a request. If the response takes longer than this time-out value, the request is canceled. This option can be used on any HINTERNET handle, including a NULL handle. It is used by InternetQueryOption and InternetSetOption.
  -- This option is not intended to represent a fine-grained, immediate timeout. You can expect the timeout to occur up to six seconds after the set timeout value.
  -- When used in reference to an FTP transaction, this option refers to the control channel.
  INTERNET_OPTION_CONTROL_RECEIVE_TIMEOUT = 6,
  -- Identical to INTERNET_OPTION_RECEIVE_TIMEOUT. This is used by InternetQueryOption and InternetSetOption.
  INTERNET_OPTION_DATA_SEND_TIMEOUT       = 7,
  -- Sets or retrieves an unsigned long integer value, in milliseconds, that contains the time-out value to send a request for the data channel of an FTP transaction. If the send takes longer than this time-out value, the send is canceled. This option can be used on any HINTERNET handle, including a NULL handle. It is used by InternetQueryOption and InternetSetOption.
  -- This flag has no impact on HTTP functionality.
  INTERNET_OPTION_DATA_RECEIVE_TIMEOUT    = 8,
  -- Sets or retrieves an unsigned long integer value that contains the time-out value, in milliseconds, to receive a response to a request for the data channel of an FTP transaction. If the response takes longer than this time-out value, the request is canceled. This option can be used on any HINTERNET handle, including a NULL handle. It is used by InternetQueryOption and InternetSetOption.
  -- This flag has no impact on HTTP functionality.
  INTERNET_OPTION_HANDLE_TYPE             = 9,
  -- Retrieves an unsigned long integer value that contains the type of the HINTERNET handles passed in. This is used by InternetQueryOption on any HINTERNET handle. Possible return values include the following.

    INTERNET_HANDLE_TYPE_INTERNET         =  1,
    INTERNET_HANDLE_TYPE_CONNECT_FTP      =  2,
    INTERNET_HANDLE_TYPE_CONNECT_GOPHER   =  3,
    INTERNET_HANDLE_TYPE_CONNECT_HTTP     =  4,
    INTERNET_HANDLE_TYPE_FTP_FIND         =  5,
    INTERNET_HANDLE_TYPE_FTP_FIND_HTML    =  6,
    INTERNET_HANDLE_TYPE_FTP_FILE         =  7,
    INTERNET_HANDLE_TYPE_FTP_FILE_HTML    =  8,
    INTERNET_HANDLE_TYPE_GOPHER_FIND      =  9,
    INTERNET_HANDLE_TYPE_GOPHER_FIND_HTML = 10,
    INTERNET_HANDLE_TYPE_GOPHER_FILE      = 11,
    INTERNET_HANDLE_TYPE_GOPHER_FILE_HTML = 12,
    INTERNET_HANDLE_TYPE_HTTP_REQUEST     = 13,
    INTERNET_HANDLE_TYPE_FILE_REQUEST     = 14,

  INTERNET_OPTION_LISTEN_TIMEOUT          = 11,
  -- Not implemented.
  INTERNET_OPTION_READ_BUFFER_SIZE        = 12,
  -- Sets or retrieves an unsigned long integer value that contains the size of the read buffer. This option can be used on HINTERNET handles returned by FtpOpenFile, FtpFindFirstFile, and InternetConnect (FTP session only). This option is used by InternetQueryOption and InternetSetOption.
  INTERNET_OPTION_WRITE_BUFFER_SIZE       = 13,
  -- Sets or retrieves an unsigned long integer value that contains the size, in bytes, of the write buffer. This option can be used on HINTERNET handles returned by FtpOpenFile and InternetConnect (FTP session only). It is used by InternetQueryOption and InternetSetOption.
  INTERNET_OPTION_ASYNC_ID                = 15,
  -- Not implemented.                 
  INTERNET_OPTION_ASYNC_PRIORITY          = 16,
  -- Not implemented.
  INTERNET_OPTION_PARENT_HANDLE           = 21,
  -- Retrieves the parent handle to this handle. This option can be used on any HINTERNET handle by InternetQueryOption.
  INTERNET_OPTION_KEEP_CONNECTION         = 22,
  -- Not implemented.
  INTERNET_OPTION_REQUEST_FLAGS           = 23,
  -- Retrieves an unsigned long integer value that contains the special status flags that indicate the status of the download in progress. This is used by InternetQueryOption. The INTERNET_OPTION_REQUEST_FLAGS option can be one of the following values:

    INTERNET_REQFLAG_FROM_CACHE           = #00000001,
	-- Response came from the cache.
    INTERNET_REQFLAG_ASYNC                = #00000002,
	-- Not implemented.
    INTERNET_REQFLAG_VIA_PROXY            = #00000004,
	-- Request was made through a proxy.
    INTERNET_REQFLAG_NO_HEADERS           = #00000008,
	-- Original response contained no headers.
    INTERNET_REQFLAG_PASSIVE              = #00000010,
	-- Not implemented.
    INTERNET_REQFLAG_CACHE_WRITE_DISABLED = #00000040,
	-- Internet request cannot be cached (an HTTPS request, for example).
    INTERNET_REQFLAG_NET_TIMEOUT          = #00000080,
	-- Internet request timed out.

  INTERNET_OPTION_EXTENDED_ERROR          = 24,
  -- Retrieves an unsigned long integer value that contains a Winsock error code mapped to the ERROR_INTERNET_ error messages last returned in this thread context. This option is used on a NULLHINTERNET handle by InternetQueryOption.
  INTERNET_OPTION_OFFLINE_MODE            = 26,
  -- Not implemented.
  INTERNET_OPTION_CACHE_STREAM_HANDLE     = 27,
  -- No longer supported.
  INTERNET_OPTION_USERNAME                = 28,
  -- Sets or retrieves a string that contains the user name associated with a handle returned by InternetConnect. This is used by InternetQueryOption and InternetSetOption.
  INTERNET_OPTION_PASSWORD                = 29,
  -- Sets or retrieves a string value that contains the password associated with a handle returned by InternetConnect. This is used by InternetQueryOption and InternetSetOption.
  INTERNET_OPTION_ASYNC                   = 30,
  -- Not implemented.                 
  INTERNET_OPTION_SECURITY_FLAGS          = 31,
  -- Retrieves an unsigned long integer value that contains the security flags for a handle. This option is used by InternetQueryOption. It can be a combination of the following values.

    SECURITY_FLAG_SECURE                   = #00000001,
	-- Uses secure transfers. This is only returned in a call to InternetQueryOption.
    SECURITY_FLAG_SSL                      = #00000002,
	-- Not implemented.
    SECURITY_FLAG_SSL3                     = #00000004,
	-- Not implemented.
    SECURITY_FLAG_PCT                      = #00000008,
	-- Not implemented.
    SECURITY_FLAG_PCT4                     = #00000010,
	-- Not implemented.
    SECURITY_FLAG_IETFSSL4                 = #00000020,
	-- Not implemented.
    SECURITY_FLAG_IGNORE_REVOCATION        = #00000080,
	-- Ignores certificate revocation problems.
    SECURITY_FLAG_IGNORE_UNKNOWN_CA        = #00000100,
	-- Ignores unknown certificate authority problems.
    SECURITY_FLAG_IGNORE_WRONG_USAGE       = #00000200,
	-- Ignores incorrect usage problems.
    SECURITY_FLAG_IGNORE_CERT_CN_INVALID   = #00001000,
	-- Ignores the ERROR_INTERNET_SEC_CERT_CN_INVALID error message.
    SECURITY_FLAG_IGNORE_CERT_DATE_INVALID = #00002000,
	-- Ignores the ERROR_INTERNET_SEC_CERT_DATE_INVALID error message.
    SECURITY_FLAG_IGNORE_REDIRECT_TO_HTTPS = #00004000,
	-- Ignores the ERROR_INTERNET_HTTP_TO_HTTPS_ON_REDIR error message.
    SECURITY_FLAG_IGNORE_REDIRECT_TO_HTTP  = #00008000,
	-- Ignores the ERROR_INTERNET_HTTPS_TO_HTTP_ON_REDIR error message.
    SECURITY_FLAG_IGNORE_WEAK_SIGNATURE    = #00010000,
	-- Ignores weak certificate signature problems.
    SECURITY_FLAG_NORMALBITNESS            = #10000000,
	-- Identical to the value SECURITY_FLAG_STRENGTH_WEAK. This is only returned in a call to InternetQueryOption.
    SECURITY_FLAG_40BIT                    = #10000000,
	-- Identical to the preferred value SECURITY_FLAG_STRENGTH_WEAK. This is only returned in a call to InternetQueryOption.
    SECURITY_FLAG_STRENGTH_WEAK            = #10000000,
	-- Uses weak (40-bit) encryption. This is only returned in a call to InternetQueryOption.
    SECURITY_FLAG_128BIT                   = #20000000,
	-- Identical to the preferred value SECURITY_FLAG_STRENGTH_STRONG. This is only returned in a call to InternetQueryOption.
    SECURITY_FLAG_STRENGTH_STRONG          = #20000000,
	-- Uses strong (128-bit) encryption. This is only returned in a call to InternetQueryOption.
    SECURITY_FLAG_56BIT                    = #40000000,
	-- Identical to the preferred value SECURITY_FLAG_STRENGTH_MEDIUM. This is only returned in a call to InternetQueryOption.
    SECURITY_FLAG_STRENGTH_MEDIUM          = #40000000,
	-- Uses medium (56-bit) encryption. This is only returned in a call to InternetQueryOption.
    SECURITY_FLAG_FORTEZZA                 = #08000000,
	-- Indicates Fortezza has been used to provide secrecy, authentication, and/or integrity for the specified connection.
    SECURITY_FLAG_UNKNOWNBIT               = #80000000,
	-- The bit size used in the encryption is unknown. This is only returned in a call to InternetQueryOption.

    -- Be aware that the data retrieved this way relates to a transaction that has occurred, whose security level can no longer be changed.
  INTERNET_OPTION_SECURITY_CERTIFICATE_STRUCT = 32,
  -- Retrieves the certificate for an SSL/PCT server into the INTERNET_CERTIFICATE_INFO structure. This is used by InternetQueryOption.
  INTERNET_OPTION_DATAFILE_NAME           = 33,
  -- Retrieves a string value that contains the name of the file backing a downloaded entity. This flag is valid after InternetOpenUrl, FtpOpenFile, GopherOpenFile, or HttpOpenRequest has completed. This option can only be queried by InternetQueryOption.
  INTERNET_OPTION_URL                     = 34,
  -- Retrieves a string value that contains the full URL of a downloaded resource. If the original URL contained any extra data, such as search strings or anchors, or if the call was redirected, the URL returned differs from the original. This option is valid on HINTERNET handles returned by InternetOpenUrl, FtpOpenFile, GopherOpenFile, or HttpOpenRequest. It is used by InternetQueryOption.
  INTERNET_OPTION_SECURITY_CERTIFICATE    = 35,
  -- Retrieves the certificate for an SSL/PCT (Secure Sockets Layer/Private Communications Technology) server into a formatted string. This is used by InternetQueryOption.
  INTERNET_OPTION_SECURITY_KEY_BITNESS    = 36,
  -- Retrieves an unsigned long integer value that contains the bit size of the encryption key. The larger the number, the greater the encryption strength used. This is used by InternetQueryOption. Be aware that the data retrieved this way relates to a transaction that has already occurred, whose security level can no longer be changed.
  INTERNET_OPTION_REFRESH                 = 37,
  -- Causes the proxy data to be reread from the registry for a handle. No buffer is required. This option can be used on the HINTERNET handle returned by InternetOpen. It is used by InternetSetOption.
  INTERNET_OPTION_PROXY                   = 38,
  -- Sets or retrieves an INTERNET_PROXY_INFO structure that contains the proxy data for an existing InternetOpen handle when the HINTERNET handle is not NULL. If the HINTERNET handle is NULL, the function sets or queries the global proxy data. This option can be used on the handle returned by InternetOpen. It is used by InternetQueryOption and InternetSetOption.
  -- Note  It is recommended that INTERNET_OPTION_PER_CONNECTION_OPTION be used instead of INTERNET_OPTION_PROXY. For more information, see KB article 226473.  
  INTERNET_OPTION_SETTINGS_CHANGED        = 39,
  -- Notifies the system that the registry settings have been changed so that it verifies the settings on the next call to InternetConnect. This is used by InternetSetOption.
  INTERNET_OPTION_VERSION                 = 40,
  -- Retrieves an INTERNET_VERSION_INFO structure that contains the version number of Wininet.dll. This option can be used on a NULLHINTERNET handle by InternetQueryOption.
  INTERNET_OPTION_USER_AGENT              = 41,
  -- Sets or retrieves the user agent string on handles supplied by InternetOpen and used in subsequent HttpSendRequest functions, as long as it is not overridden by a header added by HttpAddRequestHeaders or HttpSendRequest. This is used by InternetQueryOption and InternetSetOption.
  INTERNET_OPTION_END_BROWSER_SESSION     = 42,
  -- Flushes entries not in use from the password cache on the hard disk drive. Also resets the cache time used when the synchronization mode is once-per-session. No buffer is required for this option. This is used by InternetSetOption.
  INTERNET_OPTION_PROXY_USERNAME          = 43,
  -- Sets or retrieves a string value that contains the user name used to access the proxy. This is used by InternetQueryOption and InternetSetOption. This option can be set on the handle returned by InternetConnect or HttpOpenRequest.
  INTERNET_OPTION_PROXY_PASSWORD          = 44,
  -- Sets or retrieves a string value that contains the password used to access the proxy. This is used by InternetQueryOption and InternetSetOption. This option can be set on the handle returned by InternetConnect or HttpOpenRequest.
  INTERNET_OPTION_CONTEXT_VALUE           = 45,
  -- Sets or retrieves a DWORD_PTR that contains the address of the context value associated with this HINTERNET handle. This option can be used on any HINTERNET handle. This is used by InternetQueryOption and InternetSetOption. Previously, this set the context value to the address stored in the lpBuffer pointer. This has been corrected so that the value stored in the buffer is used and the INTERNET_OPTION_CONTEXT_VALUE flag is assigned a new value. The old value, 10, has been preserved so that applications written for the old behavior are still supported.
  INTERNET_OPTION_POLICY                  = 48,
  -- Not implemented.
  INTERNET_OPTION_DISCONNECTED_TIMEOUT    = 49,
  -- Not implemented.
  INTERNET_OPTION_CONNECTED_STATE         = 50,
  -- Sets or retrieves an unsigned long integer value that contains the connected state. This is used by InternetQueryOption and InternetSetOption.
  INTERNET_OPTION_IDLE_STATE              = 51,
  -- Not implemented.
  INTERNET_OPTION_OFFLINE_SEMANTICS       = 52,
  -- Not implemented.
  INTERNET_OPTION_SECONDARY_CACHE_KEY     = 53,
  -- Sets or retrieves a string value that contains the secondary cache key. This is used by InternetQueryOption and InternetSetOption. This option is reserved for internal use only.
  INTERNET_OPTION_CALLBACK_FILTER         = 54,
  -- Not implemented.
  INTERNET_OPTION_CONNECT_TIME            = 55,
  -- Not implemented.
  INTERNET_OPTION_SEND_THROUGHPUT         = 56,
  -- Not implemented.
  INTERNET_OPTION_RECEIVE_THROUGHPUT      = 57,
  -- Not implemented.
  INTERNET_OPTION_REQUEST_PRIORITY        = 58,
  -- Sets or retrieves an unsigned long integer value that contains the priority of requests that compete for a connection on an HTTP handle. This is used by InternetQueryOption and InternetSetOption.
  INTERNET_OPTION_HTTP_VERSION            = 59,
  -- Sets or retrieves an HTTP_VERSION_INFO structure that contains the supported HTTP version. This must be used on a NULL handle. This is used by InternetQueryOption and InternetSetOption.
  -- On Windows 7, Windows Server 2008 R2, and later, the value of the dwMinorVersion member in the HTTP_VERSION_INFO structure is overridden by Internet Explorer settings. EnableHttp1_1 is a registry value under HKLM\Software\Microsoft\InternetExplorer\AdvacnedOptions\HTTP\GENABLE controlled by Internet Options set in Internet Explorer for the system. The EnableHttp1_1 value defaults to 1. The HTTP_VERSION_INFO structure is ignored for any HTTP version less than 1.1 if EnableHttp1_1 is set to 1.
  INTERNET_OPTION_RESET_URLCACHE_SESSION  = 60,
  -- Starts a new cache session for the process. No buffer is required. This is used by InternetSetOption. This option is reserved for internal use only.
  INTERNET_OPTION_ERROR_MASK              = 62,
  -- Sets an unsigned long integer value that contains the error masks that can be handled by the client application. This can be a combination of the following values:

    INTERNET_ERROR_MASK_INSERT_CDROM      = #1,
	-- Indicates that the client application can handle the ERROR_INTERNET_INSERT_CDROM error code.
    INTERNET_ERROR_MASK_COMBINED_SEC_CERT = #2,
	-- Indicates that all certificate errors are to be reported using the same error return, namely ERROR_INTERNET_SEC_CERT_ERRORS. If this flag is set, call InternetErrorDlg upon receiving the ERROR_INTERNET_SEC_CERT_ERRORS error, so that the user can respond to a familiar dialog describing the problem.
    -- Caution  Failing to inform the user of this error exposes the user to potential spoofing attacks.
    INTERNET_ERROR_MASK_NEED_MSN_SSPI_PKG = #4,
	-- Not implemented.
    INTERNET_ERROR_MASK_LOGIN_FAILURE_DISPLAY_ENTITY_BODY = #8,
	-- Indicates that the client application can handle the ERROR_INTERNET_LOGIN_FAILURE_DISPLAY_ENTITY_BODY error code.

  INTERNET_OPTION_FROM_CACHE_TIMEOUT      = 63,
  -- Sets or retrieves a1n unsigned long integer value that contains the amount of time the system should wait for a response to a network request before checking the cache for a copy of the resource. If a network request takes longer than the time specified and the requested resource is available in the cache, the resource is retrieved from the cache. This is used by InternetQueryOption and InternetSetOption.
  INTERNET_OPTION_BYPASS_EDITED_ENTRY     = 64,
  -- Sets or retrieves the Boolean value that determines if the system should check the network for newer content and overwrite edited cache entries if a newer version is found. If set to True, the system checks the network for newer content and overwrites the edited cache entry with the newer version. The default is False, which indicates that the edited cache entry should be used without checking the network. This is used by InternetQueryOption and InternetSetOption. It is valid only in Microsoft Internet Explorer 5 and later.
  INTERNET_OPTION_HTTP_DECODING           = 65,
  -- Enables WinINet to perform decoding for the gzip and deflate encoding schemes. For more information, see Content Encoding.
  INTERNET_OPTION_DIAGNOSTIC_SOCKET_INFO  = 67,
  -- Retrieves an INTERNET_DIAGNOSTIC_SOCKET_INFO structure that contains data about a specified HTTP Request. This flag is used by InternetQueryOption.
  -- Windows 7:  This option is no longer supported.
  INTERNET_OPTION_CODEPAGE                = 68,
  -- By default, the host or authority portion of the Unicode URL is encoded according to the IDN specification. Setting this option on the request, or connection handle, when IDN is disabled, specifies a code page encoding scheme for the host portion of the URL. The lpBuffer parameter in the call to InternetSetOption contains the desired DBCS code page. If no code page is specified in lpBuffer, WinINet uses the default system code page (CP_ACP). Note: This option is ignored if IDN is not disabled. For more information about how to disable IDN, see the INTERNET_OPTION_IDN option. = Windows XP with SP2 and Windows Server 2003 with SP1:  This flag is not supported.
  -- Version:  Requires Internet Explorer 7.0.
  INTERNET_OPTION_CACHE_TIMESTAMPS        = 69,
  -- Retrieves an INTERNET_CACHE_TIMESTAMPS structure that contains the LastModified time and Expires time from the resource stored in the Internet cache. This value is used by InternetQueryOption.
  INTERNET_OPTION_DISABLE_AUTODIAL        = 70,
  -- Not implemented.
  INTERNET_OPTION_MAX_CONNS_PER_SERVER    = 73,
  -- Sets or retrieves an unsigned long integer value that contains the maximum number of connections allowed per server. This is used by InternetQueryOption and InternetSetOption. This option is only valid in Internet Explorer 5 and later.
  INTERNET_OPTION_MAX_CONNS_PER_1_0_SERVER = 74,
  -- Sets or retrieves an unsigned long integer value that contains the maximum number of connections allowed per HTTP/1.0 server. This is used by InternetQueryOption and InternetSetOption. This option is only valid in Internet Explorer 5 and later.
  INTERNET_OPTION_PER_CONNECTION_OPTION   = 75,
  -- Sets or retrieves an INTERNET_PER_CONN_OPTION_LIST structure that specifies a list of options for a particular connection. This is used by InternetQueryOption and InternetSetOption. This option is only valid in Internet Explorer 5 and later.
  -- Note  INTERNET_OPTION_PER_CONNECTION_OPTION causes the settings to be changed on a system-wide basis when a NULL handle is used in the call to InternetSetOption. To refresh the global proxy settings, you must call InternetSetOption with the INTERNET_OPTION_REFRESH option flag.   Note  To change proxy information for the entire process without affecting the global settings in Internet Explorer 5 and later, use this option on the handle that is returned from InternetOpen. The following code example changes the proxy for the whole process even though the HINTERNET handle is closed and is not used by any requests.  
  -- For more information and code examples, see KB article 226473.
  INTERNET_OPTION_DIGEST_AUTH_UNLOAD      = 76,
  -- Causes the system to log off the Digest authentication SSPI package, purging all of the credentials created for the process. No buffer is required for this option. It is used by InternetSetOption.
  INTERNET_OPTION_IGNORE_OFFLINE          = 77,
  -- Sets or retrieves whether the global offline flag should be ignored for the specified request handle. No buffer is required for this option. This is used by InternetQueryOption and InternetSetOption with a request handle. This option is only valid in Internet Explorer 5 and later.
  INTERNET_OPTION_IDENTITY                = 78,
  -- Not implemented.
  INTERNET_OPTION_REMOVE_IDENTITY         = 79,
  -- Not implemented.
  INTERNET_OPTION_ALTER_IDENTITY          = 80,
  -- Not implemented
  INTERNET_OPTION_SUPPRESS_BEHAVIOR       = 81,
  -- A general purpose option that is used to suppress behaviors on a process-wide basis. The lpBuffer parameter of the function must be a pointer to a DWORD containing the specific behavior to suppress. This option cannot be queried with InternetQueryOption. The permitted values are:

    INTERNET_SUPPRESS_RESET_ALL            = 0,
	-- Disables all suppressions, re-enabling default and configured behavior. This option is the equivalent of setting INTERNET_SUPPRESS_COOKIE_POLICY_RESET and INTERNET_SUPPRESS_COOKIE_PERSIST_RESET individually.
	-- Version:  Requires Internet Explorer 6.0 or later.
    INTERNET_SUPPRESS_COOKIE_POLICY        = 1,
	-- Ignores any configured cookie policies and allows cookies to be set.
	-- Version:  Requires Internet Explorer 6.0 or later.
    INTERNET_SUPPRESS_COOKIE_POLICY_RESET  = 2,
	-- Disables the INTERNET_SUPPRESS_COOKIE_POLICY suppression, permitting the evaluation of cookies according to the configured cookie policy.
	-- Version:  Requires Internet Explorer 6.0 or later.
    INTERNET_SUPPRESS_COOKIE_PERSIST       = 3,
	-- Suppresses the persistence of cookies, even if the server has specified them as persistent.
	-- Version:  Requires Internet Explorer 8.0 or later.
    INTERNET_SUPPRESS_COOKIE_PERSIST_RESET = 4,
	-- Disables the INTERNET_SUPPRESS_COOKIE_PERSIST suppression, re-enabling the persistence of cookies. Any previously suppressed cookies will not become persistent.
	-- Version:  Requires Internet Explorer 8.0 or later.

  INTERNET_OPTION_CLIENT_CERT_CONTEXT     = 84,
  -- This flag is not supported by InternetQueryOption. The lpBuffer parameter must be a pointer to a CERT_CONTEXT structure and not a pointer to a CERT_CONTEXT pointer. If an application receives ERROR_INTERNET_CLIENT_AUTH_CERT_NEEDED, it must call InternetErrorDlg or use InternetSetOption to supply a certificate before retrying the request. CertDuplicateCertificateContext is then called so that the certificate context passed can be independently released by the application.
  INTERNET_OPTION_PROXY_SETTINGS_CHANGED  = 95,
  -- Alerts the current WinInet instance that proxy settings have changed and that they must update with the new settings. To alert all available WinInet instances, set the Buffer parameter of InternetSetOption to NULL and BufferLength to 0 when passing this option. This option can be set on the handle returned by InternetConnect or HttpOpenRequest.
  INTERNET_OPTION_DATAFILE_EXT            = 96,
  -- Sets a string value that contains the extension of the file backing a downloaded entity. This flag should be set before calling InternetOpenUrl, FtpOpenFile, GopherOpenFile, or HttpOpenRequest. This option can only be set by InternetSetOption.
  INTERNET_OPTION_CODEPAGE_PATH           = 100,
  -- By default, the path portion of the URL is UTF8 encoded. The WinINet API performs escape character (%) encoding on the high-bit characters. Setting this option on the request, or connection handle, disables the UTF8 encoding and sets a specific code page. The lpBuffer parameter in the call to InternetSetOption contains the desired DBCS codepage for the path. If no code page is specified in lpBuffer, WinINet uses the default CP_UTF8.
  -- Windows XP with SP2 and Windows Server 2003 with SP1:  This flag is not supported.
  -- Version:  Requires Internet Explorer 7.0.
  INTERNET_OPTION_CODEPAGE_EXTRA          = 101,
  -- By default, the path portion of the URL is the default system code page (CP_ACP). The escape character (%) conversions are not performed on the extra portion. Setting this option on the request, or connection handle disables the CP_ACP encoding. The lpBuffer parameter in the call to InternetSetOption contains the desired DBCS codepage for the extra portion of the URL. If no code page is specified in lpBuffer, WinINet uses the default system code page (CP_ACP).
  -- Windows XP with SP2 and Windows Server 2003 with SP1:  This flag is not supported.
  -- Version:  Requires Internet Explorer 7.0.
  INTERNET_OPTION_IDN                     = 102,
  -- By default, the host or authority portion of the URL is encoded according to the IDN specification for both direct and proxy connections. This option can be used on the request, or connection handle to enable or disable IDN. When IDN is disabled, WinINet uses the system codepage to encode the host or authority portion of the URL. To disable IDN host conversion, set the lpBuffer parameter in the call to InternetSetOption to zero. To enable IDN conversion on only the direct connection, specify INTERNET_FLAG_IDN_DIRECT in the lpBuffer parameter in the call to InternetSetOption. To enable IDN conversion on only the proxy connection, specify INTERNET_FLAG_IDN_PROXY in the lpBuffer parameter in the call to InternetSetOption.
  -- Windows XP with SP2 and Windows Server 2003 with SP1:  This flag is not supported.
  -- Version:  Requires Internet Explorer 7.0.
  INTERNET_OPTION_MAX_CONNS_PER_PROXY     = 103,
  -- Sets or retrieves an unsigned long integer value that contains the maximum number of connections allowed per CERN proxy. When this option is set or retrieved, the hInternet parameter must set to a null handle value. A null handle value indicates that the option should be set or queried for the current process. When calling InternetSetOption with this option, all existing proxy objects will receive the new value. This value is limited to a range of 2 to 128, inclusive.
  -- Version:  Requires Internet Explorer 8.0.
  INTERNET_OPTION_SUPPRESS_SERVER_AUTH    = 104,
  -- Sets an HTTP request object such that it will not logon to origin servers, but will perform automatic logon to HTTP proxy servers. This option differs from the Request flag INTERNET_FLAG_NO_AUTH, which prevents authentication to both proxy servers and origin servers.
  -- Setting this mode will suppress the use of any credential material (either previously provided username/password or client SSL certificate) when communicating with an origin server. However, if the request must transit via an authenticating proxy, WinINet will still perform automatic authentication to the HTTP proxy per the Intranet Zone settings for the user. The default Intranet Zone setting is to permit automatic logon using the user’s default credentials.
  -- To ensure suppression of all identifying information, the caller should combine INTERNET_OPTION_SUPPRESS_SERVER_AUTH with the INTERNET_FLAG_NO_COOKIES request flag.
  -- This option may only be set on request objects before they have been sent. Attempts to set this option after the request has been sent will return ERROR_INTERNET_INCORRECT_HANDLE_STATE.
  -- No buffer is required for this option. This is used by InternetSetOption on handles returned by HttpOpenRequest only.
  -- Version:  Requires Internet Explorer 8.0 or later.
  INTERNET_OPTION_SERVER_CERT_CHAIN_CONTEXT = 105,
  -- Retrieves the server’s certificate-chain context as a duplicated PCCERT_CHAIN_CONTEXT. You may pass this duplicated context to any Crypto API function which takes a PCCERT_CHAIN_CONTEXT. You must call CertFreeCertificateChain on the returned PCCERT_CHAIN_CONTEXT when you are done with the certificate-chain context.
  -- Version:  Requires Internet Explorer 8.0.
  INTERNET_OPTION_ENABLE_REDIRECT_CACHE_READ = 122,
  -- On a request handle, sets a Boolean controlling whether redirects will be returned from the WinInet cache for a given request. The default is FALSE. Supported in Windows 8 and later.
  INTERNET_OPTION_COMPRESSED_CONTENT_LENGTH = 147,
  -- For a request where WinInet decompressed the server’s supplied Content-Encoding, retrieves the server-reported Content-Length of the response body as a ULONGLONG. Supported in Windows 10, version 1507 and later.
  INTERNET_OPTION_ENABLE_HTTP_PROTOCOL    = 148,
  -- Sets a DWORD bitmask of acceptable advanced HTTP versions. May be set on any handle type. Possible values are:

    HTTP_PROTOCOL_FLAG_HTTP2 = #2,  -- Supported on Windows 10, version 1507 and later.

  INTERNET_OPTION_HTTP_PROTOCOL_USED      = 149,
  -- Gets a DWORD indicating which advanced HTTP version was used on a given request. Possible values are:
  --   HTTP_PROTOCOL_FLAG_HTTP2 = #2,  -- Supported on Windows 10, version 1507 and later. = 0x0 indicates HTTP/1.1 or earlier; see INTERNET_OPTION_HTTP_VERSION if more precision is needed about which legacy version was used. Supported on Windows 10, version 1507 and later.
  -- Legacy versions of HTTP (1.1 and prior) cannot be disabled using this option. The default is 0x0. Supported in Windows 10, version 1507 and later.
  INTERNET_OPTION_ENCODE_EXTRA            = 155,
  -- Gets/sets a BOOL indicating whether non-ASCII characters in the query string should be percent-encoded. The default is FALSE. Supported in Windows 8.1 and later.
  INTERNET_OPTION_HSTS                    = 157,
  -- Gets/sets a BOOL indicating whether WinInet should follow HTTP Strict Transport Security (HSTS) directives from servers. If enabled, http:// schemed requests to domains which have an HSTS policy cached by WinInet will be redirected to matching https:// URLs. The default is FALSE. Supported in Windows 8.1 and later.
  INTERNET_OPTION_ENTERPRISE_CONTEXT      = 159,
  -- Sets a PWSTR containing the Enterprise ID (see https://msdn.microsoft.com/en-us/library/windows/desktop/mt759320(v=vs.85).aspx) which applies to the request. Supported in Windows 10, version 1507 and later.
  INTERNET_OPTION_OPT_IN_WEAK_SIGNATURE   = 176
--<constant>
--<name>INTERNET_OPTION_CALLBACK</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CONNECT_TIMEOUT</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CONNECT_RETRIES</name>
--<value>3</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CONNECT_BACKOFF</name>
--<value>4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_SEND_TIMEOUT</name>
--<value>5</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CONTROL_SEND_TIMEOUT</name>
--<value>5</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_RECEIVE_TIMEOUT</name>
--<value>6</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CONTROL_RECEIVE_TIMEOUT</name>
--<value>6</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_DATA_SEND_TIMEOUT</name>
--<value>7</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_DATA_RECEIVE_TIMEOUT</name>
--<value>8</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_HANDLE_TYPE</name>
--<value>9</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_INTERNET</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_CONNECT_FTP</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_CONNECT_GOPHER</name>
--<value>3</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_CONNECT_HTTP</name>
--<value>4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_FTP_FIND</name>
--<value>5</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_FTP_FIND_HTML</name>
--<value>6</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_FTP_FILE</name>
--<value>7</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_FTP_FILE_HTML</name>
--<value>8</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_GOPHER_FIND</name>
--<value>9</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_GOPHER_FIND_HTML</name>
--<value>10</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_GOPHER_FILE</name>
--<value>11</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_GOPHER_FILE_HTML</name>
--<value>12</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_HTTP_REQUEST</name>
--<value>13</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_HANDLE_TYPE_FILE_REQUEST</name>
--<value>14</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_LISTEN_TIMEOUT</name>
--<value>11</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_READ_BUFFER_SIZE</name>
--<value>12</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_WRITE_BUFFER_SIZE</name>
--<value>13</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_ASYNC_ID</name>
--<value>15</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_ASYNC_PRIORITY</name>
--<value>16</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_PARENT_HANDLE</name>
--<value>21</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_KEEP_CONNECTION</name>
--<value>22</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_REQUEST_FLAGS</name>
--<value>23</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_REQFLAG_FROM_CACHE</name>
--<value>#00000001</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_REQFLAG_ASYNC</name>
--<value>#00000002</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_REQFLAG_VIA_PROXY</name>
--<value>#00000004</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_REQFLAG_NO_HEADERS</name>
--<value>#00000008</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_REQFLAG_PASSIVE</name>
--<value>#00000010</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_REQFLAG_CACHE_WRITE_DISABLED</name>
--<value>#00000040</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_REQFLAG_NET_TIMEOUT</name>
--<value>#00000080</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_EXTENDED_ERROR</name>
--<value>24</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_OFFLINE_MODE</name>
--<value>26</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CACHE_STREAM_HANDLE</name>
--<value>27</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_USERNAME</name>
--<value>28</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_PASSWORD</name>
--<value>29</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_ASYNC</name>
--<value>30</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_SECURITY_FLAGS</name>
--<value>31</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_SECURE</name>
--<value>#00000001</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_SSL</name>
--<value>#00000002</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_SSL3</name>
--<value>#00000004</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_PCT</name>
--<value>#00000008</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_PCT4</name>
--<value>#00000010</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_IETFSSL4</name>
--<value>#00000020</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_IGNORE_REVOCATION</name>
--<value>#00000080</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_IGNORE_UNKNOWN_CA</name>
--<value>#00000100</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_IGNORE_WRONG_USAGE</name>
--<value>#00000200</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_IGNORE_CERT_CN_INVALID</name>
--<value>#00001000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_IGNORE_CERT_DATE_INVALID</name>
--<value>#00002000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_IGNORE_REDIRECT_TO_HTTPS</name>
--<value>#00004000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_IGNORE_REDIRECT_TO_HTTP</name>
--<value>#00008000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_IGNORE_WEAK_SIGNATURE</name>
--<value>#00010000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_NORMALBITNESS</name>
--<value>#10000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_40BIT</name>
--<value>#10000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_STRENGTH_WEAK</name>
--<value>#10000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_128BIT</name>
--<value>#20000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_STRENGTH_STRONG</name>
--<value>#20000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_56BIT</name>
--<value>#40000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_STRENGTH_MEDIUM</name>
--<value>#40000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_FORTEZZA</name>
--<value>#08000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SECURITY_FLAG_UNKNOWNBIT</name>
--<value>#80000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_SECURITY_CERTIFICATE_STRUCT</name>
--<value>32</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_DATAFILE_NAME</name>
--<value>33</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_URL</name>
--<value>34</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_SECURITY_CERTIFICATE</name>
--<value>35</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_SECURITY_KEY_BITNESS</name>
--<value>36</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_REFRESH</name>
--<value>37</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_PROXY</name>
--<value>38</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_SETTINGS_CHANGED</name>
--<value>39</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_VERSION</name>
--<value>40</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_USER_AGENT</name>
--<value>41</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_END_BROWSER_SESSION</name>
--<value>42</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_PROXY_USERNAME</name>
--<value>43</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_PROXY_PASSWORD</name>
--<value>44</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CONTEXT_VALUE</name>
--<value>45</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_POLICY</name>
--<value>48</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_DISCONNECTED_TIMEOUT</name>
--<value>49</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CONNECTED_STATE</name>
--<value>50</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_IDLE_STATE</name>
--<value>51</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_OFFLINE_SEMANTICS</name>
--<value>52</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_SECONDARY_CACHE_KEY</name>
--<value>53</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CALLBACK_FILTER</name>
--<value>54</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CONNECT_TIME</name>
--<value>55</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_SEND_THROUGHPUT</name>
--<value>56</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_RECEIVE_THROUGHPUT</name>
--<value>57</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_REQUEST_PRIORITY</name>
--<value>58</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_HTTP_VERSION</name>
--<value>59</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_RESET_URLCACHE_SESSION</name>
--<value>60</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_ERROR_MASK</name>
--<value>62</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_ERROR_MASK_INSERT_CDROM</name>
--<value>#1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_ERROR_MASK_COMBINED_SEC_CERT</name>
--<value>#2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_ERROR_MASK_NEED_MSN_SSPI_PKG</name>
--<value>#4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_ERROR_MASK_LOGIN_FAILURE_DISPLAY_ENTITY_BODY</name>
--<value>#8</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_FROM_CACHE_TIMEOUT</name>
--<value>63</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_BYPASS_EDITED_ENTRY</name>
--<value>64</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_HTTP_DECODING</name>
--<value>65</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_DIAGNOSTIC_SOCKET_INFO</name>
--<value>67</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CODEPAGE</name>
--<value>68</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CACHE_TIMESTAMPS</name>
--<value>69</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_DISABLE_AUTODIAL</name>
--<value>70</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_MAX_CONNS_PER_SERVER</name>
--<value>73</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_MAX_CONNS_PER_1_0_SERVER</name>
--<value>74</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_PER_CONNECTION_OPTION</name>
--<value>75</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_DIGEST_AUTH_UNLOAD</name>
--<value>76</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_IGNORE_OFFLINE</name>
--<value>77</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_IDENTITY</name>
--<value>78</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_REMOVE_IDENTITY</name>
--<value>79</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_ALTER_IDENTITY</name>
--<value>80</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_SUPPRESS_BEHAVIOR</name>
--<value>81</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SUPPRESS_RESET_ALL</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SUPPRESS_COOKIE_POLICY</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SUPPRESS_COOKIE_POLICY_RESET</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SUPPRESS_COOKIE_PERSIST</name>
--<value>3</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SUPPRESS_COOKIE_PERSIST_RESET</name>
--<value>4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CLIENT_CERT_CONTEXT</name>
--<value>84</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_PROXY_SETTINGS_CHANGED</name>
--<value>95</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_DATAFILE_EXT</name>
--<value>96</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CODEPAGE_PATH</name>
--<value>100</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_CODEPAGE_EXTRA</name>
--<value>101</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_IDN</name>
--<value>102</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_MAX_CONNS_PER_PROXY</name>
--<value>103</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_SUPPRESS_SERVER_AUTH</name>
--<value>104</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_SERVER_CERT_CHAIN_CONTEXT</name>
--<value>105</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_ENABLE_REDIRECT_CACHE_READ</name>
--<value>122</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_COMPRESSED_CONTENT_LENGTH</name>
--<value>147</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_ENABLE_HTTP_PROTOCOL</name>
--<value>148</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_PROTOCOL_FLAG_HTTP2</name>
--<value>#2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_HTTP_PROTOCOL_USED</name>
--<value>149</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_ENCODE_EXTRA</name>
--<value>155</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_HSTS</name>
--<value>157</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_ENTERPRISE_CONTEXT</name>
--<value>159</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPTION_OPT_IN_WEAK_SIGNATURE</name>
--<value>176</value>
--<desc></desc>
--</constant>
  -- Opt-in for weak signatures (e.g. SHA-1) to be treated as insecure. This will instruct WinInet to call CertGetCertificateChain using the CERT_CHAIN_OPT_IN_WEAK_SIGNATURE parameter.

  
-- =============================================================================
--                                  Query Info Flags
-- =============================================================================

-- The following lists contain the attributes and modifiers used by
-- HttpQueryInfo and IWinInetHttpInfo::QueryInfo.
--
-- Attributes
-- The attribute flags are used by HttpQueryInfo (or IWinInetHttpInfo::
-- QueryInfo) to indicate what information to retrieve. Most of the attribute
-- flags map directly to a specific HTTP header. There are also some special
-- flags, such as HTTP_QUERY_RAW_HEADERS, that are not related to a
-- specific header.

global constant
  HTTP_QUERY_MIME_VERSION              = 0,
  -- Receives the version of the MIME protocol that was used to construct the
  -- message.
  HTTP_QUERY_CONTENT_TYPE              = 1,
  -- Receives the content type of the resource (such as text/html).
  HTTP_QUERY_CONTENT_TRANSFER_ENCODING = 2,
  -- Receives the additional content coding that has been applied to the
  -- resource.
  HTTP_QUERY_CONTENT_ID                = 3,
  -- Retrieves the content identification.
  HTTP_QUERY_CONTENT_DESCRIPTION       = 4,
  -- Obsolete. Maintained for legacy application compatibility only.
  HTTP_QUERY_CONTENT_LENGTH            = 5,
  -- Retrieves the size of the resource, in bytes.
  HTTP_QUERY_CONTENT_LANGUAGE          = 6,
  -- Retrieves the language that the content is in.
  HTTP_QUERY_ALLOW                     = 7,
  -- Receives the HTTP verbs supported by the server.
  HTTP_QUERY_PUBLIC                    = 8,
  -- Receives methods available at this server.
  HTTP_QUERY_DATE                      = 9,
  -- Receives the date and time at which the message was originated.
  HTTP_QUERY_EXPIRES                   = 10,
  -- Receives the date and time after which the resource should be considered
  -- outdated.
  HTTP_QUERY_LAST_MODIFIED             = 11,
  -- Receives the date and time at which the server believes the resource was
  -- last modified.
  HTTP_QUERY_MESSAGE_ID                = 12,
  -- No longer supported.
  HTTP_QUERY_URI                       = 13,
  -- Receives some or all of the Uniform Resource Identifiers (URIs) by which
  -- the Request-URI resource can be identified.
  HTTP_QUERY_DERIVED_FROM              = 14,
  -- No longer supported.
  HTTP_QUERY_COST                      = 15,
  -- No longer supported.
  HTTP_QUERY_LINK                      = 16,
  -- Obsolete. Maintained for legacy application compatibility only.
  HTTP_QUERY_PRAGMA                    = 17,
  -- Receives the implementation-specific directives that might apply to any
  -- recipient along the request/response chain.
  HTTP_QUERY_VERSION                   = 18,
  -- Receives the last response code returned by the server.
  HTTP_QUERY_STATUS_CODE               = 19,
  -- Receives the status code returned by the server. For more information and a
  -- list of possible values, see HTTP Status Codes.
  HTTP_QUERY_STATUS_TEXT               = 20,
  -- Receives any additional text returned by the server on the response line.
  HTTP_QUERY_RAW_HEADERS               = 21,
  -- Receives all the headers returned by the server. Each header is terminated
  -- by "\0". An additional "\0" terminates the list of headers.
  HTTP_QUERY_RAW_HEADERS_CRLF          = 22,
  -- Receives all the headers returned by the server. Each header is separated
  -- by a carriage return/line feed (CR/LF) sequence.
  HTTP_QUERY_CONNECTION                = 23,
  -- Retrieves any options that are specified for a particular connection and
  -- must not be communicated by proxies over further connections.
  HTTP_QUERY_ACCEPT                    = 24,
  -- Retrieves the acceptable media types for the response.
  HTTP_QUERY_ACCEPT_CHARSET            = 25,
  -- Retrieves the acceptable character sets for the response.
  HTTP_QUERY_ACCEPT_ENCODING           = 26,
  -- Retrieves the acceptable content-coding values for the response.
  HTTP_QUERY_ACCEPT_LANGUAGE           = 27,
  -- Retrieves the acceptable natural languages for the response.
  HTTP_QUERY_AUTHORIZATION             = 28,
  -- Retrieves the authorization credentials used for a request.
  HTTP_QUERY_CONTENT_ENCODING          = 29,
  -- Retrieves any additional content codings that have been applied to the
  -- entire resource.
  HTTP_QUERY_FORWARDED                 = 30,
  -- Obsolete. Maintained for legacy application compatibility only.
  HTTP_QUERY_FROM                      = 31,
  -- Retrieves the email address for the human user who controls the requesting
  -- user agent if the From header is given.
  HTTP_QUERY_IF_MODIFIED_SINCE         = 32,
  -- Retrieves the contents of the If-Modified-Since header.
  HTTP_QUERY_LOCATION                  = 33,
  -- Retrieves the absolute Uniform Resource Identifier (URI) used in a Location
  -- response-header.
  HTTP_QUERY_ORIG_URI                  = 34,
  -- Obsolete. Maintained for legacy application compatibility only.
  HTTP_QUERY_REFERER                   = 35,
  -- Receives the Uniform Resource Identifier (URI) of the resource where the
  -- requested URI was obtained.
  HTTP_QUERY_RETRY_AFTER               = 36,
  -- Retrieves the amount of time the service is expected to be unavailable.
  HTTP_QUERY_SERVER                    = 37,
  -- Retrieves data about the software used by the origin server to handle the
  -- request.
  HTTP_QUERY_TITLE                     = 38,
  -- Obsolete. Maintained for legacy application compatibility only.
  HTTP_QUERY_USER_AGENT                = 39,
  -- Retrieves data about the user agent that made the request.
  HTTP_QUERY_WWW_AUTHENTICATE          = 40,
  -- Retrieves the authentication scheme and realm returned by the server.
  HTTP_QUERY_PROXY_AUTHENTICATE        = 41,
  -- Retrieves the authentication scheme and realm returned by the proxy.
  HTTP_QUERY_ACCEPT_RANGES             = 42,
  -- Retrieves the types of range requests that are accepted for a resource.
  HTTP_QUERY_SET_COOKIE                = 43,
  -- Receives the value of the cookie set for the request.
  HTTP_QUERY_COOKIE                    = 44,
  -- Retrieves any cookies associated with the request.
  HTTP_QUERY_REQUEST_METHOD            = 45,
  -- Receives the HTTP verb that is being used in the request, typically GET or
  -- POST.
  HTTP_QUERY_REFRESH                   = 46,
  -- Obsolete. Maintained for legacy application compatibility only.
  HTTP_QUERY_CONTENT_DISPOSITION       = 47,
  -- Obsolete. Maintained for legacy application compatibility only.
  HTTP_QUERY_AGE                       = 48,
  -- Retrieves the Age response-header field, which contains the sender's
  -- estimate of the amount of time since the response was generated at the
  -- origin server.
  HTTP_QUERY_CACHE_CONTROL             = 49,
  -- Retrieves the cache control directives.
  HTTP_QUERY_CONTENT_BASE              = 50,
  -- Retrieves the base URI (Uniform Resource Identifier) for resolving relative
  -- URLs within the entity.
  HTTP_QUERY_CONTENT_LOCATION          = 51,
  -- Retrieves the resource location for the entity enclosed in the message.
  HTTP_QUERY_CONTENT_MD5               = 52,
  -- Retrieves an MD5 digest of the entity-body for the purpose of providing an
  -- end-to-end message integrity check (MIC) for the entity-body. For more
  -- information, see RFC1864, The Content-MD5 Header Field, at
  -- http://ftp.isi.edu/in-notes/rfc1864.txt.
  HTTP_QUERY_CONTENT_RANGE             = 53,
  -- Retrieves the location in the full entity-body where the partial entity-
  -- body should be inserted and the total size of the full entity-body.
  HTTP_QUERY_ETAG                      = 54,
  -- Retrieves the entity tag for the associated entity.
  HTTP_QUERY_HOST                      = 55,
  -- Retrieves the Internet host and port number of the resource being
  -- requested.
  HTTP_QUERY_IF_MATCH                  = 56,
  -- Retrieves the contents of the If-Match request-header field.
  HTTP_QUERY_IF_NONE_MATCH             = 57,
  -- Retrieves the contents of the If-None-Match request-header field.
  HTTP_QUERY_IF_RANGE                  = 58,
  -- Retrieves the contents of the If-Range request-header field. This header
  -- enables the client application to verify that the entity related to a
  -- partial copy of the entity in the client application cache has not been
  -- updated. If the entity has not been updated, send the parts that the client
  -- application is missing. If the entity has been updated, send the entire
  -- updated entity.
  HTTP_QUERY_IF_UNMODIFIED_SINCE       = 59,
  -- Retrieves the contents of the If-Unmodified-Since request-header field.
  HTTP_QUERY_MAX_FORWARDS              = 60,
  -- Retrieves the number of proxies or gateways that can forward the request to
  -- the next inbound server.
  HTTP_QUERY_PROXY_AUTHORIZATION       = 61,
  -- Retrieves the header that is used to identify the user to a proxy that
  -- requires authentication. This header can only be retrieved before the
  -- request is sent to the server.
  HTTP_QUERY_RANGE                     = 62,
  -- Retrieves the byte range of an entity.
  HTTP_QUERY_TRANSFER_ENCODING         = 63,
  -- Retrieves the type of transformation that has been applied to the message
  -- body so it can be safely transferred between the sender and recipient.
  HTTP_QUERY_UPGRADE                   = 64,
  -- Retrieves the additional communication protocols that are supported by the
  -- server.
  HTTP_QUERY_VARY                      = 65,
  -- Retrieves the header that indicates that the entity was selected from a
  -- number of available representations of the response using server-driven
  -- negotiation.
  HTTP_QUERY_VIA                       = 66,
  -- Retrieves the intermediate protocols and recipients between the user agent
  -- and the server on requests, and between the origin server and the client on
  -- responses.
  HTTP_QUERY_WARNING                   = 67,
  -- Retrieves additional data about the status of a response that might not be
  -- reflected by the response status code.
  HTTP_QUERY_EXPECT                    = 68,
  -- Retrieves the Expect header, which indicates whether the client application
  -- should expect 100 series responses.
  HTTP_QUERY_PROXY_CONNECTION          = 69,
  -- Retrieves the Proxy-Connection header.
  HTTP_QUERY_UNLESS_MODIFIED_SINCE     = 70,
  -- Retrieves the Unless-Modified-Since header.
  HTTP_QUERY_ECHO_REQUEST              = 71,
  -- Not currently implemented.
  HTTP_QUERY_ECHO_REPLY                = 72,
  -- Not currently implemented.
  HTTP_QUERY_ECHO_HEADERS              = 73,
  -- Not currently implemented.
  HTTP_QUERY_ECHO_HEADERS_CRLF         = 74,
  -- Not currently implemented.
  HTTP_QUERY_MAX                       = 78,
  -- Not a query flag. Indicates the maximum value of an HTTP_QUERY_* value.
  HTTP_QUERY_X_CONTENT_TYPE_OPTIONS    = 79,
  -- Retrieves the X-Content-Type-Options header value.
  HTTP_QUERY_P3P                       = 80,
  -- Retrieves the P3P header value.
  HTTP_QUERY_X_P2P_PEERDIST            = 81,
  -- Retrieves the X-P2P-PeerDist header value.
  HTTP_QUERY_TRANSLATE                 = 82,
  -- Retrieves the translate header value.
  HTTP_QUERY_X_UA_COMPATIBLE           = 83,
  -- Retrieves the X-UA-Compatible header value.
  HTTP_QUERY_DEFAULT_STYLE             = 84,
  -- Retrieves the Default-Style header value.
  HTTP_QUERY_X_FRAME_OPTIONS           = 85,
  -- Retrieves the X-Frame-Options header value.
  HTTP_QUERY_X_XSS_PROTECTION          = 86,
  -- Retrieves the X-XSS-Protection header value.
  HTTP_QUERY_CUSTOM                    = 65535,
  -- Causes HttpQueryInfo to search for the header name specified in lpvBuffer
  -- and store the header data in lpvBuffer.
  -- The modifier flags are used in conjunction with an attribute flag to modify
  -- the request. Modifier flags either modify the format of the data returned
  -- or indicate where HttpQueryInfo (or QueryInfo) should search for the data.

  HTTP_QUERY_FLAG_COALESCE        = #10000000,
  -- Not implemented.
  HTTP_QUERY_FLAG_NUMBER          = #20000000,
  -- Returns the data as a 32-bit number for headers whose value is a number,
  -- such as the status code.
  HTTP_QUERY_FLAG_SYSTEMTIME      = #40000000,
  -- Returns the header value as a SYSTEMTIME structure, which does not require
  -- the application to parse the data. Use for headers whose value is a date/
  -- time string, such as "Last-Modified-Time".
  HTTP_QUERY_FLAG_REQUEST_HEADERS = #80000000
--<constant>
--<name>HTTP_QUERY_MIME_VERSION</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CONTENT_TYPE</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CONTENT_TRANSFER_ENCODING</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CONTENT_ID</name>
--<value>3</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CONTENT_DESCRIPTION</name>
--<value>4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CONTENT_LENGTH</name>
--<value>5</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CONTENT_LANGUAGE</name>
--<value>6</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_ALLOW</name>
--<value>7</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_PUBLIC</name>
--<value>8</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_DATE</name>
--<value>9</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_EXPIRES</name>
--<value>10</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_LAST_MODIFIED</name>
--<value>11</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_MESSAGE_ID</name>
--<value>12</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_URI</name>
--<value>13</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_DERIVED_FROM</name>
--<value>14</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_COST</name>
--<value>15</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_LINK</name>
--<value>16</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_PRAGMA</name>
--<value>17</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_VERSION</name>
--<value>18</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_STATUS_CODE</name>
--<value>19</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_STATUS_TEXT</name>
--<value>20</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_RAW_HEADERS</name>
--<value>21</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_RAW_HEADERS_CRLF</name>
--<value>22</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CONNECTION</name>
--<value>23</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_ACCEPT</name>
--<value>24</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_ACCEPT_CHARSET</name>
--<value>25</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_ACCEPT_ENCODING</name>
--<value>26</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_ACCEPT_LANGUAGE</name>
--<value>27</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_AUTHORIZATION</name>
--<value>28</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CONTENT_ENCODING</name>
--<value>29</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_FORWARDED</name>
--<value>30</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_FROM</name>
--<value>31</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_IF_MODIFIED_SINCE</name>
--<value>32</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_LOCATION</name>
--<value>33</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_ORIG_URI</name>
--<value>34</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_REFERER</name>
--<value>35</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_RETRY_AFTER</name>
--<value>36</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_SERVER</name>
--<value>37</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_TITLE</name>
--<value>38</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_USER_AGENT</name>
--<value>39</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_WWW_AUTHENTICATE</name>
--<value>40</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_PROXY_AUTHENTICATE</name>
--<value>41</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_ACCEPT_RANGES</name>
--<value>42</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_SET_COOKIE</name>
--<value>43</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_COOKIE</name>
--<value>44</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_REQUEST_METHOD</name>
--<value>45</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_REFRESH</name>
--<value>46</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CONTENT_DISPOSITION</name>
--<value>47</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_AGE</name>
--<value>48</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CACHE_CONTROL</name>
--<value>49</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CONTENT_BASE</name>
--<value>50</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CONTENT_LOCATION</name>
--<value>51</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CONTENT_MD5</name>
--<value>52</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CONTENT_RANGE</name>
--<value>53</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_ETAG</name>
--<value>54</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_HOST</name>
--<value>55</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_IF_MATCH</name>
--<value>56</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_IF_NONE_MATCH</name>
--<value>57</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_IF_RANGE</name>
--<value>58</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_IF_UNMODIFIED_SINCE</name>
--<value>59</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_MAX_FORWARDS</name>
--<value>60</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_PROXY_AUTHORIZATION</name>
--<value>61</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_RANGE</name>
--<value>62</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_TRANSFER_ENCODING</name>
--<value>63</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_UPGRADE</name>
--<value>64</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_VARY</name>
--<value>65</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_VIA</name>
--<value>66</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_WARNING</name>
--<value>67</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_EXPECT</name>
--<value>68</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_PROXY_CONNECTION</name>
--<value>69</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_UNLESS_MODIFIED_SINCE</name>
--<value>70</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_ECHO_REQUEST</name>
--<value>71</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_ECHO_REPLY</name>
--<value>72</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_ECHO_HEADERS</name>
--<value>73</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_ECHO_HEADERS_CRLF</name>
--<value>74</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_MAX</name>
--<value>78</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_X_CONTENT_TYPE_OPTIONS</name>
--<value>79</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_P3P</name>
--<value>80</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_X_P2P_PEERDIST</name>
--<value>81</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_TRANSLATE</name>
--<value>82</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_X_UA_COMPATIBLE</name>
--<value>83</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_DEFAULT_STYLE</name>
--<value>84</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_X_FRAME_OPTIONS</name>
--<value>85</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_X_XSS_PROTECTION</name>
--<value>86</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_CUSTOM</name>
--<value>65535</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_FLAG_COALESCE</name>
--<value>#10000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_FLAG_NUMBER</name>
--<value>#20000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_FLAG_SYSTEMTIME</name>
--<value>#40000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_QUERY_FLAG_REQUEST_HEADERS</name>
--<value>#80000000</value>
--<desc></desc>
--</constant>
  -- Queries request headers only.


-- =============================================================================
--                            Cache Group Constants
-- =============================================================================

global constant
  CACHEGROUP_ATTRIBUTE_BASIC = #00000001,
  -- Retrieves the flags, type, and disk quota attributes of the cache group.
  -- This is used by the GetUrlCacheGroupAttribute function.
  CACHEGROUP_ATTRIBUTE_FLAG = #00000002,
  -- Sets or retrieves the flags associated with the cache group.
  -- This is used by the GetUrlCacheGroupAttribute and SetUrlCacheGroupAttribute functions.
  CACHEGROUP_ATTRIBUTE_GET_ALL = #FFFFFFFF,
  -- Retrieves all the attributes of the cache group.
  -- This is used by the GetUrlCacheGroupAttribute function.
  CACHEGROUP_ATTRIBUTE_GROUPNAME = #000000010,
  -- Sets or retrieves the group name of the cache group.
  -- This is used by the GetUrlCacheGroupAttribute and SetUrlCacheGroupAttribute functions.
  CACHEGROUP_ATTRIBUTE_QUOTA = #00000008,
  -- Sets or retrieves the disk quota associated with the cache group.
  -- This is used by the GetUrlCacheGroupAttribute and SetUrlCacheGroupAttribute functions.
  CACHEGROUP_ATTRIBUTE_STORAGE = #00000020,
  -- Sets or retrieves the group owner storage associated with the cache group.
  -- This is used by the GetUrlCacheGroupAttribute and SetUrlCacheGroupAttribute functions.
  CACHEGROUP_ATTRIBUTE_TYPE = #00000004,
  -- Sets or retrieves the cache group type.
  -- This is used by the GetUrlCacheGroupAttribute and SetUrlCacheGroupAttribute functions.
  CACHEGROUP_FLAG_FLUSHURL_ONDELETE = #00000002,
  -- Indicates that all the cache entries associated with the cache group should be deleted,
  -- unless the entry belongs to another cache group.
  CACHEGROUP_FLAG_GIDONLY = #00000004,
  -- Indicates that the function should only create a unique GROUPID for the cache group and
  -- not create the actual group.
  CACHEGROUP_FLAG_NONPURGEABLE = #00000001,
  -- Indicates that the cache group cannot be purged.
  CACHEGROUP_READWRITE_MASK = #0000003C,
  -- Sets the type, disk quota, group name, and owner storage attributes of the cache group.
  -- This is used by the SetUrlCacheGroupAttribute function.
  CACHEGROUP_SEARCH_ALL = #00000000,
  -- Indicates that all of the cache groups in the user's system should be enumerated.
  CACHEGROUP_SEARCH_BYURL = #00000001,
  -- Not currently implemented.
  CACHEGROUP_TYPE_INVALID = #00000001,
  -- Indicates that the cache group type is invalid.
  GROUP_OWNER_STORAGE_SIZE = #00000004,
  -- Length of the group owner storage array.
  GROUPNAME_MAX_LENGTH = #00000078
--<constant>
--<name>CACHEGROUP_ATTRIBUTE_BASIC</name>
--<value>#00000001</value>
--<desc></desc>
--</constant>
--<constant>
--<name>CACHEGROUP_ATTRIBUTE_FLAG</name>
--<value>#00000002</value>
--<desc></desc>
--</constant>
--<constant>
--<name>CACHEGROUP_ATTRIBUTE_GET_ALL</name>
--<value>#FFFFFFFF</value>
--<desc></desc>
--</constant>
--<constant>
--<name>CACHEGROUP_ATTRIBUTE_GROUPNAME</name>
--<value>#000000010</value>
--<desc></desc>
--</constant>
--<constant>
--<name>CACHEGROUP_ATTRIBUTE_QUOTA</name>
--<value>#00000008</value>
--<desc></desc>
--</constant>
--<constant>
--<name>CACHEGROUP_ATTRIBUTE_STORAGE</name>
--<value>#00000020</value>
--<desc></desc>
--</constant>
--<constant>
--<name>CACHEGROUP_ATTRIBUTE_TYPE</name>
--<value>#00000004</value>
--<desc></desc>
--</constant>
--<constant>
--<name>CACHEGROUP_FLAG_FLUSHURL_ONDELETE</name>
--<value>#00000002</value>
--<desc></desc>
--</constant>
--<constant>
--<name>CACHEGROUP_FLAG_GIDONLY</name>
--<value>#00000004</value>
--<desc></desc>
--</constant>
--<constant>
--<name>CACHEGROUP_FLAG_NONPURGEABLE</name>
--<value>#00000001</value>
--<desc></desc>
--</constant>
--<constant>
--<name>CACHEGROUP_READWRITE_MASK</name>
--<value>#0000003C</value>
--<desc></desc>
--</constant>
--<constant>
--<name>CACHEGROUP_SEARCH_ALL</name>
--<value>#00000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>CACHEGROUP_SEARCH_BYURL</name>
--<value>#00000001</value>
--<desc></desc>
--</constant>
--<constant>
--<name>CACHEGROUP_TYPE_INVALID</name>
--<value>#00000001</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GROUP_OWNER_STORAGE_SIZE</name>
--<value>#00000004</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GROUPNAME_MAX_LENGTH</name>
--<value>#00000078</value>
--<desc></desc>
--</constant>
  -- Maximum number of characters allowed for a cache group name.


-- =============================================================================
--                                Gopher Type Values
-- =============================================================================

-- The following list contains the possible Gopher type values.

global constant
  -- Gopher Type Values
  GOPHER_TYPE_ASK            = #40000000,
  -- Ask+ item.
  GOPHER_TYPE_BINARY         = #00000200,
  -- Binary file.
  GOPHER_TYPE_BITMAP         = #00004000,
  -- Bitmap file.
  GOPHER_TYPE_CALENDAR       = #00080000,
  -- Calendar file.
  GOPHER_TYPE_CSO            = #00000004,
  -- CSO telephone book server.
  GOPHER_TYPE_DIRECTORY      = #00000002,
  -- Directory of additional Gopher items.
  GOPHER_TYPE_DOS_ARCHIVE    = #00000020,
  -- MS-DOS archive file.
  GOPHER_TYPE_ERROR          = #00000008,
  -- Indicator of an error condition.
  GOPHER_TYPE_GIF            = #00001000,
  -- GIF graphics file.
  GOPHER_TYPE_GOPHER_PLUS    = #80000000,
  -- Gopher+ item.
  GOPHER_TYPE_HTML           = #00020000,
  -- HTML document.
  GOPHER_TYPE_IMAGE          = #00002000,
  -- Image file.
  GOPHER_TYPE_INDEX_SERVER   = #00000080,
  -- Index server.
  GOPHER_TYPE_INLINE         = #00100000,
  -- Inline file.
  GOPHER_TYPE_MAC_BINHEX     = #00000010,
  -- Macintosh file in BINHEX format.
  GOPHER_TYPE_MOVIE          = #00008000,
  -- Movie file.
  GOPHER_TYPE_PDF            = #00040000,
  -- PDF file.
  GOPHER_TYPE_REDUNDANT      = #00000400,
  -- Indicator of a duplicated server.
  -- The information contained within is a duplicate of the primary server.
  -- The primary server is defined as the last directory entry that did not
  -- have a GOPHER_TYPE_REDUNDANT type.
  GOPHER_TYPE_SOUND          = #00010000,
  -- Sound file.
  GOPHER_TYPE_TELNET         = #00000100,
  -- Telnet server.
  GOPHER_TYPE_TEXT_FILE      = #00000001,
  -- ASCII text file.
  GOPHER_TYPE_TN3270         = #00000800,
  -- TN3270 server.
  GOPHER_TYPE_UNIX_UUENCODED = #00000040,
  -- UUENCODED file.
  GOPHER_TYPE_UNKNOWN        = #20000000
--<constant>
--<name>GOPHER_TYPE_ASK</name>
--<value>#40000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_BINARY</name>
--<value>#00000200</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_BITMAP</name>
--<value>#00004000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_CALENDAR</name>
--<value>#00080000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_CSO</name>
--<value>#00000004</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_DIRECTORY</name>
--<value>#00000002</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_DOS_ARCHIVE</name>
--<value>#00000020</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_ERROR</name>
--<value>#00000008</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_GIF</name>
--<value>#00001000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_GOPHER_PLUS</name>
--<value>#80000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_HTML</name>
--<value>#00020000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_IMAGE</name>
--<value>#00002000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_INDEX_SERVER</name>
--<value>#00000080</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_INLINE</name>
--<value>#00100000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_MAC_BINHEX</name>
--<value>#00000010</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_MOVIE</name>
--<value>#00008000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_PDF</name>
--<value>#00040000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_REDUNDANT</name>
--<value>#00000400</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_SOUND</name>
--<value>#00010000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_TELNET</name>
--<value>#00000100</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_TEXT_FILE</name>
--<value>#00000001</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_TN3270</name>
--<value>#00000800</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_UNIX_UUENCODED</name>
--<value>#00000040</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GOPHER_TYPE_UNKNOWN</name>
--<value>#20000000</value>
--<desc></desc>
--</constant>
  -- Item type is unknown.

--------------------------------------------------------------------------------

-- global Type tWinInetDLLVersion
--   lMajorVersion As Long
--   lMinorVersion As Long
-- End Type

global constant
  MAX_PATH = 260,
  NO_ERROR = 0
--<constant>
--<name>MAX_PATH</name>
--<value>260</value>
--<desc></desc>
--</constant>
--<constant>
--<name>NO_ERROR</name>
--<value>0</value>
--<desc></desc>
--</constant>

-- Type FILETIME
--   dwLowDateTime As Long
--   dwHighDateTime As Long
-- End Type

-- Type WIN32_FIND_DATA
--   dwFileAttributes As Long
--   ftCreationTime As FILETIME
--   ftLastAccessTime As FILETIME
--   ftLastWriteTime As FILETIME
--   nFileSizeHigh As Long
--   nFileSizeLow As Long
--   dwReserved0 As Long
--   dwReserved1 As Long
--   cFileName As String * MAX_PATH
--   cAlternate As String * 14
-- End Type

-- global Type INTERNET_BUFFERS
--   dwStructSize As Long        -- used for API versioning. Set to sizeof(INTERNET_BUFFERS)
--   Next As Long                -- INTERNET_BUFFERS chain of buffers
--   lpcszHeader As Long         -- pointer to headers (may be NULL)
--   dwHeadersLength As Long     -- length of headers if not NULL
--   dwHeadersTotal As Long      -- size of headers if not enough buffer
--   lpvBuffer As Long           -- pointer to data buffer (may be NULL)
--   dwBufferLength As Long      -- length of data buffer if not NULL
--   dwBufferTotal As Long       -- total size of chunk, or content-length if not chunked
--   dwOffsetLow As Long         -- used for read-ranges (only used in HttpSendRequest2)
--   dwOffsetHigh As Long
-- End Type

-- Use registry access settings.
global constant
  INTERNET_OPEN_TYPE_PRECONFIG = 0,
  INTERNET_OPEN_TYPE_DIRECT    = 1,
  INTERNET_OPEN_TYPE_PROXY     = 3,
  INTERNET_INVALID_PORT_NUMBER = 0
--<constant>
--<name>INTERNET_OPEN_TYPE_PRECONFIG</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPEN_TYPE_DIRECT</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_OPEN_TYPE_PROXY</name>
--<value>3</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_INVALID_PORT_NUMBER</name>
--<value>0</value>
--<desc></desc>
--</constant>

global constant
  FTP_TRANSFER_TYPE_ASCII  = #1,
  FTP_TRANSFER_TYPE_BINARY = #2
--<constant>
--<name>FTP_TRANSFER_TYPE_ASCII</name>
--<value>#1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>FTP_TRANSFER_TYPE_BINARY</name>
--<value>#2</value>
--<desc></desc>
--</constant>

-- Number of the TCP/IP port on the server to connect to.
global constant
  INTERNET_DEFAULT_FTP_PORT    = 21,
  INTERNET_DEFAULT_GOPHER_PORT = 70,
  INTERNET_DEFAULT_HTTP_PORT   = 80,
  INTERNET_DEFAULT_HTTPS_PORT  = 443,
  INTERNET_DEFAULT_SOCKS_PORT  = 1080
--<constant>
--<name>INTERNET_DEFAULT_FTP_PORT</name>
--<value>21</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_DEFAULT_GOPHER_PORT</name>
--<value>70</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_DEFAULT_HTTP_PORT</name>
--<value>80</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_DEFAULT_HTTPS_PORT</name>
--<value>443</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_DEFAULT_SOCKS_PORT</name>
--<value>1080</value>
--<desc></desc>
--</constant>

-- Type of service to access.
global constant
  INTERNET_SERVICE_FTP    = 1,
  INTERNET_SERVICE_GOPHER = 2,
  INTERNET_SERVICE_HTTP   = 3
--<constant>
--<name>INTERNET_SERVICE_FTP</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SERVICE_GOPHER</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>INTERNET_SERVICE_HTTP</name>
--<value>3</value>
--<desc></desc>
--</constant>

--
-- flags field masks
--

global constant SECURITY_INTERNET_MASK = or_all({
--<constant>
--<name>SECURITY_INTERNET_MASK</name>
--<value>or_all({</value>
--<desc></desc>
--</constant>
  INTERNET_FLAG_IGNORE_CERT_CN_INVALID, INTERNET_FLAG_IGNORE_CERT_DATE_INVALID,
  INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS, INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP})

global constant INTERNET_FLAGS_MASK = or_all({
--<constant>
--<name>INTERNET_FLAGS_MASK</name>
--<value>or_all({</value>
--<desc></desc>
--</constant>
  INTERNET_FLAG_RELOAD, INTERNET_FLAG_RAW_DATA, INTERNET_FLAG_EXISTING_CONNECT,
  INTERNET_FLAG_ASYNC, INTERNET_FLAG_PASSIVE, INTERNET_FLAG_NO_CACHE_WRITE,
  INTERNET_FLAG_MAKE_PERSISTENT, INTERNET_FLAG_FROM_CACHE, INTERNET_FLAG_SECURE,
  INTERNET_FLAG_KEEP_CONNECTION, INTERNET_FLAG_NO_AUTO_REDIRECT,
  INTERNET_FLAG_READ_PREFETCH, INTERNET_FLAG_NO_COOKIES, INTERNET_FLAG_NO_AUTH,
  INTERNET_FLAG_CACHE_IF_NET_FAIL, SECURITY_INTERNET_MASK,
  INTERNET_FLAG_RESYNCHRONIZE, INTERNET_FLAG_HYPERLINK, INTERNET_FLAG_NO_UI,
  INTERNET_FLAG_PRAGMA_NOCACHE, INTERNET_FLAG_CACHE_ASYNC,
  INTERNET_FLAG_FORMS_SUBMIT, INTERNET_FLAG_NEED_FILE,
  INTERNET_FLAG_TRANSFER_BINARY, INTERNET_FLAG_TRANSFER_ASCII})

global constant
  INTERNET_OPTIONS_MASK = not INTERNET_FLAGS_MASK
--<constant>
--<name>INTERNET_OPTIONS_MASK</name>
--<value>not INTERNET_FLAGS_MASK</value>
--<desc></desc>
--</constant>

global constant
  -- Adds the header only if it does not already exist; otherwise, an error is returned.
  HTTP_ADDREQ_FLAG_ADD_IF_NEW = #10000000,
  -- Adds the header if it does not exist. Used with REPLACE.
  HTTP_ADDREQ_FLAG_ADD        = #20000000,
  -- Replaces or removes a header. If the header value is empty and the header is found,
  -- it is removed. If not empty, the header value is replaced
  HTTP_ADDREQ_FLAG_REPLACE    = #80000000
--<constant>
--<name>HTTP_ADDREQ_FLAG_ADD_IF_NEW</name>
--<value>#10000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_ADDREQ_FLAG_ADD</name>
--<value>#20000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>HTTP_ADDREQ_FLAG_REPLACE</name>
--<value>#80000000</value>
--<desc></desc>
--</constant>

-- InternetErrorDlg constants
global constant
  FLAGS_ERROR_UI_FILTER_FOR_ERRORS    = #1,
  FLAGS_ERROR_UI_FLAGS_CHANGE_OPTIONS = #2,
  FLAGS_ERROR_UI_FLAGS_GENERATE_DATA  = #4,
  FLAGS_ERROR_UI_FLAGS_NO_UI          = #8,
  FLAGS_ERROR_UI_SERIALIZE_DIALOGS    = #10
--<constant>
--<name>FLAGS_ERROR_UI_FILTER_FOR_ERRORS</name>
--<value>#1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>FLAGS_ERROR_UI_FLAGS_CHANGE_OPTIONS</name>
--<value>#2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>FLAGS_ERROR_UI_FLAGS_GENERATE_DATA</name>
--<value>#4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>FLAGS_ERROR_UI_FLAGS_NO_UI</name>
--<value>#8</value>
--<desc></desc>
--</constant>
--<constant>
--<name>FLAGS_ERROR_UI_SERIALIZE_DIALOGS</name>
--<value>#10</value>
--<desc></desc>
--</constant>

global constant
  GENERIC_READ  = #80000000,
  GENERIC_WRITE = #40000000
--<constant>
--<name>GENERIC_READ</name>
--<value>#80000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>GENERIC_WRITE</name>
--<value>#40000000</value>
--<desc></desc>
--</constant>

-- =============================================================================
--                   Win32 Internet API Function Prototypes
-- =============================================================================

-- The following section contains the function prototypes used by the Win32(r) Internet functions.
-- *    GOPHER_ATTRIBUTE_ENUMERATOR
-- *    INTERNET_STATUS_CALLBACK

--------------------------------------------------------------------------------
-- GOPHER_ATTRIBUTE_ENUMERATOR Prototype
-- Prototype for a callback function that processes attribute information from a Gopher server. This
-- callback function is installed by a call to the GopherGetAttribute function.
--
-- Syntax
-- typedef BOOL (CALLBACK * GOPHER_ATTRIBUTE_ENUMERATOR)(
-- LPGOPHER_ATTRIBUTE_TYPE lpAttributeInfo,
-- DWORD dwError
-- );
-- Parameters
-- lpAttributeInfo
-- Address of a buffer that contains a single GOPHER_ATTRIBUTE_TYPE structure. The
-- lpBuffer parameter to GopherGetAttribute is used for storing this structure. The lpBuffer size
-- must be equal to or greater than the value of MIN_GOPHER_ATTRIBUTE_LENGTH.
--
-- dwError
-- Unsigned long integer value that contains the error value. This parameter is NO_ERROR if the
-- attribute was parsed and written to the buffer successfully. If a problem was encountered, an
-- error value is returned.
--
-- Return Value
-- Returns TRUE to continue the enumeration, or FALSE to immediately stop it. This function is
-- primarily used for returning the results of a Gopher+ ASK item.
--global function ()
--  integer result

--  result = exec_func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function

--------------------------------------------------------------------------------
-- INTERNET_STATUS_CALLBACK Prototype
-- Prototype for an application-defined status callback function.
-- Syntax
-- typedef VOID (CALLBACK * INTERNET_STATUS_CALLBACK)(
-- IN HINTERNET hInternet,
-- IN DWORD_PTR dwContext,
-- IN DWORD dwInternetStatus,
-- IN LPVOID lpvStatusInformation,
-- IN DWORD dwStatusInformationLength
-- );
-- Parameters
-- hInternet
-- Handle for which the callback function is being called.
--
-- dwContext
-- Address of an unsigned long integer value that contains the application-defined context value
-- associated with hInternet.
--
-- dwInternetStatus
-- Unsigned long integer value that contains the status code that indicates why the callback function
-- is being called. This can be one of the following values:
-- INTERNET_STATUS_CLOSING_CONNECTION
--     Closing the connection to the server. The
-- lpvStatusInformation parameter is NULL.
--
-- INTERNET_STATUS_CONNECTED_TO_SERVER
--     Successfully connected to the socket address
-- (SOCKADDR) pointed to by
-- lpvStatusInformation.
--
-- INTERNET_STATUS_CONNECTING_TO_SERVER
--     Connecting to the socket address
-- (SOCKADDR) pointed to by
-- lpvStatusInformation.
--
-- INTERNET_STATUS_CONNECTION_CLOSED
--     Successfully closed the connection to the
-- server. The lpvStatusInformation parameter is
-- NULL.
--
-- INTERNET_STATUS_CTL_RESPONSE_RECEIVED
--     Not currently implemented.
--
-- INTERNET_STATUS_DETECTING_PROXY
--     Notifies the client application that a proxy has
-- been detected.
--
-- INTERNET_STATUS_HANDLE_CLOSING
--     This handle value has been terminated.
--
-- INTERNET_STATUS_HANDLE_CREATED
--     Used by InternetConnect to indicate it has
-- created the new handle. This lets the
-- application call InternetCloseHandle from
-- another thread, if the connect is taking too
-- long. The lpvStatusInformation parameter
-- contains the address of an
-- INTERNET_ASYNC_RESULT structure.
--
-- INTERNET_STATUS_INTERMEDIATE_RESPONSE
--     Received an intermediate (100 level) status
-- code message from the server.
--
-- INTERNET_STATUS_NAME_RESOLVED
--     Successfully found the IP address of the name
-- contained in lpvStatusInformation.
--
-- INTERNET_STATUS_PREFETCH
--     Not currently implemented.
--
-- INTERNET_STATUS_RECEIVING_RESPONSE
--     Waiting for the server to respond to a request.
-- The lpvStatusInformation parameter is NULL.
--
-- INTERNET_STATUS_REDIRECT
--     An HTTP request is about to automatically
-- redirect the request. The lpvStatusInformation
-- parameter points to the new URL. At this
-- point, the application can read any data
-- returned by the server with the redirect
-- response and can query the response headers.
-- It can also cancel the operation by closing the
-- handle. This callback is not made if the
-- original request specified
-- INTERNET_FLAG_NO_AUTO_REDIRECT.
--
-- INTERNET_STATUS_REQUEST_COMPLETE
--     An asynchronous operation has been
-- completed. The lpvStatusInformation
-- parameter contains the address of an
-- INTERNET_ASYNC_RESULT structure.
--
-- INTERNET_STATUS_REQUEST_SENT
--     Successfully sent the information request to
-- the server. The lpvStatusInformation
-- parameter points to a DWORD containing the
-- number of bytes sent.
--
-- INTERNET_STATUS_RESOLVING_NAME
--     Looking up the IP address of the name
-- contained in lpvStatusInformation.
--
-- INTERNET_STATUS_RESPONSE_RECEIVED
--     Successfully received a response from the
-- server. The lpvStatusInformation parameter
-- points to a DWORD containing the number of
-- bytes received.
--
-- INTERNET_STATUS_SENDING_REQUEST
--     Sending the information request to the server.
-- The lpvStatusInformation parameter is NULL.
--
-- INTERNET_STATUS_STATE_CHANGE    Moved between a secure (HTTPS) and a
-- nonsecure (HTTP) site. This can be one of the
-- following values:
-- INTERNET_STATE_CONNECTED    Connected state (mutually exclusive with
-- disconnected state).
--
-- INTERNET_STATE_DISCONNECTED    Disconnected state. No network connection
-- could be established.
--
-- INTERNET_STATE_DISCONNECTED_BY_USER
--     Disconnected by user request.
--
-- INTERNET_STATE_IDLE    No network requests are being made by the
-- Win32(r) Internet functions.
--
-- INTERNET_STATE_BUSY    Network requests are being made by the
-- Win32 Internet functions.
--
-- INTERNET_STATUS_USER_INPUT_REQUIRED
--     The request requires user input to be
-- completed.
--
-- lpvStatusInformation
-- Address of a buffer that contains information pertinent to this call to the callback function.
--
-- dwStatusInformationLength
-- Unsigned long integer value that contains the size, in TCHAR, of the lpvStatusInformation
-- buffer.
--
-- Remarks
-- Because callbacks are made during processing of the request, the application should spend as little
-- time as possible in the callback function to avoid degrading data throughput on the network. For
-- example, displaying a dialog box in a callback function can be such a lengthy operation that the
-- server terminates the request.
-- The callback function can be called in a thread context different from the thread that initiated the
-- request.
--global function ()
--  integer result

--  result = exec_func(x, {})
--  if result =  then
--    error_msg = " failed"
--  end if
--  return result
--end function


-- =============================================================================
--                         Win32 Internet API Structures
-- =============================================================================

-- The following section describes the Win32(r) Internet function data structures and their uses.

--------------------------------------------------------------------------------
-- GOPHER_ATTRIBUTE_TYPE Structure
--
-- Contains the relevant information of a single Gopher attribute for an object.
--
-- Syntax
--   typedef struct {
--     DWORD CategoryId
--     DWORD AttributeId
--     union {
--       GOPHER_ADMIN_ATTRIBUTE Admin;
--       GOPHER_MOD_DATE_ATTRIBUTE ModDate;
--       GOPHER_SCORE_ATTRIBUTE Score;
--       GOPHER_SCORE_RANGE_ATTRIBUTE ScoreRange;
--       GOPHER_SITE_ATTRIBUTE Site;
--       GOPHER_ORGANIZATION_ATTRIBUTE Organization;
--       GOPHER_LOCATION_ATTRIBUTE Location;
--       GOPHER_GEOGRAPHICAL_LOCATION_ATTRIBUTE GeographicalLocation;
--       GOPHER_TIMEZONE_ATTRIBUTE TimeZone;
--       GOPHER_PROVIDER_ATTRIBUTE Provider;
--       GOPHER_VERSION_ATTRIBUTE Version;
--       GOPHER_ABSTRACT_ATTRIBUTE Abstract;
--       GOPHER_VIEW_ATTRIBUTE View;
--       GOPHER_VERONICA_ATTRIBUTE Veronica;
--       GOPHER_ASK_ATTRIBUTE_TYPE Ask;
--       GOPHER_UNKNOWN_ATTRIBUTE Unknown;
--     } AttributeType;
--   } GOPHER_ATTRIBUTE_TYPE, *LPGOPHER_ATTRIBUTE_TYPE;
--
-- Members
--   CategoryId
--     Unsigned long integer value that contains the Gopher name for the attribute. The possible values
--     include:
--     GOPHER_CATEGORY_ID_ABSTRACT
--     GOPHER_CATEGORY_ID_ADMIN
--     GOPHER_CATEGORY_ID_ALL
--     GOPHER_CATEGORY_ID_INFO
--     GOPHER_CATEGORY_ID_UNKNOWN
--     GOPHER_CATEGORY_ID_VERONICA
--     GOPHER_CATEGORY_ID_VIEWS
--
--   AttributeId
--     Unsigned long integer value that contains the identifier of the structure contained in the
--     AttributeType member. The possible values include:
--     GOPHER_ATTRIBUTE_ID_ABSTRACT
--     GOPHER_ATTRIBUTE_ID_ADMIN
--     GOPHER_ATTRIBUTE_ID_GEOG
--     GOPHER_ATTRIBUTE_ID_LOCATION
--     GOPHER_ATTRIBUTE_ID_MOD_DATE
--     GOPHER_ATTRIBUTE_ID_ORG
--     GOPHER_ATTRIBUTE_ID_PROVIDER
--     GOPHER_ATTRIBUTE_ID_RANGE
--     GOPHER_ATTRIBUTE_ID_SCORE
--     GOPHER_ATTRIBUTE_ID_SITE
--     GOPHER_ATTRIBUTE_ID_TIMEZONE
--     GOPHER_ATTRIBUTE_ID_TREEWALK
--     GOPHER_ATTRIBUTE_ID_TTL
--     GOPHER_ATTRIBUTE_ID_UNKNOWN
--     GOPHER_ATTRIBUTE_ID_VERSION
--     GOPHER_ATTRIBUTE_ID_VIEW
--
--   AttributeType
--     Union containing the actual setting for the Gopher attribute. The specific value of
--     AttributeType depends on the AttributeId member. The definitions of the various attribute
--     structures are available in Wininet.h.

--------------------------------------------------------------------------------
-- GOPHER_FIND_DATA Structure
-- Contains information retrieved by the GopherFindFirstFile and InternetFindNextFile functions.
--
-- Syntax
-- typedef struct {
-- TCHAR DisplayString[MAX_GOPHER_DISPLAY_TEXT + 1];
-- DWORD GopherType;
-- DWORD SizeLow;
-- DWORD SizeHigh;
-- FILETIME LastModificationTime;
-- TCHAR Locator[MAX_GOPHER_LOCATOR_LENGTH + 1];
-- } GOPHER_FIND_DATA, FAR *LPGOPHER_FIND_DATA;
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Members
-- DisplayString
-- Array of characters that contains the friendly name of an object. An application can display this
-- string to allow the user to select the object.
--
-- GopherType
-- Unsigned long integer value that contains the mask of flags that describe the item returned. This
-- can be one of the following values:
-- GOPHER_TYPE_ASK    Ask+ item.
--
-- GOPHER_TYPE_BINARY    Binary file.
--
-- GOPHER_TYPE_BITMAP    Bitmap file.
--
-- GOPHER_TYPE_CALENDAR    Calendar file.
--
-- GOPHER_TYPE_CSO    CSO telephone book server.
--
-- GOPHER_TYPE_DIRECTORY    Directory of additional Gopher items.
--
-- GOPHER_TYPE_DOS_ARCHIVE    MS-DOS archive file.
--
-- GOPHER_TYPE_ERROR    Indicator of an error condition.
--
-- GOPHER_TYPE_GIF    GIF graphics file.
--
-- GOPHER_TYPE_GOPHER_PLUS    Gopher+ item.
--
-- GOPHER_TYPE_HTML    HTML document.
--
-- GOPHER_TYPE_IMAGE    Image file.
--
-- GOPHER_TYPE_INDEX_SERVER    Index server.
--
-- GOPHER_TYPE_INLINE    Inline file.
--
-- GOPHER_TYPE_MAC_BINHEX    Macintosh file in BINHEX format.
--
-- GOPHER_TYPE_MOVIE    Movie file.
--
-- GOPHER_TYPE_PDF    PDF file.
--
-- GOPHER_TYPE_REDUNDANT    Indicator of a duplicated server. The
-- information contained within is a duplicate of
-- the primary server. The primary server is
-- defined as the last directory entry that did not
-- have a GOPHER_TYPE_REDUNDANT type.
--
-- GOPHER_TYPE_SOUND    Sound file.
--
-- GOPHER_TYPE_TELNET    Telnet server.
--
-- GOPHER_TYPE_TEXT_FILE    ASCII text file.
--
-- GOPHER_TYPE_TN3270    TN3270 server.
--
-- GOPHER_TYPE_UNIX_UUENCODED    UUENCODED file.
--
-- GOPHER_TYPE_UNKNOWN    Item type is unknown.
--
-- SizeLow
-- Unsigned long integer value that contains the low 32 bits of the file size.
--
-- SizeHigh
-- Unsigned long integer value that contains the high 32 bits of the file size.
--
-- LastModificationTime
-- FILETIME value that contains the time when the file was last modified.
--
-- Locator
-- Array of characters that identifies the file. An application can pass the locator string to
-- GopherOpenFile or GopherFindFirstFile.

--------------------------------------------------------------------------------
-- HTTP_VERSION_INFO Structure
-- Contains the global HTTP version.
--
-- Syntax
-- typedef  struct{
-- DWORD dwMajorVersion;
-- DWORD dwMinorVersion;
-- Members
-- dwMajorVersion
-- Unsigned long integer value that contains the major version number. Must be 1.
--
-- dwMinorVersion
-- Unsigned long integer value that contains the minor version number. Can be either 1 or 0.
--
--
--
-- INTERNET_ASYNC_RESULT Structure
-- Contains the result of a call to an asynchronous function. This structure is used with
-- INTERNET_STATUS_CALLBACK.
--
-- Syntax
-- typedef struct {
--     DWORD dwResult;
--     DWORD dwError;
-- } INTERNET_ASYNC_RESULT, * LPINTERNET_ASYNC_RESULT;
-- Members
-- dwResult
-- Unsigned long integer value that references an HINTERNET handle, unsigned long integer, or
-- Boolean return code from an asynchronous function.
--
-- dwError
-- Unsigned long integer value that contains the error message if dwResult indicates that the
-- function failed. If the operation succeeded, this member usually contains ERROR_SUCCESS.
--
-- Remarks
-- The value of dwResult is determined by the value of the dwInternetStatus of the status callback
-- function.
--
-- Value of dwInternetStatus    Value of dwResult
-- INTERNET_STATUS_HANDLE_CREATED    Address of the HINTERNET handle
-- INTERNET_STATUS_REQUEST_COMPLETE    Address of the HINTERNET handle

--------------------------------------------------------------------------------
-- INTERNET_AUTH_NOTIFY_DATA Structure
-- Contains the notification data for an authentication request.
--
-- Syntax
-- typedef struct {
-- DWORD cbStruct;
-- DWORD dwOptions;
-- PFN_PUTHNOTIFY pfnNotify;
-- DWORD_PTR dwContext;
-- } INTERNET_AUTH_NOTIFY_DATA;
-- Members
-- cbStruct
-- Unsigned long integer value that contains the size of the structure.
--
-- dwOptions
-- Reserved; must be set to zero.
--
-- pfnNotify
-- Notification callback to retry InternetErrorDlg.
--
-- dwContext
-- Address of an unsigned long integer value that contains an application-defined value that is used
-- to identify the application context to pass to the notification function.

--------------------------------------------------------------------------------
-- INTERNET_BUFFERS Structure
-- Contains both the data and header information.
--
-- Syntax
-- typedef struct _INTERNET_BUFFERS {
-- DWORD dwStructSize;
-- struct _INTERNET_BUFFERS% * Next;
-- LPCTSTR lpcszHeader;
-- DWORD dwHeadersLength;
-- DWORD dwHeadersTotal;
-- LPVOID lpvBuffer;
-- DWORD dwBufferLength;
-- DWORD dwBufferTotal;
-- DWORD dwOffsetLow;
-- DWORD dwOffsetHigh;
-- } INTERNET_BUFFERS, * LPINTERNET_BUFFERS;
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Members
-- dwStructSize
-- Unsigned long integer value used for API versioning. This is set to the size of the
-- INTERNET_BUFFERS structure.
--
-- Next
-- Address of the next INTERNET_BUFFERS structure.
--
-- lpcszHeader
-- Address of a string value that contains the headers. This value can be NULL.
--
-- dwHeadersLength
-- Unsigned long integer value that contains the length of the headers, in TCHARs, if lpcszHeader
-- is not NULL.
--
-- dwHeadersTotal
-- Unsigned long integer value that contains the size of the headers if there is not enough memory
-- in the buffer.
--
-- lpvBuffer
-- Address of the data buffer.
--
-- dwBufferLength
-- Unsigned long integer value that contains the length of the buffer, in TCHARs, if lpvBuffer is
-- not NULL.
--
-- dwBufferTotal
-- Unsigned long integer value that contains the total size of the resource.
--
-- dwOffsetLow
-- Unsigned long integer value that is used for read ranges.
--
-- dwOffsetHigh
-- Unsigned long integer value that is used for read ranges.

--------------------------------------------------------------------------------
-- INTERNET_CACHE_ENTRY_INFO Structure
-- Contains information about an entry in the Internet cache.
--
-- Syntax
-- typedef struct _INTERNET_CACHE_ENTRY_INFO {
-- DWORD dwStructSize;
-- LPTSTR lpszSourceUrlName;
-- LPTSTR lpszLocalFileName;
-- DWORD CacheEntryType;
-- DWORD dwUseCount;
-- DWORD dwHitRate;
-- DWORD dwSizeLow;
-- DWORD dwSizeHigh;
-- FILETIME LastModifiedTime;
-- FILETIME ExpireTime;
-- FILETIME LastAccessTime;
-- FILETIME LastSyncTime;
-- LPBYTE lpHeaderInfo;
-- DWORD dwHeaderInfoSize;
-- LPTSTR lpszFileExtension;
-- union (DWORD dwReserved; DWORD dwExemptDelta;)
-- } INTERNET_CACHE_ENTRY_INFO, *LPINTERNET_CACHE_ENTRY_INFO;
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Members
-- dwStructSize
-- Unsigned long integer value that contains the size, in TCHAR, of this structure. This value can
-- be used to help determine the version of the cache system.
--
-- lpszSourceUrlName
-- Address of a string that contains the URL name. The string occupies memory area at the end of
-- this structure.
--
-- lpszLocalFileName
-- Address of a string that contains the local file name. The string occupies memory area at the end
-- of this structure.
--
-- CacheEntryType
-- Unsigned long integer value that contains the cache type bitmask. Currently, the cache entry type
-- value of resources from the Internet is equal to zero. For History and Cookie entries, the cache
-- entry type is a combination of two values. One value determines how the cache entry is handled;
-- the second value indicates what is being cached.
-- The value that determines how the cache entry is handled can be one of the following:
-- EDITED_CACHE_ENTRY    Cache entry has been altered since it was downloaded from
-- the Internet.
--
-- NORMAL_CACHE_ENTRY    Normal cache entry; can be deleted to recover space for
-- new entries.
--
-- SPARSE_CACHE_ENTRY    Not currently implemented.
--
-- STICKY_CACHE_ENTRY     Sticky cache entry that is exempt from scavenging for
-- amount of time specified by dwExemptDelta. The default
-- value set by CommitUrlCacheEntry is one day.
--
-- TRACK_OFFLINE_CACHE_ENTRY
-- TRACK_ONLINE_CACHE_ENTRY
--         The value that indicates what is being cached can be one of
-- the following:
-- COOKIE_CACHE_ENTRY    Cookie cache entry.
--
-- URLHISTORY_CACHE_ENTRY
--     Visited link cache entry.
--
-- dwUseCount
-- Unsigned long integer value that contains the current user count of the cache entry.
--
-- dwHitRate
-- Unsigned long integer value that contains the number of times the cache entry was retrieved.
--
-- dwSizeLow
-- Unsigned long integer value that contains the low order of the file size in TCHAR.
--
-- dwSizeHigh
-- Unsigned long integer value that contains the high-order double word of the file size in TCHAR.
--
-- LastModifiedTime
-- FILETIME structure that contains the last modified time of this URL, in Greenwich mean time
-- format.
--
-- ExpireTime
-- FILETIME structure that contains the expiration time of this file, in Greenwich mean time
-- format.
--
-- LastAccessTime
-- FILETIME structure that contains the last accessed time, in Greenwich mean time format.
--
-- LastSyncTime
-- FILETIME structure that contains the last time the cache was synchronized.
--
-- lpHeaderInfo
-- Address of a buffer that contains the header information. The buffer occupies memory at the end
-- of this structure.
--
-- dwHeaderInfoSize
-- Unsigned long integer value that contains the size of the lpHeaderInfo buffer in TCHAR.
--
-- lpszFileExtension
-- Address of a string that contains the file extension used to retrieve the data as a file. The string
-- occupies memory area at the end of this structure.
--
-- union (DWORD dwReserved; DWORD dwExemptDelta;)
-- Unsigned long integer value that contains either exemption time, in seconds, from the last
-- accessed time, or zero.
--
-- Remarks
-- There is no cache entry size limit, so applications that need to enumerate the cache must be
-- prepared to allocate variable-sized buffers. For more information, see Appendix B: Using
-- Buffers.

--------------------------------------------------------------------------------
-- INTERNET_CACHE_GROUP_INFO Structure
-- Contains the information for a particular cache group.
-- Syntax
-- typedef struct _INTERNET_CACHE_GROUP_INFO{
-- DWORD dwGroupSize;
-- DWORD dwGroupFlags;
-- DWORD dwGroupType;
-- DWORD dwDiskUsage;
-- DWORD dwDiskQuota;
-- DWORD dwOwnerStorage[GROUP_OWNER_STORAGE_SIZE];
-- TCHAR szGroupName[GROUPNAME_MAX_LENGTH];
-- } INTERNET_CACHE_GROUP_INFO, *LPINTERNET_CACHE_GROUP_INFO;
-- Members
-- dwGroupSize
-- Unsigned long integer value that contains the size, in TCHAR, of the structure.
--
-- dwGroupFlags
-- Unsigned long integer value that contains the group flags. Currently, the only value defined is
-- CACHEGROUP_FLAG_NONPURGEABLE, which indicates that the cache entries in this
-- group will not be removed by the cache manager.
--
-- dwGroupType
-- Unsigned long integer value that indicates the group type. Currently, the only value defined is
-- CACHEGROUP_TYPE_INVALID.
--
-- dwDiskUsage
-- Unsigned long integer value that contains the current disk usage, in kilobytes, of this cache
-- group.
--
-- dwDiskQuota
-- Unsigned long integer value that indicates the disk quota, in kilobytes, for this cache group.
--
-- dwOwnerStorage
-- Array of unsigned long integers that can be used by a client application to store information
-- related to the group.
--
-- szGroupName
-- Array of characters that contains the group name.

--------------------------------------------------------------------------------
-- INTERNET_CACHE_TIMESTAMPS Structure
-- Contains the LastModified and Expire times for a resource stored in the Internet cache.
--
-- Syntax
-- typedef struct _INTERNET_CACHE_TIMESTAMPS{
-- FILETIME ftExpires;
-- FILETIME ftLastModified;
-- } INTERNET_CACHE_TIMESTAMPS, *LPINTERNET_CACHE_TIMESTAMPS;
-- Members
-- ftExpires
-- FILETIME value that contains the Expires time.
--
-- ftLastModified
-- FILETIME value that contains the LastModified time.
--
-- Remarks
-- This structure is returned in the buffer when calling InternetQueryOption with the
-- INTERNET_OPTION_CACHE_TIMESTAMPS flag.

--------------------------------------------------------------------------------
-- INTERNET_CERTIFICATE_INFO Structure
-- Contains certificate information returned from the server. This structure is used by the
-- InternetQueryOption function.
--
-- Syntax
-- typedef struct {
-- FILETIME ftExpiry;
-- FILETIME ftStart;
-- LPTSTR lpszSubjectInfo;
-- LPTSTR lpszIssuerInfo;
-- LPTSTR lpszProtocolName;
-- LPTSTR lpszSignatureAlgName;
-- LPTSTR lpszEncryptionAlgName;
-- DWORD dwKeySize;
-- } INTERNET_CERTIFICATE_INFO, * LPINTERNET_CERTIFICATE_INFO;
-- Members
-- ftExpiry
-- FILETIME structure that contains the date the certificate expires.
--
-- ftStart
-- FILETIME structure that contains the date the certificate becomes valid.
--
-- lpszSubjectInfo
-- Address of a buffer that contains the name of the organization, site, and server for which the
-- certificate was issued.
--
-- lpszIssuerInfo
-- Address of a buffer that contains the name of the organization, site, and server that issued the
-- certificate.
--
-- lpszProtocolName
-- Address of a buffer that contains the name of the protocol used to provide the secure connection.
--
-- lpszSignatureAlgName
-- Address of a buffer that contains the name of the algorithm used for signing the certificate.
--
-- lpszEncryptionAlgName
-- Address of a buffer that contains the name of the algorithm used for doing encryption over the
-- secure channel (SSL/PCT) connection.
--
-- dwKeySize
-- Unsigned long integer value that contains the size, in TCHAR, of the key.
--
-- Remarks
-- Applications requesting this information must free pointers that are allocated and placed in the
-- returned structure.

--------------------------------------------------------------------------------
-- INTERNET_CONNECTED_INFO Structure
-- Contains the information to set the global online/offline state.
--
-- Syntax
-- typedef struct {
-- DWORD dwConnectedState;
-- DWORD dwFlags
-- } INTERNET_CONNECTED_INFO, * LPINTERNET_CONNECTED_INFO;
-- Members
-- dwConnectedState
-- Unsigned long integer value that contains the state information. This can be one of the following
-- values:
-- INTERNET_STATE_CONNECTED    Connected to network. Replaces
-- INTERNET_STATE_ONLINE.
--
-- INTERNET_STATE_DISCONNECTED    Disconnected from network. Replaces
-- INTERNET_STATE_OFFLINE.
--
-- INTERNET_STATE_DISCONNECTED_BY_USER
--     Disconnected by user request. Replaces
-- INTERNET_STATE_OFFLINE_USER.
--
-- INTERNET_STATE_IDLE    No network requests are being made by
-- Win32(r) Internet functions.
--
-- INTERNET_STATE_BUSY    Network requests are being made by the
-- Win32 Internet functions.
--
-- dwFlags
-- Unsigned long integer value that controls the transition between states. This can be
-- ISO_FORCE_DISCONNECTED, which puts Win32 Internet functions into offline mode. All
-- outstanding requests will be aborted with a canceled error.

--------------------------------------------------------------------------------
-- INTERNET_PER_CONN_OPTION Structure
-- Contains the value of an option.
--
-- Syntax
-- typedef struct {
-- DWORD   dwOption;
-- union {
-- DWORD    dwValue;
-- LPTSTR   pszValue;
-- FILETIME ftValue;
-- } Value;
-- } INTERNET_PER_CONN_OPTION, *LPINTERNET_PER_CONN_OPTION;
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Members
-- dwOption
-- Unsigned long integer value that contains the option to be queried or set. This can be one of the
-- following values:
-- INTERNET_PER_CONN_AUTOCONFIG_URL
--     Sets or retrieves a string containing the URL to
-- the automatic configuration script.
--
-- INTERNET_PER_CONN_AUTODISCOVERY_FLAGS
--     Sets or retrieves the automatic discovery
-- settings. The Value parameter will contain one
-- or more of the following values:
-- AUTO_PROXY_FLAG_ALWAYS_DETECT
--     Always automatically detect settings.
--
-- AUTO_PROXY_FLAG_CACHE_INIT_RUN
--     Indicates that the cached results of the
-- automatic proxy configuration script should be
-- used, instead of actually running the script,
-- unless the cached file has expired.
--
-- AUTO_PROXY_FLAG_DETECTION_RUN    Automatic detection has been run at least once
-- on this connection.
--
-- AUTO_PROXY_FLAG_DETECTION_SUSPECT
--     Not currently supported.
--
-- AUTO_PROXY_FLAG_DONT_CACHE_PROXY_RESULT
--     Do not allow the caching of the result of the
-- automatic proxy configuration script.
--
-- AUTO_PROXY_FLAG_MIGRATED    The setting was migrated from a Microsoft(r)
-- Internet Explorer 4.0 installation and
-- automatic detection should be attempted once.
--
-- AUTO_PROXY_FLAG_USER_SET    The user has explicitly set the automatic
-- detection.
--
-- INTERNET_PER_CONN_FLAGS    Sets or retrieves the connection type. The
-- Value parameter will contain one or more of
-- the following values:
-- PROXY_TYPE_DIRECT    The connection does not use a proxy server.
--
-- PROXY_TYPE_PROXY    The connection uses an explicitly set proxy
-- server.
--
-- PROXY_TYPE_AUTO_PROXY_URL    The connection downloads and processes an
-- automatic configuration script at a specified
-- URL.
--
-- PROXY_TYPE_AUTO_DETECT    The connection automatically detects settings.
--
-- INTERNET_PER_CONN_PROXY_BYPASS    Sets or retrieves a string containing the URLs
-- that do not use the proxy server.
--
-- INTERNET_PER_CONN_PROXY_SERVER    Sets or retrieves a string containing the proxy
-- servers.
--
-- Value
-- Union of an unsigned long integer value (dwValue), an address of a string value (pszValue), and
-- a FILETIME structure (ftValue) that contains the value for the option.
--
-- Remarks
-- In Microsoft(r) Internet Explorer 5, only the ANSI versions of InternetQueryOption and
-- InternetSetOption will work with the INTERNET_PER_CONN_OPTION structure. The
-- Unicode versions will support using the INTERNET_PER_CONN_OPTION structure in later
-- versions of Internet Explorer.
--
-- For queries that return strings, InternetQueryOption allocates the memory for the pszValue
-- member of the structure. The calling application must free this memory using the GlobalFree
-- function when it has finished using the string.

--------------------------------------------------------------------------------
-- INTERNET_PER_CONN_OPTION_LIST Structure
-- Contains the list of options for a particular Internet connection.
--
-- Syntax
-- typedef struct {
-- DWORD    dwSize;
-- LPTSTR   pszConnection;
-- DWORD    dwOptionCount;
-- DWORD    dwOptionError;
-- LPINTERNET_PER_CONN_OPTION  pOptions;
-- } INTERNET_PER_CONN_OPTION_LIST, *LPINTERNET_PER_CONN_OPTION_LIST;
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Members
-- dwSize
-- Unsigned long integer value that contains the size of the
-- INTERNET_PER_CON_OPTION_LIST structure.
--
-- pszConnection
-- Address of a string value that contains the name of the RAS connection or NULL, which
-- indicates the default or LAN connection, to set or query options on.
--
-- dwOptionCount
-- Unsigned long integer value that contains the number of options to query or set.
--
-- dwOptionError
-- Unsigned long integer value that contains the options that failed if an error occurs.
--
-- pOptions
-- Address of an array of INTERNET_PER_CONN_OPTION structures containing the options to
-- query or set.
--
-- Remarks
-- In Microsoft(r) Internet Explorer 5, only the ANSI versions of InternetQueryOption and
-- InternetSetOption will work with the INTERNET_PER_CONN_OPTION_LIST structure. The
-- Unicode versions will support using the INTERNET_PER_CONN_OPTION_LIST structure in
-- later versions of Internet Explorer.

--------------------------------------------------------------------------------
-- INTERNET_PROXY_INFO Structure
--
-- Contains information that is supplied with the INTERNET_OPTION_PROXY value to get or set
-- proxy information on a handle obtained from a call to the InternetOpen function.
-- Syntax
-- typedef struct {
-- DWORD dwAccessType;
-- LPCTSTR lpszProxy;
-- LPCTSTR lpszProxyBypass;
-- } INTERNET_PROXY_INFO, * LPINTERNET_PROXY_INFO;
-- Members
-- dwAccessType
-- Unsigned long integer value that contains the access type. This can be one of the following
-- values:
-- INTERNET_OPEN_TYPE_DIRECT    Internet accessed through a direct connection.
--
-- INTERNET_OPEN_TYPE_PRECONFIG    Applies only when setting proxy information.
--
-- INTERNET_OPEN_TYPE_PROXY    Internet accessed using a proxy.
--
-- lpszProxy
-- Address of a string value that contains the proxy server list.
--
-- lpszProxyBypass
-- Address of a string value that contains the proxy bypass list.

--------------------------------------------------------------------------------
-- INTERNET_VERSION_INFO Structure
-- Contains the HTTP version number of the server. This structure is used when passing the
-- INTERNET_OPTION_VERSION flag to the InternetQueryOption function.
--
-- Syntax
-- typedef struct {
-- DWORD dwMajorVersion;
-- DWORD dwMinorVersion;
-- } INTERNET_VERSION_INFO, * LPINTERNET_VERSION_INFO;
-- Members
-- dwMajorVersion
-- Unsigned long integer value that contains the major version number.
--
-- dwMinorVersion
-- Unsigned long integer value that contains the minor version number.

--------------------------------------------------------------------------------
-- URL_COMPONENTS Structure
--
-- Contains the constituent parts of a URL. This structure is used with the InternetCrackUrl and
-- InternetCreateUrl functions.
--
-- Syntax
-- typedef struct {
-- DWORD dwStructSize;
-- LPTSTR lpszScheme;
-- DWORD dwSchemeLength;
-- INTERNET_SCHEME nScheme;
-- LPTSTR lpszHostName;
-- DWORD dwHostNameLength;
-- INTERNET_PORT nPort;
-- LPTSTR lpszUserName;
-- DWORD dwUserNameLength;
-- LPTSTR lpszPassword;
-- DWORD dwPasswordLength;
-- LPTSTR lpszUrlPath;
-- DWORD dwUrlPathLength;
-- LPTSTR lpszExtraInfo;
-- DWORD dwExtraInfoLength;
-- } URL_COMPONENTS, *LPURL_COMPONENTS;
--   The actual syntax of this function varies between its ANSI and Unicode
--   implementations. For more information, see Win32 Internet Functions Syntax.
--
-- Members
-- dwStructSize
-- Unsigned long integer value that contains the size, in TCHAR, of this structure. Used for version
-- checking. The size of this structure must be set to initialize this structure properly.
--
-- lpszScheme
-- Address of a string value that contains the scheme name.
--
-- dwSchemeLength
-- Unsigned long integer value that contains the length of the scheme name in TCHAR.
--
-- nScheme
-- INTERNET_SCHEME value that indicates the Internet protocol scheme.
--
-- lpszHostName
-- Address of a string value that contains the host name.
--
-- dwHostNameLength
-- Unsigned long integer value that contains the length of the host name in TCHAR.
--
-- nPort
-- Converted port number.
--
-- lpszUserName
-- Address of a string value that contains the user name.
--
-- dwUserNameLength
-- Unsigned long integer value that contains the length of the user name in TCHAR.
--
-- lpszPassword
-- Address of a string value that contains the password.
--
-- dwPasswordLength
-- Unsigned long integer value that contains the length of the password in TCHAR.
--
-- lpszUrlPath
-- Address of a string value that contains the URL path.
--
-- dwUrlPathLength
-- Unsigned long integer value that contains the length of the URL path in TCHAR.
--
-- lpszExtraInfo
-- Address of a string value that contains the extra information (for example, ?something or
-- #something).
--
-- dwExtraInfoLength
-- Unsigned long integer value that contains the length of the extra information in TCHAR.
--
-- Remarks
-- For InternetCrackUrl, if a pointer member and its corresponding length member are both zero,
-- that component is not returned. If the pointer member is NULL but the length member is not zero,
-- both the pointer and length members are returned. If both pointer and corresponding length
-- members are nonzero, the pointer member points to a buffer where the component is copied. The
-- component can be un-escaped, depending on the dwFlags parameter of InternetCrackUrl.
--
-- For InternetCreateUrl, the pointer members should be NULL if the component is not required. If
-- the corresponding length member is zero, the pointer member is the address of a zero-terminated
-- string. If the length member is not zero, it is the string length of the corresponding pointer member.
  
