--
-- libxlsxwriter
--
-- Copyright 2014-2015, John McNamara, jmcnamara@cpan.org. See LICENSE.txt.
--

-- namespace xlsxwriter

include misc.e
include dll.e
include machine.e
include _types_.e
include _external_.e

global constant
  LXW_ROW_MAX            = 1048576,
--<constant>
--<name>LXW_ROW_MAX</name>
--<value>1048576</value>
--<desc></desc>
--</constant>
  LXW_COL_MAX            = 16384,
--<constant>
--<name>LXW_COL_MAX</name>
--<value>16384</value>
--<desc></desc>
--</constant>
  LXW_COL_META_MAX       = 128,
--<constant>
--<name>LXW_COL_META_MAX</name>
--<value>128</value>
--<desc></desc>
--</constant>
  LXW_HEADER_FOOTER_MAX  = 255
--<constant>
--<name>LXW_HEADER_FOOTER_MAX</name>
--<value>255</value>
--<desc></desc>
--</constant>

-- The Excel 2007 specification says that the maximum number of page
-- breaks is 1026. However, in practice it is actually 1023.
global constant LXW_BREAKS_MAX = 1023
--<constant>
--<name>LXW_BREAKS_MAX</name>
--<value>1023</value>
--<desc></desc>
--</constant>

-- Default column width in Excel
global constant LXW_DEF_COL_WIDTH = 8.43
--<constant>
--<name>LXW_DEF_COL_WIDTH</name>
--<value>8.43</value>
--<desc></desc>
--</constant>

-- Default row height in Excel
global constant LXW_DEF_ROW_HEIGHT = 15
--<constant>
--<name>LXW_DEF_ROW_HEIGHT</name>
--<value>15</value>
--<desc></desc>
--</constant>

-- Errors conditions encountered when closing the Workbook and writing the Excel file to disk.
global constant
  LXW_CLOSE_ERROR_NONE = 0,
--<constant>
--<name>LXW_CLOSE_ERROR_NONE</name>
--<value>0</value>
--<desc>No error</desc>
--</constant>
  LXW_CLOSE_ERROR_ZIP  = 1
--<constant>
--<name>LXW_CLOSE_ERROR_ZIP</name>
--<value>1</value>
--<desc>Error encountered when creating file zip container</desc>
--</constant>

global type lxw_close_error(integer x)
--<eu_type>
--<name>lxw_close_error</name>
--<desc></desc>
--<param>
--<type>integer</type>
--<name>x</name>
--<desc></desc>
--</param>
--</eu_type>
  return find(x, {LXW_CLOSE_ERROR_NONE, LXW_CLOSE_ERROR_ZIP})
end type

global constant
  stqh_first = 0,
--<constant>
--<name>stqh_first</name>
--<value>0</value>
--<desc>first element</desc>
--</constant>
  stqh_last          = 4,
--<constant>
--<name>stqh_last</name>
--<value>4</value>
--<desc>addr of last next element</desc>
--</constant>
  SIZEOF_STAILQ_HEAD = 8
--<constant>
--<name>SIZEOF_STAILQ_HEAD</name>
--<value>8</value>
--<desc></desc>
--</constant>

global constant
  stqe_next = 0,
--<constant>
--<name>stqe_next</name>
--<value>0</value>
--<desc>next element</desc>
--</constant>
  SIZEOF_STAILQ_ENTRY = 4
--<constant>
--<name>SIZEOF_STAILQ_ENTRY</name>
--<value>4</value>
--<desc></desc>
--</constant>

global constant
  lxw_datetime__year  =  0, -- int
  lxw_datetime__month =  4, -- int
  lxw_datetime__day   =  8, -- int
  lxw_datetime__hour  = 12, -- int
  lxw_datetime__min   = 16, -- int
  lxw_datetime__sec   = 20, -- double
  SIZEOF_LXW_DATETIME = 32
--<constant>
--<name>lxw_datetime__year</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_datetime__month</name>
--<value>4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_datetime__day</name>
--<value>8</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_datetime__hour</name>
--<value>12</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_datetime__min</name>
--<value>16</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_datetime__sec</name>
--<value>20</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SIZEOF_LXW_DATETIME</name>
--<value>32</value>
--<desc></desc>
--</constant>

global constant
  lxw_workbook_options__constant_memory = 0, -- uint8_t
  SIZEOF_LXW_WORKBOOK_OPTIONS            = 1
--<constant>
--<name>lxw_workbook_options__constant_memory</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SIZEOF_LXW_WORKBOOK_OPTIONS</name>
--<value>1</value>
--<desc></desc>
--</constant>

global constant
  lxw_workbook__file             =   0, -- FILE*
  lxw_workbook__worksheets       =   4, -- lxw_worksheets*
  lxw_workbook__formats          =   8, -- lxw_formats*
  lxw_workbook__defined_names    =  12, -- lxw_defined_names*
  lxw_workbook__sst              =  16, -- lxw_sst*
  lxw_workbook__properties       =  20, -- lxw_properties*
  lxw_workbook__filename         =  24, -- const char*
  lxw_workbook__options          =  28, -- lxw_workbook_options*
  lxw_workbook__num_sheets       =  30, -- uint16_t
  lxw_workbook__first_sheet      =  32, -- uint16_t
  lxw_workbook__active_sheet     =  36, -- uint32_t
  lxw_workbook__num_xf_formats   =  40, -- uint16_t
  lxw_workbook__num_format_count =  42, -- uint16_t
  lxw_workbook__font_count       =  44, -- uint16_t
  lxw_workbook__border_count     =  46, -- uint16_t
  lxw_workbook__fill_count       =  48, -- uint16_t
  lxw_workbook__optimize         =  50, -- uint8_t
  lxw_workbook__used_xf_formats  =  52, -- lxw_hash_table*
  SIZEOF_LXW_WORKBOOK             =  56
--<constant>
--<name>lxw_workbook__file</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__worksheets</name>
--<value>4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__formats</name>
--<value>8</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__defined_names</name>
--<value>12</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__sst</name>
--<value>16</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__properties</name>
--<value>20</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__filename</name>
--<value>24</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__options</name>
--<value>28</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__num_sheets</name>
--<value>30</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__first_sheet</name>
--<value>32</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__active_sheet</name>
--<value>36</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__num_xf_formats</name>
--<value>40</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__num_format_count</name>
--<value>42</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__font_count</name>
--<value>44</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__border_count</name>
--<value>46</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__fill_count</name>
--<value>48</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__optimize</name>
--<value>50</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_workbook__used_xf_formats</name>
--<value>52</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SIZEOF_LXW_WORKBOOK</name>
--<value>56</value>
--<desc></desc>
--</constant>

global constant
  lxw_worksheet__file                  =   0, -- FILE *
  lxw_worksheet__optimize_tmpfile      =   4, -- FILE *
  lxw_worksheet__table                 =   8, -- lxw_table_rows *
  lxw_worksheet__hyperlinks            =  12, -- lxw_table_rows *
  lxw_worksheet__array                 =  16, -- lxw_cell**
  lxw_worksheet__merged_ranges         =  20, -- lxw_merged_ranges*
  lxw_worksheet__dim_rowmin            =  24, -- lxw_row_t
  lxw_worksheet__dim_rowmax            =  28, -- lxw_row_t
  lxw_worksheet__dim_colmin            =  32, -- lxw_col_t
  lxw_worksheet__dim_colmax            =  34, -- lxw_col_t
  lxw_worksheet__sst                   =  36, -- lxw_sst*
  lxw_worksheet__name                  =  40, -- char*
  lxw_worksheet__quoted_name           =  44, -- char*
  lxw_worksheet__index                 =  48, -- uint32_t
  lxw_worksheet__active                =  52, -- uint8_t
  lxw_worksheet__selected              =  53, -- uint8_t
  lxw_worksheet__hidden                =  54, -- uint8_t
  lxw_worksheet__active_sheet          =  56, -- uint32_t
  lxw_worksheet__col_options           =  60, -- lxw_col_options**
  lxw_worksheet__col_options_max       =  64, -- uint16_t
  lxw_worksheet__col_sizes             =  68, -- double*
  lxw_worksheet__col_sizes_max         =  72, -- uint16_t
  lxw_worksheet__col_formats           =  76, -- lxw_format**
  lxw_worksheet__col_formats_max       =  80, -- uint16_t
  lxw_worksheet__col_size_changed      =  82, -- uint8_t
  lxw_worksheet__optimize              =  83, -- uint8_t
  lxw_worksheet__optimize_row          =  84, -- lxw_row*
  lxw_worksheet__fit_width             =  90, -- uint16_t
  lxw_worksheet__fit_height            =  88, -- uint16_t
  lxw_worksheet__horizontal_dpi        =  92, -- uint16_t
  lxw_worksheet__hlink_count           =  94, -- uint16_t
  lxw_worksheet__page_start            =  96, -- uint16_t
  lxw_worksheet__print_scale           =  98, -- uint16_t
  lxw_worksheet__rel_count             = 100, -- uint16_t
  lxw_worksheet__vertical_dpi          = 102, -- uint16_t
  lxw_worksheet__filter_on             = 104, -- uint8_t
  lxw_worksheet__fit_page              = 105, -- uint8_t
  lxw_worksheet__hcenter               = 106, -- uint8_t
  lxw_worksheet__orientation           = 107, -- uint8_t
  lxw_worksheet__outline_changed       = 108, -- uint8_t
  lxw_worksheet__page_order            = 109, -- uint8_t
  lxw_worksheet__page_setup_changed    = 110, -- uint8_t
  lxw_worksheet__page_view             = 111, -- uint8_t
  lxw_worksheet__paper_size            = 112, -- uint8_t
  lxw_worksheet__print_gridlines       = 113, -- uint8_t
  lxw_worksheet__print_headers         = 114, -- uint8_t
  lxw_worksheet__print_options_changed = 115, -- uint8_t
  lxw_worksheet__screen_gridlines      = 116, -- uint8_t
  lxw_worksheet__tab_color             = 117, -- uint8_t
  lxw_worksheet__vba_codename          = 118, -- uint8_t
  lxw_worksheet__vcenter               = 119, -- uint8_t
  lxw_worksheet__margin_left           = 120, -- double
  lxw_worksheet__margin_right          = 128, -- double
  lxw_worksheet__margin_top            = 136, -- double
  lxw_worksheet__margin_bottom         = 144, -- double
  lxw_worksheet__margin_header         = 152, -- double
  lxw_worksheet__margin_footer         = 160, -- double
  lxw_worksheet__header_footer_changed = 168, -- uint8_t
  lxw_worksheet__header                = 169, -- char[LXW_HEADER_FOOTER_MAX]
  lxw_worksheet__footer                = 424, -- char[LXW_HEADER_FOOTER_MAX]
  lxw_worksheet__repeat_rows           = 680, -- lxw_repeat_rows
  lxw_worksheet__repeat_cols           = 692, -- lxw_repeat_cols
  lxw_worksheet__print_area            = 700, -- lxw_print_area
  lxw_worksheet__autofilter            = 716, -- lxw_autofilter
  lxw_worksheet__merged_range_count    = 732, -- uint16_t
  lxw_worksheet__hbreaks               = 736, -- lxw_row_t*
  lxw_worksheet__vbreaks               = 740, -- lxw_col_t*
  lxw_worksheet__external_hyperlinks   = 744, -- lxw_rel_tuples*
  lxw_worksheet__list_pointers         = 748, -- lxw_worksheet*
  SIZEOF_LXW_WORKSHEET                  = 752
--<constant>
--<name>lxw_worksheet__file</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__optimize_tmpfile</name>
--<value>4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__table</name>
--<value>8</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__hyperlinks</name>
--<value>12</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__array</name>
--<value>16</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__merged_ranges</name>
--<value>20</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__dim_rowmin</name>
--<value>24</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__dim_rowmax</name>
--<value>28</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__dim_colmin</name>
--<value>32</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__dim_colmax</name>
--<value>34</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__sst</name>
--<value>36</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__name</name>
--<value>40</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__quoted_name</name>
--<value>44</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__index</name>
--<value>48</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__active</name>
--<value>52</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__selected</name>
--<value>53</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__hidden</name>
--<value>54</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__active_sheet</name>
--<value>56</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__col_options</name>
--<value>60</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__col_options_max</name>
--<value>64</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__col_sizes</name>
--<value>68</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__col_sizes_max</name>
--<value>72</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__col_formats</name>
--<value>76</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__col_formats_max</name>
--<value>80</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__col_size_changed</name>
--<value>82</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__optimize</name>
--<value>83</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__optimize_row</name>
--<value>84</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__fit_width</name>
--<value>90</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__fit_height</name>
--<value>88</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__horizontal_dpi</name>
--<value>92</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__hlink_count</name>
--<value>94</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__page_start</name>
--<value>96</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__print_scale</name>
--<value>98</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__rel_count</name>
--<value>100</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__vertical_dpi</name>
--<value>102</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__filter_on</name>
--<value>104</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__fit_page</name>
--<value>105</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__hcenter</name>
--<value>106</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__orientation</name>
--<value>107</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__outline_changed</name>
--<value>108</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__page_order</name>
--<value>109</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__page_setup_changed</name>
--<value>110</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__page_view</name>
--<value>111</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__paper_size</name>
--<value>112</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__print_gridlines</name>
--<value>113</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__print_headers</name>
--<value>114</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__print_options_changed</name>
--<value>115</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__screen_gridlines</name>
--<value>116</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__tab_color</name>
--<value>117</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__vba_codename</name>
--<value>118</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__vcenter</name>
--<value>119</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__margin_left</name>
--<value>120</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__margin_right</name>
--<value>128</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__margin_top</name>
--<value>136</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__margin_bottom</name>
--<value>144</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__margin_header</name>
--<value>152</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__margin_footer</name>
--<value>160</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__header_footer_changed</name>
--<value>168</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__header</name>
--<value>169</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__footer</name>
--<value>424</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__repeat_rows</name>
--<value>680</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__repeat_cols</name>
--<value>692</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__print_area</name>
--<value>700</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__autofilter</name>
--<value>716</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__merged_range_count</name>
--<value>732</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__hbreaks</name>
--<value>736</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__vbreaks</name>
--<value>740</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__external_hyperlinks</name>
--<value>744</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheet__list_pointers</name>
--<value>748</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SIZEOF_LXW_WORKSHEET</name>
--<value>752</value>
--<desc></desc>
--</constant>

global constant
  lxw_worksheets__stqh_first = 0, -- pointer
  lxw_worksheets__stqh_last  = 4, -- pointer
  SIZEOF_LXW_WORKSHEETS      = 8
--<constant>
--<name>lxw_worksheets__stqh_first</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>lxw_worksheets__stqh_last</name>
--<value>4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>SIZEOF_LXW_WORKSHEETS</name>
--<value>8</value>
--<desc></desc>
--</constant>

-- Error codes from `worksheet_write*()` functions.

global constant
  LXW_WRITE_ERROR_NONE = 0,  -- No error.
  LXW_RANGE_ERROR         = 1,  -- Row or column index out of range.
  LXW_STRING_LENGTH_ERROR = 2,  -- String exceeds Excel's LXW_STRING_LENGTH_ERROR limit.
  LXW_STRING_HASH_ERROR   = 3   -- Error finding string index.
--<constant>
--<name>LXW_WRITE_ERROR_NONE</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_RANGE_ERROR</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_STRING_LENGTH_ERROR</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_STRING_HASH_ERROR</name>
--<value>3</value>
--<desc></desc>
--</constant>

global type lxw_write_error(integer x)
--<eu_type>
--<name>lxw_write_error</name>
--<desc></desc>
--<param>
--<type>integer</type>
--<name>x</name>
--<desc></desc>
--</param>
--</eu_type>
  return find(x, {LXW_WRITE_ERROR_NONE, LXW_RANGE_ERROR, LXW_STRING_LENGTH_ERROR,
                  LXW_STRING_HASH_ERROR})
end type

-- Gridline options using in `worksheet_gridlines()`.

global constant
  LXW_HIDE_ALL_GRIDLINES = 0,  -- Hide screen and print gridlines.
  LXW_SHOW_SCREEN_GRIDLINES = 1,  -- Show screen gridlines.
  LXW_SHOW_PRINT_GRIDLINES  = 2,  -- Show print gridlines.
  LXW_SHOW_ALL_GRIDLINES    = 3   -- Show screen and print gridlines.
--<constant>
--<name>LXW_HIDE_ALL_GRIDLINES</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_SHOW_SCREEN_GRIDLINES</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_SHOW_PRINT_GRIDLINES</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_SHOW_ALL_GRIDLINES</name>
--<value>3</value>
--<desc></desc>
--</constant>

global type lxw_gridlines(integer x)
--<eu_type>
--<name>lxw_gridlines</name>
--<desc></desc>
--<param>
--<type>integer</type>
--<name>x</name>
--<desc></desc>
--</param>
--</eu_type>
  return find(x, {LXW_HIDE_ALL_GRIDLINES, LXW_SHOW_SCREEN_GRIDLINES,
                  LXW_SHOW_PRINT_GRIDLINES, LXW_SHOW_ALL_GRIDLINES})
end type

global constant
  LXW_FORMAT_FIELD_LEN    = 128,
  LXW_DEFAULT_FONT_NAME   = "Calibri",
  LXW_DEFAULT_FONT_FAMILY = 2,
  LXW_DEFAULT_FONT_THEME  = 1,
  LXW_PROPERTY_UNSET      = -1,
  LXW_COLOR_UNSET         = -1,
  LXW_COLOR_MASK          = #FFFFFF,
  LXW_MIN_FONT_SIZE       = 1,
  LXW_MAX_FONT_SIZE       = 409
--<constant>
--<name>LXW_FORMAT_FIELD_LEN</name>
--<value>128</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_DEFAULT_FONT_NAME</name>
--<value>"Calibri"</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_DEFAULT_FONT_FAMILY</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_DEFAULT_FONT_THEME</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PROPERTY_UNSET</name>
--<value>-1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_UNSET</name>
--<value>-1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_MASK</name>
--<value>#FFFFFF</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_MIN_FONT_SIZE</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_MAX_FONT_SIZE</name>
--<value>409</value>
--<desc></desc>
--</constant>

-- Format underline values for format_set_underline().
global constant
  LXW_UNDERLINE_SINGLE            = 1,  -- Single underline
  LXW_UNDERLINE_DOUBLE            = 2,  -- Double underline
  LXW_UNDERLINE_SINGLE_ACCOUNTING = 3,  -- Single accounting underline
  LXW_UNDERLINE_DOUBLE_ACCOUNTING = 4   -- Double accounting underline
--<constant>
--<name>LXW_UNDERLINE_SINGLE</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_UNDERLINE_DOUBLE</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_UNDERLINE_SINGLE_ACCOUNTING</name>
--<value>3</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_UNDERLINE_DOUBLE_ACCOUNTING</name>
--<value>4</value>
--<desc></desc>
--</constant>

global type lxw_format_underlines(integer x)
--<eu_type>
--<name>lxw_format_underlines</name>
--<desc></desc>
--<param>
--<type>integer</type>
--<name>x</name>
--<desc></desc>
--</param>
--</eu_type>
  return find(x, {LXW_UNDERLINE_SINGLE, LXW_UNDERLINE_DOUBLE,
                  LXW_UNDERLINE_SINGLE_ACCOUNTING, LXW_UNDERLINE_DOUBLE_ACCOUNTING})
end type

-- Superscript and subscript values for format_set_font_script().
global constant
  LXW_FONT_SUPERSCRIPT = 1,  -- Superscript font
  LXW_FONT_SUBSCRIPT   = 2   -- Subscript font
--<constant>
--<name>LXW_FONT_SUPERSCRIPT</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_FONT_SUBSCRIPT</name>
--<value>2</value>
--<desc></desc>
--</constant>

global type lxw_format_scripts(integer x)
--<eu_type>
--<name>lxw_format_scripts</name>
--<desc></desc>
--<param>
--<type>integer</type>
--<name>x</name>
--<desc></desc>
--</param>
--</eu_type>
  return find(x, {LXW_FONT_SUPERSCRIPT, LXW_FONT_SUBSCRIPT})
end type

-- Alignment values for format_set_align().
global constant
  LXW_ALIGN_NONE                 =  0,  -- No alignment. Cell will use Excel's default for the data type
  LXW_ALIGN_LEFT                 =  1,  -- Left horizontal alignment
  LXW_ALIGN_CENTER               =  2,  -- Center horizontal alignment
  LXW_ALIGN_RIGHT                =  3,  -- Right horizontal alignment
  LXW_ALIGN_FILL                 =  4,  -- Cell fill horizontal alignment
  LXW_ALIGN_JUSTIFY              =  5,  -- Justify horizontal alignment
  LXW_ALIGN_CENTER_ACROSS        =  6,  -- Center Across horizontal alignment
  LXW_ALIGN_DISTRIBUTED          =  7,  -- Left horizontal alignment
  LXW_ALIGN_VERTICAL_TOP         =  8,  -- Top vertical alignment
  LXW_ALIGN_VERTICAL_BOTTOM      =  9,  -- Bottom vertical alignment
  LXW_ALIGN_VERTICAL_CENTER      = 10,  -- Center vertical alignment
  LXW_ALIGN_VERTICAL_JUSTIFY     = 11,  -- Justify vertical alignment
  LXW_ALIGN_VERTICAL_DISTRIBUTED = 12  -- Distributed vertical alignment
--<constant>
--<name>LXW_ALIGN_NONE</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_ALIGN_LEFT</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_ALIGN_CENTER</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_ALIGN_RIGHT</name>
--<value>3</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_ALIGN_FILL</name>
--<value>4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_ALIGN_JUSTIFY</name>
--<value>5</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_ALIGN_CENTER_ACROSS</name>
--<value>6</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_ALIGN_DISTRIBUTED</name>
--<value>7</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_ALIGN_VERTICAL_TOP</name>
--<value>8</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_ALIGN_VERTICAL_BOTTOM</name>
--<value>9</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_ALIGN_VERTICAL_CENTER</name>
--<value>10</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_ALIGN_VERTICAL_JUSTIFY</name>
--<value>11</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_ALIGN_VERTICAL_DISTRIBUTED</name>
--<value>12</value>
--<desc></desc>
--</constant>

global type lxw_format_alignments(integer x)
--<eu_type>
--<name>lxw_format_alignments</name>
--<desc></desc>
--<param>
--<type>integer</type>
--<name>x</name>
--<desc></desc>
--</param>
--</eu_type>
  return find(x, {LXW_ALIGN_NONE, LXW_ALIGN_LEFT, LXW_ALIGN_CENTER,
                  LXW_ALIGN_RIGHT, LXW_ALIGN_FILL, LXW_ALIGN_JUSTIFY,
                  LXW_ALIGN_CENTER_ACROSS, LXW_ALIGN_DISTRIBUTED,
                  LXW_ALIGN_VERTICAL_TOP, LXW_ALIGN_VERTICAL_BOTTOM,
                  LXW_ALIGN_VERTICAL_CENTER, LXW_ALIGN_VERTICAL_JUSTIFY,
                  LXW_ALIGN_VERTICAL_DISTRIBUTED})
end type

global constant
  LXW_DIAGONAL_BORDER_UP      = 1,
  LXW_DIAGONAL_BORDER_DOWN    = 2,
  LXW_DIAGONAL_BORDER_UP_DOWN = 3
--<constant>
--<name>LXW_DIAGONAL_BORDER_UP</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_DIAGONAL_BORDER_DOWN</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_DIAGONAL_BORDER_UP_DOWN</name>
--<value>3</value>
--<desc></desc>
--</constant>

global type lxw_format_diagonal_types(integer x)
--<eu_type>
--<name>lxw_format_diagonal_types</name>
--<desc></desc>
--<param>
--<type>integer</type>
--<name>x</name>
--<desc></desc>
--</param>
--</eu_type>
  return find(x, {LXW_DIAGONAL_BORDER_UP, LXW_DIAGONAL_BORDER_DOWN,
                  LXW_DIAGONAL_BORDER_UP_DOWN})
end type

-- Predefined values for common colors.
global constant
  LXW_COLOR_BLACK   = #000000,  -- Black
  LXW_COLOR_BLUE    = #0000FF,  -- Blue
  LXW_COLOR_BROWN   = #800000,  -- Brown
  LXW_COLOR_CYAN    = #00FFFF,  -- Cyan
  LXW_COLOR_GRAY    = #808080,  -- Gray
  LXW_COLOR_GREEN   = #008000,  -- Green
  LXW_COLOR_LIME    = #00FF00,  -- Lime
  LXW_COLOR_MAGENTA = #FF00FF,  -- Magenta
  LXW_COLOR_NAVY    = #000080,  -- Navy
  LXW_COLOR_ORANGE  = #FF6600,  -- Orange
  LXW_COLOR_PINK    = #FF00FF,  -- Pink
  LXW_COLOR_PURPLE  = #800080,  -- Purple
  LXW_COLOR_RED     = #FF0000,  -- Red
  LXW_COLOR_SILVER  = #C0C0C0,  -- Silver
  LXW_COLOR_WHITE   = #FFFFFF,  -- White
  LXW_COLOR_YELLOW  = #FFFF00   -- Yellow
--<constant>
--<name>LXW_COLOR_BLACK</name>
--<value>#000000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_BLUE</name>
--<value>#0000FF</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_BROWN</name>
--<value>#800000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_CYAN</name>
--<value>#00FFFF</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_GRAY</name>
--<value>#808080</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_GREEN</name>
--<value>#008000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_LIME</name>
--<value>#00FF00</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_MAGENTA</name>
--<value>#FF00FF</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_NAVY</name>
--<value>#000080</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_ORANGE</name>
--<value>#FF6600</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_PINK</name>
--<value>#FF00FF</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_PURPLE</name>
--<value>#800080</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_RED</name>
--<value>#FF0000</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_SILVER</name>
--<value>#C0C0C0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_WHITE</name>
--<value>#FFFFFF</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_COLOR_YELLOW</name>
--<value>#FFFF00</value>
--<desc></desc>
--</constant>

global type lxw_defined_colors(integer x)
--<eu_type>
--<name>lxw_defined_colors</name>
--<desc></desc>
--<param>
--<type>integer</type>
--<name>x</name>
--<desc></desc>
--</param>
--</eu_type>
  return find(x, {LXW_COLOR_BLACK, LXW_COLOR_BLUE, LXW_COLOR_BROWN,
                  LXW_COLOR_CYAN, LXW_COLOR_GRAY, LXW_COLOR_GREEN,
                  LXW_COLOR_LIME, LXW_COLOR_MAGENTA, LXW_COLOR_NAVY,
                  LXW_COLOR_ORANGE, LXW_COLOR_PINK, LXW_COLOR_PURPLE,
                  LXW_COLOR_RED, LXW_COLOR_SILVER, LXW_COLOR_WHITE,
                  LXW_COLOR_YELLOW})
end type

-- Pattern value for use with format_set_pattern().
global constant
  LXW_PATTERN_NONE             =  0,  -- Empty pattern
  LXW_PATTERN_SOLID            =  1,  -- Solid pattern
  LXW_PATTERN_MEDIUM_GRAY      =  2,  -- Medium gray pattern
  LXW_PATTERN_DARK_GRAY        =  3,  -- Dark gray pattern
  LXW_PATTERN_LIGHT_GRAY       =  4,  -- Light gray pattern
  LXW_PATTERN_DARK_HORIZONTAL  =  5,  -- Dark horizontal line pattern
  LXW_PATTERN_DARK_VERTICAL    =  6,  -- Dark vertical line pattern
  LXW_PATTERN_DARK_DOWN        =  7,  -- Dark diagonal stripe pattern
  LXW_PATTERN_DARK_UP          =  8,  -- Reverse dark diagonal stripe pattern
  LXW_PATTERN_DARK_GRID        =  9,  -- Dark grid pattern
  LXW_PATTERN_DARK_TRELLIS     = 10,  -- Dark trellis pattern
  LXW_PATTERN_LIGHT_HORIZONTAL = 11,  -- Light horizontal Line pattern
  LXW_PATTERN_LIGHT_VERTICAL   = 12,  -- Light vertical line pattern
  LXW_PATTERN_LIGHT_DOWN       = 13,  -- Light diagonal stripe pattern
  LXW_PATTERN_LIGHT_UP         = 14,  -- Reverse light diagonal stripe pattern
  LXW_PATTERN_LIGHT_GRID       = 15,  -- Light grid pattern
  LXW_PATTERN_LIGHT_TRELLIS    = 16,  -- Light trellis pattern
  LXW_PATTERN_GRAY_125         = 17,  -- 12.5% gray pattern
  LXW_PATTERN_GRAY_0625        = 18  -- 6.25% gray pattern
--<constant>
--<name>LXW_PATTERN_NONE</name>
--<value>0</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_SOLID</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_MEDIUM_GRAY</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_DARK_GRAY</name>
--<value>3</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_LIGHT_GRAY</name>
--<value>4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_DARK_HORIZONTAL</name>
--<value>5</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_DARK_VERTICAL</name>
--<value>6</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_DARK_DOWN</name>
--<value>7</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_DARK_UP</name>
--<value>8</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_DARK_GRID</name>
--<value>9</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_DARK_TRELLIS</name>
--<value>10</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_LIGHT_HORIZONTAL</name>
--<value>11</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_LIGHT_VERTICAL</name>
--<value>12</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_LIGHT_DOWN</name>
--<value>13</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_LIGHT_UP</name>
--<value>14</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_LIGHT_GRID</name>
--<value>15</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_LIGHT_TRELLIS</name>
--<value>16</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_GRAY_125</name>
--<value>17</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_PATTERN_GRAY_0625</name>
--<value>18</value>
--<desc></desc>
--</constant>

global type lxw_format_patterns(integer x)
--<eu_type>
--<name>lxw_format_patterns</name>
--<desc></desc>
--<param>
--<type>integer</type>
--<name>x</name>
--<desc></desc>
--</param>
--</eu_type>
  return find(x, {LXW_PATTERN_NONE, LXW_PATTERN_SOLID,
                  LXW_PATTERN_MEDIUM_GRAY, LXW_PATTERN_DARK_GRAY,
                  LXW_PATTERN_LIGHT_GRAY, LXW_PATTERN_DARK_HORIZONTAL,
                  LXW_PATTERN_DARK_VERTICAL, LXW_PATTERN_DARK_DOWN,
                  LXW_PATTERN_DARK_UP, LXW_PATTERN_DARK_GRID,
                  LXW_PATTERN_DARK_TRELLIS, LXW_PATTERN_LIGHT_HORIZONTAL,
                  LXW_PATTERN_LIGHT_VERTICAL, LXW_PATTERN_LIGHT_DOWN,
                  LXW_PATTERN_LIGHT_UP, LXW_PATTERN_LIGHT_GRID,
                  LXW_PATTERN_LIGHT_TRELLIS, LXW_PATTERN_GRAY_125,
                  LXW_PATTERN_GRAY_0625})
end type

-- Cell border styles for use with format_set_border().
global constant
  LXW_BORDER_NONE                =  1,  -- No border
  LXW_BORDER_THIN                =  2,  -- Thin border style
  LXW_BORDER_MEDIUM              =  3,  -- Medium border style
  LXW_BORDER_DASHED              =  4,  -- Dashed border style
  LXW_BORDER_DOTTED              =  5,  -- Dotted border style
  LXW_BORDER_THICK               =  6,  -- Thick border style
  LXW_BORDER_DOUBLE              =  7,  -- Double border style
  LXW_BORDER_HAIR                =  8,  -- Hair border style
  LXW_BORDER_MEDIUM_DASHED       =  9,  -- Medium dashed border style
  LXW_BORDER_DASH_DOT            = 10,  -- Dash-dot border style
  LXW_BORDER_MEDIUM_DASH_DOT     = 11,  -- Medium dash-dot border style
  LXW_BORDER_DASH_DOT_DOT        = 12,  -- Dash-dot-dot border style
  LXW_BORDER_MEDIUM_DASH_DOT_DOT = 13,  -- Medium dash-dot-dot border style
  LXW_BORDER_SLANT_DASH_DOT      = 14   -- Slant dash-dot border style
--<constant>
--<name>LXW_BORDER_NONE</name>
--<value>1</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_BORDER_THIN</name>
--<value>2</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_BORDER_MEDIUM</name>
--<value>3</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_BORDER_DASHED</name>
--<value>4</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_BORDER_DOTTED</name>
--<value>5</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_BORDER_THICK</name>
--<value>6</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_BORDER_DOUBLE</name>
--<value>7</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_BORDER_HAIR</name>
--<value>8</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_BORDER_MEDIUM_DASHED</name>
--<value>9</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_BORDER_DASH_DOT</name>
--<value>10</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_BORDER_MEDIUM_DASH_DOT</name>
--<value>11</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_BORDER_DASH_DOT_DOT</name>
--<value>12</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_BORDER_MEDIUM_DASH_DOT_DOT</name>
--<value>13</value>
--<desc></desc>
--</constant>
--<constant>
--<name>LXW_BORDER_SLANT_DASH_DOT</name>
--<value>14</value>
--<desc></desc>
--</constant>

global type lxw_format_borders(integer x)
--<eu_type>
--<name>lxw_format_borders</name>
--<desc></desc>
--<param>
--<type>integer</type>
--<name>x</name>
--<desc></desc>
--</param>
--</eu_type>
  return find(x, {LXW_BORDER_NONE, LXW_BORDER_THIN, LXW_BORDER_MEDIUM,
                  LXW_BORDER_DASHED, LXW_BORDER_DOTTED, LXW_BORDER_THICK,
                  LXW_BORDER_DOUBLE, LXW_BORDER_HAIR, LXW_BORDER_MEDIUM_DASHED,
                  LXW_BORDER_DASH_DOT, LXW_BORDER_MEDIUM_DASH_DOT, LXW_BORDER_DASH_DOT_DOT,
                  LXW_BORDER_MEDIUM_DASH_DOT_DOT, LXW_BORDER_SLANT_DASH_DOT})
end type

-----------------------------------------------------------------------------

-- next routines try to partially replace map

function map_find(sequence map, object key)
  for i = 1 to length(map) do
    if equal(map[i][1], key) then  -- key matches
      return i
    end if
  end for
  return 0
end function

function map_get(sequence map, object key, object default)
  integer i

  i = map_find(map, key)
  if i then  -- key matches
    return map[i][2]
  else
    return default
  end if
end function

function map_set(sequence map, object key, object value, integer add)
  integer i

  i = map_find(map, key)
  if i then  -- key matches
    map[i][2] = value
  else
    if add then
      map = append(map, {key, value})
    end if
  end if
  return map
end function

function map_remove(sequence map, object key)
  sequence result
  
  result = {}
  for i = 1 to length(map) do
    if not equal(map[i][1], key) then
      result = append(result, map[i])
    end if
  end for
  return result
end function

-----------------------------------------------------------------------------

function peek_ptr(atom addr)
  return peek4u(addr)
end function

procedure poke_int(atom addr, atom value)
  poke4(addr, value)
end procedure

procedure poke_dbl(atom addr, atom value)
  poke(addr, atom_to_float64(value))
end procedure

global function allocate_datetime(sequence datetime)
--<function>
--<name>allocate_datetime</name>
--<digest>Allocate a datetime structure from a Euphoria datetime value.</digest>
--<desc>
-- 'datetime' should be {year, month, day, hour, minute, second}
--</desc>
--<param>
--<type>sequence</type>
--<name>datetime</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom ptr
  integer lg

  ptr = allocate(SIZEOF_LXW_DATETIME)
  mem_set(ptr, NULL, SIZEOF_LXW_DATETIME)
  -- not-so-easy!
  lg = length(datetime)
  if lg = 6 then poke_dbl(ptr + lxw_datetime__sec,   datetime[6])
  elsif lg = 5 then poke_int(ptr + lxw_datetime__min,   datetime[5])
  elsif lg = 4 then poke_int(ptr + lxw_datetime__hour,   datetime[4])
  elsif lg = 3 then poke_int(ptr + lxw_datetime__day,   datetime[3])
  elsif lg = 2 then poke_int(ptr + lxw_datetime__month, datetime[2])
  elsif lg = 1 then poke_int(ptr + lxw_datetime__year,   datetime[1])
  end if
  return ptr
end function

global function allocate_array4(sequence data)
--<function>
--<name>allocate_array4</name>
--<digest>Allocate a NULL terminated array of 4-byte values.</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>data</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom size, ptr

  size = 4 * (length(data)+1)
  ptr = allocate(size)
  poke4(ptr, data & {NULL})
  return ptr
end function

function peek_string(atom a)
  integer i
  sequence s

  s = ""
  if a then
    i = peek(a)
    while i do
      s = append(s, i)
      a += 1
      i = peek(a)
    end while
  end if
  return s
end function

-- N.B. We have to track the filename object in memory for the lifetime of each Workbook.
sequence m_filename
m_filename = {}

-----------------------------------------------------------------------------

integer x_new_workbook, x_new_workbook_opt
x_new_workbook = 0
x_new_workbook_opt = 0

global function new_workbook(object filename, atom options)
--<function>
--<name>new_workbook</name>
--<digest>Create a new workbook object with (optional) additional workbook options.</digest>
--<desc>
-- options = NULL by default
--</desc>
--<param>
--<type>object</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>options</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom result

  result = NULL
  if atom(filename) then filename = peek_string(filename) end if
  if sequence(filename) then filename = allocate_string(filename) end if
  if options = NULL then
    result = exec_func(x_new_workbook, {filename}, "+new_workbook")
  else
    result = exec_func(x_new_workbook_opt, {filename,options}, "+new_workbook_opt")
  end if
  m_filename = map_set(m_filename, result, filename, 1)
  return result
end function

-----------------------------------------------------------------------------

global function new_workbook_opt(object filename, atom options)
--<function>
--<name>new_workbook_opt</name>
--<digest>Create a new workbook object with additional workbook options.</digest>
--<desc>
--</desc>
--<param>
--<type>object</type>
--<name>filename</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>options</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom result

  if atom(filename) then filename = peek_string(filename) end if
  if sequence(filename) then filename = allocate_string(filename) end if
  result = exec_func(x_new_workbook_opt, {filename,options},
                     "+new_workbook_opt")
  m_filename = map_set(m_filename, result, filename, 1)
  return result
end function

-----------------------------------------------------------------------------

integer x_workbook_add_format
x_workbook_add_format = 0

global function workbook_add_format(atom workbook)
--<function>
--<name>workbook_add_format</name>
--<digest>Create a new Format object to format cells in worksheets.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>workbook</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  return exec_func(x_workbook_add_format, {workbook},
                    "+workbook_add_format")
end function

-----------------------------------------------------------------------------

integer x_workbook_add_worksheet  
x_workbook_add_worksheet = 0

global function workbook_add_worksheet(atom workbook, object sheetname)
--<function>
--<name>workbook_add_worksheet</name>
--<digest>Add a new worksheet to a workbook with (optional) the provided name.</digest>
--<desc>
-- sheetname = NULL by default
--</desc>
--<param>
--<type>atom</type>
--<name>workbook</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>sheetname</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  if sequence(sheetname) then sheetname = allocate_string(sheetname) end if
  return exec_func(x_workbook_add_worksheet, {workbook,sheetname},
                    "+workbook_add_worksheet")
end function

-----------------------------------------------------------------------------

integer x_workbook_close      
x_workbook_close = 0

global function workbook_close(atom workbook)
--<function>
--<name>workbook_close</name>
--<digest>Close the Workbook object and write the XLSX file.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>workbook</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom result, filename
  
  result = exec_func(x_workbook_close, {workbook}, "+workbook_close")
  if result = 0 then
    -- clean up the workbook filename object
    filename = map_get(m_filename, workbook, NULL)
    if filename != NULL then free(filename) end if
    m_filename = map_remove(m_filename, workbook)
  end if
  return result
end function

-----------------------------------------------------------------------------

global function workbook_get_worksheets(atom workbook)
--<function>
--<name>workbook_get_worksheets</name>
--<digest>Return a list of all Worksheets in the Workbook.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>workbook</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  sequence result
  atom worksheets, next

  result = {}
  worksheets = peek_ptr(workbook + lxw_workbook__worksheets)
  next = peek_ptr(worksheets + stqh_first)
  while next != NULL do
    result = append(result, next)
    next = peek_ptr(next + lxw_worksheet__list_pointers + stqe_next)
  end while
  return result
end function

-----------------------------------------------------------------------------

integer x_workbook_define_name
x_workbook_define_name = 0

global function workbook_define_name(atom workbook, object name, object formula)
--<function>
--<name>workbook_define_name</name>
--<digest>Create a defined name in the workbook to use as a variable.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>workbook</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>name</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>formula</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  if sequence(name) then name = allocate_string(name) end if
  if sequence(formula) then formula = allocate_string(formula) end if
  return exec_func(x_workbook_define_name, {workbook,name,formula},
                    "+workbook_define_name")
end function

-----------------------------------------------------------------------------

integer x_worksheet_write_number  
x_worksheet_write_number = 0

global function worksheet_write_number(atom worksheet, sequence cell, atom number, atom format)
--<function>
--<name>worksheet_write_number</name>
--<digest>Write a number to a worksheet cell.</digest>
--<desc>
-- format = NULL by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>cell</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>number</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  return exec_func(x_worksheet_write_number, {worksheet, cell[1], cell[2],
                   number, format}, "+worksheet_write_number")
end function

-----------------------------------------------------------------------------

integer x_worksheet_write_string  
x_worksheet_write_string = 0

global function worksheet_write_string(atom worksheet, sequence cell,
                                        object string, atom format)
--<function>
--<name>worksheet_write_string</name>
--<digest>Write a string to a worksheet cell.</digest>
--<desc>
-- format = NULL by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>cell</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>string</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  if sequence(string) then string = allocate_string(string) end if
  return exec_func(x_worksheet_write_string, {worksheet, cell[1], cell[2],
                   string, format}, "+worksheet_write_string")
end function

-----------------------------------------------------------------------------

integer x_worksheet_write_formula
x_worksheet_write_formula = 0

global function worksheet_write_formula(atom worksheet, sequence cell,
                                          object formula, atom format)
--<function>
--<name>worksheet_write_formula</name>
--<digest>Write a formula to a worksheet cell.</digest>
--<desc>
-- format = NULL by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>cell</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>formula</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  if sequence(formula) then formula = allocate_string(formula) end if
  return exec_func(x_worksheet_write_formula, {worksheet, cell[1], cell[2],
                   formula, format}, "+worksheet_write_formula")
end function

-----------------------------------------------------------------------------

integer x_worksheet_write_array_formula 
x_worksheet_write_array_formula = 0

global function worksheet_write_array_formula(atom worksheet,
                     sequence range, object formula, atom format)
--<function>
--<name>worksheet_write_array_formula</name>
--<digest>Write an array formula to a worksheet cell.</digest>
--<desc>
-- format = NULL by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>range</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>formula</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  if sequence(formula) then formula = allocate_string(formula) end if
  return exec_func(x_worksheet_write_array_formula, {worksheet,range[1],
                   range[2],range[3],range[4],formula,format},
                   "+worksheet_write_array_formula")
end function

-----------------------------------------------------------------------------

integer x_worksheet_write_datetime
x_worksheet_write_datetime = 0

global function worksheet_write_datetime(atom worksheet, sequence cell,
                                          object datetime, atom format)
--<function>
--<name>worksheet_write_datetime</name>
--<digest>Write a date or time to a worksheet cell.</digest>
--<desc>
-- format = NULL by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>cell</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>datetime</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  if sequence(datetime) then datetime = allocate_datetime(datetime) end if
  return exec_func(x_worksheet_write_datetime, {worksheet, cell[1], cell[2],
                   datetime, format}, "+worksheet_write_datetime")
end function

-----------------------------------------------------------------------------

integer x_worksheet_write_url, x_worksheet_write_url_opt
x_worksheet_write_url = 0
x_worksheet_write_url_opt = 0

global function worksheet_write_url(atom worksheet, sequence cell, object url, object string,
    object tooltip, atom format)
--<function>
--<name>worksheet_write_url</name>
--<digest>Write a URL/hyperlink to a worksheet cell.</digest>
--<desc>
-- string, tooltip and format = NULL by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>cell</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>url</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>string</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>tooltip</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>

  if sequence(url) then url = allocate_string(url) end if
  if sequence(string) then string = allocate_string(string) end if
  if sequence(tooltip) then tooltip = allocate_string(tooltip) end if

  if string = NULL and tooltip = NULL then
    return exec_func(x_worksheet_write_url, {worksheet, cell[1], cell[2],
                     url, format}, "+worksheet_write_url")
  else
    return exec_func(x_worksheet_write_url_opt, {worksheet, cell[1],
                     cell[2], url, string, tooltip, format},
                     "+worksheet_write_url_opt")
  end if

end function

-----------------------------------------------------------------------------

integer x_worksheet_write_blank   
x_worksheet_write_blank = 0

global function worksheet_write_blank(atom worksheet, sequence cell,
                                       atom format)
--<function>
--<name>worksheet_write_blank</name>
--<digest>Write a formatted blank worksheet cell.</digest>
--<desc>
-- format = NULL by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>cell</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  return exec_func(x_worksheet_write_blank, {worksheet, cell[1], cell[2],
                   format}, "+worksheet_write_blank")
end function

-----------------------------------------------------------------------------

integer x_worksheet_set_row   
x_worksheet_set_row = 0

global function worksheet_set_row(atom worksheet, atom row, atom height, atom format, atom options)
--<function>
--<name>worksheet_set_row</name>
--<digest>Set the properties for a row of cells.</digest>
--<desc>
-- format and options = NULL by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>row</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>height</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>options</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  return exec_func(x_worksheet_set_row, {worksheet,row,height,format,
                   options}, "+worksheet_set_row")
end function

-----------------------------------------------------------------------------

integer x_worksheet_set_column
x_worksheet_set_column = 0

global function worksheet_set_column(atom worksheet, sequence cols, atom width, atom format, atom options)
--<function>
--<name>worksheet_set_column</name>
--<digest>Set the properties for one or more columns of cells.</digest>
--<desc>
-- format and options = NULL by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>cols</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>width</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>options</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  return exec_func(x_worksheet_set_column, {worksheet, cols[1], cols[2],
                   width,format,options}, "+worksheet_set_column")
end function

-----------------------------------------------------------------------------

integer x_worksheet_merge_range   
x_worksheet_merge_range = 0

global function worksheet_merge_range(atom worksheet, sequence range, object string, atom format)
--<function>
--<name>worksheet_merge_range</name>
--<digest>Merge a range of cells.</digest>
--<desc>
-- format = NULL by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>range</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>string</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  if sequence(string) then string = allocate_string(string) end if
  return exec_func(x_worksheet_merge_range, {worksheet,range[1],range[2],
                   range[3],range[4],string,format},
                   "+worksheet_merge_range")
end function

-----------------------------------------------------------------------------

integer x_worksheet_autofilter
x_worksheet_autofilter = 0

global function worksheet_autofilter(atom worksheet, sequence range)
--<function>
--<name>worksheet_autofilter</name>
--<digest>Set the autofilter area in the worksheet.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>range</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  return exec_func(x_worksheet_autofilter, {worksheet,range[1],range[2],
                   range[3],range[4]}, "+worksheet_autofilter")
end function

-----------------------------------------------------------------------------

integer x_worksheet_activate  
x_worksheet_activate = 0

global procedure worksheet_activate(atom worksheet)
--<procedure>
--<name>worksheet_activate</name>
--<digest>Make a worksheet the active, i.e., visible worksheet.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_activate, {worksheet}, "+worksheet_activate")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_select    
x_worksheet_select = 0

global procedure worksheet_select(atom worksheet)
--<procedure>
--<name>worksheet_select</name>
--<digest>Set a worksheet tab as selected.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_select, {worksheet}, "+worksheet_select")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_set_landscape 
x_worksheet_set_landscape = 0

global procedure worksheet_set_landscape(atom worksheet)
--<procedure>
--<name>worksheet_set_landscape</name>
--<digest>Set the page orientation as landscape.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_set_landscape, {worksheet}, "+worksheet_set_landscape")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_set_portrait  
x_worksheet_set_portrait = 0

global procedure worksheet_set_portrait(atom worksheet)
--<procedure>
--<name>worksheet_set_portrait</name>
--<digest>Set the page orientation as portrait.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_set_portrait, {worksheet}, "+worksheet_set_portrait")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_set_page_view 
x_worksheet_set_page_view = 0

global procedure worksheet_set_page_view(atom worksheet)
--<procedure>
--<name>worksheet_set_page_view</name>
--<digest>Set the page layout to page view mode.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_set_page_view, {worksheet}, "+worksheet_set_page_view")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_set_paper 
x_worksheet_set_paper = 0

global procedure worksheet_set_paper(atom worksheet, atom paper_type)
--<procedure>
--<name>worksheet_set_paper</name>
--<digest>Set the paper type for printing.</digest>
--<desc>
--<code>
--   | Index | Paper format         | Paper size        |
--   | ----- | -------------------- | ----------------- |
--   |   0   | Printer default      | Printer default   |
--   |   1   | Letter               | 8 1/2 x 11 in     |
--   |   2   | Letter Small         | 8 1/2 x 11 in     |
--   |   3   | Tabloid              | 11 x 17 in        |
--   |   4   | Ledger               | 17 x 11 in        |
--   |   5   | Legal                | 8 1/2 x 14 in     |
--   |   6   | Statement            | 5 1/2 x 8 1/2 in  |
--   |   7   | Executive            | 7 1/4 x 10 1/2 in |
--   |   8   | A3                   | 297 x 420 mm      |
--   |   9   | A4                   | 210 x 297 mm      |
--   |  10   | A4 Small             | 210 x 297 mm      |
--   |  11   | A5                   | 148 x 210 mm      |
--   |  12   | B4                   | 250 x 354 mm      |
--   |  13   | B5                   | 182 x 257 mm      |
--   |  14   | Folio                | 8 1/2 x 13 in     |
--   |  15   | Quarto               | 215 x 275 mm      |
--   |  16   | ---                  | 1#14 in           |
--   |  17   | ---                  | 11x17 in          |
--   |  18   | Note                 | 8 1/2 x 11 in     |
--   |  19   | Envelope 9           | 3 7/8 x 8 7/8     |
--   |  20   | Envelope 10          | 4 1/8 x 9 1/2     |
--   |  21   | Envelope 11          | 4 1/2 x 10 3/8    |
--   |  22   | Envelope 12          | 4 3/4 x 11        |
--   |  23   | Envelope 14          | 5 x 11 1/2        |
--   |  24   | C size sheet         | ---               |
--   |  25   | D size sheet         | ---               |
--   |  26   | E size sheet         | ---               |
--   |  27   | Envelope DL          | 110 x 220 mm      |
--   |  28   | Envelope C3          | 324 x 458 mm      |
--   |  29   | Envelope C4          | 229 x 324 mm      |
--   |  30   | Envelope C5          | 162 x 229 mm      |
--   |  31   | Envelope C6          | 114 x 162 mm      |
--   |  32   | Envelope C65         | 114 x 229 mm      |
--   |  33   | Envelope B4          | 250 x 353 mm      |
--   |  34   | Envelope B5          | 176 x 250 mm      |
--   |  35   | Envelope B6          | 176 x 125 mm      |
--   |  36   | Envelope             | 110 x 230 mm      |
--   |  37   | Monarch              | 3.875 x 7.5 in    |
--   |  38   | Envelope             | 3 5/8 x 6 1/2 in  |
--   |  39   | Fanfold              | 14 7/8 x 11 in    |
--   |  40   | German Std Fanfold   | 8 1/2 x 12 in     |
--   |  41   | German Legal Fanfold | 8 1/2 x 13 in     |
--</code>
--
-- paper_type = 0 by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>paper_type</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_set_paper, {worksheet, paper_type},
         "+worksheet_set_paper")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_set_margins   
x_worksheet_set_margins = 0

global procedure worksheet_set_margins(atom worksheet, atom left, atom right, atom top, atom bottom)
--<procedure>
--<name>worksheet_set_margins</name>
--<digest>Set the worksheet margins (in inches) for the printed page.</digest>
--<desc>
-- left, right, top and bottom = -1 by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>left</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>right</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>top</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>bottom</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_set_margins, {worksheet,left,right,top,bottom},
         "+worksheet_set_margins")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_set_header, x_worksheet_set_header_opt
x_worksheet_set_header = 0
x_worksheet_set_header_opt = 0

global function worksheet_set_header(atom worksheet, object string, atom options)
--<function>
--<name>worksheet_set_header</name>
--<digest>Set the printed page header caption with (optional) additional options.</digest>
--<desc>
--<code>
--   | Control         | Category      | Description           |
--   | --------------- | ------------- | --------------------- |
--   | `&L`            | Justification | Left                  |
--   | `&C`            |               | Center                |
--   | `&R`            |               | Right                 |
--   | `&P`            | Information   | Page number           |
--   | `&N`            |               | Total number of pages |
--   | `&D`            |               | Date                  |
--   | `&T`            |               | Time                  |
--   | `&F`            |               | File name             |
--   | `&A`            |               | Worksheet name        |
--   | `&Z`            |               | Workbook path         |
--   | `&fontsize`     | Font          | Font size             |
--   | `&"font,style"` |               | Font name and style   |
--   | `&U`            |               | Single underline      |
--   | `&E`            |               | Double underline      |
--   | `&S`            |               | Strikethrough         |
--   | `&X`            |               | Superscript           |
--   | `&Y`            |               | Subscript             |
--</code>
-- options = NULL by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>string</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>options</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>

  if sequence(string) then string = allocate_string(string) end if

  if options = NULL then
    return exec_func(x_worksheet_set_header, {worksheet,string},
                     "+worksheet_set_header")
  else
    return exec_func(x_worksheet_set_header_opt, {worksheet,string,options},
                     "+worksheet_set_header_opt")
  end if

end function

-----------------------------------------------------------------------------

integer x_worksheet_set_footer, x_worksheet_set_footer_opt
x_worksheet_set_footer = 0
x_worksheet_set_footer_opt = 0

global function worksheet_set_footer(atom worksheet, object string, atom options)
--<function>
--<name>worksheet_set_footer</name>
--<digest>Set the printed page footer caption with (optional) additional options.</digest>
--<desc>
-- options = NULL by default
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>string</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>options</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>

  if sequence(string) then string = allocate_string(string) end if

  if options = NULL then
    return exec_func(x_worksheet_set_footer, {worksheet,string},
                      "+worksheet_set_footer")
  else
    return exec_func(x_worksheet_set_footer_opt, {worksheet,string,options},
                      "+worksheet_set_footer_opt")
  end if

end function

-----------------------------------------------------------------------------

integer x_worksheet_set_h_pagebreaks
x_worksheet_set_h_pagebreaks = 0

global procedure worksheet_set_h_pagebreaks(atom worksheet, object breaks)
--<procedure>
--<name>worksheet_set_h_pagebreaks</name>
--<digest>Set the horizontal page breaks on a worksheet.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>breaks</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  if sequence(breaks) then breaks = allocate_array4(breaks) end if
  c_proc(x_worksheet_set_h_pagebreaks, {worksheet,breaks},
         "+worksheet_set_h_pagebreaks")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_set_v_pagebreaks
x_worksheet_set_v_pagebreaks = 0

global procedure worksheet_set_v_pagebreaks(atom worksheet, object breaks)
--<procedure>
--<name>worksheet_set_v_pagebreaks</name>
--<digest>Set the vertical page breaks on a worksheet.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>breaks</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  if sequence(breaks) then breaks = allocate_array4(breaks) end if
  c_proc(x_worksheet_set_v_pagebreaks, {worksheet,breaks},
         "+worksheet_set_v_pagebreaks")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_print_across  
x_worksheet_print_across = 0

global procedure worksheet_print_across(atom worksheet)
--<procedure>
--<name>worksheet_print_across</name>
--<digest>Set the order in which pages are printed.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_print_across, {worksheet}, "+worksheet_print_across")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_gridlines 
x_worksheet_gridlines = 0

global procedure worksheet_gridlines(atom worksheet, atom option)
--<procedure>
--<name>worksheet_gridlines</name>
--<digest>Set the option to display or hide gridlines on the screen and the printed page.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>option</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_gridlines, {worksheet,option}, "+worksheet_gridlines")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_center_horizontally
x_worksheet_center_horizontally = 0

global procedure worksheet_center_horizontally(atom worksheet)
--<procedure>
--<name>worksheet_center_horizontally</name>
--<digest>Center the printed page horizontally.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_center_horizontally, {worksheet},
         "+worksheet_center_horizontally")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_center_vertically 
x_worksheet_center_vertically = 0

global procedure worksheet_center_vertically(atom worksheet)
--<procedure>
--<name>worksheet_center_vertically</name>
--<digest>Center the printed page vertically.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_center_vertically, {worksheet},
         "+worksheet_center_vertically")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_print_row_col_headers
x_worksheet_print_row_col_headers = 0

global procedure worksheet_print_row_col_headers(atom worksheet)
--<procedure>
--<name>worksheet_print_row_col_headers</name>
--<digest>Set the option to print the row and column headers on the printed page.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_print_row_col_headers, {worksheet},
         "+worksheet_print_row_col_headers")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_repeat_rows   
x_worksheet_repeat_rows = 0

global function worksheet_repeat_rows(atom worksheet, sequence rows)
--<function>
--<name>worksheet_repeat_rows</name>
--<digest>Set the number of rows to repeat at the top of each printed page.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>rows</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  return exec_func(x_worksheet_repeat_rows, {worksheet,rows[1],rows[2]},
                   "+worksheet_repeat_rows")
end function

-----------------------------------------------------------------------------

integer x_worksheet_repeat_columns
x_worksheet_repeat_columns = 0

global function worksheet_repeat_columns(atom worksheet, sequence cols)
--<function>
--<name>worksheet_repeat_columns</name>
--<digest>Set the number of columns to repeat at the top of each printed page.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>cols</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  return exec_func(x_worksheet_repeat_columns, {worksheet, cols[1],
                   cols[2]}, "+worksheet_repeat_columns")
end function

-----------------------------------------------------------------------------

integer x_worksheet_print_area
x_worksheet_print_area = 0

global function worksheet_print_area(atom worksheet, sequence range)
--<function>
--<name>worksheet_print_area</name>
--<digest>Set the print area for a worksheet.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>sequence</type>
--<name>range</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  return exec_func(x_worksheet_print_area, {worksheet,range[1],range[2],
                   range[3],range[4]}, "+worksheet_print_area")
end function

-----------------------------------------------------------------------------

integer x_worksheet_fit_to_pages  
x_worksheet_fit_to_pages = 0

global procedure worksheet_fit_to_pages(atom worksheet, atom width, atom height)
--<procedure>
--<name>worksheet_fit_to_pages</name>
--<digest>Fit the printed area to a specific number of pages both vertically and horizontally.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>width</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>height</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_fit_to_pages, {worksheet,width,height},
         "+worksheet_fit_to_pages")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_set_start_page
x_worksheet_set_start_page = 0

global procedure worksheet_set_start_page(atom worksheet, atom start_page)
--<procedure>
--<name>worksheet_set_start_page</name>
--<digest>Set the start page number when printing.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>start_page</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_set_start_page, {worksheet,start_page},
         "+worksheet_set_start_page")
end procedure

-----------------------------------------------------------------------------

integer x_worksheet_set_print_scale
x_worksheet_set_print_scale = 0

global procedure worksheet_set_print_scale(atom worksheet, atom scale)
--<procedure>
--<name>worksheet_set_print_scale</name>
--<digest>Set the scale factor for the printed page.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>worksheet</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>scale</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_worksheet_set_print_scale, {worksheet,scale},
         "+worksheet_set_print_scale")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_font_name   
x_format_set_font_name = 0

global procedure format_set_font_name(atom format, object name)
--<procedure>
--<name>format_set_font_name</name>
--<digest>Set the font used in the cell.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>name</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  if sequence(name) then name = allocate_string(name) end if
  c_proc(x_format_set_font_name, {format,name}, "+format_set_font_name")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_font_size   
x_format_set_font_size = 0

global procedure format_set_font_size(atom format, atom size)
--<procedure>
--<name>format_set_font_size</name>
--<digest>Set the size of the font used in the cell.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>size</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_font_size, {format,size}, "+format_set_font_size")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_font_color  
x_format_set_font_color = 0

global procedure format_set_font_color(atom format, atom color)
--<procedure>
--<name>format_set_font_color</name>
--<digest>Set the color of the font used in the cell.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>color</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_font_color, {format, color}, "+format_set_font_color")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_bold    
x_format_set_bold = 0

global procedure format_set_bold(atom format)
--<procedure>
--<name>format_set_bold</name>
--<digest>Turn on bold for the format font.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_bold, {format}, "+format_set_bold")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_italic  
x_format_set_italic = 0

global procedure format_set_italic(atom format)
--<procedure>
--<name>format_set_italic</name>
--<digest>Turn on italic for the format font.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_italic, {format}, "+format_set_italic")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_underline   
x_format_set_underline = 0

global procedure format_set_underline(atom format, atom style)
--<procedure>
--<name>format_set_underline</name>
--<digest>Turn on underline for the format.</digest>
--<desc>
-- The available underline styles are:
-- - LXW_UNDERLINE_SINGLE
-- - LXW_UNDERLINE_DOUBLE
-- - LXW_UNDERLINE_SINGLE_ACCOUNTING
-- - LXW_UNDERLINE_DOUBLE_ACCOUNTING
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>style</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_underline, {format,style}, "+format_set_underline")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_font_strikeout
x_format_set_font_strikeout = 0

global procedure format_set_font_strikeout(atom format)
--<procedure>
--<name>format_set_font_strikeout</name>
--<digest>Set the strikeout property of the font.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_font_strikeout, {format}, "+format_set_font_strikeout")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_font_script 
x_format_set_font_script = 0

global procedure format_set_font_script(atom format, atom script)
--<procedure>
--<name>format_set_font_script</name>
--<digest>Set the superscript/subscript property of the font.</digest>
--<desc>
-- The available script styles are:
-- - LXW_FONT_SUPERSCRIPT
-- - LXW_FONT_SUBSCRIPT
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>script</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_font_script, {format,script}, "+format_set_font_script")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_num_format  
x_format_set_num_format = 0

global procedure format_set_num_format(atom format, object num_format)
--<procedure>
--<name>format_set_num_format</name>
--<digest>Set the number format for a cell.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>num_format</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  if sequence(num_format) then num_format = allocate_string(num_format) end if
  c_proc(x_format_set_num_format, {format,num_format}, "+format_set_num_format")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_num_format_index
x_format_set_num_format_index = 0

global procedure format_set_num_format_index(atom format, atom index)
--<procedure>
--<name>format_set_num_format_index</name>
--<digest>Set the Excel built-in number format for a cell.</digest>
--<desc>
-- The Excel built-in number formats as shown in the table below:
--
--<code>
--   | Index | Index | Format String                                        |
--   | ----- | ----- | ---------------------------------------------------- |
--   |   0   |  #00  | `General`                                            |
--   |   1   |  #01  | `0`                                                  |
--   |   2   |  #02  | `0.00`                                               |
--   |   3   |  #03  | `#,##0`                                              |
--   |   4   |  #04  | `#,##0.00`                                           |
--   |   5   |  #05  | `($#,##0_);($#,##0)`                                 |
--   |   6   |  #06  | `($#,##0_);[Red]($#,##0)`                            |
--   |   7   |  #07  | `($#,##0.00_);($#,##0.00)`                           |
--   |   8   |  #08  | `($#,##0.00_);[Red]($#,##0.00)`                      |
--   |   9   |  #09  | `0%`                                                 |
--   |  10   |  #0a  | `0.00%`                                              |
--   |  11   |  #0b  | `0.00E+00`                                           |
--   |  12   |  #0c  | `# ?/?`                                              |
--   |  13   |  #0d  | `# ??/??`                                            |
--   |  14   |  #0e  | `m/d/yy`                                             |
--   |  15   |  #0f  | `d-mmm-yy`                                           |
--   |  16   |  #10  | `d-mmm`                                              |
--   |  17   |  #11  | `mmm-yy`                                             |
--   |  18   |  #12  | `h:mm AM/PM`                                         |
--   |  19   |  #13  | `h:mm:ss AM/PM`                                      |
--   |  20   |  #14  | `h:mm`                                               |
--   |  21   |  #15  | `h:mm:ss`                                            |
--   |  22   |  #16  | `m/d/yy h:mm`                                        |
--   | ...   |  ...  | ...                                                  |
--   |  37   |  #25  | `(#,##0_);(#,##0)`                                   |
--   |  38   |  #26  | `(#,##0_);[Red](#,##0)`                              |
--   |  39   |  #27  | `(#,##0.00_);(#,##0.00)`                             |
--   |  40   |  #28  | `(#,##0.00_);[Red](#,##0.00)`                        |
--   |  41   |  #29  | `_(* #,##0_);_(* (#,##0);_(* "-"_);_(@_)`            |
--   |  42   |  #2a  | `_($* #,##0_);_($* (#,##0);_($* "-"_);_(@_)`         |
--   |  43   |  #2b  | `_(* #,##0.00_);_(* (#,##0.00);_(* "-"??_);_(@_)`    |
--   |  44   |  #2c  | `_($* #,##0.00_);_($* (#,##0.00);_($* "-"??_);_(@_)` |
--   |  45   |  #2d  | `mm:ss`                                              |
--   |  46   |  #2e  | `[h]:mm:ss`                                          |
--   |  47   |  #2f  | `mm:ss.0`                                            |
--   |  48   |  #30  | `##0.0E+0`                                           |
--   |  49   |  #31  | `@`                                                  |
--</code>
--
-- Notes:
--  -  Numeric formats 23 to 36 are not documented by Microsoft and may differ in international
--     versions. The listed date and currency formats may also vary depending on system settings.
--
--  - The dollar sign in the above format appears as the defined local currency symbol.
--
--  - These formats can also be set via format_set_num_format().
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>index</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_num_format_index, {format, index}, "+format_set_num_format_index")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_unlocked
x_format_set_unlocked = 0

global procedure format_set_unlocked(atom format)
--<procedure>
--<name>format_set_unlocked</name>
--<digest>Set the cell unlocked state.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_unlocked, {format}, "+format_set_unlocked")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_hidden  
x_format_set_hidden = 0

global procedure format_set_hidden(atom format)
--<procedure>
--<name>format_set_hidden</name>
--<digest>Hide formulas in a cell.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_hidden, {format}, "+format_set_hidden")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_align
x_format_set_align = 0

global procedure format_set_align(atom format, atom align)
--<procedure>
--<name>format_set_align</name>
--<digest>Set the alignment for data in the cell.</digest>
--<desc>
-- The following are the available horizontal alignments:
-- - LXW_ALIGN_LEFT
-- - LXW_ALIGN_CENTER
-- - LXW_ALIGN_RIGHT
-- - LXW_ALIGN_FILL
-- - LXW_ALIGN_JUSTIFY
-- - LXW_ALIGN_CENTER_ACROSS
-- - LXW_ALIGN_DISTRIBUTED
--
-- The following are the available vertical alignments:
-- - LXW_ALIGN_VERTICAL_TOP
-- - LXW_ALIGN_VERTICAL_BOTTOM
-- - LXW_ALIGN_VERTICAL_CENTER
-- - LXW_ALIGN_VERTICAL_JUSTIFY
-- - LXW_ALIGN_VERTICAL_DISTRIBUTED
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>align</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_align, {format,align}, "+format_set_align")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_text_wrap   
x_format_set_text_wrap = 0

global procedure format_set_text_wrap(atom format)
--<procedure>
--<name>format_set_text_wrap</name>
--<digest>Wrap text in a cell.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_text_wrap, {format}, "+format_set_text_wrap")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_rotation
x_format_set_rotation = 0

global procedure format_set_rotation(atom format, atom angle)
--<procedure>
--<name>format_set_rotation</name>
--<digest>Set the rotation of the text in a cell.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>angle</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_rotation, {format,angle}, "+format_set_rotation")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_indent  
x_format_set_indent = 0

global procedure format_set_indent(atom format, atom indent)
--<procedure>
--<name>format_set_indent</name>
--<digest>Set the cell text indentation level.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>indent</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_indent, {format, indent}, "+format_set_indent")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_shrink  
x_format_set_shrink = 0

global procedure format_set_shrink(atom format)
--<procedure>
--<name>format_set_shrink</name>
--<digest>Turn on the text "shrink to fit" for a cell.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_shrink, {format}, "+format_set_shrink")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_pattern 
x_format_set_pattern = 0

global procedure format_set_pattern(atom format, atom index)
--<procedure>
--<name>format_set_pattern</name>
--<digest>Set the background fill pattern for a cell</digest>
--<desc>
--<code>
--   | Fill Type                     | Define                       |
--   | ----------------------------- | ---------------------------- |
--   | Solid                         | LXW_PATTERN_SOLID            |
--   | Medium gray                   | LXW_PATTERN_MEDIUM_GRAY      |
--   | Dark gray                     | LXW_PATTERN_DARK_GRAY        |
--   | Light gray                    | LXW_PATTERN_LIGHT_GRAY       |
--   | Dark horizontal line          | LXW_PATTERN_DARK_HORIZONTAL  |
--   | Dark vertical line            | LXW_PATTERN_DARK_VERTICAL    |
--   | Dark diagonal stripe          | LXW_PATTERN_DARK_DOWN        |
--   | Reverse dark diagonal stripe  | LXW_PATTERN_DARK_UP          |
--   | Dark grid                     | LXW_PATTERN_DARK_GRID        |
--   | Dark trellis                  | LXW_PATTERN_DARK_TRELLIS     |
--   | Light horizontal line         | LXW_PATTERN_LIGHT_HORIZONTAL |
--   | Light vertical line           | LXW_PATTERN_LIGHT_VERTICAL   |
--   | Light diagonal stripe         | LXW_PATTERN_LIGHT_DOWN       |
--   | Reverse light diagonal stripe | LXW_PATTERN_LIGHT_UP         |
--   | Light grid                    | LXW_PATTERN_LIGHT_GRID       |
--   | Light trellis                 | LXW_PATTERN_LIGHT_TRELLIS    |
--   | 12.5% gray                    | LXW_PATTERN_GRAY_125         |
--   | 6.25% gray                    | LXW_PATTERN_GRAY_0625        |
--</code>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>index</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_pattern, {format, index}, "+format_set_pattern")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_bg_color
x_format_set_bg_color = 0

global procedure format_set_bg_color(atom format, atom color)
--<procedure>
--<name>format_set_bg_color</name>
--<digest>Set the pattern background color for a cell.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>color</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_bg_color, {format, color}, "+format_set_bg_color")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_fg_color
x_format_set_fg_color = 0

global procedure format_set_fg_color(atom format, atom color)
--<procedure>
--<name>format_set_fg_color</name>
--<digest>Set the pattern foreground color for a cell.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>color</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_fg_color, {format, color}, "+format_set_fg_color")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_bottom  
x_format_set_bottom = 0

global procedure format_set_bottom(atom format, atom style)
--<procedure>
--<name>format_set_bottom</name>
--<digest>Set the cell bottom border style.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>style</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_bottom, {format,style}, "+format_set_bottom")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_top     
x_format_set_top = 0

global procedure format_set_top(atom format, atom style)
--<procedure>
--<name>format_set_top</name>
--<digest>Set the cell top border style.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>style</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_top, {format,style}, "+format_set_top")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_left    
x_format_set_left = 0

global procedure format_set_left(atom format, atom style)
--<procedure>
--<name>format_set_left</name>
--<digest>Set the cell left border style.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>style</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_left, {format,style}, "+format_set_left")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_right   
x_format_set_right = 0

global procedure format_set_right(atom format, atom style)
--<procedure>
--<name>format_set_right</name>
--<digest>Set the cell right border style.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>style</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_right, {format,style}, "+format_set_right")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_border  
x_format_set_border = 0

global procedure format_set_border(atom format, object style)
--<procedure>
--<name>format_set_border</name>
--<digest>Set the cell border style.</digest>
--<desc>
-- The following border styles are available:
-- - LXW_BORDER_THIN
-- - LXW_BORDER_MEDIUM
-- - LXW_BORDER_DASHED
-- - LXW_BORDER_DOTTED
-- - LXW_BORDER_THICK
-- - LXW_BORDER_DOUBLE
-- - LXW_BORDER_HAIR
-- - LXW_BORDER_MEDIUM_DASHED
-- - LXW_BORDER_DASH_DOT
-- - LXW_BORDER_MEDIUM_DASH_DOT
-- - LXW_BORDER_DASH_DOT_DOT
-- - LXW_BORDER_MEDIUM_DASH_DOT_DOT
-- - LXW_BORDER_SLANT_DASH_DOT
--
-- The 'style' can be a single atom for all borders, or a sequence
-- of four values for each border, e.g. {left, top, right, bottom}.
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>style</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  if sequence(style) then
    c_proc(x_format_set_left,   {format,style[1]}, "+format_set_left")
    c_proc(x_format_set_top,    {format,style[2]}, "+format_set_top")
    c_proc(x_format_set_right,  {format,style[3]}, "+format_set_right")
    c_proc(x_format_set_bottom, {format,style[4]}, "+format_set_bottom")
  else
    c_proc(x_format_set_border, {format,style}, "+format_set_border")
  end if
end procedure

-----------------------------------------------------------------------------

integer x_format_set_bottom_color
x_format_set_bottom_color = 0

global procedure format_set_bottom_color(atom format, atom color)
--<procedure>
--<name>format_set_bottom_color</name>
--<digest>Set the color of the bottom cell border.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>color</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_bottom_color, {format, color}, "+format_set_bottom_color")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_top_color   
x_format_set_top_color = 0

global procedure format_set_top_color(atom format, atom color)
--<procedure>
--<name>format_set_top_color</name>
--<digest>Set the color of the top cell border.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>color</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_top_color, {format, color}, "+format_set_top_color")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_left_color  
x_format_set_left_color = 0

global procedure format_set_left_color(atom format, atom color)
--<procedure>
--<name>format_set_left_color</name>
--<digest>Set the color of the left cell border.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>color</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_left_color, {format, color}, "+format_set_left_color")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_right_color 
x_format_set_right_color = 0

global procedure format_set_right_color(atom format, atom color)
--<procedure>
--<name>format_set_right_color</name>
--<digest>Set the color of the right cell border.</digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>color</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_right_color, {format, color}, "+format_set_right_color")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_border_color
x_format_set_border_color = 0

global procedure format_set_border_color(atom format, object color)
--<procedure>
--<name>format_set_border_color</name>
--<digest>Set the color of the cell border.</digest>
--<desc>
-- The 'color' can be a single atom for all borders, or a sequence
-- of four values for each border, e.g. {left, top, right, bottom}.
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>color</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  if sequence(color) then
    c_proc(x_format_set_left_color,   {format, color[1]}, "+format_set_left_color")
    c_proc(x_format_set_top_color,    {format, color[2]}, "+format_set_top_color")
    c_proc(x_format_set_right_color,  {format, color[3]}, "+format_set_right_color")
    c_proc(x_format_set_bottom_color, {format, color[4]}, "+format_set_bottom_color")
  else
    c_proc(x_format_set_border_color, {format, color}, "+format_set_border_color")
  end if
end procedure

-----------------------------------------------------------------------------

integer x_format_set_diag_type   
x_format_set_diag_type = 0

global procedure format_set_diag_type(atom format, atom value)
--<procedure>
--<name>format_set_diag_type</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>value</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_diag_type, {format,value}, "+format_set_diag_type")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_diag_color  
x_format_set_diag_color = 0

global procedure format_set_diag_color(atom format, atom color)
--<procedure>
--<name>format_set_diag_color</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>color</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_diag_color, {format, color}, "+format_set_diag_color")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_diag_border 
x_format_set_diag_border = 0

global procedure format_set_diag_border(atom format, atom value)
--<procedure>
--<name>format_set_diag_border</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>value</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_diag_border, {format,value}, "+format_set_diag_border")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_font_outline
x_format_set_font_outline = 0

global procedure format_set_font_outline(atom format)
--<procedure>
--<name>format_set_font_outline</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_font_outline, {format}, "+format_set_font_outline")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_font_shadow 
x_format_set_font_shadow = 0

global procedure format_set_font_shadow(atom format)
--<procedure>
--<name>format_set_font_shadow</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_font_shadow, {format}, "+format_set_font_shadow")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_font_family 
x_format_set_font_family = 0

global procedure format_set_font_family(atom format, atom value)
--<procedure>
--<name>format_set_font_family</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>value</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_font_family, {format,value}, "+format_set_font_family")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_font_charset
x_format_set_font_charset = 0

global procedure format_set_font_charset(atom format, atom value)
--<procedure>
--<name>format_set_font_charset</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>value</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_font_charset, {format,value}, "+format_set_font_charset")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_font_scheme 
x_format_set_font_scheme = 0

global procedure format_set_font_scheme(atom format, object scheme)
--<procedure>
--<name>format_set_font_scheme</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>object</type>
--<name>scheme</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  if sequence(scheme) then scheme = allocate_string(scheme) end if
  c_proc(x_format_set_font_scheme, {format,scheme}, "+format_set_font_scheme")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_font_condense
x_format_set_font_condense = 0

global procedure format_set_font_condense(atom format)
--<procedure>
--<name>format_set_font_condense</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_font_condense, {format}, "+format_set_font_condense")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_font_extend 
x_format_set_font_extend = 0

global procedure format_set_font_extend(atom format)
--<procedure>
--<name>format_set_font_extend</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_font_extend, {format}, "+format_set_font_extend")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_reading_order
x_format_set_reading_order = 0

global procedure format_set_reading_order(atom format, atom value)
--<procedure>
--<name>format_set_reading_order</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>value</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_reading_order, {format,value},
         "+format_set_reading_order")
end procedure

-----------------------------------------------------------------------------

integer x_format_set_theme   
x_format_set_theme = 0

global procedure format_set_theme(atom format, atom value)
--<procedure>
--<name>format_set_theme</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>atom</type>
--<name>format</name>
--<desc></desc>
--</param>
--<param>
--<type>atom</type>
--<name>value</name>
--<desc></desc>
--</param>
--<example>
--</example>
--<see_also>
--</see_also>
--</procedure>
  c_proc(x_format_set_theme, {format,value}, "+format_set_theme")
end procedure

-----------------------------------------------------------------------------

integer x_lxw_quote_sheetname 
x_lxw_quote_sheetname = 0

global function lxw_quote_sheetname(object str)
--<function>
--<name>lxw_quote_sheetname</name>
--<digest></digest>
--<desc>
--</desc>
--<param>
--<type>object</type>
--<name>str</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom ptr
  
  if sequence(str) then str = allocate_string(str) end if
  ptr = exec_func(x_lxw_quote_sheetname, {str}, "+lxw_quote_sheetname")
  return peek_string(ptr)
end function

-----------------------------------------------------------------------------

integer x_lxw_get_row         
x_lxw_get_row = 0

function lxw_get_row(object row_str)
  if sequence(row_str) then row_str = allocate_string(row_str) end if
  return exec_func(x_lxw_get_row, {row_str}, "+lxw_get_row")
end function

-----------------------------------------------------------------------------

integer x_lxw_get_col         
x_lxw_get_col = 0

function lxw_get_col(object row_str)
  if sequence(row_str) then row_str = allocate_string(row_str) end if
  return exec_func(x_lxw_get_col, {row_str}, "+lxw_get_col")
end function

-----------------------------------------------------------------------------

integer x_lxw_get_row_2       
x_lxw_get_row_2 = 0

function lxw_get_row_2(object row_str)
  if sequence(row_str) then row_str = allocate_string(row_str) end if
  return exec_func(x_lxw_get_row_2, {row_str}, "+lxw_get_row_2")
end function

-----------------------------------------------------------------------------

integer x_lxw_get_col_2       
x_lxw_get_col_2 = 0

function lxw_get_col_2(object row_str)
  if sequence(row_str) then row_str = allocate_string(row_str) end if
  return exec_func(x_lxw_get_col_2, {row_str}, "+lxw_get_col_2")
end function

-----------------------------------------------------------------------------

global function CELL(sequence cell)
--<function>
--<name>CELL</name>
--<digest>Convert an Excel `A1` cell string into a `(row, col)` sequence.</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>cell</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom p_cell

  p_cell = allocate_string(cell)
  return { lxw_get_row(p_cell), lxw_get_col(p_cell) }
end function

-----------------------------------------------------------------------------

global function COLS(sequence cols)
--<function>
--<name>COLS</name>
--<digest>Convert an Excel `A:B` column range into a `(col1, col2)` sequence.</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>cols</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom p_cols

  p_cols = allocate_string(cols)
  return { lxw_get_col(p_cols), lxw_get_col_2(p_cols) }
end function

-----------------------------------------------------------------------------

global function RANGE(sequence range)
--<function>
--<name>RANGE</name>
--<digest>Convert an Excel `A1:B2` range into a `(first_row, first_col, last_row, last_col)` sequence.</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>range</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom p_range

  p_range = allocate_string(range)
  return { lxw_get_row(range), lxw_get_col(range), lxw_get_row_2(range), lxw_get_col_2(range) }
end function

-----------------------------------------------------------------------------

global function ROWS(sequence rows)
--<function>
--<name>ROWS</name>
--<digest>Convert an Excel `A1:B2` range into a `(row1, row2)` sequence.</digest>
--<desc>
--</desc>
--<param>
--<type>sequence</type>
--<name>rows</name>
--<desc></desc>
--</param>
--<return>
--</return>
--<example>
--</example>
--<see_also>
--</see_also>
--</function>
  atom p_rows

  p_rows = allocate_string(rows)
  return { lxw_get_row(p_rows), lxw_get_row_2(p_rows) }
end function
